// CablanDoc.h : interface of the CCablanDoc class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_CABLANDOC_H__1DF2159F_8ED8_472F_B753_F170381AA08E__INCLUDED_)
#define AFX_CABLANDOC_H__1DF2159F_8ED8_472F_B753_F170381AA08E__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


class CCablanDoc : public CDocument
{
protected: // create from serialization only
	CCablanDoc();
	DECLARE_DYNCREATE(CCablanDoc)

// Attributes
public:



//****************************************************************************
//Cable analsys
//****************************************************************************


#define MAXCOLUMNS 700//go to DialogPathAnalysis.h. 
#define MAXROWS 700
#define MAXLANDINGS 91//also is in DialogLandingCosts.h
#define MAXCABLEWAYS 37
#define MAXCABLESYSTEMS 6
#define MAXCELLS 101 //maximum number of cells in a cableway - change MAXCELLS, if you change this.
#define MAXINTSUPPORT 20  //maximum number of intermediate supports in a cableway - change MAXCELLS, if you change this.
#define MAXROADLENGTH 1000 //maximum number of cells that a road route can hold.
#define MAXCONTOURLINES 10//maximum number of contour lines passing through each cell.
#define MAXUNITBOUNDARYVERTICES 50//maximum number of vertices of harvet unit boundary.
#define MAXTIMBEREXITS 20//maximum number of timber exits.
#define MINCELLALONGCABLEROAD 5

	//Woodam's Global variables

	struct DTMCOORD//coordinate should be all positive values. Program would recognize negative value is beyond boundary. 
	{
		int x;
		int y;
		int z;
	} DTMCoord[MAXCOLUMNS][MAXROWS];//[column;x][row;y]      10mx10m grids  -- covering 100ha ;

	struct CONTOUR
	{
		CPoint Start;
		CPoint End;
		int Elevation;
	} Contour[MAXCOLUMNS][MAXROWS][MAXCONTOURLINES];//keep coordinate of the line passes through the cell. 
	short NumberofContourlinesWithinThisCell[MAXCOLUMNS][MAXROWS];//keep how many contour lines passing through each cell-->maximum 5 lines.
	
	int ContourInterval;

	float Volume[MAXCOLUMNS][MAXROWS];//column first then rows -> following EXCEL address and x & y axis
	float OriginalVolume[MAXCOLUMNS][MAXROWS];//contains original volume from GIS layer. Volume[] will change after QUADTREE.

	short FishStreams[MAXCOLUMNS][MAXROWS];//input stream label
	short FishStreamBuffer[MAXCOLUMNS][MAXROWS];//volume remove will be restricted inside stream buffer.
	short FishStreamBufferWidth;//in meters.

	short ExistingRoads[MAXCOLUMNS][MAXROWS];//input road layer
	
	short ChokerLength;//used for clumping volume cells together; in meters.

	short HarvestArea[MAXCOLUMNS][MAXROWS];//if the cell falls within the unit boundary, then have integer value from the GIS layer.


	struct LANDINGS
	{
		int row;
		int column;//when [0], this keeps the total number of landing alternatives available
		int CableSystem;//Cable system ID.	
	} Landings[MAXLANDINGS];//maximum 100 alternative landings 

	bool boolLandingPayloadAnalysis[MAXLANDINGS];//if payload analysis is done for this landing, then this variable has 'TRUE' until the user changes the operational data and system parameters.


	int DTMcolumns;
	int DTMrows;
	int DTMminX;
	int DTMminY;
	int DTMminZ;
	int DTMmaxX;
	int DTMmaxY;
	int DTMmaxZ;
	int DTMcellsize;


	struct UNITBOUNDARY
	{
		int startCol;
		int startRow;
		int endCol;
		int endRow;
	} UnitBoundary;

	struct HARVESTUNITBOUNDARY
	{
		int col;
		int row;
	} HarvestUnitBoundary[MAXUNITBOUNDARYVERTICES];
	
	short TotalNumberofUnitBoundaryVertices;
	bool CellsWithinUnitBoundary[MAXCOLUMNS][MAXROWS];//this is the main variable. cf) HarvestArea[][] is defined from the GIS layer and the data will be transfered to CellsWithinUnitBoundary[][] in SettingHarvestingUnitBoundary() in View.


	struct CABLEWAYS
	{
		int tailcolumn;//after set tail location from payload analysis
		int tailrow;//after set tail location from payload analysis
		int endcolumn;//the end of cableway from the miximum yarding distance
		int endrow;//the end of cableway from the miximum yarding distance
		int taillocationID;//taillocationID for Profile[]
		int totalnumberofprofilecells;//total number of cells on profile
		float slope;//linear equation based on real coordinate (x, y)
		float intercept;//linear equation based on real coordinate (x, y)
		int totalnumberofINTs;//totalNumberof INT supports.
	} Cableways[MAXLANDINGS][MAXCABLEWAYS];//each landing, each system can have 360 alternative cableways.

	
	CABLEWAYS tempCableways[MAXCABLEWAYS];//temporary global variable which is used for calling OnCablewaysGenerate() subroutine in View.

	struct CABLESYSTEMS
	{
		char CableSystemName[20];//name (system and carriage)
		float maxyarddist;//in feet
		float maxlateraldist;//in feet  when [0], this keeps the total number of cable systems available
		float towerheight;//in feet
		float intermediatesupportheight;//in feet
		float maxspartreeheight;
		float minspartreeheight;
		//user-defined payload anaylsis data and variables. All are in metric units
		float gbTmax;//maximum skyline tension.
		float gbMmax;//maximum mainline tension.
		float gbW1;
		float gbW3;
		float gbW4;
		float gbWc;
		float gbLC;
		float gbCarriageDepth;
		float gbL;
		float gbDLog;
		float gbBeta;
		float gbMyu;
		float gbClearance;
		float gbAllowableDeflection;
		float gbMinSlopewithoutHBL;//minimum slope for tower without haulback in percent.
		float gbMaxSlopeChange;//maximum allowable change in chord slope between adjacent spans
		int gbMaxINTSupports;//limit of number of INT supports.
		bool boolSpecChangebyUser;//if any of user-defined variables changes, then TRUE;
		bool English;//TRUE:English unit
		bool HaulbackLine;//TRUE:Haulback line available
		bool Downhill;//TRUE:Downhill available

		//other opeartion data
		float gbDensity;//2200 lb / cubic meter  -> kg / cubic meter
		float gbEngineEfficiency;//65% of engine power
		float gbEnginePower;//horse power
		float gbHookTime;// sec/ton(cubic meter)
		float gbRiggingCosts;//sec. per intermediate support and tailspar
		float gbUnhookTime;// sec/ton(cubic meter)
		float gbUnitCost;//hourly cost of tower operation
		float gbDelayPercentage;//delay time as a percentage of total operating time
		float gbFellingCosts;//felling costs per cubic meter
		float gbMoveInCosts;//system move-in and installing costs
		float gbYarderSetupHour;//initial yarder setup hour per landing
		float gbCableRoadChangeHour;//skyline (cableway) road change time
		float gbUserMaxPayload;//6600 lb  -> kg
		float gbUserMinPayload;//2200 lb  -> kg
		float gbLoadFactor;//percent of skyline capacity actually used
		float gbInhaulSpeedLimit;//speed limit when partial suspension.. speed should not be greater than this limit.(m/sec)
		float gbInhaulSpeedLimitFull;//speed limit when full suspension.. 
		float gbOuthaulSpeedLimit;//speed limit when outhauling..
		float gbLateralYardingSpeed;//average speed for lateral yarding
		float gbPenaltyForBeyondLateralYarding;//penalize the cells which are beyond the leach of the carriage.
		float gbMultiplierLYD1;//multiplier to lateral yarding time for the first zone (0-1/3 of LYD)
		float gbMultiplierLYD2;//multiplier to lateral yarding time for the first zone (1/3 - 2/3 of LYD)
		float gbMultiplierLYD3;//multiplier to lateral yarding time for the first zone (2/3 - 1 of LYD)
		float gbGiveUpCellDistance;//if the cell is located beyond this distance from the cable, then give up the cell.
	} CableSystems[MAXCABLESYSTEMS];

	int gbCurrentCableSystem;//global variable for CableSystem
	int TotalNumberofLandings;//number of candidate tower location.
	int TotalNumberofActualLandings;//this is less than TotalNumberofLandings since some landings can share the same location.
	int TotalNumberofSystems;


	//results from cable path analysis
	float Payload[MAXLANDINGS][MAXCABLEWAYS];//[landingID][CablewayID]
	float PayloadAtEachTP[MAXLANDINGS][MAXCABLEWAYS][MAXCELLS];
	float CycleTime[MAXLANDINGS][MAXCABLEWAYS];//[landingID][CablewayID]
	float CycleTimeAtEachTP[MAXLANDINGS][MAXCABLEWAYS][MAXCELLS];
	float TailSparTreeHeight[MAXLANDINGS][MAXCABLEWAYS];
	
		
	
	//COSTS
	struct COSTS
	{
		float Fixed;
		float Variable;
	} CableCosts;


	//Road costs
	float HaulingCosts;
	float RGFactor1;
	float RGFactor2;
	float RGFactor3;
	float RGFactor4;
	float RGFactor5;
	float RGFactor6;
	float RGFactor7;
	float RoadCosts;
	float SSFactor1;
	float SSFactor2;
	float SSFactor3;
	float SSFactor4;
	float SSFactor5;
	float SSFactor6;
	float SSFactor7;
	float RGLimit;
	float SSLimit;
	float MultiplierForRoadsOnStream;//penalty for increasing construction costs.

	//Landing costs
	float LandingCosts[MAXLANDINGS];//each landing either has landing construction costs or shares the same landing with another tower location.
	int LandingSharingTowerID[MAXLANDINGS];
	

	//file name
	CString DEMFile, VolumeFile, StreamFile, RoadFile, HarvestAreaFile;
	CString LandingFile, CableRoadFile;

	


	//ROAD
//	CPoint AccessRoadtoLanding[MAXLANDINGS][MAXROADLENGTH];

	
	//automated landing location search
	CPoint BoundaryStart, BoundaryEnd;//keep coordinates for starting and ending
	bool blLandingBoundarySetting;//if this is true, then user is trying to set landing boundary.




//*********************************************************************************
//NETWORK Analysis
//*********************************************************************************
	
	//NETWORK
	//cable system network
	bool bCoveredCell[MAXCOLUMNS][MAXROWS];//True, if the cell is covered any of cable lines. Otherwise, false.

	//destination
	struct TIMBEREXITS
	{
		int row;
		int column;
	} TimberExits[MAXTIMBEREXITS]; //maximum 20 destinations in whole unit.

	int TotalNumberofTimberExits;


	/////////////////////////////////////////////////////
	//From NETWORK2000
	/////////////////////////////////////////////////////
//Woodam's code
#define MAX_LINKS 500002//Maximum number of links//add 2 more than real capacity just for safety--not more than 99999 since CurrentPath string include 5 digit nodenumber.
#define MAX_SALES 20002//Maximum number of links//add 2 more than real capacity just for safety--should not be bigger than 60000 (maximum value for unsigned short).
#define MAX_CHAR 11//Maximum length of label for nodes. 10 letters max.
#define MAX_COL 4//number of column in MSFlexGrid


	int TotalNumberofLinks;
	int TotalNumberofNodes;

	struct LINKS
	{
		char FromNode[MAX_CHAR];
		char ToNode[MAX_CHAR];
		float VariableCost;
		float FixedCost;
	} Links[MAX_LINKS];

	//temporary variable
	float tempDist[MAX_LINKS];
	float tempSlope[MAX_LINKS];
	float tempSideSlope[MAX_LINKS];


	bool LinkExclusion[MAX_LINKS];
	int IndexforLinkOrder[MAX_LINKS];//using when open editor to restore original input order

	char TempCharacter[MAX_CHAR];


	//for DialogSaleEditor
	int TotalNumberofSales;

	struct SALES
	{
		char EntryNode[MAX_CHAR];
		char DestNode[MAX_CHAR];
		float Volume;
		short Year;
	} Sales[MAX_SALES];

	bool SaleExclusion[MAX_SALES];
	float TotalCostsperSale[MAX_SALES];
	float InterestRate;
	short FirstUseYearofLink[MAX_LINKS];//this variable will be initialized as -1.
	unsigned short IndexforSaleOrder[MAX_SALES];//using when open editor to restore original input order

	bool Error;//is true if there is an error while conducting shortest path algorithm
	bool IsDataReady;//FALSE indicates data changed in link or sale editor and not go through the DataManager()
					 //is will be set TRUE at DataManager()
	bool IsFirstIterationDone;//also used in View


	//temporary variable for storing Cable links file or Road link file temporarily.
	LINKS RoadLinks[MAX_LINKS];
	int RoadTotalNumberofLinks;
	SALES RoadSales[MAX_SALES];
	int RoadTotalNumberofSales;

	LINKS CableLinks[MAX_LINKS];
	int CableTotalNumberofLinks;
	SALES CableSales[MAX_SALES];
	int CableTotalNumberofSales;


	//for ShortestPath Algorithm
	int Node[MAX_LINKS][2];
	float Cost[MAX_LINKS];
	float Cost_BackUp[MAX_LINKS];
	char Label_NumberToString[MAX_LINKS][MAX_CHAR];//[new number labe] gives original string labels
	int Entry[MAX_SALES];//EntryNode in Number label
	int Destination[MAX_SALES];//DestNode in Number label
	int IO;
	int DEST;
	unsigned short SaleOrder[MAX_SALES];
	short Method;


	///  Declare variables for ShortestPath Algorithm //////////////
	int IPTOR[MAX_LINKS];
	int NDPT[MAX_LINKS];
	float TSUM[MAX_LINKS];
	float TCUM[MAX_LINKS];
	int NCUM[MAX_LINKS];
	int NOTO[MAX_LINKS];
	int NPRED[MAX_LINKS];
	int NNN;
	int NCM;
	int M;
	int KPLK;
	int KPNCM;
	float TMIN;



	//results from ShortestPath Algorithm
	CString csFileName;
	CString csFilePath;
	CString csTempOutputFile;
	int NumberofLinksinPath;
	int FinalPath[MAX_LINKS];
	CString CurrentPath[MAX_SALES];
	CString BestPath[MAX_SALES];
	float TotalCostfromShortestPathAlgorithm;//keep the result to compare with other heuristics
	short IterationNo;
	short BestIterationNo;



	//NETWORK heuristic methods
	float AccumulatedVolumeperLink[MAX_LINKS];
	float TemporaryAccumulatedVolumeperLink[MAX_LINKS];
	float TotalVariableCosts;
	float VariableCostperSale[MAX_SALES];//just for report writer
	float TotalFixedCosts;
	float FixedCostperSale[MAX_SALES];//just for report writer
	short FirstUseSaleofLink[MAX_LINKS];//just for report writer
	float TotalVariableandFixedCosts;
	float TemporaryVolume[MAX_SALES];
	float TemporaryVariableCost[MAX_LINKS];
	float TemporaryFixedCost[MAX_LINKS];

	






// Operations
public:

	//Woodam's sub-routines
	void OpenFileforVolume();
	void OpenFileforStream();
	void OpenFileforRoad();
	void OpenFileforHarvestArea();
	void SettingHarvestUnitBoundary();//convert from raster to vector.
	int CorrectionTerrainData(int column, int row);
	void ChangecsFileName(CString tempString);//Alternative way to change csFileName(global Doc variable) in View... Using pDoc->csFileName generates an error--don't know why.

	void FileGenerator();

	void SaveFileCableRoads();
	void OpenFileCableRoads();


	///////////////////////////////////////////////////////////////////////////////////////
	//From NETWORK2000
	///////////////////////////////////////////////////////////////////////////////////////
	
	//Woodam's codes
	void Shell_Sorting(CString x[], int o[], int n);//shell sorting --used for sorting link file
	void SortingSalesbyYear(short x[], unsigned short o[], unsigned short n);
	void SortingSalesbyVolume(float x[], unsigned short o[], unsigned short n);
	void DataManager();//managing data from link and sale editor
	void PreparingDataforShortestPath();
	void InitializingShortestPathAlgorithm();
	void ShortestPathAlgorithm(int IDNumber);
	void ShortestPathRun();

	void AddAccumulatedVolume(int IDNumber);
	void CalculateRevisedVariableCost();
	int FindLinkNumberbetweenTwoNodes(int fromNode, int toNode);
	void CalculateTotalCosts();
	float CalculateCostsperSale(int SaleNo, CString NodeString);
	void SaveResultsToFile(CString NodeString[], CString csFileName);
	void ChangeSaleOrder();
	void CheckLinkData();//check link duplicates
	
	void ConvertfromCStringToChar(char* character, CString cstring);
	CString ConvertfromCharToCString(char* character);





// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CCablanDoc)
	public:
	virtual BOOL OnNewDocument();
	virtual void Serialize(CArchive& ar);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CCablanDoc();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CCablanDoc)
	afx_msg void OnFileOpendtm();
	afx_msg void OnFileOpenvolume();
	afx_msg void OnFileOpenstreams();
	afx_msg void OnFileOpenroads();
	afx_msg void OnInputdataHarvestunitboundaryByfile();
	afx_msg void OnLandingsFilemanagerOpenlandings();
	afx_msg void OnLandingsFilemanagerSavelandings();
	afx_msg void OnCablewaysFilemanagerOpenfile();
	afx_msg void OnCablewaysFilemanagerSavefile();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

	// Generated OLE dispatch map functions
	//{{AFX_DISPATCH(CCablanDoc)
		// NOTE - the ClassWizard will add and remove member functions here.
		//    DO NOT EDIT what you see in these blocks of generated code !
	//}}AFX_DISPATCH
	DECLARE_DISPATCH_MAP()
	DECLARE_INTERFACE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_CABLANDOC_H__1DF2159F_8ED8_472F_B753_F170381AA08E__INCLUDED_)
