// CablanView.h : interface of the CCablanView class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_CABLANVIEW_H__1FCF61C3_A38B_4D43_8508_53FE9CB3BC59__INCLUDED_)
#define AFX_CABLANVIEW_H__1FCF61C3_A38B_4D43_8508_53FE9CB3BC59__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


class CCablanView : public CScrollView
{
protected: // create from serialization only
	CCablanView();
	DECLARE_DYNCREATE(CCablanView)
	
		
	//WOODAM'S VARIABLES
	CClientDC* pDC;
	CPoint DrawFrom;
	CPoint DrawTo;
	CPoint cpMargin;
	COLORREF clrElevation[30];  // each clrElevation shows 100m range.
	COLORREF clrVolume[10];//
	CBrush brsElevation[30];
	CBrush brsVolume[11];//brsVolume[10] is white brush
	CPen penColor[10];

	int ViewScale;
	
	bool boolAddLandings;
	bool boolCheckProfile;
	bool boolSettingHarvestUnitBoundary;//if this is TRUE, then user is ready for selecting boundary by right mouse click.
	bool boolTimberExits;//if this is TRUE, then user is ready to place timber exit locations.
	bool boolVolumeLayerShowing;//if this is true, then Volume Layer is showed on the scress.
	

	struct PROFILE
	{
		int column;//for elevation.. cell address on which the line falls
		int row;//for elevation.. cell address on which the line falls
		int elevation;
		float flatdistancefromlanding;
		bool fishstreambuffer;//true, if the cell is on fishstream buffer
	} Profile[MAXCELLS];
	int TotalNumberofProfileCells;


	//contour lines
	bool CellAlreadyChecked[MAXCOLUMNS][MAXROWS];//[5]--4 lateral sides
	bool CellsonContour[MAXCOLUMNS][MAXROWS];


	void DrawDTM();
	void DrawVolumeLayer();
	void DrawTailLocationsandCableways(int landingid, int systemid);
	void DrawCablewayProfile(int landingid, int systemid, int cablewayid);
	void DrawEachCell(int column, int row, COLORREF RGB);
	void DrawEachLine(int startcolumn, int startrow, int endcolumn, int endrow, int width, COLORREF RGB);
	CPoint FindCellAddressbasedonCoordinates(double x, double y);

	
	//QUADTREE-Variables
	bool VolumeCellLessthanDesignPayload[MAXCOLUMNS][MAXROWS];//TRUE if available timber volume within the cell is less than design payload and greater than 0.
	int TotalNumberofVolumeCellsLessthanDesignPayload;
	bool boolTimberParcelAlreadySet;

	struct SORTEDVOLUMECELLS
	{
		int col;
		int row;
	} SortedVolumeCells[MAXCOLUMNS * MAXROWS];
	float DesignPayload;

	//QUADTREE-Subroutines
	void SettingTimberVolumeCells();
	void DrawTimberParcels();
	void IdentifyingVolumeCells();
	void SortingVolumeCells();
	void Shell_Sorting(float x[], int o[], int n);

	
	//Set Harvesting Unit Boundary
	void SettingHarvestingUnitBoundary();


	//Fish stream
	void DrawStreamBuffer();

	//Existing roads
	void DrawExistingRoads();

	//Harvest Area
	void DrawHarvestArea();



	//******************************
	//Cable Analysis
	//******************************

	//tail spar
	int TailLocationID;//cell number in the possible cells fall into profile.

	int TotalNumberofINTs;
	bool boolINTsupport[MAXCELLS];


	//intermediate results 
	bool boolLocatingINTSupport;
	int ChangingINTSupportID;//INT support will change to this ID.
	int ChangingTailsparID;//tailspar location will change to this ID.
	int SelectedINTSLocation;


	//payload analysis
	int TotalSpans;
	float CableLengthatEachSpan[MAXINTSUPPORT];
	float GlobalD1, GlobalD2, GlobalD3, GlobalD4, GlobalH1, GlobalH2, GlobalH3, GlobalH4;//geometry of cables
	float HC;//distance from the ground to the bottom of carriage
	float Alpha;//angle in radian between choker line attached to log end.
	float Phi;//ground slope in radian
	float NormalForce[MAXCELLS];//for saving Normal force
	

	struct SEGMENT
	{
		double Left, Right;
	} T1, H1, V1, T2, H2, V2, T3, H3, V3, T4, H4, V4;


	//results
	float MinPayload;
	float CycleTime;
	bool DownHill;
	bool HaulBackNeeded;
	bool NotEnoughPayload;
	bool SteepChange;
	bool PartialOnStream;
	bool MoreINTSupportsRequired;



	//subroutines
	void WriteText(CPoint Position, CString tempString);
	void WriteTextAboveGridCell(int col, int row, CString tempString);
	void DrawAnythingToTest(CPoint From, CPoint To, int PenColor);//Location is ID number of Profile cells
	double Radians(double Degree);
	int LeftSparLocation(int CurrentTP);
	int RightSparLocation(int CurrentTP);

	//Cableway generators
	void GenerateCablewaysperLanding(int LandingCol, int LandingRow, int CableSystem);

	//payload analysis
	void GenerateIntermediateSupports(int CableSystemNo, float TailSparHeight);
	void PayloadAnalysis(int CableSystemNo, int LandingID, int CablewayID, float TailSparHeight);
	float CalculateStrechedLength(int TP, float Beta, float Myu, float CarriageDepth, float LC, float L, float HeadSparHeight, float INTHeight, float TailSparHeight, bool Draw);
	float CalculateStrechedLengthbyHC(float HC, int TP, float HeadSparHeight, float INTHeight, float TailSparHeight, bool Draw);
	void Segment1(double UpperTension, float d1, float h1, float Length1, float w1);
	void Segment2(double LowerTension, float d2, float h2, float Length2, float w2);
	void Segment3(float d3, float h3, float Length3, float w3, float Alpha, float CarriageWeight);
	void DownhillSegment2(double UpperTension, float d2, float h2, float length2, float w2);
	void DownhillSegment1(double LowerTension, float d1, float h1, float length1, float w1);
	void DownhillSegment4(float d4, float h4, float length4, float w4, float Alpha, float CarriageWeight);
	float PayloadPartial(float Myu, float CarriageWeight, float Phi, float HC, int TP, int leftsparlocation, int rightsparlocation, float HeadSparHeight, float INTHeight, float TailSparHeight);
	float PayloadFull(float CarriageWeight, float HC, int TP, int leftsparlocation, int rightsparlocation, float HeadSparHeight, float INTHeight, float TailSparHeight);

	//economic analysis
	float CalculateUnitCost(float MinPayload, float Myu, float HeadSparHeight, float YardingDist, float MinStLength, float CableWeight, float CarriageWeight, int CableSystem);


	//Auotmated search of landing location within a boundary
	float FindBestLandingLocationWithinBoundary(int Landingcol, int Landingrow, int CableSystem);//returning the total areas this landing and cable system can cover.

	//drawing contour maps
	bool CheckContourCells(int col, int row, double ContourElevation, int SideID);//return TRUE when it finds contour line pass at least one of four sides of the cell.
	CPoint LinearInterpolationforContourlines(int col1, int row1, int col2, int row2, double ContourElevation);


	//******************************
	//ROAD
	//******************************

	bool blRoadRoutingbyUser;
	int blRoadMovingbyUser;//keeps selected cell id number in SelectedRoadCellsbyUser[] array
	int MaxRoadLength;
	CPoint ExistingRoad;
	CPoint SelectedRoadCellsbyUser[MAXROADLENGTH];
	CPoint tempRoadCells[MAXROADLENGTH];
	int NumberofSelectedRoadCellsbyUser;
	int gbLandingNumber;




	void GenerateRoadRoutefromExistingRoadtoLandings(int landingNo, CPoint StartingCell, CPoint EndingCell);
	void GenerateRoadRoutefromOnePointtoTheother(int landingNo, CPoint StartingCell, CPoint EndingCell);
	CPoint RandomlyFindingNextCellonRoute(CPoint CurrentCell, CPoint PreviousCell, CPoint TargetCell);
	void SimulatedAnnealingProcessforTruckRoadRouting(int landingNo);
	void DrawRoadRoute(int landingNo);

	//reporting in textfile (using column and row numbers)
	void CableReports(CString FileName);
	void RoadReports(CString FileName);

	
	//******************************
	//NETWORK
	//******************************
	//check node connections
	int SelectedNode;
	CString ConnectedNodes[MAX_LINKS];//[fromNode]; consist of 5digit numbers
	bool blConnectedNodes[MAX_LINKS];

	float LandingVolume[MAXLANDINGS];//data transfered from cable system network to road network
	float RoadVariableCostsperLanding[MAXLANDINGS];//data transfered from road network to cable system network
	float RoadFixedCostsperLanding[MAXLANDINGS];//data transfered from road network to cable system network
	
	void GenerateCableSystemNetwork(float RoadVariableCostsperLanding[], float RoadFixedCostsperLanding[], int Iteration);
	void GenerateCableSystemSaleData(int Iteration);
	void GenerateRoadNetwork(int Iteration);
	void GenerateRoadNetwork_WritingtoFile(int fromcol, int fromrow, int tocol, int torow, float variable, float fixed, FILE *outfile_ptr, float dist, float slope, float sideslope);
	void GenerateRoadSaleData(float LandingVolume[], int Iteration);

	void RunShortestPathHeuristic();
	void CalculateVariableandFixedCostsperSale();
	float RecalculateTotalVariableandFixedCostsfromEachCelltoDestination(int Iteration);//returning the total overall costs
	float RecalculateBestTotalVariableandFixedCostsfromEachCelltoDestination(int Iteration);//returning the total overall costs
	float FindVolumeatLanding(int LandingID);
	float VariableCostsRoadCell(double dist, double slope);
	float FixedCostsRoadCell(double dist, double slope, double sidedist, double sideslope);

	



	//show results
	struct LADNINGLOCATION
	{
		int col;
		int row;
		int nodeno;
	} LandingLocationResults[MAXLANDINGS];//keep landing location as column and row
	int TotalNumberofFinalLandings;

	struct ROADCELLS
	{
		int col;
		int row;
		int nodeno;//keep node number given from DataManager()
	} RoadCellResults[MAXROADLENGTH];//keep road cells as column and row
	int TotalNumberofFinalRoadCells;

	
	struct CABLES
	{
		int landing;
		int system;
		int cableway;
	} CableResults[MAXLANDINGS * MAXCABLESYSTEMS * MAXCABLEWAYS];//keep cableways to be selected as the results of optimization
	int TotalNumberofFinalCableways;

	struct FROMCELLTOCABLE
	{
		int x;
		int y;
	} FromCellToCableResults[MAXCOLUMNS][MAXROWS];//keeps x and y coordinates of one point on any cableway where the volume should go.

	
	void GenerateCableSystemNetworkAllAtOnce();
	void GenerateCableSystemSaleDataAllAtOnce();
	void GenerateRoadNetworkAllAtOnce();

	

	//drawing the results
	void DrawCablePath(CString FileName);
	void DrawRoadPath(CString FileName);
	void DrawAllPath();

	//keep the results as text file
	void SaveResultstoFile();
	void OpenResultstoFile();

	CString ResultFile;





	//END OF WOODAM'S VARIABLES


	













// Attributes
public:
	CCablanDoc* GetDocument();

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CCablanView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	protected:
	virtual void OnInitialUpdate(); // called first time after construct
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CCablanView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CCablanView)
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLandingsPlacelanding();
	afx_msg void OnCablewaysGenerate();
	afx_msg void OnCablewaysCheckprofile();
	afx_msg void OnAnalysesCableways();
	afx_msg void OnInputdataCablesystem();
	afx_msg void OnInputdataOperationdata();
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnNetworkCablesystemsGeneratenetwork();
	afx_msg void OnNetworkRoadsDrawroads();
	afx_msg void OnNetworkRoadsGenerate();
	afx_msg void OnNetworkRoadsGeneratenetwork();
	afx_msg void OnNetworkOptimizing();
	afx_msg void OnNetworkOptimizingallatonce();
	afx_msg void OnResultsShowcableways();
	afx_msg void OnResultsShowlandings();
	afx_msg void OnResultsShowroads();
	afx_msg void OnResultsShowtimbercells();
	afx_msg void OnNetworkHarvestingunit();
	afx_msg void OnInputdataRoadcosts();
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnResultsClearharvestunit();
	afx_msg void OnZoomIn();
	afx_msg void OnZoomOut();
	afx_msg void OnNetworkRoadsTimberexits();
	afx_msg void OnInputdataHarvestunitboundaryBymouse();
	afx_msg void OnLayersDemShowdem();
	afx_msg void OnLayersHarvestareaShowharvestarea();
	afx_msg void OnLayersRoadsShowroads();
	afx_msg void OnLayersStreamsCreatebuffer();
	afx_msg void OnLayersStreamsShowstreams();
	afx_msg void OnLayersVolumeClumpcells();
	afx_msg void OnLayersVolumeShowvolume();
	afx_msg void OnLayersDemChangecontourinterval();
	afx_msg void OnLayersDemDrawcontourlines();
	afx_msg void OnInputdataLandingcosts();
	afx_msg void OnResultsFilemanagerOpenfile();
	afx_msg void OnResultsFilemanagerSavefile();
	afx_msg void OnLandingsDeletelandinglocation();
	afx_msg void OnResultsShowallresults();
	afx_msg void OnResultsRoadreports();
	afx_msg void OnCablewaysWriteidnumbers();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

#ifndef _DEBUG  // debug version in CablanView.cpp
inline CCablanDoc* CCablanView::GetDocument()
   { return (CCablanDoc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_CABLANVIEW_H__1FCF61C3_A38B_4D43_8508_53FE9CB3BC59__INCLUDED_)
