#if !defined(AFX_DIALOGPATHANALYSIS_H__703C1088_4F37_48DD_A5AC_B2FDD1769F95__INCLUDED_)
#define AFX_DIALOGPATHANALYSIS_H__703C1088_4F37_48DD_A5AC_B2FDD1769F95__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// DialogPathAnalysis.h : header file
//

/////////////////////////////////////////////////////////////////////////////
// CDialogPathAnalysis dialog

class CDialogPathAnalysis : public CDialog
{
// Construction
public:
	CDialogPathAnalysis(CWnd* pParent = NULL);   // standard constructor

	//************************************************************
	//Woody's variables///////////////////////////////////////////
	//************************************************************

#define MAXCELLS 101 //maximum number of cells in a cableway
#define MAXINTSUPPORT 20  //maximum number of intermediate supports in a cableway

	struct PROFILE
	{
		int column;//for elevation.. cell address on which the line falls
		int row;//for elevation.. cell address on which the line falls
		int elevation;
		float flatdistancefromlanding;
		bool fishstreambuffer;//true, if the cell is on fishstream buffer
	} Profile[MAXCELLS];

	int TotalNumberofProfileCells;
	
	struct CABLESYSTEMS
	{
		float maxyarddist;//in feet
		float maxlateraldist;//in feet  when [0], this keeps the total number of cable systems available
		float towerheight;//in feet
		float intermediatesupportheight;//in feet
		float maxspartreeheight;
		float minspartreeheight;
	} CableSystems;


	//user-defined variables. These are from Doc...
	float gbTmax;
	float gbMmax;
	float gbW1;
	float gbW3;
	float gbW4;
	float gbWc;
	float gbLC;
	float gbCarriageDepth;
	float gbL;
	float gbDLog;
	float gbBeta;
	float gbMyu;
	float gbClearance;
	float gbAllowableDeflection;
	float gbMinSlopewithoutHBL;//minimum slope for tower without haulback in percent.
	float gbMaxSlopeChange;//maximum allowable change in chord slope between adjacent spans
	int gbMaxINTSupports;//limit of the number of Intermediate Supports.


	//user-defined cable operation data
	float gbHooktimeperCM;// sec/ton(cubic meter)
	float gbUnhooktimeperCM;// sec/ton(cubic meter)
	float gbRiggingCosts;//sec. per intermediate support and tailspar
	float gbUserMinPayload;//2200 lb  -> kg
	float gbDensity;//2200 lb / cubic meter  -> kg / cubic meter
	float gbUserMaxPayload;//6600 lb  -> kg
	float gbLoadFactor;//percent of skyline capacity actually used
	float gbCableOperationUnitCost;//hourly cost of tower operation
	float gbDelayPercentage;
	float gbFellingCosts;
	float gbMoveInCosts;//system move in costs
	float gbYarderSetupHour;//system initial setup hour
	float gbCableRoadChangeHour;//sklyine reinstalling hour
	float gbEnginePower;//horse power
	float gbEngineEfficiency;//65% of engine power
	float gbInhaulSpeedLimit;//speed limit when partial suspension.. speed should not be greater than this limit.(m/sec)
	float gbInhaulSpeedLimitFull;//speed limit when full suspension.. 
	float gbOuthaulSpeedLimit;//speed limit when outhauling..
	float gbLateralYardingSpeed;//average speed for lateral yarding.
	float gbPenaltyForBeyondLateralYarding;//beyond lateryarding distance.
	float gbMultiplierLYD1;
	float gbMultiplierLYD2;
	float gbMultiplierLYD3;
	float gbGiveUpCellDistance;//distance from cable road



	//tail spar
	float SparTreeHeight;//in feet
	int TailLocationID;//cell number in the possible cells fall into profile.

	int TotalNumberofINTs;
	bool boolINTsupport[MAXCELLS];


	//intermediate results 
	bool boolLocatingINTSupport;
	int ChangingINTSupportID;//INT support will change to this ID.
	int ChangingTailsparID;//tailspar location will change to this ID.
	int SelectedINTSLocation;


	//payload analysis
	int TotalSpans;
	float CableLengthatEachSpan[MAXINTSUPPORT];
	float GlobalD1, GlobalD2, GlobalD3, GlobalD4, GlobalH1, GlobalH2, GlobalH3, GlobalH4;//geometry of cables
	float HC;//distance from the ground to the bottom of carriage
	float Alpha;//angle in radian between choker line attached to log end.
	float Phi;//ground slope in radian
	float NormalForce[MAXCELLS];//for saving Normal force
	
	struct SEGMENT
	{
		double Left, Right;
	} T1, H1, V1, T2, H2, V2, T3, H3, V3, T4, H4, V4;


	//results
	float MinPayload;
	bool PartialOnStream;//see view()
	bool MoreINTSupportsRequired;//see view()

	//cable road number and landing number
	int landingNo;
	int cablewayNo;
	int systemNo;




	//for scale and drawing
	int MaxElev;
	int MinElev;
	int Scale, Margin;
	CRect Coord;
	CBrush brsColor[5];
	CPen penColor[5];






	//subroutines
	void DrawAnythingToTest(CPoint From, CPoint To, int PenColor);//Location is ID number of Profile cells
	void WriteText(CPoint Position, CString tempString);
	void DrawLog(int i, float LC, float L, float CarriageDepth, float Alpha, float HC, float Beta, float Phi);
	double Radians(double Degree);
	int LeftSparLocation(int CurrentTP);
	int RightSparLocation(int CurrentTP);

	//payload analysis
	void GenerateIntermediateSupports();
	void PayloadAnalysis();
	float CalculateStrechedLength(int TP, float Beta, float Myu, float CarriageDepth, float LC, float L, float HeadSparHeight, float INTHeight, float TailSparHeight, bool Draw);
	float CalculateStrechedLengthbyHC(float HC, int TP, float HeadSparHeight, float INTHeight, float TailSparHeight, bool Draw);
	void Segment1(double UpperTension, float d1, float h1, float Length1, float w1);
	void Segment2(double LowerTension, float d2, float h2, float Length2, float w2);
	void Segment3(float d3, float h3, float Length3, float w3, float Alpha, float CarriageWeight);
	void DownhillSegment2(double UpperTension, float d2, float h2, float length2, float w2);
	void DownhillSegment1(double LowerTension, float d1, float h1, float length1, float w1);
	void DownhillSegment4(float d4, float h4, float length4, float w4, float Alpha, float CarriageWeight);
	float PayloadPartial(float Myu, float CarriageWeight, float Phi, float HC, int TP, int leftsparlocation, int rightsparlocation, float HeadSparHeight, float INTHeight, float TailSparHeight);
	float PayloadFull(float CarriageWeight, float HC, int TP, int leftsparlocation, int rightsparlocation, float HeadSparHeight, float INTHeight, float TailSparHeight);

	//economic analysis
	float CalculateUnitCost(float MinPayload, float Myu, float HeadSparHeight, float MinStLength, float CableWeight, float CarriageWeight);//doesn't include system id number since this dialog class gets only one system.


// Dialog Data
	//{{AFX_DATA(CDialogPathAnalysis)
	enum { IDD = IDD_DIALOGPATHANALYSIS };
	CStatic	m_Picture1;
	//}}AFX_DATA


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CDialogPathAnalysis)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:


	// Generated message map functions
	//{{AFX_MSG(CDialogPathAnalysis)
	virtual BOOL OnInitDialog();
	afx_msg void OnDrawprofile();
	afx_msg void OnLocateintsupport();
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnRemoveintsupport();
	afx_msg void OnButtonPayloadanalysis();
	afx_msg void OnButtonGenerateints();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_DIALOGPATHANALYSIS_H__703C1088_4F37_48DD_A5AC_B2FDD1769F95__INCLUDED_)
