// CablanView.cpp : implementation of the CCablanView class
//

#include "stdafx.h"
#include "Cablan.h"

#include "CablanDoc.h"
#include "CablanView.h"
#include "math.h"
#include "DialogPathAnalysis.h"
#include "CableSystemDataInput.h"
#include "DialogCableOperationData.h"
#include "DialogLandingCosts.h"
#include "DialogRoadCosts.h"
#include "DialogContourMap.h"
#include "DialogFishStreamBuffer.h"
#include "DialogChokerLength.h"
#include "DialogDeleteLandings.h"



#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCablanView

IMPLEMENT_DYNCREATE(CCablanView, CScrollView)

BEGIN_MESSAGE_MAP(CCablanView, CScrollView)
	//{{AFX_MSG_MAP(CCablanView)
	ON_WM_LBUTTONDOWN()
	ON_COMMAND(ID_LANDINGS_PLACELANDING, OnLandingsPlacelanding)
	ON_COMMAND(ID_CABLEWAYS_GENERATE, OnCablewaysGenerate)
	ON_COMMAND(ID_CABLEWAYS_CHECKPROFILE, OnCablewaysCheckprofile)
	ON_COMMAND(ID_ANALYSES_CABLEWAYS, OnAnalysesCableways)
	ON_COMMAND(ID_INPUTDATA_CABLESYSTEM, OnInputdataCablesystem)
	ON_COMMAND(ID_INPUTDATA_OPERATIONDATA, OnInputdataOperationdata)
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_COMMAND(ID_NETWORK_CABLESYSTEMS_GENERATENETWORK, OnNetworkCablesystemsGeneratenetwork)
	ON_COMMAND(ID_NETWORK_ROADS_DRAWROADS, OnNetworkRoadsDrawroads)
	ON_COMMAND(ID_NETWORK_ROADS_GENERATE, OnNetworkRoadsGenerate)
	ON_COMMAND(ID_NETWORK_ROADS_GENERATENETWORK, OnNetworkRoadsGeneratenetwork)
	ON_COMMAND(ID_NETWORK_OPTIMIZING, OnNetworkOptimizing)
	ON_COMMAND(ID_NETWORK_OPTIMIZINGALLATONCE, OnNetworkOptimizingallatonce)
	ON_COMMAND(ID_RESULTS_SHOWCABLEWAYS, OnResultsShowcableways)
	ON_COMMAND(ID_RESULTS_SHOWLANDINGS, OnResultsShowlandings)
	ON_COMMAND(ID_RESULTS_SHOWROADS, OnResultsShowroads)
	ON_COMMAND(ID_RESULTS_SHOWTIMBERCELLS, OnResultsShowtimbercells)
	ON_COMMAND(ID_NETWORK_HARVESTINGUNIT, OnNetworkHarvestingunit)
	ON_COMMAND(ID_INPUTDATA_ROADCOSTS, OnInputdataRoadcosts)
	ON_WM_LBUTTONDBLCLK()
	ON_COMMAND(ID_RESULTS_CLEARHARVESTUNIT, OnResultsClearharvestunit)
	ON_COMMAND(ID_ZOOM_IN, OnZoomIn)
	ON_COMMAND(ID_ZOOM_OUT, OnZoomOut)
	ON_COMMAND(ID_NETWORK_ROADS_TIMBEREXITS, OnNetworkRoadsTimberexits)
	ON_COMMAND(ID_INPUTDATA_HARVESTUNITBOUNDARY_BYMOUSE, OnInputdataHarvestunitboundaryBymouse)
	ON_COMMAND(ID_LAYERS_DEM_SHOWDEM, OnLayersDemShowdem)
	ON_COMMAND(ID_LAYERS_HARVESTAREA_SHOWHARVESTAREA, OnLayersHarvestareaShowharvestarea)
	ON_COMMAND(ID_LAYERS_ROADS_SHOWROADS, OnLayersRoadsShowroads)
	ON_COMMAND(ID_LAYERS_STREAMS_CREATEBUFFER, OnLayersStreamsCreatebuffer)
	ON_COMMAND(ID_LAYERS_STREAMS_SHOWSTREAMS, OnLayersStreamsShowstreams)
	ON_COMMAND(ID_LAYERS_VOLUME_CLUMPCELLS, OnLayersVolumeClumpcells)
	ON_COMMAND(ID_LAYERS_VOLUME_SHOWVOLUME, OnLayersVolumeShowvolume)
	ON_COMMAND(ID_LAYERS_DEM_CHANGECONTOURINTERVAL, OnLayersDemChangecontourinterval)
	ON_COMMAND(ID_LAYERS_DEM_DRAWCONTOURLINES, OnLayersDemDrawcontourlines)
	ON_COMMAND(ID_INPUTDATA_LANDINGCOSTS, OnInputdataLandingcosts)
	ON_COMMAND(ID_RESULTS_FILEMANAGER_OPENFILE, OnResultsFilemanagerOpenfile)
	ON_COMMAND(ID_RESULTS_FILEMANAGER_SAVEFILE, OnResultsFilemanagerSavefile)
	ON_COMMAND(ID_LANDINGS_DELETELANDINGLOCATION, OnLandingsDeletelandinglocation)
	ON_COMMAND(ID_RESULTS_SHOWALLRESULTS, OnResultsShowallresults)
	ON_COMMAND(ID_RESULTS_ROADREPORTS, OnResultsRoadreports)
	ON_COMMAND(ID_CABLEWAYS_WRITEIDNUMBERS, OnCablewaysWriteidnumbers)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CScrollView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCablanView construction/destruction

CCablanView::CCablanView()
{
	cpMargin.x = 100;
	cpMargin.y = 100;
	ViewScale = 5000;//10000 for south zone

	//define color for DEM
	for (int i=0; i<10; i++)
		clrElevation[i] = RGB(i*28, 255, 0);  //green to yellow for the elevation of 0/3 ~ 1/3 of (MaxZ-MinZ)
	for (i=10; i<20; i++)
		clrElevation[i] = RGB(255, 255-(i-10)*20, 100);  //yellow to red for the elevation of 1/3 ~ 2/3 of (MaxZ-MinZ)
	for (i=20; i<30; i++)
		clrElevation[i] = RGB(255-(i-20)*20, 100, 100);  //red to black for the elevation of 2/3 ~ 3/3 of (MaxZ-MinZ)

	//define color for Volume
	for (i=0; i<10; i++)
		clrVolume[i] = RGB((10-i)*10, 100+((10-i)*15), (10-i)*10);  //green to dark green

	clrVolume[10] = RGB(255,255,255);
}


CCablanView::~CCablanView()
{
}

BOOL CCablanView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CScrollView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CCablanView drawing

void CCablanView::OnDraw(CDC* pDC)
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here
}

void CCablanView::OnInitialUpdate()
{
	CScrollView::OnInitialUpdate();

	int i;
	CSize sizeTotal;

	sizeTotal.cx = 9000;
	sizeTotal.cy = 9000;
	
	SetScrollSizes(MM_LOMETRIC, sizeTotal);


	//*****************************************
	//Initial user data
	//*****************************************

	//user-defined data
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	for (i = 1; i < MAXCABLESYSTEMS; ++i)
	{
/*
		//user-defined payload analysis data. All are in metric units
		pDoc->CableSystems[i].gbTmax = (float)(20000. * 0.4536);//conver lb to kg -> 20000lb
		pDoc->CableSystems[i].gbMmax = (float)(20000. * 0.4536);
		pDoc->CableSystems[i].gbW1 = (float)(1.04 * 1.4882);//unit weight of skyline -> convert 1.04 lb/ft to kg/m
		pDoc->CableSystems[i].gbW3 = (float)(.8 * 1.4882);//unit weight of mainline
		pDoc->CableSystems[i].gbW4 = (float)(.6 * 1.4882);//unit weight of haulbackline
		pDoc->CableSystems[i].gbWc = (float)(400 * 0.4536);//carriage weight -> conver lb to kg
		pDoc->CableSystems[i].gbLC = (float)(10. * 0.3048);//effective choker length
		pDoc->CableSystems[i].gbCarriageDepth = (float)(2. * 0.3048);//carriage depth
		pDoc->CableSystems[i].gbL = (float)(36. * 0.3048);//log length, choker hooked at end of log
		pDoc->CableSystems[i].gbDLog = (float)(1. * 0.3048);//log diameter
		pDoc->CableSystems[i].gbBeta = (float)(10. * 3.141592 / 180.);//front end log clearance in radian
		pDoc->CableSystems[i].gbMyu = (float)(.6);//coefficient of ground friction
		pDoc->CableSystems[i].gbClearance = (float) (20.0 * 0.3048);//minimum allowable skyline height from the ground--used for locating intermediate support
		pDoc->CableSystems[i].gbAllowableDeflection = (float)0.07;//--used for locating intermediate support
		pDoc->CableSystems[i].gbMinSlopewithoutHBL = -20.;//minimum slope for tower without haulback in percent.
		pDoc->CableSystems[i].gbMaxSlopeChange = 50.;//maximum allowable change in chord slope between adjacent spans
		pDoc->CableSystems[i].gbMaxINTSupports = 5;//limit of number of INT supports.

		
		pDoc->CableSystems[i].towerheight = (float)(40 * 0.3048);//in meter
		pDoc->CableSystems[i].maxyarddist = (float)(1000 * 0.3048);//in meter
		pDoc->CableSystems[i].maxlateraldist = (float)(200 * 0.3048);//in meter
		pDoc->CableSystems[i].intermediatesupportheight = (float)(40 * 0.3048);//in meter
		pDoc->CableSystems[i].maxspartreeheight = (float)(50 * 0.3048);//in meter
		pDoc->CableSystems[i].minspartreeheight = (float)(40 * 0.3048);//in meter

		//user-defined operation data
		pDoc->CableSystems[i].gbHookTime = (float)60.;// sec/ton(cubic meter)
		pDoc->CableSystems[i].gbUnhookTime = (float)30.;// sec/ton(cubic meter)
		pDoc->CableSystems[i].gbRiggingCosts = (float)200.;//$100 per intermediate support and tailspar
		pDoc->CableSystems[i].gbUserMinPayload = (float)(2200.0 * 0.4536);//2200 lb  -> kg
		pDoc->CableSystems[i].gbDensity = (float)(1500.0 * 0.4536);//1500 lb / cubic meter  -> kg / cubic meter
		pDoc->CableSystems[i].gbUserMaxPayload = (float)(6600.0 * 0.4536);//6600 lb  -> kg
		pDoc->CableSystems[i].gbLoadFactor = (float).8;//percent of skyline capacity actually used
		pDoc->CableSystems[i].gbUnitCost = (float)30.0;//hourly cost of tower operation
		pDoc->CableSystems[i].gbDelayPercentage = (float)20.0;//20%
		pDoc->CableSystems[i].gbFellingCosts = (float)15.0;//$15 per cubic meter
		pDoc->CableSystems[i].gbMoveInCosts = (float)5000.0;//$5000 for move-in and installing costs
		pDoc->CableSystems[i].gbYarderSetupHour = (float)4.0;//initial yarder setup hour per landing
		pDoc->CableSystems[i].gbCableRoadChangeHour = (float)1.;//1 hour for reinstalling skyline (cableway)
		pDoc->CableSystems[i].gbEnginePower = (float) 300.0;//horse power
		pDoc->CableSystems[i].gbEngineEfficiency = (float)0.65;//65% of engine power
		pDoc->CableSystems[i].gbInhaulSpeedLimit = (float)1.5;//1.5 m/sec -- speed limit for partial suspension
		pDoc->CableSystems[i].gbInhaulSpeedLimitFull = (float) 15.;//m/sec -- speed limit when full suspension.. 
		pDoc->CableSystems[i].gbOuthaulSpeedLimit = (float) 20.;//m/sec -- speed limit when outhauling..
		pDoc->CableSystems[i].gbLateralYardingSpeed = (float) 1.;//m/sec -- average speed for lateral yarding
		pDoc->CableSystems[i].gbPenaltyForBeyondLateralYarding = (float) 3.;
		pDoc->CableSystems[i].gbMultiplierLYD1 = (float) 1.;//multiplier to lateral yarding time for the first zone (0-1/3 of LYD)
		pDoc->CableSystems[i].gbMultiplierLYD2 = (float) 1.5;//multiplier to lateral yarding time for the first zone (1/3 - 2/3 of LYD)
		pDoc->CableSystems[i].gbMultiplierLYD3 = (float) 2.;//multiplier to lateral yarding time for the first zone (2/3 - 1 of LYD)
		pDoc->CableSystems[i].gbGiveUpCellDistance = (float)(3 * 200 * 0.3048) ;//if the cell is located beyond this distance from the cable, then give up the cell.
*/

		pDoc->CableSystems[i].gbTmax = (float)(10000.);//convert lb to kg -> 20000lb
		pDoc->CableSystems[i].gbMmax = (float)(10000.);
		pDoc->CableSystems[i].gbW1 = (float)(1.5);//unit weight of skyline -> convert 1.04 lb/ft to kg/m
		pDoc->CableSystems[i].gbW3 = (float)(1.2);//unit weight of mainline
		pDoc->CableSystems[i].gbW4 = (float)(0.9);//unit weight of haulbackline
		pDoc->CableSystems[i].gbWc = (float)(180);//carriage weight -> conver lb to kg
		pDoc->CableSystems[i].gbLC = (float)(3);//effective choker length????????????????????
		pDoc->CableSystems[i].gbCarriageDepth = (float)(0.7);//carriage depth
		pDoc->CableSystems[i].gbL = (float)(12);//log length, choker hooked at end of log
		pDoc->CableSystems[i].gbDLog = (float)(0.3);//log diameter
		pDoc->CableSystems[i].gbBeta = (float)(10. * 3.141592 / 180.);//front end log clearance in radian
		pDoc->CableSystems[i].gbMyu = (float)(.6);//coefficient of ground friction
		pDoc->CableSystems[i].gbClearance = (float) (6);//minimum allowable skyline height from the ground--used for locating intermediate support
		pDoc->CableSystems[i].gbAllowableDeflection = (float)0.07;//--used for locating intermediate support
		pDoc->CableSystems[i].gbMinSlopewithoutHBL = -20.;//minimum slope for tower without haulback in percent.
		pDoc->CableSystems[i].gbMaxSlopeChange = 50.;//maximum allowable change in chord slope between adjacent spans
		pDoc->CableSystems[i].gbMaxINTSupports = 5;//limit of number of INT supports.
		
		pDoc->CableSystems[i].towerheight = (float)(12);//in meter
		pDoc->CableSystems[i].maxyarddist = (float)(300);//in meter
		pDoc->CableSystems[i].maxlateraldist = (float)(50);//in meter
		pDoc->CableSystems[i].intermediatesupportheight = (float)(12);//in meter
		pDoc->CableSystems[i].maxspartreeheight = (float)(15);//in meter
		pDoc->CableSystems[i].minspartreeheight = (float)(12);//in meter

		//user-defined operation data
		pDoc->CableSystems[i].gbHookTime = (float)60.;// sec/ton(cubic meter)
		pDoc->CableSystems[i].gbUnhookTime = (float)30.;// sec/ton(cubic meter)
		pDoc->CableSystems[i].gbRiggingCosts = (float)200.;//$100 per intermediate support and tailspar
		pDoc->CableSystems[i].gbUserMinPayload = (float)(1000);//2200 lb  -> kg
		pDoc->CableSystems[i].gbDensity = (float)(700);//1500 lb / cubic meter  -> kg / cubic meter
		pDoc->CableSystems[i].gbUserMaxPayload = (float)(3000);//6600 lb  -> kg
		pDoc->CableSystems[i].gbLoadFactor = (float).8;//percent of skyline capacity actually used
		pDoc->CableSystems[i].gbUnitCost = (float)300.0;//hourly cost of tower operation
		pDoc->CableSystems[i].gbMoveInCosts = (float)3000.0;//$5000 for move-in and installing costs
		pDoc->CableSystems[i].gbDelayPercentage = (float)20.0;//20%
		pDoc->CableSystems[i].gbFellingCosts = (float)15.0;//$15 per cubic meter
		pDoc->CableSystems[i].gbYarderSetupHour = (float)4.0;//initial yarder setup hour per landing
		pDoc->CableSystems[i].gbCableRoadChangeHour = (float)1.0;//1 hour for reinstalling skyline (cableway)
		pDoc->CableSystems[i].gbEnginePower = (float) 300.0;//horse power
		pDoc->CableSystems[i].gbEngineEfficiency = (float)0.65;//65% of engine power
		pDoc->CableSystems[i].gbInhaulSpeedLimit = (float)5;//1.5 m/sec -- speed limit for partial suspension
		pDoc->CableSystems[i].gbInhaulSpeedLimitFull = (float) 15.;//m/sec -- speed limit when full suspension.. 
		pDoc->CableSystems[i].gbOuthaulSpeedLimit = (float) 20.;//m/sec -- speed limit when outhauling..
		pDoc->CableSystems[i].gbLateralYardingSpeed = (float) 1.;//m/sec -- average speed for lateral yarding
		pDoc->CableSystems[i].gbPenaltyForBeyondLateralYarding = (float)10.;
		pDoc->CableSystems[i].gbMultiplierLYD1 = (float) 1.;//multiplier to lateral yarding time for the first zone (0-1/3 of LYD)
		pDoc->CableSystems[i].gbMultiplierLYD2 = (float) 1.5;//multiplier to lateral yarding time for the first zone (1/3 - 2/3 of LYD)
		pDoc->CableSystems[i].gbMultiplierLYD3 = (float) 2.;//multiplier to lateral yarding time for the first zone (2/3 - 1 of LYD)
		pDoc->CableSystems[i].gbGiveUpCellDistance = (float)(3 * 50) ;//if the cell is located beyond this distance from the cable, then give up the cell.

		pDoc->CableSystems[i].CableSystemName[0] = '?';
		pDoc->CableSystems[i].boolSpecChangebyUser = FALSE;
		pDoc->CableSystems[i].Downhill = FALSE;
		pDoc->CableSystems[i].English = FALSE;
		pDoc->CableSystems[i].HaulbackLine = FALSE;
	}

	
	//from Jeff, Tom, and Ben for South Zone of McDunn Forest.
	//System 1 -- TTY 6150 with ACME 10 --- matches with specs in the LoggerPC new version.
	pDoc->CableSystems[1].gbTmax = (float)(12000);//South Zone - TTY 6150  --50,000lb
	pDoc->CableSystems[1].gbMmax = (float)(6200);//South Zone - TTY 6150  --50,000lb
	pDoc->CableSystems[1].gbW1 = (float)(2.1);//unit weight of skyline -> convert 1.42 lb/ft to kg/m (0.875 in)
	pDoc->CableSystems[1].gbW3 = (float)(1.1);//unit weight of mainline -> 0.72 lb/ft (0.625in)
	pDoc->CableSystems[1].gbW4 = (float)(0.7);//unit weight of haulbackline ->0.46 lb/ft (0.5 in)  -->0.68 kg/m
	pDoc->CableSystems[1].gbWc = (float)(750);//carriage weight -> kg//South Zone - TTY 6150-ACME 10
	pDoc->CableSystems[1].gbLC = (float)(3);//effective choker length//South Zone - TTY 6150
	pDoc->CableSystems[1].gbCarriageDepth = (float)(1);//carriage depth
	pDoc->CableSystems[1].gbL = (float)(12);//log length, choker hooked at end of log
	pDoc->CableSystems[1].gbDLog = (float)(0.3);//log diameter
	pDoc->CableSystems[1].gbBeta = (float)(10. * 3.141592 / 180.);//front end log clearance in radian
	pDoc->CableSystems[1].gbMyu = (float)(.6);//coefficient of ground friction
	pDoc->CableSystems[1].gbClearance = (float) (6);//minimum allowable skyline height from the ground--used for locating intermediate support
	pDoc->CableSystems[1].gbAllowableDeflection = (float)0.07;//--used for locating intermediate support
	pDoc->CableSystems[1].gbMinSlopewithoutHBL = -10.;//minimum slope for tower without haulback in percent.
	pDoc->CableSystems[1].gbMaxSlopeChange = (float)45.;//maximum allowable change in chord slope (%) between adjacent spans
	pDoc->CableSystems[1].gbMaxINTSupports = 2;//limit of number of INT supports.

	pDoc->CableSystems[1].towerheight = (float)(15);//in meter//South Zone - TTY 6150
	pDoc->CableSystems[1].maxyarddist = (float)(600);//in meter//South Zone - TTY 6150
	pDoc->CableSystems[1].maxlateraldist = (float)(50);//in meter//South Zone - TTY 6150
	pDoc->CableSystems[1].intermediatesupportheight = (float)(12);//in meter//South Zone - TTY 6150
	pDoc->CableSystems[1].maxspartreeheight = (float)(15);//in meter//South Zone - TTY 6150
	pDoc->CableSystems[1].minspartreeheight = (float)(12);//in meter

	//user-defined operation data
	pDoc->CableSystems[1].gbHookTime = (float)30.;// sec/m3
	pDoc->CableSystems[1].gbUnhookTime = (float)10.;// sec/m3
	pDoc->CableSystems[1].gbRiggingCosts = (float)200.;//South Zone - TTY 6150
	pDoc->CableSystems[1].gbUserMinPayload = (float)(2500);//South Zone - TTY 6150
	pDoc->CableSystems[1].gbDensity = (float)(700);//1500 lb / cubic meter  -> kg / cubic meter
	pDoc->CableSystems[1].gbUserMaxPayload = (float)(7500);//South Zone - TTY 6150
	pDoc->CableSystems[1].gbLoadFactor = (float).8;//percent of skyline capacity actually used
	pDoc->CableSystems[1].gbUnitCost = (float)465.0;//hourly cost of tower operation (from felling to truck)
	pDoc->CableSystems[1].gbMoveInCosts = (float)465.0;//for move-in costs (landing to landing)
	pDoc->CableSystems[1].gbDelayPercentage = (float)20.0;//20%
	pDoc->CableSystems[1].gbFellingCosts = (float)3.5;//$3.5 per cubic meter--->$20/mbf
	pDoc->CableSystems[1].gbYarderSetupHour = (float)4.0;//initial yarder setup hour per landing
	pDoc->CableSystems[1].gbCableRoadChangeHour = (float)1.0;//1 hour for reinstalling skyline (cableway)
	pDoc->CableSystems[1].gbEnginePower = (float) 230.0;//horse power//South Zone - TTY 6150
	pDoc->CableSystems[1].gbEngineEfficiency = (float)0.65;//65% of engine power
	pDoc->CableSystems[1].gbInhaulSpeedLimit = (float)3;//speed limit for partial suspension//South Zone - TTY 6150
	pDoc->CableSystems[1].gbInhaulSpeedLimitFull = (float) 4;//speed limit when full suspension.. //South Zone - TTY 6150
	pDoc->CableSystems[1].gbOuthaulSpeedLimit = (float) 5;//speed limit when outhauling..//South Zone - TTY 6150
	pDoc->CableSystems[1].gbLateralYardingSpeed = (float) .33;//m/sec -- average speed for lateral yarding//South Zone - TTY 6150
	pDoc->CableSystems[1].gbPenaltyForBeyondLateralYarding = (float) 10.;
	pDoc->CableSystems[1].gbMultiplierLYD1 = (float) 1.;//multiplier to lateral yarding time for the first zone (0-1/3 of LYD)
	pDoc->CableSystems[1].gbMultiplierLYD2 = (float) 1.5;//multiplier to lateral yarding time for the first zone (1/3 - 2/3 of LYD)
	pDoc->CableSystems[1].gbMultiplierLYD3 = (float) 2.;//multiplier to lateral yarding time for the first zone (2/3 - 1 of LYD)
	pDoc->CableSystems[1].gbGiveUpCellDistance = (float)(2 * 50) ;//if the cell is located beyond this distance from the cable, then give up the cell.

	pDoc->CableSystems[1].CableSystemName[0] = 'M';
	pDoc->CableSystems[1].CableSystemName[1] = 'A';
	pDoc->CableSystems[1].CableSystemName[2] = 'D';
	pDoc->CableSystems[1].CableSystemName[3] = 'I';
	pDoc->CableSystems[1].CableSystemName[4] = 'L';
	pDoc->CableSystems[1].CableSystemName[5] = 'L';
	pDoc->CableSystems[1].CableSystemName[6] = '6';
	pDoc->CableSystems[1].CableSystemName[7] = '1';
	pDoc->CableSystems[1].CableSystemName[8] = '5';
	pDoc->CableSystems[1].CableSystemName[9] = '0';
	pDoc->CableSystems[1].boolSpecChangebyUser = FALSE;
	pDoc->CableSystems[1].Downhill = FALSE;
	pDoc->CableSystems[1].English = FALSE;
	pDoc->CableSystems[1].HaulbackLine = TRUE;

	//from Jeff, Tom, and Ben for South Zone of McDunn Forest.
	//System 2 -- Koller K-300 with Koller carriage - matches with the spec in the LoggerPC new version
	pDoc->CableSystems[2].gbTmax = (float)(5000);//South Zone --Koller300  --12,000lb
	pDoc->CableSystems[2].gbMmax = (float)(2200);//South Zone --Koller300  --12,000lb
	pDoc->CableSystems[2].gbW1 = (float)(0.9);//unit weight of skyline -> 1 lb = 0.4536 kg, 1 m = 3.2808 ft.   (0.72lb/ft = 1.07 kg/m)
	pDoc->CableSystems[2].gbW3 = (float)(0.4);//unit weight of mainline (0.26 lb/ft = 0.39 kg/m)
	pDoc->CableSystems[2].gbW4 = (float)(0.4);//unit weight of haulbackline
	pDoc->CableSystems[2].gbWc = (float)(150);//carriage weight -> kg//South Zone
	pDoc->CableSystems[2].gbLC = (float)(2.5);//effective choker length//South Zone
	pDoc->CableSystems[2].gbCarriageDepth = (float)(0.7);//carriage depth
	pDoc->CableSystems[2].gbL = (float)(12);//log length, choker hooked at end of log
	pDoc->CableSystems[2].gbDLog = (float)(0.3);//log diameter
	pDoc->CableSystems[2].gbBeta = (float)(10. * 3.141592 / 180.);//front end log clearance in radian
	pDoc->CableSystems[2].gbMyu = (float)(.6);//coefficient of ground friction
	pDoc->CableSystems[2].gbClearance = (float) (6);//minimum allowable skyline height from the ground--used for locating intermediate support
	pDoc->CableSystems[2].gbAllowableDeflection = (float)0.07;//--used for locating intermediate support
	pDoc->CableSystems[2].gbMinSlopewithoutHBL = -10.;//minimum slope for tower without haulback in percent.
	pDoc->CableSystems[2].gbMaxSlopeChange = (float)45.;//maximum allowable change in chord slope (%) between adjacent spans
	pDoc->CableSystems[2].gbMaxINTSupports = 2;//limit of number of INT supports.
		
	pDoc->CableSystems[2].towerheight = (float)(7);//in meter//South Zone
	pDoc->CableSystems[2].maxyarddist = (float)(300);//in meter//South Zone
	pDoc->CableSystems[2].maxlateraldist = (float)(30);//in meter//South Zone
	pDoc->CableSystems[2].intermediatesupportheight = (float)(8);//in meter//South Zone
	pDoc->CableSystems[2].maxspartreeheight = (float)(15);//in meter//South Zone
	pDoc->CableSystems[2].minspartreeheight = (float)(12);//in meter

	//user-defined operation data (K-300 with student crew = $11/hour labor)
	pDoc->CableSystems[2].gbHookTime = (float)30.;// sec/ton(cubic meter)
	pDoc->CableSystems[2].gbUnhookTime = (float)10.;// sec/ton(cubic meter)
	pDoc->CableSystems[2].gbRiggingCosts = (float)60.;//South Zone//$44/tree = 2hours 2 men ($11*1.4/hour/man)
	pDoc->CableSystems[2].gbUserMinPayload = (float)(1000);//South Zone 
	pDoc->CableSystems[2].gbDensity = (float)(700);//1500 lb / cubic meter  -> kg / cubic meter
	pDoc->CableSystems[2].gbUserMaxPayload = (float)(3000);//South Zone 
	pDoc->CableSystems[2].gbLoadFactor = (float).8;//percent of skyline capacity actually used
	pDoc->CableSystems[2].gbUnitCost = (float)160.0;//hourly cost of tower operation (from felling to truck)
	pDoc->CableSystems[2].gbMoveInCosts = (float)160.0;//for move-in (Landing to landing 1 hour)
	pDoc->CableSystems[2].gbDelayPercentage = (float)20.0;//20%
	pDoc->CableSystems[2].gbFellingCosts = (float)3.5;//$3.5 per cubic meter--->$20/mbf
	pDoc->CableSystems[2].gbYarderSetupHour = (float)1.0;//initial yarder setup hour per landing
	pDoc->CableSystems[2].gbCableRoadChangeHour = (float)1.0;//1 hour for reinstalling skyline (cableway)
	pDoc->CableSystems[2].gbEnginePower = (float) 65.0;//horse power//South Zone -> approximation of tractor engine
	pDoc->CableSystems[2].gbEngineEfficiency = (float)0.65;//65% of engine power
	pDoc->CableSystems[2].gbInhaulSpeedLimit = (float)1;//speed limit for partial suspension//South Zone --m/sec  (600ft/min = 3m/sec)
	pDoc->CableSystems[2].gbInhaulSpeedLimitFull = (float) 2;//speed limit when full suspension.. //South Zone (600ft/min = 3m/sec)
	pDoc->CableSystems[2].gbOuthaulSpeedLimit = (float) 3;//speed limit when outhauling..//South Zone (800ft/min = 4 m/sec)
	pDoc->CableSystems[2].gbLateralYardingSpeed = (float) .17;//m/sec -- average speed for lateral yarding//South Zone 
	pDoc->CableSystems[2].gbPenaltyForBeyondLateralYarding = (float) 10.;
	pDoc->CableSystems[2].gbMultiplierLYD1 = (float) 1.;//multiplier to lateral yarding time for the first zone (0-1/3 of LYD)
	pDoc->CableSystems[2].gbMultiplierLYD2 = (float) 1.5;//multiplier to lateral yarding time for the first zone (1/3 - 2/3 of LYD)
	pDoc->CableSystems[2].gbMultiplierLYD3 = (float) 2.;//multiplier to lateral yarding time for the first zone (2/3 - 1 of LYD)
	pDoc->CableSystems[2].gbGiveUpCellDistance = (float)(2 * 30) ;//if the cell is located beyond this distance from the cable, then give up the cell.

	pDoc->CableSystems[2].CableSystemName[0] = 'K';
	pDoc->CableSystems[2].CableSystemName[1] = 'O';
	pDoc->CableSystems[2].CableSystemName[2] = 'L';
	pDoc->CableSystems[2].CableSystemName[3] = 'L';
	pDoc->CableSystems[2].CableSystemName[4] = 'E';
	pDoc->CableSystems[2].CableSystemName[5] = 'R';
	pDoc->CableSystems[2].CableSystemName[6] = '-';
	pDoc->CableSystems[2].CableSystemName[7] = '3';
	pDoc->CableSystems[2].CableSystemName[8] = '0';
	pDoc->CableSystems[2].CableSystemName[9] = '0';
	pDoc->CableSystems[2].boolSpecChangebyUser = FALSE;
	pDoc->CableSystems[2].Downhill = FALSE;
	pDoc->CableSystems[2].English = FALSE;
	pDoc->CableSystems[2].HaulbackLine = FALSE;



	//System 3 -- TY-90 for Rock Creek application
	pDoc->CableSystems[3].gbTmax = (float)(32700);
	pDoc->CableSystems[3].gbMmax = (float)(11800);
	pDoc->CableSystems[3].gbW1 = (float)(6.15);
	pDoc->CableSystems[3].gbW3 = (float)(2.1);
	pDoc->CableSystems[3].gbW4 = (float)(0.4);
	pDoc->CableSystems[3].gbWc = (float)(1785);
	pDoc->CableSystems[3].gbLC = (float)(2.5);//??
	pDoc->CableSystems[3].gbCarriageDepth = (float)(1.2);
	pDoc->CableSystems[3].gbL = (float)(12.5);
	pDoc->CableSystems[3].gbDLog = (float)(0.9);
	pDoc->CableSystems[3].gbBeta = (float)(10. * 3.141592 / 180.);
	pDoc->CableSystems[3].gbMyu = (float)(.6);
	pDoc->CableSystems[3].gbClearance = (float) (6);
	pDoc->CableSystems[3].gbAllowableDeflection = (float)0.07;
	pDoc->CableSystems[3].gbMinSlopewithoutHBL = -10.;
	pDoc->CableSystems[3].gbMaxSlopeChange = (float)45.;//maximum allowable change in chord slope (%) between adjacent spans
	pDoc->CableSystems[3].gbMaxINTSupports = 2;
		
	pDoc->CableSystems[3].towerheight = (float)(27);
	pDoc->CableSystems[3].maxyarddist = (float)(610);
	pDoc->CableSystems[3].maxlateraldist = (float)(61);
	pDoc->CableSystems[3].intermediatesupportheight = (float)(18);
	pDoc->CableSystems[3].maxspartreeheight = (float)(18);
	pDoc->CableSystems[3].minspartreeheight = (float)(18);

	pDoc->CableSystems[3].gbHookTime = (float)30.;
	pDoc->CableSystems[3].gbUnhookTime = (float)10.;
	pDoc->CableSystems[3].gbRiggingCosts = (float)1000.;
	pDoc->CableSystems[3].gbUserMinPayload = (float)(6000);
	pDoc->CableSystems[3].gbDensity = (float)(860);
	pDoc->CableSystems[3].gbUserMaxPayload = (float)(15000);
	pDoc->CableSystems[3].gbLoadFactor = (float).8;
	pDoc->CableSystems[3].gbUnitCost = (float)522.0;
	pDoc->CableSystems[3].gbMoveInCosts = (float)1500.0;
	pDoc->CableSystems[3].gbDelayPercentage = (float)15.0;
	pDoc->CableSystems[3].gbFellingCosts = (float)3.5;
	pDoc->CableSystems[3].gbYarderSetupHour = (float)8.0;
	pDoc->CableSystems[3].gbCableRoadChangeHour = (float)1.2;
	pDoc->CableSystems[3].gbEnginePower = (float) 450.0;
	pDoc->CableSystems[3].gbEngineEfficiency = (float)0.7;
	pDoc->CableSystems[3].gbInhaulSpeedLimit = (float)4;
	pDoc->CableSystems[3].gbInhaulSpeedLimitFull = (float) 10;
	pDoc->CableSystems[3].gbOuthaulSpeedLimit = (float) 5;
	pDoc->CableSystems[3].gbLateralYardingSpeed = (float) .33;
	pDoc->CableSystems[3].gbPenaltyForBeyondLateralYarding = (float) 10.;
	pDoc->CableSystems[3].gbMultiplierLYD1 = (float) 1.;//multiplier to lateral yarding time for the first zone (0-1/3 of LYD)
	pDoc->CableSystems[3].gbMultiplierLYD2 = (float) 1.5;//multiplier to lateral yarding time for the first zone (1/3 - 2/3 of LYD)
	pDoc->CableSystems[3].gbMultiplierLYD3 = (float) 2.;//multiplier to lateral yarding time for the first zone (2/3 - 1 of LYD)
	pDoc->CableSystems[3].gbGiveUpCellDistance = (float)(2 * 61) ;//if the cell is located beyond this distance from the cable, then give up the cell.

	pDoc->CableSystems[3].CableSystemName[0] = 'T';
	pDoc->CableSystems[3].CableSystemName[1] = 'Y';
	pDoc->CableSystems[3].CableSystemName[2] = '-';
	pDoc->CableSystems[3].CableSystemName[3] = '2';
	pDoc->CableSystems[3].CableSystemName[4] = '9';
	pDoc->CableSystems[3].CableSystemName[5] = '0';
	pDoc->CableSystems[3].CableSystemName[6] = ' ';
	pDoc->CableSystems[3].CableSystemName[7] = ' ';
	pDoc->CableSystems[3].CableSystemName[8] = ' ';
	pDoc->CableSystems[3].CableSystemName[9] = ' ';
	pDoc->CableSystems[3].boolSpecChangebyUser = FALSE;
	pDoc->CableSystems[3].Downhill = FALSE;
	pDoc->CableSystems[3].English = FALSE;
	pDoc->CableSystems[3].HaulbackLine = FALSE;


	pDoc->TotalNumberofSystems = 3;//total numberof systems


	//ROAD--does not need any more.... but some of subroutines require these variables.
	//set up the existing road location
	blRoadRoutingbyUser = FALSE;
	ExistingRoad.x = 70;
	ExistingRoad.y = 70;
	NumberofSelectedRoadCellsbyUser = 2;
	SelectedRoadCellsbyUser[2].x = ExistingRoad.x;//[1] will be each landing location
	SelectedRoadCellsbyUser[2].y = ExistingRoad.y;

	//Road costs--Road costs can be revised in OnInputdataRoadcosts()
/*
	pDoc->HaulingCosts = (float)0.3;
	pDoc->RGFactor1 = (float)0.8;
	pDoc->RGFactor2 = (float)0.9;
	pDoc->RGFactor3 = (float)1.0;
	pDoc->RGFactor4 = (float)2.0;
	pDoc->RGFactor5 = (float)5.0;
	pDoc->RGFactor6 = (float)10.0;
	pDoc->RGFactor7 = (float)20.0;
	pDoc->RoadCosts = (float)20000.0;
	pDoc->SSFactor1 = (float)0.8;
	pDoc->SSFactor2 = (float)1.0;
	pDoc->SSFactor3 = (float)1.2;
	pDoc->SSFactor4 = (float)1.5;
	pDoc->SSFactor5 = (float)2.0;
	pDoc->SSFactor6 = (float)3.0;
	pDoc->SSFactor7 = (float)5.0;
	pDoc->RGLimit = (float)0.3;
	pDoc->SSLimit = (float)0.9;
	pDoc->MultiplierForRoadsOnStream = (float)5.;//penalty for increasing construction costs.
*/

	//from Tom Edwards, Jeff Starnes
	pDoc->HaulingCosts = (float)0.08;//$0.71/mbf/mile--from Tom Edwards
	pDoc->RGFactor1 = (float)0.9;//needs to do literature review.
	pDoc->RGFactor2 = (float)1.0;
	pDoc->RGFactor3 = (float)1.5;
	pDoc->RGFactor4 = (float)2.5;
	pDoc->RGFactor5 = (float)5.0;
	pDoc->RGFactor6 = (float)10.0;
	pDoc->RGFactor7 = (float)20.0;
	pDoc->RoadCosts = (float)30000.0;//$50000 per mile
	pDoc->SSFactor1 = (float)0.8;
	pDoc->SSFactor2 = (float)0.9;
	pDoc->SSFactor3 = (float)1.0;
	pDoc->SSFactor4 = (float)1.5;
	pDoc->SSFactor5 = (float)2.5;
	pDoc->SSFactor6 = (float)5.0;
	pDoc->SSFactor7 = (float)10.0;
	pDoc->RGLimit = (float)0.3;
	pDoc->SSLimit = (float)0.9;
	pDoc->MultiplierForRoadsOnStream = (float)3.;//penalty for increasing construction costs.


	//Landing costs--Road costs can be revised in OnLButtonUp()
	for (i = 1; i < MAXLANDINGS; ++i)
	{
		pDoc->LandingCosts[i] = 5000.0;//from Jeff and Tom
		pDoc->LandingSharingTowerID[i] = 0;
	}


	//Contour lines
	pDoc->Contour[0][0][0].Start.x = -1;//will be used in OnMapContour()
	pDoc->ContourInterval = 10;

	//other boolean variables
	memset(pDoc->boolLandingPayloadAnalysis, FALSE, sizeof(pDoc->boolLandingPayloadAnalysis));
	boolAddLandings = FALSE;
	boolCheckProfile = FALSE;
	boolSettingHarvestUnitBoundary = FALSE;
	boolTimberExits = FALSE;
	boolVolumeLayerShowing = FALSE;
	boolTimberParcelAlreadySet = FALSE;

	pDoc->TotalNumberofUnitBoundaryVertices = 0;
	pDoc->FishStreamBufferWidth = 0;
	pDoc->TotalNumberofTimberExits = 0;
	pDoc->ChokerLength = 20;


	



	//temporory file name

	//central zone
//	pDoc->DEMFile = "E:/thesis/mcdunn/centralZone/centraldem";
//	pDoc->VolumeFile = "E:/thesis/mcdunn/centralZone/centvolCM.dat";
//	pDoc->StreamFile = "E:/thesis/mcdunn/centralZone/streamcent";
//	pDoc->RoadFile = "E:/thesis/mcdunn/centralZone/roadcent";
//	pDoc->HarvestAreaFile = "E:/thesis/mcdunn/centralZone/central zone harvest";


//	Rock Creek
//	pDoc->DEMFile = "E:/thesis/rockcreek/dem.txt";
//	pDoc->VolumeFile = "E:/thesis/rockcreek/volume.txt";
//	pDoc->StreamFile = "E:/thesis/rockcreek/stream.txt";
//	pDoc->RoadFile = "E:/thesis/rockcreek/roads.txt";



	///////////////////////////////////////////
	//south zone
	pDoc->DEMFile = "E:/thesis/McDunn/SouthZone/dem.dat";
//	pDoc->VolumeFile = "E:/thesis/McDunn/SouthZone/southvolCM.dat";
	pDoc->VolumeFile = "E:/thesis/McDunn/SouthZone/thinnvolume.dat";
	pDoc->StreamFile = "E:/thesis/McDunn/SouthZone/streamsouth";
	pDoc->RoadFile = "E:/thesis/McDunn/SouthZone/roadsouth";

	//patch-cut
//	pDoc->HarvestAreaFile = "E:/thesis/output/koller patch/input/study1ascii";
//	pDoc->LandingFile = "E:/thesis/output/koller patch/input/Landings.dat";
//	pDoc->CableRoadFile = "E:/thesis/output/koller patch/input/cableroaddata.dat";
//	ResultFile = "E:/thesis/output/koller patch/input/ResultFile.dat";

	//patch-cut with long SKL
//	pDoc->HarvestAreaFile = "E:/thesis/output/koller patch long/input/study1ascii";
//	pDoc->LandingFile = "E:/thesis/output/koller patch long/input/Landings.dat";
//	pDoc->CableRoadFile = "E:/thesis/output/koller patch long/input/cableroaddata.dat";
//	ResultFile = "E:/thesis/output/koller patch long/input/ResultFile.dat";

	//patch-cut with long SKL and INT limitation (3)
//	pDoc->HarvestAreaFile = "E:/thesis/output/koller patch long limited INT/input/study1ascii";
//	pDoc->LandingFile = "E:/thesis/output/koller patch long limited INT/input/Landings.dat";
//	pDoc->CableRoadFile = "E:/thesis/output/koller patch long limited INT/input/cableroaddata.dat";
//	ResultFile = "E:/thesis/output/koller patch long limited INT/input/ResultFile.dat";

//	pDoc->LandingFile = "E:/thesis/output/landings.dat";
//	pDoc->CableRoadFile = "E:/thesis/output/cableroaddata.dat";

	//Koller All
//	pDoc->HarvestAreaFile = "E:/thesis/output/kollerAll/input/kollerallascii";
//	pDoc->LandingFile = "E:/thesis/output/kollerAll/input/LandingsAll.dat";
//	pDoc->CableRoadFile = "E:/thesis/output/kollerAll/input/cableroaddataAll.dat";
//	ResultFile = "E:/thesis/output/kollerAll/input/ResultFile.dat";


	//Koller All Stream
//	pDoc->HarvestAreaFile = "E:/thesis/output/kollerAllStream/input/kollerallascii";
//	pDoc->LandingFile = "E:/thesis/output/kollerAllStream/input/LandingsAll.dat";
//	pDoc->CableRoadFile = "E:/thesis/output/kollerAllStream/input/cableroaddataAllstream.dat";
//	ResultFile = "E:/thesis/output/kollerAllStream/input/ResultFile.dat";


	//Koller All27
//	pDoc->HarvestAreaFile = "E:/thesis/output/kollerAll27/input/kollerallascii";
//	pDoc->LandingFile = "E:/thesis/output/kollerAll27/input/LandingsAll27.dat";
//	pDoc->CableRoadFile = "E:/thesis/output/kollerAll27/input/cableroaddataAll27.dat";
//	ResultFile = "E:/thesis/output/kollerAll27/input/ResultFile.dat";


	//COFE 2002
//	pDoc->HarvestAreaFile = "E:/thesis/output/cofe/input/study1ascii";
//	pDoc->LandingFile = "E:/thesis/output/cofe/input/Landings.dat";
//	pDoc->CableRoadFile = "E:/thesis/output/cofe/input/cableroaddata.dat";
//	ResultFile = "E:/thesis/output/cofe/output/ResultFile.dat";


	//COFE 2002-One INT no Streams 10 landings
//	pDoc->HarvestAreaFile = "E:/thesis/output/example1/One INT no Stream/input/study1ascii";
//	pDoc->LandingFile = "E:/thesis/output/example1/One INT no Stream/input/Landings.dat";
//	pDoc->CableRoadFile = "E:/thesis/output/example1/One INT no Stream/input/cableroaddata.dat";
//	ResultFile = "E:/thesis/output/example1/One INT no Stream/output/ResultFile.dat";


	//Planning area 1
	pDoc->HarvestAreaFile = "E:/thesis/output/planningarea1/input/planning1final";
	pDoc->LandingFile = "E:/thesis/output/planningarea1/input/Landings.dat";
	pDoc->CableRoadFile = "E:/thesis/output/planningarea1/input/cableroaddata.dat";
	ResultFile = "E:/thesis/output/planningarea1/output/ResultFile.dat";





//	pDoc->HarvestAreaFile = "E:/thesis/mcdunn/SouthZone/South zone harvest";
//	pDoc->HarvestAreaFile = "E:/thesis/mcdunn/SouthZone/patchasciiutm";
	
//	pDoc->HarvestAreaFile = "E:/thesis/mcdunn/SouthZone/kollerallascii";
//	pDoc->LandingFile = "E:/thesis/McDunn/SouthZone/Textfiles/LandingsAll27.dat";
//	pDoc->CableRoadFile = "E:/thesis/McDunn/SouthZone/Textfiles/cableroaddataAll27.dat";

//	pDoc->DEMFile = "E:/thesis/data/elev100.dat";
//	pDoc->VolumeFile = "E:/thesis/data/smallvolume1.vol";
	

}

/////////////////////////////////////////////////////////////////////////////
// CCablanView printing

BOOL CCablanView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CCablanView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CCablanView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CCablanView diagnostics

#ifdef _DEBUG
void CCablanView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CCablanView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}

CCablanDoc* CCablanView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CCablanDoc)));
	return (CCablanDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CCablanView message handlers


void CCablanView::DrawDTM()
{
	//temp. variables
	int i, j;
	CPoint cpFill;
	int nColor = 0;
	
	
	//CClientDC* pDC = new CClientDC(this);
	pDC = new CClientDC(this);
	OnPrepareDC(pDC);

	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pDC->SetMapMode(MM_ISOTROPIC);
	pDC->SetWindowExt(ViewScale, ViewScale);
	
	
	//create brush colored by elevation
	for (i=0; i<30; i++)
		brsElevation[i].CreateSolidBrush(clrElevation[i]);
	

	CBrush *pOldBrush = pDC->SelectObject(&brsElevation[0]);

	for (j = 1; j <= pDoc->DTMrows; ++j)
	{
		for(i = 1; i <= pDoc->DTMcolumns; ++i)
		{
			if (pDoc->DTMCoord[i][j].z > 0)//data are existing
			{
				//generate coordinate to draw grids on the screen
				DrawFrom.x = (long)(pDoc->DTMCoord[i][j].x - pDoc->DTMminX - (pDoc->DTMcellsize/2.0) + cpMargin.x);// (CellHeight/2) integer problem??
				DrawFrom.y = (long)(pDoc->DTMCoord[i][j].y - pDoc->DTMmaxY - (pDoc->DTMcellsize/2.0) - cpMargin.y);
				DrawTo.x = (long)(pDoc->DTMCoord[i][j].x - pDoc->DTMminX + (pDoc->DTMcellsize/2.0) + cpMargin.x);
				DrawTo.y = (long)(pDoc->DTMCoord[i][j].y - pDoc->DTMmaxY + (pDoc->DTMcellsize/2.0) - cpMargin.y);
			
				// choose a fill color to show iso-contour
				nColor = (int)( (double)(pDoc->DTMCoord[i][j].z - pDoc->DTMminZ) / (double)(pDoc->DTMmaxZ - pDoc->DTMminZ) * 100.0/3.4);
			
				cpFill.x = (long)(pDoc->DTMCoord[i][j].x - pDoc->DTMminX + cpMargin.x);
				cpFill.y = (long)(pDoc->DTMmaxY - pDoc->DTMCoord[i][j].y - cpMargin.y);
			
				// drawing
				CRect rcSquare(DrawFrom.x, DrawFrom.y, DrawTo.x, DrawTo.y);
			
				pDC->FillRect(rcSquare, &brsElevation[nColor]);
			}
		}
	}


	//draw harvest unit boundary
	DrawHarvestArea();



	pDC->SelectObject(pOldBrush);

	//delete CBrush Object
	for (i=0; i<30; i++)
		brsElevation[i].DeleteObject();
	

	boolVolumeLayerShowing = FALSE;//now DEM shows up on the screen.
}



void CCablanView::DrawVolumeLayer()
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	//temp. variables
	int i, j;
	CPoint cpFill;
	int nColor = 0;
	int tempint;
	
	
	//CClientDC* pDC = new CClientDC(this);
	pDC = new CClientDC(this);
	OnPrepareDC(pDC);


	pDC->SetMapMode(MM_ISOTROPIC);
	pDC->SetWindowExt(ViewScale, ViewScale);
	
	
	//create brush colored by elevation
	for (i=0; i<11; i++)
		brsVolume[i].CreateSolidBrush(clrVolume[i]);
	

	CBrush *pOldBrush = pDC->SelectObject(&brsVolume[0]);

	double maxVolume, minVolume;
	maxVolume = 0.;
	minVolume = 9999999999999.9;

	for (j = pDoc->UnitBoundary.startRow; j <= pDoc->UnitBoundary.endRow; ++j)
	{
		for (i = pDoc->UnitBoundary.startCol; i <= pDoc->UnitBoundary.endCol; ++i)
		{
//	for (j = 1; j <= pDoc->DTMrows; ++j)
//	{
//		for(i = 1; i <= pDoc->DTMcolumns; ++i)
//		{
			if (pDoc->OriginalVolume[i][j] > 0)
			{
				if (maxVolume < pDoc->OriginalVolume[i][j])
					maxVolume = pDoc->OriginalVolume[i][j];
				if (minVolume > pDoc->OriginalVolume[i][j])
					minVolume = pDoc->OriginalVolume[i][j];
			}
		}
	}

	tempint = (int)((maxVolume - minVolume)* 20. / 10.0);//?????????????*100 because the volume is too small (in decimal).


	for (j = 1; j <= pDoc->DTMrows; ++j)
	{
		for(i = 1; i <= pDoc->DTMcolumns; ++i)
		{
			if (pDoc->DTMCoord[i][j].z > 0)//data are existing
			{
				//generate coordinate to draw grids on the screen
				DrawFrom.x = (long)(pDoc->DTMCoord[i][j].x - pDoc->DTMminX - (pDoc->DTMcellsize/2.0) + cpMargin.x);// (CellHeight/2) integer problem??
				DrawFrom.y = (long)(pDoc->DTMCoord[i][j].y - pDoc->DTMmaxY - (pDoc->DTMcellsize/2.0) - cpMargin.y);
				DrawTo.x = (long)(pDoc->DTMCoord[i][j].x - pDoc->DTMminX + (pDoc->DTMcellsize/2.0) + cpMargin.x);
				DrawTo.y = (long)(pDoc->DTMCoord[i][j].y - pDoc->DTMmaxY + (pDoc->DTMcellsize/2.0) - cpMargin.y);

				if (pDoc->OriginalVolume[i][j] > 0)
				{
					// choose a fill color to show iso-contour
					nColor = (int)( (double)(pDoc->OriginalVolume[i][j] - minVolume)*20. / tempint);//?????????????*100 because the volume is too small (in decimal).
					//for Example I
					
					if (pDoc->OriginalVolume[i][j] > 1.05)
						nColor = 9;
					else if (pDoc->OriginalVolume[i][j] > 0.9)
						nColor = 6;
					else if (pDoc->OriginalVolume[i][j] > 0.7)
						nColor = 4;
					else if (pDoc->OriginalVolume[i][j] > 0.5)
						nColor = 2;
					else if (pDoc->OriginalVolume[i][j] > 0.3)
						nColor = 0;
					else 
						nColor = 0;
					

		
					if (nColor >= 10)
						nColor = 9;
				}
				else
					nColor = 10;

				// drawing
				CRect rcSquare(DrawFrom.x, DrawFrom.y, DrawTo.x, DrawTo.y);
				pDC->FillRect(rcSquare, &brsVolume[nColor]);
			}
		}
	}


	//draw harvest unit boundary
	DrawHarvestArea();



	pDC->SelectObject(pOldBrush);

	//delete CBrush Object
	for (i=0; i<11; i++)
		brsVolume[i].DeleteObject();	


	boolVolumeLayerShowing = TRUE;//now DEM shows up on the screen.

}



void CCablanView::OnLayersHarvestareaShowharvestarea() 
{
	DrawHarvestArea();
}


void CCablanView::DrawHarvestArea()
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pDC = new CClientDC(this);
	OnPrepareDC(pDC);
	pDC->SetMapMode(MM_ISOTROPIC);
	pDC->SetWindowExt(ViewScale, ViewScale);

	int i, j, k, l;
	bool tempbool;



	//draw harvest unit boundary which is defined by user (mouse clicking)
	if (pDoc->TotalNumberofUnitBoundaryVertices > 0)
	{
		for (i = 1; i <= pDoc->TotalNumberofUnitBoundaryVertices; ++i)
		{
			if (i <  pDoc->TotalNumberofUnitBoundaryVertices)
				DrawEachLine(pDoc->HarvestUnitBoundary[i].col, pDoc->HarvestUnitBoundary[i].row, pDoc->HarvestUnitBoundary[i+1].col, pDoc->HarvestUnitBoundary[i+1].row, 5, RGB(0,0,0));
			else
				DrawEachLine(pDoc->HarvestUnitBoundary[i].col, pDoc->HarvestUnitBoundary[i].row, pDoc->HarvestUnitBoundary[1].col, pDoc->HarvestUnitBoundary[1].row, 5, RGB(0,0,0));
		}

		//Draw timber exit locations
		for (i = 1; i <= pDoc->TotalNumberofTimberExits; ++i)
			DrawEachCell(pDoc->TimberExits[i].column, pDoc->TimberExits[i].row, RGB(0,255,0));
	}



	//draw harvest area by lining each grid 
	pDC->SelectStockObject(BLACK_PEN);

	for (j = 1; j <= pDoc->DTMrows; ++j)
	{
		for (i = 1; i <= pDoc->DTMcolumns; ++i)
		{
			if (pDoc->HarvestArea[i][j] > 0)
			{
				tempbool = FALSE;
				if (j == 1 || i == 1 || j == pDoc->DTMrows || i == pDoc->DTMcolumns)//boundary of the DEM layer.
				{
					tempbool = TRUE;
				}
				else
				{
					for (k = i - 1; k <= i + 1; ++k)
					{
						for (l = j - 1; l <= j + 1; ++l)
						{
							if (pDoc->HarvestArea[k][l] <= 0)//if the cell[i][j] touches the outside boundary, then draw the grid cell with black pen
							{
								tempbool = TRUE;
								k = i + 2;
								break;
							}
						}
					}
				}

				if (tempbool == TRUE)
				{
					DrawFrom.x = (long)(pDoc->DTMCoord[i][j].x - pDoc->DTMminX - (pDoc->DTMcellsize/2.0) + cpMargin.x);// (CellHeight/2) integer problem??
					DrawFrom.y = (long)(pDoc->DTMCoord[i][j].y - pDoc->DTMmaxY - (pDoc->DTMcellsize/2.0) - cpMargin.y);
					DrawTo.x = (long)(pDoc->DTMCoord[i][j].x - pDoc->DTMminX + (pDoc->DTMcellsize/2.0) + cpMargin.x);
					DrawTo.y = (long)(pDoc->DTMCoord[i][j].y - pDoc->DTMmaxY + (pDoc->DTMcellsize/2.0) - cpMargin.y);
				
					pDC->MoveTo(DrawFrom.x, DrawFrom.y);
					pDC->LineTo(DrawTo.x, DrawFrom.y);
					pDC->LineTo(DrawTo.x, DrawTo.y);
					pDC->LineTo(DrawFrom.x, DrawTo.y);
					pDC->LineTo(DrawFrom.x, DrawFrom.y);
				}
			}
		}
	}


}




void CCablanView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int i,j;
	int Foundi = -1;
	int Foundj = -1;

	//reset LandingBoundary address
	pDoc->BoundaryEnd.x = -1;
	pDoc->BoundaryEnd.y = -1;
	pDoc->BoundaryStart.x = -1;
	pDoc->BoundaryStart.y = -1;
	pDoc->blLandingBoundarySetting = FALSE;


	if (boolAddLandings == TRUE || boolSettingHarvestUnitBoundary == TRUE || boolTimberExits == TRUE)
	{
		pDC = new CClientDC(this);
		OnPrepareDC(pDC);
		pDC->SetMapMode(MM_ISOTROPIC);
		pDC->SetWindowExt(ViewScale, ViewScale);

		pDC->DPtoLP(&point);

		point.x = (long)(point.x + pDoc->DTMminX - cpMargin.x);
		point.y = (long)(point.y + pDoc->DTMmaxY + cpMargin.y);


		//find column
		for (i = 1; i <= pDoc->DTMcolumns; ++i)
		{
			j = 1;
			//take any row within the boundary
			while (pDoc->DTMCoord[i][j].x < 0)//if < 0, it means beyond the boundary
			{
				++j;
				if (j >= pDoc->DTMrows)
					j = 1;
			}
			if (point.x < pDoc->DTMCoord[i][j].x + pDoc->DTMcellsize/2.0 && point.x > pDoc->DTMCoord[i][j].x - pDoc->DTMcellsize/2.0)
			{
				Foundi = i;
				break;
			}
		}	

		//find row
		if (Foundi >= 0)
		{
			for (j = 1; j <= pDoc->DTMrows; ++j)
			{
				if (point.y < pDoc->DTMCoord[Foundi][j].y + pDoc->DTMcellsize/2.0 && point.y > pDoc->DTMCoord[Foundi][j].y - pDoc->DTMcellsize/2.0)
				{
					Foundj = j;
					break;
				}
			}
		}

		//keep this address for setting landing boundary.
		if (Foundi >= 0 && Foundj >= 0)
		{
			if (boolAddLandings == TRUE)
			{
				//landing boundary.
				pDoc->BoundaryStart.x = Foundi;
				pDoc->BoundaryStart.y = Foundj;
				pDoc->blLandingBoundarySetting = TRUE;
			}
			else if (boolSettingHarvestUnitBoundary == TRUE)//harvest unit boundary
			{
				if (pDoc->TotalNumberofUnitBoundaryVertices < MAXUNITBOUNDARYVERTICES - 1)
				{
					++pDoc->TotalNumberofUnitBoundaryVertices;
					pDoc->HarvestUnitBoundary[pDoc->TotalNumberofUnitBoundaryVertices].col = Foundi;
					pDoc->HarvestUnitBoundary[pDoc->TotalNumberofUnitBoundaryVertices].row = Foundj;
					if (pDoc->TotalNumberofUnitBoundaryVertices > 1)
						DrawEachLine(pDoc->HarvestUnitBoundary[pDoc->TotalNumberofUnitBoundaryVertices-1].col, pDoc->HarvestUnitBoundary[pDoc->TotalNumberofUnitBoundaryVertices-1].row, Foundi, Foundj, 5, RGB(0,0,0));
				}
				else
					AfxMessageBox("No more points for the unit boundary can be accepted.", MB_ICONSTOP);
			}
			else if (boolTimberExits == TRUE)
			{
				//Final destination
				pDoc->TotalNumberofTimberExits++;
				pDoc->TimberExits[pDoc->TotalNumberofTimberExits].column = Foundi;
				pDoc->TimberExits[pDoc->TotalNumberofTimberExits].row = Foundj;
				boolTimberExits = FALSE;
				
				//draw the cell
				for (i = 1; i < pDoc->TotalNumberofTimberExits; ++i)
					DrawEachCell(pDoc->TimberExits[i].column, pDoc->TimberExits[i].row, RGB(0,0,0));//column and row
			}
		}

	}

/*
		////////////
		//ROAD
		///////////
		if (blRoadRoutingbyUser == TRUE)
		{
			blRoadMovingbyUser = -1;
			for(k = 1; k <= NumberofSelectedRoadCellsbyUser; ++k)
			{
				//select a cell already in the array
				if (i == SelectedRoadCellsbyUser[k].x && j == SelectedRoadCellsbyUser[k].y)
				{
					blRoadMovingbyUser = k;
					break;
				}
			}
			
			//new vertex
			if (blRoadMovingbyUser < 0)
			{
				n = 1;
				for (k = 0; k <= MaxRoadLength; ++k)//keep new vertex in order...
				{
					if (pDoc->AccessRoadtoLanding[gbLandingNumber][k].x == SelectedRoadCellsbyUser[n].x && 
						pDoc->AccessRoadtoLanding[gbLandingNumber][k].y == SelectedRoadCellsbyUser[n].y)
						++n;
					if (pDoc->AccessRoadtoLanding[gbLandingNumber][k].x == i && 
						pDoc->AccessRoadtoLanding[gbLandingNumber][k].y == j)
					{
						blRoadMovingbyUser = n;
						break;
					}
				}
				
				if (blRoadMovingbyUser > 0)
				{
					for (k = NumberofSelectedRoadCellsbyUser; k >= n; --k)
					{
						SelectedRoadCellsbyUser[k+1].x = SelectedRoadCellsbyUser[k].x;
						SelectedRoadCellsbyUser[k+1].y = SelectedRoadCellsbyUser[k].y;
					}
				
					SelectedRoadCellsbyUser[n].x = i;
					SelectedRoadCellsbyUser[n].y = j;
					++NumberofSelectedRoadCellsbyUser;
				}				
			}
		}
	}
*/

	CScrollView::OnLButtonDown(nFlags, point);
}


void CCablanView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	//Double click --> Finishing unit boundary input by mouse
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	//stop inputting boundary points (vertices).
	int i,j;
	int Foundi = -1;
	int Foundj = -1;

	//reset LandingBoundary address
	pDoc->BoundaryEnd.x = -1;
	pDoc->BoundaryEnd.y = -1;
	pDoc->BoundaryStart.x = -1;
	pDoc->BoundaryStart.y = -1;
	pDoc->blLandingBoundarySetting = FALSE;


	if (boolSettingHarvestUnitBoundary == TRUE)
	{
		pDC = new CClientDC(this);
		OnPrepareDC(pDC);
		pDC->SetMapMode(MM_ISOTROPIC);
		pDC->SetWindowExt(ViewScale, ViewScale);

		pDC->DPtoLP(&point);

		point.x = (long)(point.x + pDoc->DTMminX - cpMargin.x);
		point.y = (long)(point.y + pDoc->DTMmaxY + cpMargin.y);


		//find column
		for (i = 1; i <= pDoc->DTMcolumns; ++i)
		{
			j = 1;
			//take any row within the boundary
			while (pDoc->DTMCoord[i][j].x < 0)//if < 0, it means beyond the boundary; including 0 for release mode
			{
				++j;
				if (j >= pDoc->DTMrows)
					j = 1;
			}
			if (point.x < pDoc->DTMCoord[i][j].x + pDoc->DTMcellsize/2.0 && point.x > pDoc->DTMCoord[i][j].x - pDoc->DTMcellsize/2.0)
			{
				Foundi = i;
				break;
			}
		}	

		//find row
		if (Foundi >= 0)
		{
			for (j = 1; j <= pDoc->DTMrows; ++j)
			{
				if (point.y < pDoc->DTMCoord[Foundi][j].y + pDoc->DTMcellsize/2.0 && point.y > pDoc->DTMCoord[Foundi][j].y - pDoc->DTMcellsize/2.0)
				{
					Foundj = j;
					break;
				}
			}
		}

		//keep this address for setting landing boundary.
		if (Foundi >= 0 && Foundj >= 0)
		{
			boolSettingHarvestUnitBoundary = FALSE;
			DrawEachLine(pDoc->HarvestUnitBoundary[1].col, pDoc->HarvestUnitBoundary[1].row, pDoc->HarvestUnitBoundary[pDoc->TotalNumberofUnitBoundaryVertices].col, pDoc->HarvestUnitBoundary[pDoc->TotalNumberofUnitBoundaryVertices].row, 5, RGB(0,0,0));
		}
	}

	//select grid cells inside the boundary.
	SettingHarvestingUnitBoundary();
	boolSettingHarvestUnitBoundary = FALSE;

	CScrollView::OnLButtonDblClk(nFlags, point);
}




void CCablanView::OnMouseMove(UINT nFlags, CPoint point) 
{

	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);


	int i,j;
	int Foundi = -1;
	int Foundj = -1;
	CString tempString;

	pDC = new CClientDC(this);
	OnPrepareDC(pDC);
	pDC->SetMapMode(MM_ISOTROPIC);
	pDC->SetWindowExt(ViewScale, ViewScale);
	pDC->DPtoLP(&point);

	point.x = (long)(point.x + pDoc->DTMminX - cpMargin.x);
	point.y = (long)(point.y + pDoc->DTMmaxY + cpMargin.y);


	//find column
	for (i = 1; i <= pDoc->DTMcolumns; ++i)
	{
		j = 1;
		//take any row within the boundary
		while (pDoc->DTMCoord[i][j].x < 0)//if < 0, it means beyond the boundary
		{
			++j;
			if (j >= pDoc->DTMrows)
				j = 1;
		}
		if (point.x < pDoc->DTMCoord[i][j].x + pDoc->DTMcellsize/2.0 && point.x > pDoc->DTMCoord[i][j].x - pDoc->DTMcellsize/2.0)
		{
			Foundi = i;
			break;
		}
	}

	//find row
	if (Foundi >= 0)
	{
		for (j = 1; j <= pDoc->DTMrows; ++j)
		{
			if (point.y < pDoc->DTMCoord[Foundi][j].y + pDoc->DTMcellsize/2.0 && point.y > pDoc->DTMCoord[Foundi][j].y - pDoc->DTMcellsize/2.0)
			{	
				Foundj = j;
				break;
			}
		}
	}


	if (Foundi >= 0 && Foundj >= 0)
	{
		//legend
		if (boolVolumeLayerShowing == FALSE)
			tempString.Format("x:%d  y:%d  z:%d  c:%d  r:%d      ", pDoc->DTMCoord[Foundi][Foundj].x, pDoc->DTMCoord[Foundi][Foundj].y, pDoc->DTMCoord[Foundi][Foundj].z, Foundi, Foundj);
		else//now DEM shows up on the screen.
			tempString.Format("x:%d  y:%d  z:%.1f  c:%d  r:%d         ", pDoc->DTMCoord[Foundi][Foundj].x, pDoc->DTMCoord[Foundi][Foundj].y, pDoc->OriginalVolume[Foundi][Foundj], Foundi, Foundj);
	}


	if(nFlags == MK_LBUTTON)//when left mouse button is held down.
	{
		if (pDoc->blLandingBoundarySetting == TRUE)
		{
			if (Foundi >= 0 && Foundj >= 0)
			{
				pDoc->BoundaryEnd.x = Foundi;
				pDoc->BoundaryEnd.y = Foundj;

				if (pDoc->BoundaryStart.x >= 0 && pDoc->BoundaryStart.y >= 0)
				{
					DrawEachLine(pDoc->BoundaryStart.x, pDoc->BoundaryStart.y, pDoc->BoundaryEnd.x, pDoc->BoundaryStart.y, -1, RGB(0,0,255));//width -1 means dotted line.
					DrawEachLine(pDoc->BoundaryEnd.x, pDoc->BoundaryStart.y, pDoc->BoundaryEnd.x, pDoc->BoundaryEnd.y, -1, RGB(0,0,255));
					DrawEachLine(pDoc->BoundaryEnd.x, pDoc->BoundaryEnd.y, pDoc->BoundaryStart.x, pDoc->BoundaryEnd.y, -1, RGB(0,0,255));
					DrawEachLine(pDoc->BoundaryStart.x, pDoc->BoundaryEnd.y, pDoc->BoundaryStart.x, pDoc->BoundaryStart.y, -1, RGB(0,0,255));
				}
			}
		}
	}



	//set a different mapmode for text output (legend)
	pDC = new CClientDC(this);
	OnPrepareDC(pDC);
	pDC->SetMapMode(MM_TEXT);//just for legend
	CRect WinRect;
	GetClientRect(WinRect);
	//writing guide
	pDC->TextOut(WinRect.left + 50, 3, tempString);



/*
//road placement by user mouse moving.
	if (blRoadMovingbyUser > 0)
	{
		CCablanDoc* pDoc = GetDocument();
		ASSERT_VALID(pDoc);

	
		int i,j;
		int Foundi = -1;
		int Foundj = -1;

		pDC = new CClientDC(this);
		OnPrepareDC(pDC);
		pDC->SetMapMode(MM_ISOTROPIC);
		pDC->SetWindowExt(ViewScale, ViewScale);

		pDC->DPtoLP(&point);

		point.x = (long)(point.x + pDoc->DTMminX - cpMargin.x);
		point.y = (long)(point.y + pDoc->DTMmaxY + cpMargin.y);


		//find column
		for (i = 1; i <= pDoc->DTMcolumns; ++i)
		{
			j = 1;
			//take any row within the boundary
			while (pDoc->DTMCoord[i][j].x < 0)//if < 0, it means beyond the boundary
			{
				++j;
				if (j >= pDoc->DTMrows)
					j = 1;
			}
			if (point.x < pDoc->DTMCoord[i][j].x + pDoc->DTMcellsize/2.0 && point.x > pDoc->DTMCoord[i][j].x - pDoc->DTMcellsize/2.0)
			{
				Foundi = i;
				break;
			}
		}

		//find row
		for (j = 1; j <= pDoc->DTMrows; ++j)
		{
			if (point.y < pDoc->DTMCoord[Foundi][j].y + pDoc->DTMcellsize/2.0 && point.y > pDoc->DTMCoord[Foundi][j].y - pDoc->DTMcellsize/2.0)
			{	
				Foundj = j;
				break;
			}
		}


		CString tempString;
		
		if (Foundi >= 0 && Foundj >= 0)
		{
			SelectedRoadCellsbyUser[blRoadMovingbyUser].x = Foundi;
			SelectedRoadCellsbyUser[blRoadMovingbyUser].y = Foundj;
			

			//for next and find route between two verteces.
		}
	}
*/

	CScrollView::OnMouseMove(nFlags, point);
}




void CCablanView::DrawCablewayProfile(int landingid, int systemid, int cablewayid)
{
	//select Grid cells on the cable profile
	//keep Grid cell data into Profile[], which is used in payload analysis
	//Profile[] array starts from 1.
	//set Cableways[][].totalnumberofoprofilecells
	pDC = new CClientDC(this);
	OnPrepareDC(pDC);
	
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pDC->SetMapMode(MM_ISOTROPIC);
	pDC->SetWindowExt(ViewScale, ViewScale);

	int i, j;
	double x, y;//real coordinate in linear equation which developed by slop and intercept
	int start, ending;
	int landingcolumn = pDoc->Landings[landingid].column;
	int landingrow = pDoc->Landings[landingid].row;
	CPoint CellAddress;

	
	//determine parameters for for-next loop -- select one axis as independent variable based on the slope
	if (fabs(pDoc->Cableways[landingid][cablewayid].slope) > 1 && fabs(pDoc->Cableways[landingid][cablewayid].slope) < 10000.0)//select y-axis as an independent variable
	{
		if (landingrow < pDoc->Cableways[landingid][cablewayid].endrow)
		{
			start = landingrow;
			ending = pDoc->Cableways[landingid][cablewayid].endrow;
		}
		else
		{
			start = pDoc->Cableways[landingid][cablewayid].endrow;
			ending = landingrow;
		}

		TotalNumberofProfileCells = 0;//initializing
		for (j = start; j <= ending; ++j)//changing in y direction
		{
			//find real coordinates
			y = pDoc->DTMCoord[landingcolumn][j].y;
			x = ( y - pDoc->Cableways[landingid][cablewayid].intercept) / pDoc->Cableways[landingid][cablewayid].slope;//calculate real x coordinate from y
			CellAddress = FindCellAddressbasedonCoordinates(x, y);

			if (pDoc->DTMCoord[CellAddress.x][CellAddress.y].z > 0)//only if this cell has elevation. (not no data)
			{
				++TotalNumberofProfileCells;
				Profile[TotalNumberofProfileCells].column = CellAddress.x;
				Profile[TotalNumberofProfileCells].row = CellAddress.y;
				Profile[TotalNumberofProfileCells].elevation = pDoc->DTMCoord[CellAddress.x][CellAddress.y].z;
				Profile[TotalNumberofProfileCells].flatdistancefromlanding = (float)sqrt( pow((double)pDoc->DTMCoord[landingcolumn][landingrow].x - x, 2.0) + pow((double)pDoc->DTMCoord[landingcolumn][landingrow].y - y, 2.0) );
				if (pDoc->FishStreamBuffer[CellAddress.x][CellAddress.y] > 0)
					Profile[TotalNumberofProfileCells].fishstreambuffer = TRUE;
				else
					Profile[TotalNumberofProfileCells].fishstreambuffer = FALSE;
			}
		}
	}
	else if (fabs(pDoc->Cableways[landingid][cablewayid].slope) > 10000.0)//when slope is infinite ->North and South direction
	{
		if (landingrow < pDoc->Cableways[landingid][cablewayid].endrow)
		{
			start = landingrow;
			ending = pDoc->Cableways[landingid][cablewayid].endrow;
		}
		else
		{
			start = pDoc->Cableways[landingid][cablewayid].endrow;
			ending = landingrow;
		}


		TotalNumberofProfileCells = 0;//initializing
		for (j = start; j <= ending; ++j)//changing in y direction
		{
			y = pDoc->DTMCoord[landingcolumn][j].y;
			x = pDoc->DTMCoord[landingcolumn][landingrow].x;
			CellAddress = FindCellAddressbasedonCoordinates(x, y);

			if (pDoc->DTMCoord[CellAddress.x][CellAddress.y].z > 0)//only if this cell has elevation. (not no data)
			{
				++TotalNumberofProfileCells;
				Profile[TotalNumberofProfileCells].column = CellAddress.x;
				Profile[TotalNumberofProfileCells].row = CellAddress.y;
				Profile[TotalNumberofProfileCells].elevation = pDoc->DTMCoord[CellAddress.x][CellAddress.y].z;
				Profile[TotalNumberofProfileCells].flatdistancefromlanding = (float)sqrt( pow((double)pDoc->DTMCoord[landingcolumn][landingrow].x - x, 2.0) + pow((double)pDoc->DTMCoord[landingcolumn][landingrow].y - y, 2.0) );
				if (pDoc->FishStreamBuffer[CellAddress.x][CellAddress.y] > 0)
					Profile[TotalNumberofProfileCells].fishstreambuffer = TRUE;
				else
					Profile[TotalNumberofProfileCells].fishstreambuffer = FALSE;
			}
		}
	}
	else//select x-axis as an independent variable
	{
		if (landingcolumn < pDoc->Cableways[landingid][cablewayid].endcolumn)
		{
			start = landingcolumn;
			ending = pDoc->Cableways[landingid][cablewayid].endcolumn;
		}
		else
		{
			start = pDoc->Cableways[landingid][cablewayid].endcolumn;
			ending = landingcolumn;
		}

		TotalNumberofProfileCells = 0;//initializing
		for (i = start; i <= ending; ++i)//changing in x direction
		{
			x = pDoc->DTMCoord[i][landingrow].x;
			y = x * pDoc->Cableways[landingid][cablewayid].slope + pDoc->Cableways[landingid][cablewayid].intercept;//calculate real x coordinate from y
			CellAddress = FindCellAddressbasedonCoordinates(x, y);

			if (pDoc->DTMCoord[CellAddress.x][CellAddress.y].z > 0)//only if this cell has elevation. (not no data)
			{
				++TotalNumberofProfileCells;
				Profile[TotalNumberofProfileCells].column = CellAddress.x;
				Profile[TotalNumberofProfileCells].row = CellAddress.y;
				Profile[TotalNumberofProfileCells].elevation = pDoc->DTMCoord[CellAddress.x][CellAddress.y].z;
				Profile[TotalNumberofProfileCells].flatdistancefromlanding = (float)sqrt( pow((double)pDoc->DTMCoord[landingcolumn][landingrow].x - x, 2.0) + pow((double)pDoc->DTMCoord[landingcolumn][landingrow].y - y, 2.0) );
				if (pDoc->FishStreamBuffer[CellAddress.x][CellAddress.y] > 0)
					Profile[TotalNumberofProfileCells].fishstreambuffer = TRUE;
				else
					Profile[TotalNumberofProfileCells].fishstreambuffer = FALSE;
			}
		}
	}


	
	//reordering profile cells in order to be from the landing
	PROFILE tempProfile[MAXCELLS];
	if (Profile[1].flatdistancefromlanding > Profile[TotalNumberofProfileCells].flatdistancefromlanding)
	{
		memcpy(tempProfile, Profile, sizeof(tempProfile));
		for (i = 1; i <= TotalNumberofProfileCells; ++i)
		{
			Profile[i].column = tempProfile[TotalNumberofProfileCells + 1 - i].column;
			Profile[i].elevation = tempProfile[TotalNumberofProfileCells + 1 - i].elevation;
			Profile[i].flatdistancefromlanding = tempProfile[TotalNumberofProfileCells + 1 - i].flatdistancefromlanding;
			Profile[i].row = tempProfile[TotalNumberofProfileCells + 1 - i].row;
			Profile[i].fishstreambuffer = tempProfile[TotalNumberofProfileCells + 1 - i].fishstreambuffer;
		}
	}


	//if there is a cell having -99999 nodata, then get rid of this cell.
/*
	short countcell;
	float elevation;
	
	for (i = 1; i <= TotalNumberofProfileCells; ++i)
	{
		if (Profile[i].elevation < 0)
		{
			elevation = 0.;
			countcell = 0;

			for (j = Profile[i].column - 2; j <= Profile[i].column + 2; ++j)
			{
				for (k = Profile[i].row - 2; k <= Profile[i].row + 2; ++k)
				{
					if (j > 0 && j < pDoc->DTMcolumns && k > 0 && k < pDoc->DTMrows)
					{
						if (pDoc->DTMCoord[j][k].z > 0)
						{
							elevation += (float)pDoc->DTMCoord[j][k].z;
							countcell++;
						}
					}
				}
			}

			elevation /= (float)countcell;
			Profile[i].elevation = (int)elevation;
		}
	}
*/



	////set Cableways[][].totalnumberofoprofilecells/////////////////////////////
	pDoc->Cableways[landingid][cablewayid].totalnumberofprofilecells = TotalNumberofProfileCells;

	
	//reinitiallize Cableways[][].taillocationID
	if (pDoc->CableSystems[systemid].boolSpecChangebyUser == TRUE)
		pDoc->Cableways[landingid][cablewayid].taillocationID = TotalNumberofProfileCells;

	

/*
	//draw cells on the profile until tailspar location
	if (pDoc->Cableways[landingid][cablewayid].taillocationID > 0)//tail spar is already set up.
	{
		for (i = 1; i < pDoc->Cableways[landingid][cablewayid].taillocationID; ++i)
			DrawEachCell(Profile[i].column, Profile[i].row, RGB(50,50,50));
	}
	else
	{
		for (i = 1; i < TotalNumberofProfileCells; ++i)
			DrawEachCell(Profile[i].column, Profile[i].row, RGB(50,50,50));
	}
*/


}





void CCablanView::OnZoomIn() 
{
	// TODO: Add your command handler code here
	if (ViewScale > 1000)
		ViewScale -= 1000;
	
	//cleaning the screen
	pDC = new CClientDC(this);
	OnPrepareDC(pDC);
	pDC->SetMapMode(MM_TEXT);//just for legend
	pDC->SelectStockObject(WHITE_BRUSH);
	pDC->Rectangle(0, 0, 5000, 5000);

	DrawDTM();
}

void CCablanView::OnZoomOut() 
{
	// TODO: Add your command handler code here
	ViewScale += 1000;

	//cleaning the screen
	pDC = new CClientDC(this);
	OnPrepareDC(pDC);
	pDC->SetMapMode(MM_TEXT);//just for legend
	pDC->SelectStockObject(WHITE_BRUSH);
	pDC->Rectangle(0, 0, 5000, 5000);

	DrawDTM();
}











void CCablanView::OnLandingsPlacelanding() 
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	boolAddLandings = TRUE;
	CString tempString;
	tempString.Format("You cannot add more than %d landing alternatives.", MAXLANDINGS);

	if (pDoc->TotalNumberofLandings <= 0)//if this is the first landing alternative
		pDoc->TotalNumberofLandings = 1;
	else if (pDoc->TotalNumberofLandings >= MAXLANDINGS)
		AfxMessageBox(tempString, MB_ICONSTOP);
	else if (pDoc->Landings[pDoc->TotalNumberofLandings].CableSystem > 0)//only in case that the previous landing has real data.
		++pDoc->TotalNumberofLandings;//add one more to the total number of landing alternatives
}



void CCablanView::OnCablewaysGenerate() 
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	DrawDTM();

	int i, j;
	int cs;
	CString tempString;
	

	for (i = 1; i <= pDoc->TotalNumberofLandings; ++i)//i starts from 1.. Zero is used as a temprorary variable in FindBestLandingLocationWithinBoundary().
	{
		cs = pDoc->Landings[i].CableSystem;
		//copy current cableway data to temporary global variable
		memcpy(pDoc->tempCableways, pDoc->Cableways[i], sizeof(pDoc->tempCableways));
		GenerateCablewaysperLanding(pDoc->Landings[i].column, pDoc->Landings[i].row, cs);
		//copy back to the current cableway variable.
		memcpy(pDoc->Cableways[i], pDoc->tempCableways, sizeof(pDoc->tempCableways));


		for (j = 0; j <=35; ++j)
		{
			if (pDoc->Cableways[i][j].tailcolumn <= 0 || pDoc->CableSystems[cs].boolSpecChangebyUser == TRUE)
			{
				//return to initial values.
				pDoc->Cableways[i][j].tailcolumn = pDoc->Cableways[i][j].endcolumn;
				pDoc->Cableways[i][j].tailrow = pDoc->Cableways[i][j].endrow;
				//run profile and keep totalnumberofprofilecells in order to reflect system spec. changes.
				DrawCablewayProfile(i, cs, j);//set Cableways[][].totalnumberofprofilecells
				pDoc->Cableways[i][j].taillocationID = pDoc->Cableways[i][j].totalnumberofprofilecells;
			}
		}

		//return boolean value
		pDoc->CableSystems[cs].boolSpecChangebyUser = FALSE;


		//To print only landing location, delete below line temporarily
		DrawTailLocationsandCableways(i, cs);//draw tail location and line between landing and tails

		//write tower ID
		tempString.Format("%d", i);
		WriteTextAboveGridCell(pDoc->Landings[i].column, pDoc->Landings[i].row, tempString);
	}

	//write tower ID after draw all cable roads.
	for (i = 1; i <= pDoc->TotalNumberofLandings; ++i)//i starts from 1.. Zero is used as a temprorary variable in FindBestLandingLocationWithinBoundary().
	{
		tempString.Format("%d", i);
		WriteTextAboveGridCell(pDoc->Landings[i].column, pDoc->Landings[i].row, tempString);
	}


	boolCheckProfile = TRUE;//enable to open DialodPathAnalysis dialog box to see each profile.
}


void CCablanView::OnCablewaysWriteidnumbers() 
{
	//just writing cable road ID number
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int i;
	CString tempString;
	
	//write tower ID after draw all cable roads.
	for (i = 1; i <= pDoc->TotalNumberofLandings; ++i)//i starts from 1.. Zero is used as a temprorary variable in FindBestLandingLocationWithinBoundary().
	{
		tempString.Format("%d", i);
		WriteTextAboveGridCell(pDoc->Landings[i].column, pDoc->Landings[i].row, tempString);
	}


}


void CCablanView::GenerateCablewaysperLanding(int LandingCol, int LandingRow, int CableSystem)
{	
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int j;
	int celldist;
		
	double newx, newy;
	CPoint TailLocation;
	float slope, intercept;
	double a, b, c;


	celldist = (int) (pDoc->CableSystems[CableSystem].maxyarddist / pDoc->DTMcellsize);//in meter

	//Cableways[landingID][SystemID][0~35]
	//the first alternative -- North
	if (LandingRow - celldist > 0)
		pDoc->tempCableways[0].endrow = LandingRow - celldist;
	else
		pDoc->tempCableways[0].endrow = 1;
	pDoc->tempCableways[0].endcolumn = LandingCol;
	pDoc->tempCableways[0].slope = (float)999999.9;
	pDoc->tempCableways[0].intercept = (float)999999.9;

	//South
	if (LandingRow + celldist <= pDoc->DTMrows)
		pDoc->tempCableways[18].endrow = LandingRow + celldist;
	else
		pDoc->tempCableways[18].endrow = pDoc->DTMrows;
	pDoc->tempCableways[18].endcolumn = LandingCol;
	pDoc->tempCableways[18].slope = (float)999999.9;
	pDoc->tempCableways[18].intercept = (float)999999.9;

	//East 
	pDoc->tempCableways[9].endrow = LandingRow;
	if (LandingCol + celldist <= pDoc->DTMcolumns)
		pDoc->tempCableways[9].endcolumn = LandingCol + celldist;
	else
		pDoc->tempCableways[9].endcolumn = pDoc->DTMcolumns;
	pDoc->tempCableways[9].slope = (float)0.0;
	pDoc->tempCableways[9].intercept = (float)pDoc->DTMCoord[LandingCol][LandingRow].y;
	
	//West
	pDoc->tempCableways[27].endrow = LandingRow;
	if (LandingCol - celldist > 0)
		pDoc->tempCableways[27].endcolumn = LandingCol - celldist;
	else
		pDoc->tempCableways[27].endcolumn = 1;
	pDoc->tempCableways[27].slope = (float)0.0;
	pDoc->tempCableways[27].intercept = (float)pDoc->DTMCoord[LandingCol][LandingRow].y;

	
	for (j = 1; j <= 17; ++j)//j is in 10 degrees
	{
		if (j != 9)
		{
			slope = (float)tan( (double)j*10.0*3.14/180.0 ) ;
			intercept = (float)pDoc->DTMCoord[LandingCol][LandingRow].y - slope * (float)pDoc->DTMCoord[LandingCol][LandingRow].x;

			a = 1.0 + pow(slope, 2.0);
			b = slope*intercept - (double)pDoc->DTMCoord[LandingCol][LandingRow].x - slope*(double)pDoc->DTMCoord[LandingCol][LandingRow].y;
			c = pow((double)pDoc->DTMCoord[LandingCol][LandingRow].x, 2.0) + pow(intercept, 2.0) + pow((double)pDoc->DTMCoord[LandingCol][LandingRow].y, 2.0) - pow((double)pDoc->CableSystems[CableSystem].maxyarddist, 2.0) - 2.0*intercept*(double)pDoc->DTMCoord[LandingCol][LandingRow].y;

			newx = ( -b + sqrt(pow(b, 2.0) - a*c) ) / a;
			newy = slope * newx + intercept;

			TailLocation = FindCellAddressbasedonCoordinates(newx, newy);
			pDoc->tempCableways[j].endcolumn = TailLocation.x;
			pDoc->tempCableways[j].endrow = TailLocation.y;

			//another solution for opposite direction-- need to calculate before changing slope and intercept
			newx = ( -b - sqrt(pow(b, 2.0) - a*c) ) / a;
			newy = slope * newx + intercept;

			//recalculate slope for the case that cable ways hit the boundary.. slope and intercept should be changed in accordence with new tail location
			slope = (float)(pDoc->DTMCoord[LandingCol][LandingRow].y - pDoc->DTMCoord[TailLocation.x][TailLocation.y].y) / (float)(pDoc->DTMCoord[LandingCol][LandingRow].x - pDoc->DTMCoord[TailLocation.x][TailLocation.y].x);
			intercept = (float)pDoc->DTMCoord[LandingCol][LandingRow].y - slope * (float)pDoc->DTMCoord[LandingCol][LandingRow].x;
			pDoc->tempCableways[j].slope = slope;
			pDoc->tempCableways[j].intercept = intercept;

			//use newx and newy resulted from the above
			TailLocation = FindCellAddressbasedonCoordinates(newx, newy);

			pDoc->tempCableways[j+18].endcolumn = TailLocation.x;
			pDoc->tempCableways[j+18].endrow = TailLocation.y;
			//recalculate slope for the case that cable ways hit the boundary.. slope and intercept should be changed in accordence with new tail location
			slope = (float)(pDoc->DTMCoord[LandingCol][LandingRow].y - pDoc->DTMCoord[TailLocation.x][TailLocation.y].y) / (float)(pDoc->DTMCoord[LandingCol][LandingRow].x - pDoc->DTMCoord[TailLocation.x][TailLocation.y].x);
			intercept = (float)pDoc->DTMCoord[LandingCol][LandingRow].y - slope * (float)pDoc->DTMCoord[LandingCol][LandingRow].x;
			pDoc->tempCableways[j+18].slope = slope;
			pDoc->tempCableways[j+18].intercept = intercept;
		}
	}

}




CPoint CCablanView::FindCellAddressbasedonCoordinates(double x, double y)
{
	//x and y is real coordinates

	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int i, j;
	int column, row;
	double mindistance;
	double distance;
	CPoint CellAddress;
	CellAddress.x = 0;
	CellAddress.y = 0;

	//find column
	column = 0;
	mindistance = 9999999999999.9;
	for (i = 1; i <= pDoc->DTMcolumns; ++i)
	{
		//take any row within the boundary
		j = 1;
		while (pDoc->DTMCoord[i][j].x < 0)//if < 0, it means beyond the boundary or outof data
		{
			++j;
			if (j > pDoc->DTMrows)
				j = 1;
		}

		distance = fabs((double)pDoc->DTMCoord[i][j].x - x);
		if (distance < mindistance)
		{
			mindistance = distance;//find the closest column to prevent error... this loop must find one...
			column = i;					
		}
	}

	
	if (column != 0)
	{
		//find row
		row = 0;
		mindistance = 9999999999999.9;
		for (j = 1; j <= pDoc->DTMrows; ++j)
		{
			//take any column within the boundary
			i = 1;
			while (pDoc->DTMCoord[i][j].y <= 0)//if < 0, it means beyond the boundary; including 0 for release mode
			{
				++i;
				if (i > pDoc->DTMcolumns)
					i = 1;
			}

			distance = fabs((double)pDoc->DTMCoord[i][j].y - y);
			if (distance < mindistance)
			{
				mindistance = distance;
				row = j;
			}
		}
	}


	CellAddress.x = column;
	CellAddress.y = row;


	//just in case: if the above for-next statement fails to find a right cell address, search all grid cells
	if (CellAddress.x == 0 || CellAddress.y ==0)
	{
		for (j = 1; j <= pDoc->DTMrows; ++j)
		{
			for (i = 1; i <= pDoc->DTMcolumns; ++i)
			{
				distance = pow(x - pDoc->DTMCoord[i][j].x, 2.0) + pow(y - pDoc->DTMCoord[i][j].y, 2.0);
				if (distance < pow(pDoc->DTMcellsize, 2.0))
				{
					CellAddress.x = i;
					CellAddress.y = j;
					break;
				}
			}
			
			if (CellAddress.x != 0)
				break;
		}
	}

	return (CellAddress);

}




void CCablanView::DrawTailLocationsandCableways(int landingid, int systemid) 
{
	//drawing ending points
	//draw 36 cable ways from the landing.
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int i,j,k;
	int landingrow, landingcolumn;
	CString tempString;
		
	
	for (k = 0; k <= 35; ++k)
	{
		if (pDoc->Cableways[landingid][k].taillocationID > MINCELLALONGCABLEROAD)
		{
			i = pDoc->Cableways[landingid][k].tailcolumn;
			j = pDoc->Cableways[landingid][k].tailrow;
		
			landingcolumn = pDoc->Landings[landingid].column;
			landingrow = pDoc->Landings[landingid].row;

			if (i > 0 && j > 0 && i <= pDoc->DTMcolumns && j <= pDoc->DTMrows)
			{
				DrawFrom.x = (long)(pDoc->DTMCoord[i][j].x - pDoc->DTMminX - (pDoc->DTMcellsize/2.0) + cpMargin.x);// (CellHeight/2) integer problem??
				DrawFrom.y = (long)(pDoc->DTMCoord[i][j].y - pDoc->DTMmaxY - (pDoc->DTMcellsize/2.0) - cpMargin.y);
				DrawTo.x = (long)(pDoc->DTMCoord[i][j].x - pDoc->DTMminX + (pDoc->DTMcellsize/2.0) + cpMargin.x);
				DrawTo.y = (long)(pDoc->DTMCoord[i][j].y - pDoc->DTMmaxY + (pDoc->DTMcellsize/2.0) - cpMargin.y);
	
				// drawing
				CRect rcSquare(DrawFrom.x, DrawFrom.y, DrawTo.x, DrawTo.y);
				pDC->FillSolidRect(rcSquare, RGB(0,0,0));

				//draw a line-cable way
				DrawFrom.x = (long)(pDoc->DTMCoord[i][j].x - pDoc->DTMminX + cpMargin.x);
				DrawFrom.y = (long)(pDoc->DTMCoord[i][j].y - pDoc->DTMmaxY - cpMargin.y);
				DrawTo.x = (long)(pDoc->DTMCoord[landingcolumn][landingrow].x - pDoc->DTMminX + cpMargin.x);
				DrawTo.y = (long)(pDoc->DTMCoord[landingcolumn][landingrow].y - pDoc->DTMmaxY - cpMargin.y);

				pDC->MoveTo(DrawFrom);
				pDC->LineTo(DrawTo);
		
				//legend
				tempString.Format("x:%d  y:%d  z:%d", pDoc->DTMCoord[i][j].x, pDoc->DTMCoord[i][j].y, pDoc->DTMCoord[i][j].z);
			}
		}
	}
}


void CCablanView::DrawEachLine(int startcolumn, int startrow, int endcolumn, int endrow, int width, COLORREF RGB)
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	if (width > 0)
		penColor[1].CreatePen(PS_SOLID, width, RGB);
	else
		penColor[1].CreatePen(PS_DOT, 5, RGB);
	
	CPen *pOldPen = pDC->SelectObject(&penColor[1]);

	//draw a line-cable way
	DrawFrom.x = (long)(pDoc->DTMCoord[startcolumn][startrow].x - pDoc->DTMminX + cpMargin.x);
	DrawFrom.y = (long)(pDoc->DTMCoord[startcolumn][startrow].y - pDoc->DTMmaxY - cpMargin.y);
	DrawTo.x = (long)(pDoc->DTMCoord[endcolumn][endrow].x - pDoc->DTMminX + cpMargin.x);
	DrawTo.y = (long)(pDoc->DTMCoord[endcolumn][endrow].y - pDoc->DTMmaxY - cpMargin.y);

	pDC->MoveTo(DrawFrom);
	pDC->LineTo(DrawTo);

	pDC->SelectObject(pOldPen);

	//delete CPen Object
	penColor[1].DeleteObject();

}







void CCablanView::DrawEachCell(int column, int row, COLORREF RGB)
{
	//drawing ending points
	//refill cells with different color 
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CRect rcSquare;

	
	if (column > 0 && row > 0 && row <= pDoc->DTMrows && column <= pDoc->DTMcolumns)
	{
		DrawFrom.x = (long)(pDoc->DTMCoord[column][row].x - pDoc->DTMminX - (pDoc->DTMcellsize/2.0) + cpMargin.x);// (CellHeight/2) integer problem??
		DrawFrom.y = (long)(pDoc->DTMCoord[column][row].y - pDoc->DTMmaxY - (pDoc->DTMcellsize/2.0) - cpMargin.y);
		DrawTo.x = (long)(pDoc->DTMCoord[column][row].x - pDoc->DTMminX + (pDoc->DTMcellsize/2.0) + cpMargin.x);
		DrawTo.y = (long)(pDoc->DTMCoord[column][row].y - pDoc->DTMmaxY + (pDoc->DTMcellsize/2.0) - cpMargin.y);
	
		// drawing
		rcSquare.left = DrawFrom.x;
		rcSquare.bottom = DrawFrom.y;
		rcSquare.right = DrawTo.x;
		rcSquare.top = DrawTo.y;
		pDC->FillSolidRect(rcSquare, RGB);
	}
		
}






void CCablanView::OnCablewaysCheckprofile() 
{
	boolCheckProfile = TRUE;
}








//*************************************************************************************************
//***************************************
//Payload analysis
//***************************************
//*************************************************************************************************


void CCablanView::GenerateIntermediateSupports(int CableSystemNo, float TailSparHeight)
{
	//also counting the total number of INT supports for this cable.
	//uses Profile[] which was set up in DrawCablewayProfile().
	//set up a global array in View which is boolINTsupport[] for further use in PayloadAnalysis()
	//set up boolean variables: DownHill, HaulBackNeeded, SteepChange, and MoreINTSupportsRequired.
	//call PayloadAnalysis() with landingID of 0, in order to determine the INT support that has the least effects on payload.
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	float aveslope, slope1, slope2, diffx, diffy;
	int i;
	float diff, diff1, diff2;

	//user-defined variables
	float HeadSparHeight = (float)((double)pDoc->CableSystems[CableSystemNo].towerheight);//intermediate support height
	float INTHeight = (float)((double)pDoc->CableSystems[CableSystemNo].intermediatesupportheight);//intermediate support height
	

	float MaxSpan = pDoc->CableSystems[CableSystemNo].maxyarddist;
	float MinSlope = pDoc->CableSystems[CableSystemNo].gbMinSlopewithoutHBL;//minimum slope for tower without haulback in percent.
	float MaxSlope = (float)(pDoc->CableSystems[CableSystemNo].gbMaxSlopeChange);//(in degree) maximum allowable change in chord slope between adjacent spans->convert from percentage.
	int MaxINTSupports = pDoc->CableSystems[CableSystemNo].gbMaxINTSupports;//limit of number of INT supports.
		

	//initializing output variables
	memset(boolINTsupport, 0, sizeof(boolINTsupport));
	TotalNumberofINTs = 0;
	


		 
	//select possible intermediate supports
	//1. set intermediate supports in each terrain point
	//2. if the angle between adjacent int. supports
	
	//change on April 12, 2002
	aveslope = (float)100. * (float)(Profile[1].elevation - Profile[TailLocationID].elevation) / (float)(Profile[1].flatdistancefromlanding - Profile[TailLocationID].flatdistancefromlanding);


	//check if downhill or haulbackline is necessary
	if (aveslope > 0.)//downhill yarding
	{
		//AfxMessageBox("Downhill yarding!", MB_ICONSTOP);
		DownHill = TRUE;
	}
	else if (aveslope > MinSlope)
	{
		HaulBackNeeded = TRUE;
		//AfxMessageBox("Haulbackline is necessary", MB_ICONSTOP);
	}
	
	
	//**1. generate all possible intermediate supports -- from Dr.Sessions code
	for (i = 2; i < TailLocationID; ++i)//not include 1 nor TailLocationID because of i-1 and i+1
	{
		if (i == 2)
			diff = HeadSparHeight - INTHeight;
		else if (i == TailLocationID - 1)
			diff = INTHeight - TailSparHeight;
		else 
			diff = 0;

		diffx = (float)(Profile[i].flatdistancefromlanding - Profile[i-1].flatdistancefromlanding);
		diffy = (float)(Profile[i].elevation - Profile[i-1].elevation - diff);
		slope1 = diffy / diffx;//slope from behind to current pt.
		diffx = (float)(Profile[i+1].flatdistancefromlanding - Profile[i-1].flatdistancefromlanding);
		diffy = (float)(Profile[i+1].elevation - Profile[i-1].elevation - diff);
		slope2 = diffy / diffx;//slope from behind to ahead pt.

		//permit an intermediate support at convex point (slope1 > slope2)
		//or at a point which is slightly concave (slope1 > slope 2 - 0.02)
		if (slope2 < slope1)//?????????????????????????????????????no carriage move by gravitiy is considered.
		{//this works for both uphill and downhill.
			++TotalNumberofINTs;
			boolINTsupport[i] = TRUE;
		}
	}

	//**2. eliminate unecessary intermediate spars -- Woodam's code
	//temporary setting
	boolINTsupport[1] = TRUE;//headspar
	boolINTsupport[TailLocationID] = TRUE;//tailspar

	bool booltemp = FALSE;
	int tempb, tempc;
	while (booltemp == FALSE)
	{
		booltemp = TRUE;
		for (i = 2; i < TailLocationID; ++i)
		{
			if (boolINTsupport[i] == TRUE)
			{
				tempb = LeftSparLocation(i);//find left closest spar location from ith TP.
				tempc = RightSparLocation(i);//find right closest spar location from ith TP.

				if (tempb == 1)//from headspar
					diff1 = HeadSparHeight - INTHeight;
				else
					diff1 = 0;
				if (tempc == TailLocationID)//to tailspar
					diff2 = INTHeight - TailSparHeight;
				else
					diff2 = 0;

				diffx = (float)(Profile[i].flatdistancefromlanding - Profile[tempb].flatdistancefromlanding);
				diffy = (float)(Profile[i].elevation - Profile[tempb].elevation - diff1);
				slope1 = diffy / diffx;//slope from behind to current pt.
				diffx = (float)(Profile[tempc].flatdistancefromlanding - Profile[i].flatdistancefromlanding);
				diffy = (float)(Profile[tempc].elevation - Profile[i].elevation - diff2);
				slope2 = diffy / diffx;//slope from current to ahead pt.

				//slopes are negative when uphill
				//get rid of this INT support
				if (slope2 >= slope1)//?????????????????????????????????????no carriage move by gravitiy is considered.
				{
					//works both for uphill (negative slope) and downhill (positive slope).
					//concave geographic shape
					boolINTsupport[i] = FALSE;
					--TotalNumberofINTs;
					booltemp = FALSE;
				}
			}
		}
	}


	
	
	//**3. calculate deflection at each terrain point to eliminate unnecessary intermediate supports.
	//temporary variables
	float dx, dy, slope, L, D, P05;
	float his1, his2;
//	int j;

	//user-defined variables
	float Clearance = pDoc->CableSystems[CableSystemNo].gbClearance;
	float AllowableDeflection = pDoc->CableSystems[CableSystemNo].gbAllowableDeflection;
	


	if (TotalNumberofINTs > 0)//if there are INT supports.
	{
		for (i = 2; i < TailLocationID; ++i)
		{
			if (boolINTsupport[i] == TRUE)
			{
				tempb = LeftSparLocation(i);//find left closest spar location from ith TP.
				tempc = RightSparLocation(i);//find right closest spar location from ith TP.

				if (tempb == 1)//from headspar
					his1 = HeadSparHeight;
				else 
					his1 = INTHeight;

				if (tempc == TailLocationID)
					his2 = TailSparHeight;
				else
					his2 = INTHeight;
			
				dx = (float) (Profile[tempc].flatdistancefromlanding - Profile[tempb].flatdistancefromlanding);
				dy = (float) (Profile[tempc].elevation + his2 - (Profile[tempb].elevation + his1));
				slope = dy/dx;
	
				L = Profile[i].flatdistancefromlanding - Profile[tempb].flatdistancefromlanding;//distance from i to left INT suppport.
				D = (float)(Profile[tempb].elevation + his1) + slope * L - (float)(Profile[i].elevation + Clearance);//elevation difference without an intermediate support.

				//allowable defelction at point j is being prorated if point j is not at midspan (dx/2)
				if (L < dx/2.)
					P05 = AllowableDeflection * L / (dx/(float)2.);
				else
					P05 = AllowableDeflection * (dx - L) / (dx/(float)2.);
				
				if (D > 0 && D >= P05 * dx)//intermediate support is not necessary.
				{
					boolINTsupport[i] = FALSE;//eliminate intermediate support here 
					//make sure all front TPs are ok without this INT support.
					//go through each TP from the headspar to check clearance
/*
					for (j = 2; j < i; ++j)
					{
						tempb = LeftSparLocation(i);//find left closest spar location from ith TP.
						tempc = RightSparLocation(i);//find right closest spar location from ith TP.

						if (tempb == 1)//from headspar
							his1 = HeadSparHeight;
						else 
							his1 = INTHeight;
			
						if (tempc == TailLocationID)
							his2 = TailSparHeight;
						else
							his2 = INTHeight;
			
						dx = (float) (Profile[tempc].flatdistancefromlanding - Profile[tempb].flatdistancefromlanding);
						dy = (float) (Profile[tempc].elevation + his2 - (Profile[tempb].elevation + his1));
						slope = dy/dx;
	
						L = Profile[j].flatdistancefromlanding - Profile[tempb].flatdistancefromlanding;//distance from j to left INT suppport.
						D = (float)(Profile[tempb].elevation + his1) + slope * L - (float)(Profile[j].elevation);//skyline height from the ground at j terrain point.

						if (D < Clearance)//intermediate support is necessary.
						{
							boolINTsupport[i] = TRUE;//restore INT support at ith TP.
							break;
						}
					}
*/
				}
			}
		}
	}
			


	//reset Total number of INTs
	TotalNumberofINTs = 0;
	for (i = 2; i < TailLocationID; ++i)//do not include 1 and TailLocationID. They are temporary set as TRUE
	{
		if (boolINTsupport[i] == TRUE)
			++TotalNumberofINTs;
	}



	//**4. Check for too steep break in chord slope
	///////
	if (TotalNumberofINTs > 0)//if there are INT supports.
	{
		for (i = 2; i < TailLocationID; ++i)
		{
			if (boolINTsupport[i] == TRUE)
			{
				tempb = LeftSparLocation(i);//find left closest spar location from ith TP.
				tempc = RightSparLocation(i);//find right closest spar location from ith TP.

				if (tempb == 1)//from headspar
					diff1 = HeadSparHeight - INTHeight;
				else 
					diff1 = 0;

				if (tempc == TailLocationID)//to tailspar
					diff2 = INTHeight - TailSparHeight;
				else 
					diff2 = 0;

				diffx = (float)(Profile[i].flatdistancefromlanding - Profile[tempb].flatdistancefromlanding);
				diffy = (float)(Profile[i].elevation - Profile[tempb].elevation - diff1);
				slope1 = diffy / diffx;//slope from behind to current pt.
				//if cord slope shows downhill, assign TRUE to DownHill boolean variable.
				if (slope1 > 0) //Sep.3,2002
					DownHill = TRUE;
				
				
				slope1 = (float)atan(slope1);//in radians
				
				diffx = (float)(Profile[tempc].flatdistancefromlanding - Profile[i].flatdistancefromlanding);
				diffy = (float)(Profile[tempc].elevation - Profile[i].elevation - diff2);
				slope2 = diffy / diffx;//slope from current to ahead pt.
				if (slope2 > 0) //Sep.3,2002
					DownHill = TRUE;

				
				slope2 = (float)atan(slope2);//in radians


				//not available INT support
				if (fabs(slope2 - slope1) > (float)atan(MaxSlope/100.))//Sep.9,2002
				{
					//AfxMessageBox("Too steep break in chord slope!", MB_ICONSTOP);
					SteepChange = TRUE;
				}
			}
		}
	}

	
	//**5. conduct payload analysis to eliminate the least important intermediate support if there is any limitation on the numberof INT supports
	float tempMaxPayload = 0.;
	int keptINT;

	if (TotalNumberofINTs > 0 && SteepChange != TRUE)
	{
		while (TotalNumberofINTs > MaxINTSupports)
		{
			keptINT = -1;//initial value
			for (i = 2; i < TailLocationID; ++i)
			{
				if (boolINTsupport[i] == TRUE)
				{
					boolINTsupport[i] = FALSE;//try to eliminate this INT.
					PayloadAnalysis(CableSystemNo, 0, 0, TailSparHeight);//system, landing, cableway, 0 is temporary to skip some of procedure in PayloadAnalysis().
					if (tempMaxPayload < MinPayload)
					{
						tempMaxPayload = MinPayload;//MinPayload is determined in PayloadAnalysis()
						keptINT = i;
					}
					boolINTsupport[i] = TRUE;//restore the value.
				}
			}
			
			if (keptINT > 1)
			{
				boolINTsupport[keptINT] = FALSE;//eliminate the selected INT.
				--TotalNumberofINTs;//reduce the number by one.
			}
			else
			{
				MoreINTSupportsRequired = TRUE;
				break;
			}
		}
	}



	//**7. try to increase tail spar height to eliminate intermediate supports.
	//     conduct payload analysis
	//     calculate unit cost
	//     select the best cable design	



	//temporary resetting
	boolINTsupport[1] = FALSE;//headspar
	boolINTsupport[TailLocationID] = FALSE;//tailspar

	//reset TotalNumber of INTs
	TotalNumberofINTs = 0;
	for (i = 2; i < TailLocationID; ++i)//do not include 1 and TailLocationID. They are temporary set as TRUE
	{
		if (boolINTsupport[i] == TRUE)
			++TotalNumberofINTs;
	}
	
}



int CCablanView::LeftSparLocation(int CurrentTP)
{
	int tempa, tempb;

	tempa = CurrentTP - 1;//start from one cell behind Current TP
	tempb = -1;
	
	while (tempb < 0 && tempa > 0)//get the behind INT support location
	{
		if (boolINTsupport[tempa] == TRUE)
			tempb = tempa;
		else
			--tempa;
	}

	if (tempa < 1)
		tempb = 1;//Headspar-->no intermediate support between headspar and current TP.

	return(tempb);
}

int CCablanView::RightSparLocation(int CurrentTP)
{
	int tempa, tempc;
	
	tempa = CurrentTP + 1;//start from one cell after Current TP
	tempc = -1;
	
	while (tempc < 0 && tempa < TailLocationID)//get the next INT support location
	{
		if (boolINTsupport[tempa] == TRUE)
			tempc = tempa;
		else
			++tempa;
	}

	if (tempa >= TailLocationID)
		tempc = TailLocationID;//Tailspar  --> no intermediate support between tailspar and current TP.

	return(tempc);
}





void CCablanView::PayloadAnalysis(int CableSystemNo, int LandingID, int CablewayID, float TailSparHeight)
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);


	//input data from previous subroutines
	//1. Profile[]->information on terrain geometry
	//2. boolINTsupport[]->information on intermediate supports
	//3. LandingID and CablewayID are used for pDoc->PayloadAtEachTP[][][] and pDoc->CycleTimeAtEachTP[][][]
	//   LandingID and CablewayID will be 0 (temporary-no use), when this subroutine is called from GenerateIntermediateSupports() to reduce the number of INT supports.
	//4. Take care of maximum mainline tension - do loop until mainline tension is below the limit.
	float HeadSparHeight = (float)((double)pDoc->CableSystems[CableSystemNo].towerheight);//intermediate support height
	float INTHeight = (float)((double)pDoc->CableSystems[CableSystemNo].intermediatesupportheight);//intermediate support height
		
	//user-defined variables. All are in metric units
	float Tmax = pDoc->CableSystems[CableSystemNo].gbTmax;
	float Mmax = pDoc->CableSystems[CableSystemNo].gbMmax;
	float W1 = pDoc->CableSystems[CableSystemNo].gbW1;
	float W3 = pDoc->CableSystems[CableSystemNo].gbW3;
	float W4 = pDoc->CableSystems[CableSystemNo].gbW4;
	float Wc = pDoc->CableSystems[CableSystemNo].gbWc;
	float LC = pDoc->CableSystems[CableSystemNo].gbLC;
	float CarriageDepth = pDoc->CableSystems[CableSystemNo].gbCarriageDepth;//carriage depth
	float L = pDoc->CableSystems[CableSystemNo].gbL;//log length, choker hooked at end of log
	float DLog = pDoc->CableSystems[CableSystemNo].gbDLog;//log diameter
	float Beta = pDoc->CableSystems[CableSystemNo].gbBeta;//front end log clearance in radian
	float Myu = pDoc->CableSystems[CableSystemNo].gbMyu;//coefficient of ground friction
	


	//temporary variables
	int i, j;
	float MinS = (float)999999999.;
	float S;
	int MinSOccuredAt;
	float d1,h1,d2,h2;
	float N[MAXCELLS];
	

	//calculate span distance
	TotalSpans = 1;
	d1 = Profile[1].flatdistancefromlanding;//coordinate rather than distance
	h1 = (float)Profile[1].elevation + HeadSparHeight;

	for (i = 2; i < TailLocationID; ++i)//not include 1 (headspar)
	{
		if (boolINTsupport[i] == TRUE)
		{
			d2 = Profile[i].flatdistancefromlanding;
			h2 = (float)Profile[i].elevation + INTHeight;
			CableLengthatEachSpan[TotalSpans] = (float)sqrt( pow(d2-d1, 2.0) + pow(h2-h1, 2.0) );//global variable to used in other subroutines
			d1 = d2;
			h1 = h2;
			++TotalSpans;//global variable
		}
	}

	d2 = Profile[TailLocationID].flatdistancefromlanding;
	h2 = (float)Profile[TailLocationID].elevation + TailSparHeight;
	CableLengthatEachSpan[TotalSpans] = (float)sqrt( pow(d2-d1, 2.0) + pow(h2-h1, 2.0) );


	//find the shortest streched length of the skyline
	for (i = 2; i < TailLocationID; ++i)//not include 1 (headspar)
	{
		if (boolINTsupport[i] != TRUE)
		{
			if (Profile[i].fishstreambuffer != TRUE)
				S = CalculateStrechedLength(i, Beta, Myu, CarriageDepth, LC, L, HeadSparHeight, INTHeight, TailSparHeight, FALSE);
			else//added on June 4, 2002.
				S = CalculateStrechedLengthbyHC(CarriageDepth+L+LC, i, HeadSparHeight, INTHeight, TailSparHeight, FALSE);//HC = CarriageDepth+L+LC  --> indicates full suspension.
			if (MinS > S)
			{
				MinSOccuredAt = i;
				MinS = S;
			}
		}
	}


	
	///////////////////////////////////////
	//find the minimum payload 	
	///////////////////////////////////////
	
	//temporary variables
	float tBeta1 = (float)0.0;//temporary Beta's
	float tBeta;
	float tempPhi;
	float z1, z2;
	int CriticalTP;
	float NetPayload[MAXCELLS];
	float tempHC[MAXCELLS];
	float slope, slope1, intercept;
	int closesthigherslopepoint;//to check if mainline touches the ground
	float Length1, Length2, Length3, Length4;//cable length for each segment
	float Tav;//upper skyline tension.. (Tav = Tmax - W1*h) It varies depending on headspar or int. support.
	float Mav;//upper mainline tension..
	int leftsparlocation, rightsparlocation;
	float leftsparelevation, rightsparelevation;
	float hc1 = (float)0.0;
	float hc;
	float maxhc;//maximum hc available at each T.P. -> height from flat skyline to the ground
	bool FullSuspension = FALSE;
	CPoint textPosition;
	CString tempString;
	float Tav_reductionfactor;//for reduce Skyline maximum tension when mainline tension is larger than the maximum.
	
	MinPayload = (float)999999999.;


	//initializing 
	for (i = 0; i < MAXCELLS; ++i)
	{
		NormalForce[i] = -1.0;//this variable is set up in the PayloadPartial() and is used in CalculateUnitCosts()
		NetPayload[i] = 0.;//April 20,2002
		pDoc->PayloadAtEachTP[LandingID][CablewayID][i] = -1.;
		pDoc->CycleTimeAtEachTP[LandingID][CablewayID][i] = -1.0;
	}


		
	for (i = 2; i < TailLocationID; ++i)//not include 1 (headspar)
	{
		FullSuspension = FALSE;
		
		if (boolINTsupport[i] != TRUE && Profile[i].elevation >= 0)//not go through, if either at the intermediate support TP or at no elevation data TPs.
		{
			//check if full suspension or partial suspension
			leftsparlocation = 0;//if 0, headspar. Otherwise intermediate support.
			rightsparlocation = 0;//if 0, tailspar. Otherwise intermediate support.
			for (j = 2; j < i; ++j)
			{	
				if (boolINTsupport[j] == TRUE)
					leftsparlocation = j;//the closest intermediate support location.
			}
			for (j = TailLocationID + 1; j > i; --j)
			{
				if (boolINTsupport[j] == TRUE)
					rightsparlocation = j;//the closest intermediate support location.
			}

			//calculate elevation of the top of the spars
			if (leftsparlocation > 0)
				leftsparelevation = (float) ( Profile[leftsparlocation].elevation + INTHeight );
			else
			{
				leftsparlocation = 1;//headspar
				leftsparelevation = (float) ( Profile[1].elevation + HeadSparHeight );
			}

			if (rightsparlocation > 0)
				rightsparelevation = (float) ( Profile[rightsparlocation].elevation + INTHeight );
			else
			{
				rightsparlocation = TailLocationID;//tailspar
				rightsparelevation = (float) ( Profile[TailLocationID].elevation + TailSparHeight );
			}

			//slope between two end spars
			slope = (leftsparelevation - rightsparelevation) / (Profile[leftsparlocation].flatdistancefromlanding - Profile[rightsparlocation].flatdistancefromlanding);
			intercept = leftsparelevation - Profile[leftsparlocation].flatdistancefromlanding*slope;
			maxhc = slope * Profile[i].flatdistancefromlanding + intercept;//y = slope*x + intercept
			maxhc -= Profile[i].elevation;//maximum hc available
			
			//guess HC including Carriage depth (from the ground to the skyline)
			hc1 = (float)0.0;
			z2 = (float)99999999.;//keep the minimum z2
			while (hc1 <= maxhc)
			{
				z1 = CalculateStrechedLengthbyHC(hc1, i, HeadSparHeight, INTHeight, TailSparHeight, FALSE) - MinS;
				if (fabs(double(z1)) < fabs(double(z2)))
				{
					hc = hc1;
					z2 = z1;
				}
				hc1 += (float)0.1;
			}
			
					
			if (hc >= CarriageDepth + L + LC)//full
			{
				FullSuspension = TRUE;
				HC = hc;//keep in global variable
				Alpha = (float)(Radians(90.0));//90 degree from the horizon,
				//drawline by TRUE
				z2 = CalculateStrechedLengthbyHC(hc, i, HeadSparHeight, INTHeight, TailSparHeight, TRUE) - MinS;//just for drawing geometry
//				DrawLog(i, LC, L, CarriageDepth, Alpha, HC, (float)Radians(90.0)-Phi, Phi);//make Beta + Phi is 90 degree.
			}
			else//partial
			{
				FullSuspension = FALSE;
				//groundslope would be negative when uphill because we locate headspar in left side. 
				tempPhi = (float)atan(-(float)(Profile[i+1].elevation - Profile[i].elevation) / (Profile[i+1].flatdistancefromlanding - Profile[i].flatdistancefromlanding));//we put negative sign because we need to get positive angle for uphill yarding to calculate Alpha. If downhill Phi will be negative value.
				tBeta = 0.0;
				z2 = (float)99999999.;//keep the minimum z2

				//Guess - increase Beta by fixed value
				tBeta1 = 0.0;
				while (tBeta1 <= (float)(90.0*3.141592/180 - tempPhi))//tempPhi is positive when uphill, and negative when downhill
				{
					z1 = CalculateStrechedLength(i, tBeta1, Myu, CarriageDepth, LC, L, HeadSparHeight, INTHeight, TailSparHeight, FALSE) - MinS;
					if (fabs(double(z1)) < fabs(double(z2)) && HC < maxhc)
					{
						z2 = z1;
						tBeta = tBeta1;
					}
					tBeta1 += (float)Radians(0.5);//increase by 0.1 degree
				}
				
				z1 = CalculateStrechedLength(i, tBeta, Myu, CarriageDepth, LC, L, HeadSparHeight, INTHeight, TailSparHeight, TRUE) - MinS;//just for drawing geometry and saving N(normal force)
//				DrawLog(i, LC, L, CarriageDepth, Alpha, HC, tBeta, Phi);
			}


			//GlobalHs,GlobalDs, Alpha, Phi, and HC(which includes CarriageDepth) are from CalculatedStrechedLength()
			
			//MAINLINE - Uphill
			//mainline geometry
			//For D, always right - left. For H, always left - right.
			GlobalD3 = Profile[i].flatdistancefromlanding;//right - left-> always positive
			GlobalH3 = (float)(Profile[1].elevation + HeadSparHeight - (Profile[i].elevation + HC - CarriageDepth/2.0));//reverse order (left - right) to make positive slope for uphill yarding (headspar is located in leftside).
			slope = GlobalH3/GlobalD3;//positive slope->Uphill
			
			//check if mainline touches the ground
			closesthigherslopepoint = 0;
			for (j = 1; j < i; ++j)
			{	
				slope1 = (float)(Profile[j].elevation - (Profile[i].elevation + HC - CarriageDepth/2.0)) / (Profile[i].flatdistancefromlanding - Profile[j].flatdistancefromlanding);//reverse order to make positive slope for uphill yarding (headspar is located in leftside).
				if (slope < slope1 && slope > 0)
					closesthigherslopepoint = j;
			}

			//mainline touches the ground-> revise H3 and D3
			if (closesthigherslopepoint > 0)//only if slope > 0 -> only uphill yarding...(do not care downhill)
			{
				GlobalD3 = Profile[i].flatdistancefromlanding - Profile[closesthigherslopepoint].flatdistancefromlanding;//right - left
				GlobalH3 = (float)(Profile[closesthigherslopepoint].elevation - (Profile[i].elevation + HC - CarriageDepth/2.0));//left - right
			}



			//HAULBACKLINE	- Downhill
			//haulbackline geometry --for downhill
			//For D, always right - left. For H, always left - right.
			GlobalD4 = Profile[TailLocationID].flatdistancefromlanding -  Profile[i].flatdistancefromlanding;//right - left-> always positive
			GlobalH4 = (float)(Profile[TailLocationID].elevation + TailSparHeight - (Profile[i].elevation + HC - CarriageDepth/2.0));//right order (right - left) to make positive slope for downhill yarding (tailspar is located in rightside).
			slope = GlobalH4/GlobalD4;//positive slope->Downhill

			//check if mainline touches the ground
			closesthigherslopepoint = 0;
			for (j = TailLocationID - 1; j > i; --j)
			{	
				slope1 = (float)(Profile[j].elevation - (Profile[i].elevation + HC - CarriageDepth/2.0)) / (Profile[j].flatdistancefromlanding - Profile[i].flatdistancefromlanding);
				if (slope < slope1 && slope > 0)
					closesthigherslopepoint = j;
			}

			//mainline touches the ground-> revise H3 and D3
			if (closesthigherslopepoint > 0)//only if slope > 0 -> only uphill yarding...(do not care downhill)
			{
				GlobalD4 = Profile[closesthigherslopepoint].flatdistancefromlanding - Profile[i].flatdistancefromlanding;//right - left
				GlobalH4 = (float)( (Profile[i].elevation + HC - CarriageDepth/2.0) - Profile[closesthigherslopepoint].elevation );//left - right
			}
			else
				GlobalH4 = (float)( (Profile[i].elevation + HC - CarriageDepth/2.0) - Profile[TailLocationID].elevation + TailSparHeight);//(left - right) reverse order so that H4 can be negative value with downhill. This needs for DownhillSegment4()


			//calculate cable length
			Length1 = (float) ( sqrt(pow(GlobalD1,2.0) + pow(GlobalH1,2.0)) );
			Length2 = (float) ( sqrt(pow(GlobalD2,2.0) + pow(GlobalH2,2.0)) );
			Length3 = (float) ( sqrt(pow(GlobalD3,2.0) + pow(GlobalH3,2.0)) );			
			Length4 = (float) ( sqrt(pow(GlobalD4,2.0) + pow(GlobalH4,2.0)) );			



			//Payload analysis*******************
			/////////////////////////////////////
			//Step 1: Line tensions
			if (leftsparelevation > rightsparelevation)//only uphill yarding
			{
				//revised on April 27, 2002.
				if (leftsparlocation > 0)//there is left int support.
				{
					Tav = Tmax - W1 * (Profile[1].elevation + HeadSparHeight - leftsparelevation);//at the headspar, Tav = Tmax
					Mav = Mmax - W3 * (Profile[1].elevation + HeadSparHeight - leftsparelevation);//at the headspar, Tav = Tmax
				}
				else
				{
					Tav = Tmax;//at the headspar
					Mav = Mmax;
				}
				
				do
				{
					Segment1(Tav, GlobalD1, GlobalH1, Length1, W1);
					Segment2(T1.Right, GlobalD2, GlobalH2, Length2, W1);
					Segment3(GlobalD3, GlobalH3, Length3, W3, Alpha, Wc);

					if (T3.Left > Mav)//if mainline tension is bigger than maximum.
					{
						Tav_reductionfactor = (float)((double)Mav / T3.Left / 2.0);
						Tav *= (float)(1.0 - Tav_reductionfactor);
					}

				} while (T3.Left > Mav);
			}
			else//downhill
			{
				//determine maximum tension at the upper location in Segment2 
				if (rightsparlocation > 0)//there is right int support.
					Tav = Tmax - W1 * (Profile[TailLocationID].elevation + TailSparHeight - rightsparelevation);
				else
					Tav = Tmax;//at the tailspar

				DownhillSegment2(Tav, GlobalD2, GlobalH2, Length2, W1);
				DownhillSegment1(T2.Left, GlobalD1, GlobalH1, Length1, W1);
				DownhillSegment4(GlobalD4, GlobalH4, Length4, W4, Alpha, Wc);
			}



			//Step 2: Calculate Payload
			N[i] = (float)0.0;
			//calculate Payload
			if (FullSuspension == TRUE)
				NetPayload[i] = PayloadFull(Wc, HC, i, leftsparlocation, rightsparlocation, HeadSparHeight, INTHeight, TailSparHeight);
			else
			{
				//////////////////////////////////////
				//Fish stream logging restriction
				if (pDoc->FishStreamBuffer[Profile[i].column][Profile[i].row] > 0)
					PartialOnStream = TRUE;
				else
					NetPayload[i] = PayloadPartial(Myu, Wc, Phi, HC, i, leftsparlocation, rightsparlocation, HeadSparHeight, INTHeight, TailSparHeight);
			}


			//save payload at each point to the global variable************
			pDoc->PayloadAtEachTP[LandingID][CablewayID][i] = NetPayload[i];


			textPosition.x = (long)Profile[i].flatdistancefromlanding;
			textPosition.y = Profile[i].elevation - 15;
			tempString.Format("%6d", (int)NetPayload[i]);
			WriteText(textPosition, tempString);
			tempHC[i] = HC;
		}
		else//if this point is intermediate support
			pDoc->PayloadAtEachTP[LandingID][CablewayID][i] = 0.0;//intermediate support, otherwise either positive number(payload) of negative number(not feasible cable road)
	}

	MinPayload = (float)99999999.9;
	float tempYardingDist;
	float elevDiff;
	float UserMax = pDoc->CableSystems[CableSystemNo].gbUserMaxPayload;
	
	for (i = 2; i < TailLocationID; ++i)//not include 1 (headspar)
	{
		if (NetPayload[i] > 0)//excluding intermediate support TPs.
		{
			elevDiff = (float)fabs(Profile[1].elevation - Profile[i].elevation);
			tempYardingDist = (float)sqrt(pow(elevDiff, 2.0) + pow(Profile[i].flatdistancefromlanding, 2.0));
			//CalculateUnitCost in View() considers only LoadedTime and EmptyTime.. Other time factors such as hooking and unhooking are taken care of in GenerateCableSystemNetwork();
			//This is different from DialogPathAnalysis()
			pDoc->CycleTimeAtEachTP[LandingID][CablewayID][i] = CalculateUnitCost(NetPayload[i], Myu, HeadSparHeight, tempYardingDist, MinS, W3, Wc, CableSystemNo);
		}
		else
		{
			//revised on April 29, 2002
			//at the intermediate support
			elevDiff = (float)fabs(Profile[1].elevation - Profile[i].elevation);
			tempYardingDist = (float)sqrt(pow(elevDiff, 2.0) + pow(Profile[i].flatdistancefromlanding, 2.0));
			//CalculateUnitCost in View() considers only LoadedTime and EmptyTime.. Other time factors such as hooking and unhooking are taken care of in GenerateCableSystemNetwork();
			//This is different from DialogPathAnalysis()
			pDoc->CycleTimeAtEachTP[LandingID][CablewayID][i] = CalculateUnitCost(UserMax, Myu, HeadSparHeight, tempYardingDist, MinS, W3, Wc, CableSystemNo);
		}
			
		//April 20, 2002
		if (MinPayload > NetPayload[i] && boolINTsupport[i] == FALSE)
		{
			MinPayload = NetPayload[i];
			CriticalTP = i;
		}
	}

	//April 20, 2002
	if (MinPayload < 0.)//all netpayload is negative value.
		MinPayload = 0.;
	
	
	//April 21, 2002
	//if LandingID = 0, this subroutine is called from GenerateIntermediateSupports() in order to reduce the number of INT supports. -> No use to run the below routines.
	if (LandingID > 0)
	{
		//April 20, 2002
		//redefine PayloadAtEachTP considering the minimum payload ahead of each TP.
		float tempMinPayload = 9999999.;
		pDoc->PayloadAtEachTP[LandingID][CablewayID][1] = tempMinPayload;//payload at the landing could be infinite.

		for (i = 2; i < TailLocationID; ++i)
		{
			if (pDoc->PayloadAtEachTP[LandingID][CablewayID][i] > 0.)//excluding intermediate supports
			{
				if (tempMinPayload < pDoc->PayloadAtEachTP[LandingID][CablewayID][i])
					pDoc->PayloadAtEachTP[LandingID][CablewayID][i] = tempMinPayload;
				else
					tempMinPayload = pDoc->PayloadAtEachTP[LandingID][CablewayID][i];
			}
			else//at the intermediate supports
				pDoc->PayloadAtEachTP[LandingID][CablewayID][i] = pDoc->PayloadAtEachTP[LandingID][CablewayID][i-1];

		}//now pDoc->PayloadAtEachTP has the payload we can carry from the TP to the landing.



		//at the intermediate support --> estimate cycletime by proportion of the cycle time of the BEHIND cell to the distance.
		//added on April 15, 2002.
		int closestcell;
		int celldiff;
	
		for (i = 2; i < TailLocationID; ++i)//not include 1 (headspar)
		{
			celldiff = 999999;
			if (pDoc->CycleTimeAtEachTP[LandingID][CablewayID][i] == 0.0)
			{
				//find out the closest cell having Cycletime.
				for (j = 2; j < TailLocationID; ++j)
				{
					if (pDoc->CycleTimeAtEachTP[LandingID][CablewayID][j] > 0)
					{
						if (abs(i-j) != 0 && abs(i-j) < celldiff)
						{
							celldiff = abs(i-j);
							closestcell = j;
						}
					}
				}

				//input cycle time estimation based on the closestcell.
				pDoc->CycleTimeAtEachTP[LandingID][CablewayID][i] = pDoc->CycleTimeAtEachTP[LandingID][CablewayID][closestcell] / Profile[closestcell].flatdistancefromlanding * Profile[i].flatdistancefromlanding;
			}
		}


		//overall average  cycle time--2/3 of external yarding distance
		tempYardingDist = (float) (0.67 * MinS);
		CycleTime = CalculateUnitCost(MinPayload, Myu, HeadSparHeight, tempYardingDist, MinS, W3, Wc, CableSystemNo);
	}

}




float CCablanView::CalculateUnitCost(float MinPayload, float Myu, float HeadSparHeight, float YardingDist, float MinStLength, float CableWeight, float CarriageWeight, int CableSystem)
{
	//this subroutine is identical to CalculateUnitCost() in DialogPathAnalysis.cpp -- not any more since 4/8/02
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	//Fixed variables
	float G = (float)9.81;// gravity acceleration= 9.81m/sec2
	int cs = CableSystem;


	//User-defined variables
	float HooktimeperCM = pDoc->CableSystems[cs].gbHookTime;// sec/cubic meter
	float UnhooktimeperCM = pDoc->CableSystems[cs].gbUnhookTime;// sec/cubic meter
	float RiggingCosts = pDoc->CableSystems[cs].gbRiggingCosts;//$ per intermediate support and tailspar
	float MinpayloadbyUser = pDoc->CableSystems[cs].gbUserMinPayload;//
	float Density = pDoc->CableSystems[cs].gbDensity;//2200 lb / cubic meter  -> kg / cubic meter
	float MaxpayloadbyUser = pDoc->CableSystems[cs].gbUserMaxPayload;//
	float LoadFactor = pDoc->CableSystems[cs].gbLoadFactor;//percent of skyline capacity actually used
	float Cost = pDoc->CableSystems[cs].gbUnitCost;//hourly cost of tower operation $/m3/hour
	float DelayPercentage = pDoc->CableSystems[cs].gbDelayPercentage;
	float FellingCosts = pDoc->CableSystems[cs].gbFellingCosts;
	float MoveInCosts = pDoc->CableSystems[cs].gbMoveInCosts;//System move-in costs
	float YarderSetupHour = pDoc->CableSystems[cs].gbYarderSetupHour;//System setup hour
	float CableRoadChangeHour = pDoc->CableSystems[cs].gbCableRoadChangeHour;//skyline reinstalling hour
	float EnginePower = pDoc->CableSystems[cs].gbEnginePower;//horse power
	float PowerConverterEfficiency = pDoc->CableSystems[cs].gbEngineEfficiency;//65% of engine power
	
	
	//temporary variables
	int i;
	float averageYD;//average yarding distance
	float loadsize;
	float load1, load2;
	float cycletime;
	float EmptyTime;//unit: sec
	float LoadedTime;//unit: sec
	float WorkMyuN;
	float slopedist, totalslopedist;
	float liftheight;
	int averageYDlocation;
	float totalwork;
	float availableWatts = EnginePower * PowerConverterEfficiency * (float)746.0;//1hp = 746 Watts  -> Watts ((1N-m)/sec)
	bool PartialSuspension;
	CString tempString;
	CPoint textPosition;
	int NumberofINTs = 0;

	//temporary
	float minimumhaulingtimefromSpeedLimit;

	//here is difference from DialogPathAnalysis.cpp
//	averageYD = (float)0.67 * MinStLength;//2/3 of total chord distance
	averageYD = YardingDist;//depending on where the log is located.


	totalslopedist = 0.0;
	WorkMyuN = 0.0;
	PartialSuspension = FALSE;

	//finding where this averageYD falls along the skyline corridor.
	for (i = 2; i < TailLocationID; ++i)//TailLocationID is global variable and needs to be set right before this subroutine is called.
	{
		//changed on April 8, 2002
		totalslopedist = (float)sqrt( pow((double)(Profile[i].elevation - Profile[1].elevation), 2.0) + pow((double)(Profile[i].flatdistancefromlanding), 2.0) );
		averageYDlocation = i;//will be used to get liftheight.
		if (totalslopedist >= averageYD)
			break;

		if (NormalForce[i] > 0)//until averageYD
		{
			slopedist = (float)sqrt( pow((double)(Profile[i+1].elevation - Profile[i].elevation), 2.0) + pow((double)(Profile[i+1].flatdistancefromlanding - Profile[i].flatdistancefromlanding), 2.0) );
			WorkMyuN += NormalForce[i] * Myu * slopedist;//WorkMyuN is in kg*m, not N*m
			PartialSuspension = TRUE;
		}

		//count number of INT supports
		if (boolINTsupport[i] == TRUE)
			++NumberofINTs;
	}

	liftheight = (float)( Profile[1].elevation + HeadSparHeight - (Profile[averageYDlocation].elevation) );//if liftheight is less than 0, that means downhill..

	//Temporarily set as positive value *******
	liftheight = (float)fabs(liftheight);//in case of downhill, the energy will be consumed by haulback line drum. -- assume the same mechanism with mainline drum for uphill.

	
	//determine loadsize---three different load size available:MaxpayloadbyUser, MinPayload from PayloadAnalysis, and MinPayloadbyUser.
	//the largest load available is MaxpayloadbyUser.
	//the smallest load available is MinpayloadbyUser.
	//otherwise, the load is in between them depending on MinPaylaod
	if (MinPayload > MaxpayloadbyUser)
		loadsize = MaxpayloadbyUser;// * LoadFactor;//???????????????? why loadfactor?
	else
	{
		load1 = MinPayload;;// * LoadFactor;
		load2 = MinpayloadbyUser;//does not need to apply LoadFactor because it is small enough.
		if (load1 > load2)
			loadsize = load1;
		else
			loadsize = load2;
	}

	cycletime = 0.0;
	
	if (liftheight > 0)	
	{
		totalwork = (loadsize + CarriageWeight + MinStLength * CableWeight) * G * liftheight + WorkMyuN * G;//Unit: N-m
		LoadedTime = totalwork / availableWatts;//second
		//once there is at least one partial suspension span, then consider it is partial suspension yarding.....????????
		if (PartialSuspension == TRUE)
		{
			minimumhaulingtimefromSpeedLimit = averageYD / pDoc->CableSystems[cs].gbInhaulSpeedLimit;//m / (m/sec) = sec
			if (minimumhaulingtimefromSpeedLimit > LoadedTime)//if speed from egine work is faster than maximum yarding speed from the user.
				LoadedTime = minimumhaulingtimefromSpeedLimit;
		}
		else//full suspension
		{
			minimumhaulingtimefromSpeedLimit = averageYD / pDoc->CableSystems[cs].gbInhaulSpeedLimitFull;//m / (m/sec) = sec
			if (minimumhaulingtimefromSpeedLimit > LoadedTime)
				LoadedTime = minimumhaulingtimefromSpeedLimit;
		}			
	}

	//EmptyTime = ((CarriageWeight + MinStLength * CableWeight)* G * liftheight) / availableWatts;//second
	minimumhaulingtimefromSpeedLimit = averageYD / pDoc->CableSystems[cs].gbOuthaulSpeedLimit;//gbOuthaulSpeedLimit is average outhaul speed
	//if (minimumhaulingtimefromSpeedLimit > EmptyTime)
		EmptyTime = minimumhaulingtimefromSpeedLimit;



	//cycletime is one cycletime for average yarding distance which is 2/3 of feasible maximum yarding distance.
	//cycletime is based on loadsize which is determined by either Minpayload from payload analysis or design payload defined by the user.
	//cycletime does not include Hooking time and unhooking time--->this is different from cycletime in DialogPathAnalysis.cpp...
	//hooking time and unhooking will be added in GenerateCableNetwork() while considering volume per turn.

	cycletime = EmptyTime + LoadedTime;
	return(cycletime);
}




void CCablanView::WriteText(CPoint Position, CString tempString)
{
	/*
	CClientDC* pDC = new CClientDC(this);
	pDC->SetMapMode(MM_ISOTROPIC);
	pDC->SetWindowExt(ViewScale, ViewScale);
	pDC->SetWindowOrg(0, 0);//negative y is down, positive x is right

	Position.x += Coord.left + Margin;
	Position.y = -(Coord.top + Margin + MaxElev - Position.y);
	
	pDC->TextOut(Position.x, Position.y, tempString);
	*/
}


void CCablanView::WriteTextAboveGridCell(int col, int row, CString tempString)
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);


	pDC = new CClientDC(this);
	OnPrepareDC(pDC);

	pDC->SetMapMode(MM_ISOTROPIC);
	pDC->SetWindowExt(ViewScale, ViewScale);


	CFont tempFont;
	tempFont.CreateFont(
		50,                        // nHeight
		0,                         // nWidth
		0,                         // nEscapement
		0,                         // nOrientation
		FW_NORMAL,                 // nWeight
		FALSE,                     // bItalic
		FALSE,                     // bUnderline
		0,                         // cStrikeOut
		ANSI_CHARSET,              // nCharSet
		OUT_DEFAULT_PRECIS,        // nOutPrecision
		CLIP_DEFAULT_PRECIS,       // nClipPrecision
		DEFAULT_QUALITY,           // nQuality
		DEFAULT_PITCH | FF_SWISS,  // nPitchAndFamily
		"Arial");                 // lpszFacename

	CFont *pOldFont = pDC->SelectObject(&tempFont);
	
	DrawFrom.x = (long)(pDoc->DTMCoord[col][row].x - pDoc->DTMminX - (pDoc->DTMcellsize/2.0) + cpMargin.x);// (CellHeight/2) integer problem??
	DrawFrom.y = (long)(pDoc->DTMCoord[col][row].y - pDoc->DTMmaxY - (pDoc->DTMcellsize/2.0) - cpMargin.y);


	pDC->TextOut(DrawFrom.x, DrawFrom.y, tempString);
		
	pDC->SelectObject(pOldFont);

	tempFont.DeleteObject();
}


void CCablanView::DownhillSegment2(double UpperTension, float d2, float h2, float length2, float w2)
{
	T2.Right = UpperTension;//h2 is negative. This will be UpperTension for Segment 2.
	
	H2.Right = (w2*h2*d2)/(2*length2) + (UpperTension*d2/length2)
				*sqrt(1-pow(((w2*d2)/(2*UpperTension)),2.0));//h2 should be negative because downhill. The first term will be negative because of negative h2.
	
	V2.Right = sqrt(pow(T2.Right,2.0)-pow(H2.Right, 2.0));

	H2.Left = H2.Right;

	V2.Left = (H2.Left*(-h2) - w2*length2*d2/2) / d2;//h2 should be negative->downhill

	T2.Left = sqrt(pow(H2.Left,2.0) + pow(V2.Left, 2.0));
}

void CCablanView::DownhillSegment1(double LowerTension, float d1, float h1, float length1, float w1)
{
	T1.Right = LowerTension;

	H1.Right = (w1*h1*d1)/(2*length1) + (LowerTension*d1/length1)
				*sqrt(1-pow(((w1*d1)/(2*LowerTension)),2.0));//if h1 could be either positive or negative.. depending on carraige elevation and headspar elevation.
															 //if h1 is negative the sign in the eqation will change-->it's ok.
	//if h1 is positive, which means carriage is lower than headspar, the first term should be positive, since Tension in left becomes lower Tension.
	//if h1 is negative, which means carriage is higher than headspar, the first term should be negative, since Tension in left becomes upper tension.


	V1.Right = sqrt(pow(T1.Right,2.0)-pow(H1.Right, 2.0));
	
	H1.Left = H1.Right;
	
	if (h1 < 0)//carriage is higher than headspar
		V1.Left = (H1.Left*(-h1) - w1*length1*d1/2) / d1;//since h1 is negative, the first term will be positive
	else//carriage is lower than headspar
		V1.Left = (H1.Left*h1 + w1*length1*d1/2) / d1;//since h1 is positive, the first term will be positive

	T1.Left = sqrt(pow(H1.Left,2.0) + pow(V1.Left, 2.0));
}



void CCablanView::DownhillSegment4(float d4, float h4, float length4, float w4, float Alpha, float CarriageWeight)
{
	//h4 is negative since downhill.
	double sign1;
	if (GlobalH1 < 0)//carriage is higher than headspar.
		sign1 = -1.0;
	else//carriage is lower than headspar.
		sign1 = 1.0;


	double numerator = tan(Alpha)*(H2.Left-H1.Right)
					   - (V1.Right*sign1+V2.Left-CarriageWeight)//change V1 sign depending on geometry. if h1 is negative V1 should be negative.
		               + (w4*length4)/2;
	double denomenator = fabs((double)(h4/d4)) - tan(Alpha);
	H4.Left = numerator / denomenator;
	
	H4.Right = H4.Left;

	V4.Left = (H4.Left*(-h4) - w4*length4*d4/2) / d4;//h4 is negative.
	
	V4.Right = V4.Left + w4*length4;
	
	T4.Right =  sqrt(pow(H4.Right,2.0) + pow(V4.Right, 2.0));
	T4.Left =  sqrt(pow(H4.Left,2.0) + pow(V4.Left, 2.0));
}




void CCablanView::Segment1(double UpperTension, float d1, float h1, float length1, float w1)
{
	T1.Left = UpperTension;

	H1.Left = -(w1*h1*d1)/(2*length1) + (UpperTension*d1/length1)
				*sqrt(1-pow(((w1*d1)/(2*UpperTension)),2.0));//if h1 should be positive, because uphill

	V1.Left = sqrt(pow(T1.Left,2.0)-pow(H1.Left, 2.0));
	
	H1.Right = H1.Left;

	V1.Right = (H1.Right*h1 - w1*length1*d1/2) / d1;//h1 is positive

	T1.Right = sqrt(pow(H1.Right,2.0) + pow(V1.Right, 2.0));
}


void CCablanView::Segment2(double LowerTension, float d2, float h2, float length2, float w2)
{
	T2.Left = LowerTension;//if h2 is positive (carriage is higher than tailspar). This will be UpperTension for Segment 2.
	
	H2.Left = -(w2*h2*d2)/(2*length2) + (LowerTension*d2/length2)
				*sqrt(1-pow(((w2*d2)/(2*LowerTension)),2.0));//if h2 is negative, it automatically change the sign in the equation. O.K.

	//if the geometry shows carriage is SLIGHTLY lower than tail spar (negative h2 here), but because of cable weight T2.left could be upper tension.
	if (H2.Left > T2.Left)
	{
		H2.Left = (w2*h2*d2)/(2*length2) + (LowerTension*d2/length2)
				*sqrt(1-pow(((w2*d2)/(2*LowerTension)),2.0));
		
		V2.Left = -sqrt(pow(T2.Left,2.0)-pow(H2.Left, 2.0));//put negative so that this will be counted as force against payload.
	}
	else
		V2.Left = sqrt(pow(T2.Left,2.0)-pow(H2.Left, 2.0));



	H2.Right = H2.Left;

	if (h2 >= 0)//carriage is higher than tailspar
		V2.Right = (H2.Right*h2 - w2*length2*d2/2) / d2;//since h2 is positive, the first term will be positive
	else//carriage is lower than tailspar
		V2.Right = (H2.Right*(-h2) + w2*length2*d2/2) / d2;//since h2 is negative, the first term will be positive.

	T2.Right = sqrt(pow(H2.Right,2.0) + pow(V2.Right, 2.0));

}


void CCablanView::Segment3(float d3, float h3, float length3, float w3, float Alpha, float CarriageWeight)
{
	double sign2;
	if (GlobalH2 >= 0)//carriage is higher than tailspar.
		sign2 = -1.0;
	else//carriage is lower than tailspar.
		sign2 = 1.0;


	double numerator = tan(Alpha)*(H1.Left-H2.Right)
					   - (V1.Right+V2.Left*sign2-CarriageWeight)//change V2 sign depending on geometry. if h2 is positive(carriage is higher than tail) V2 should be negative.
		               + (w3*length3)/2;
	double denomenator = fabs((double)(h3/d3)) - tan(Alpha);//h3 should be positive, since uphill yarding.

	H3.Right = numerator / denomenator;
	
	H3.Left = H3.Right;

	V3.Right = (H3.Right*h3 - w3*length3*d3/2) / d3;//applicable for only uphill yarding. h3 should be positive
	
	V3.Left = V3.Right + w3*length3;
	
	T3.Right =  sqrt(pow(H3.Right,2.0) + pow(V3.Right, 2.0));
	T3.Left =  sqrt(pow(H3.Left,2.0) + pow(V3.Left, 2.0));
}


float CCablanView::PayloadPartial(float Myu, float CarriageWeight, float Phi, float HC, int TP, int leftsparlocation, int rightsparlocation, float HeadSparHeight, float INTHeight, float TailSparHeight)
{
	float NetLoad = 0.0;
	float N = 0.0;
	float sparheight;
	double sign1, sign2, sign3, sign4;
	float leftsparelevation, rightsparelevation;
	float tempMyu;

	if (leftsparlocation > 1)
		sparheight = (float)INTHeight;
	else
		sparheight =  (float)HeadSparHeight;

	leftsparelevation = (float) ( Profile[leftsparlocation].elevation + sparheight );

	if (rightsparlocation < TailLocationID && rightsparlocation > 0)
		sparheight = (float)INTHeight;
	else
		sparheight =  (float)TailSparHeight;

	rightsparelevation = (float) ( Profile[rightsparlocation].elevation + sparheight );
	

	//decide uphill or downhill
	if (leftsparelevation > rightsparelevation)//uphill
	{
		sign3 = 1.0;
		sign4 = 0.0;
		N = (float) ( (H1.Right + H3.Right - H2.Left) / (sin(Phi)+Myu*cos(Phi)) );//denomenator (sin and cos) works for both downhill and uphill. Down hill has negative Phi.
	}
	else//downhill
	{
		sign3 = 0.0;
		sign4 = 1.0;
	
		tempMyu = Myu;
		while (sin(Phi)+tempMyu*cos(Phi) < 0)//if Phi is steep downhill slope, N could be negative value. So increase Myu to avoid a negative N.
			tempMyu += (float)Radians(1.0);
		
		N = (float) ( (-H1.Right + H2.Left + H4.Left) / (sin(Phi)+tempMyu*cos(Phi)) );//sin and cos works for both downhill and uphill. Down hill has negative Phi.
	}
	
	
	if (Profile[TP].elevation + HC > leftsparelevation)//carriage is higher than headspar -> h1 is negative (downhill)
		sign1 = -1.0;//downhill
	else
		sign1 = 1.0;//uphill
	
	if (Profile[TP].elevation + HC > rightsparelevation)//carriage is higher than tailspar -> h2 is positive (uphill)
		sign2 = -1.0;
	else //downhill
		sign2 = 1.0;
	
	
	NetLoad = (float) ( (sign1*V1.Right + sign3*V3.Right + sign2*V2.Left + sign4*V4.Left - CarriageWeight) + N*cos(Phi) - Myu*N*sin(Phi) );
	
	NormalForce[TP] = N;
	return(NetLoad);
}



float CCablanView::PayloadFull(float CarriageWeight, float HC, int TP, int leftsparlocation, int rightsparlocation, float HeadSparHeight, float INTHeight, float TailSparHeight)
{
	float NetLoad = 0.0;
	double sign1, sign2, sign3, sign4;
	float leftsparelevation, rightsparelevation;

	if (leftsparlocation > 1)
		leftsparelevation = (float) ( Profile[leftsparlocation].elevation + INTHeight );
	else
		leftsparelevation = (float) ( Profile[1].elevation + HeadSparHeight );

	if (rightsparlocation < TailLocationID && rightsparlocation > 0)
		rightsparelevation = (float) ( Profile[rightsparlocation].elevation + INTHeight );
	else
		rightsparelevation = (float) ( Profile[TailLocationID].elevation + TailSparHeight );

	//decide uphill or downhill
	if (leftsparelevation > rightsparelevation)//uphill
	{
		sign3 = 1.0;
		sign4 = 0.0;
	}
	else//downhill
	{
		sign3 = 0.0;
		sign4 = 1.0;
	}


	if (Profile[TP].elevation + HC > leftsparelevation)//carriage is higher than headspar -> h1 is negative (downhill)
		sign1 = -1.0;//downhill
	else
		sign1 = 1.0;//uphill
	
	if (Profile[TP].elevation + HC > rightsparelevation)//carriage is higher than tailspar -> h2 is positive (uphill)
		sign2 = -1.0;
	else //downhill
		sign2 = 1.0;


	NetLoad = (float) ( (sign1*V1.Right + sign3*V3.Right + sign2*V2.Left + sign4*V4.Left - CarriageWeight) );

	return(NetLoad);

}




float CCablanView::CalculateStrechedLength(int TP, float Beta, float Myu, float CarriageDepth, float LC, float L, float HeadSparHeight, float INTHeight, float TailSparHeight, bool Draw)
{
	//temporary variables
	int j;
	float groundslope;
	int leftINTsupportlocation;//to identify we are heading to another INT support
	int rightINTsupportlocation;//to identify we are coming from another INT support
	float S;
	float d1,h1,d2,h2;
	CPoint From, To, From1;
	int SpanNo;


	//calculate streched length of the skyline at the given T.P.

	//let's figure out where I am. (which span)
	leftINTsupportlocation = 1;
	rightINTsupportlocation = TailLocationID;
	SpanNo = 1;//count spans to know where I am. 1 means we are going directly to headspar.
		
	for (j = 2; j < TP; ++j)//not include 1 (headspar)
	{
		if (boolINTsupport[j] == TRUE)
		{
			leftINTsupportlocation = j;//keep the closest int. support location in left side
			++SpanNo;
		}
	}	

	for (j = TailLocationID - 1; j > TP ; --j)//start checking backward
	{
		if (boolINTsupport[j] == TRUE)
			rightINTsupportlocation = j;//keep the closest int. support location in right side
	}


	//calculate Phi (ground angle from the current T.P. to left T.P. toward headspar)
	groundslope = (float)(Profile[TP+1].elevation - Profile[TP].elevation) / (Profile[TP+1].flatdistancefromlanding - Profile[TP].flatdistancefromlanding);
	//groundslope would be negative when uphill because we locate headspar in left side. 
	Phi = (float)atan(-groundslope);//we put negative sign because we need to get positive angle for uphill yarding to calculate Alpha. If downhill Phi will be negative value.-->global variable
			
	//calculate Alpha
	Alpha = (float)fabs(atan(2.*tan(Phi+Beta) + (cos(Phi)-Myu*sin(Phi))/(sin(Phi)+Myu*cos(Phi))));//alpha should be in positive no matter of downhill or uphill.-->global variable

	//calculate HC
	HC = (float) ( (LC*sin(Alpha - Phi) + L*sin(Beta)) / cos(Phi) + CarriageDepth ); //global variable
	

	//calculate d and h
	//d's are alwayas positive (right - left)
	//h's could be positive or negative (left - right)
	if (leftINTsupportlocation > 1)//heading to an intermediate support
	{
		d1 = (float)(Profile[TP].flatdistancefromlanding - Profile[leftINTsupportlocation].flatdistancefromlanding);//distance between Int. support and current T.P.  -> always right T.P. - left T.P.. Always be positive
		h1 = (float)(Profile[leftINTsupportlocation].elevation + INTHeight - (Profile[TP].elevation + HC));//always left T.P. - right T.P. -> with down hill h1 will be negative.  -> useful when calculate H1L
		From.x = (long)Profile[leftINTsupportlocation].flatdistancefromlanding;
		From.y = (long)( Profile[leftINTsupportlocation].elevation + INTHeight );
	}
	else//heading to the headspar
	{
		d1 = (float)Profile[TP].flatdistancefromlanding;//distance between headspar and current T.P.
		h1 = (float)(Profile[1].elevation + HeadSparHeight - (Profile[TP].elevation + HC));
		From.x = (long)Profile[1].flatdistancefromlanding;
		From.y = (long)( Profile[1].elevation + HeadSparHeight );
	}
		
	To.x = (long) Profile[TP].flatdistancefromlanding;
	To.y = (long)( Profile[TP].elevation + HC );
	

	if (rightINTsupportlocation < TailLocationID)//from an intermediate support
	{
		d2 = (float)(Profile[rightINTsupportlocation].flatdistancefromlanding - Profile[TP].flatdistancefromlanding);//distance between Int. support and current T.P.; right - left
		h2 = (float)((Profile[TP].elevation + HC) - (Profile[rightINTsupportlocation].elevation + INTHeight));//left - right
		From1.x = (long)Profile[rightINTsupportlocation].flatdistancefromlanding;
		From1.y = (long) ( Profile[rightINTsupportlocation].elevation + INTHeight );
	}
	else//from tailspar
	{
		d2 = (float)(Profile[TailLocationID].flatdistancefromlanding - Profile[TP].flatdistancefromlanding);//distance between current T.P. and tailspar
		h2 = (float)((Profile[TP].elevation + HC) - (Profile[TailLocationID].elevation + TailSparHeight) );//left - right
		From1.x = (long)Profile[TailLocationID].flatdistancefromlanding;
		From1.y = (long)( Profile[TailLocationID].elevation + TailSparHeight );
	}
	
	

	//calculate S
	S = (float)( sqrt(pow(d1,2.0)+pow(h1,2.0)) + sqrt(pow(d2,2.0)+pow(h2,2.0)) );
	
	for (j = 1; j <= TotalSpans; ++j)//add other spans (assumed as a straight line) if there are.
	{
		if (j != SpanNo && CableLengthatEachSpan[j] > 0)
			S += CableLengthatEachSpan[j];//CableLengthatEachSpan[] is global variable and already set in PayloadAnalysis().
	}

	//store the value into global variables
	GlobalD1 = d1;
	GlobalD2 = d2;
	GlobalH1 = h1;
	GlobalH2 = h2;

	return(S);
}





float CCablanView::CalculateStrechedLengthbyHC(float HC, int TP, float HeadSparHeight, float INTHeight, float TailSparHeight, bool Draw)
{
	//if we know HC (including CarriageDepth, so from the ground to Skyline),
	//we can calculate total streched lenght of skyline

	//temporary variables
	int j;
	int leftINTsupportlocation;//to identify we are heading to another INT support
	int rightINTsupportlocation;//to identify we are coming from another INT support
	float S;
	float d1,h1,d2,h2;
	CPoint From, To, From1;
	int SpanNo;


	//calculate streched length of the skyline at the given T.P.

	//let's figure out where I am. (which span)
	leftINTsupportlocation = 1;
	rightINTsupportlocation = TailLocationID;
	SpanNo = 1;//count spans to know where I am. 1 means we are going directly to headspar.
		
	for (j = 2; j < TP; ++j)//not include 1 (headspar)
	{
		if (boolINTsupport[j] == TRUE)
		{
			leftINTsupportlocation = j;//keep the closest int. support location in left side
			++SpanNo;
		}
	}	

	for (j = TailLocationID - 1; j > TP ; --j)//start checking backward
	{
		if (boolINTsupport[j] == TRUE)
			rightINTsupportlocation = j;//keep the closest int. support location in right side
	}


	//calculate d and h
	if (leftINTsupportlocation > 1)//heading to an intermediate support
	{
		d1 = (float)(Profile[TP].flatdistancefromlanding - Profile[leftINTsupportlocation].flatdistancefromlanding);//distance between Int. support and current T.P.  -> always right T.P. - left T.P.. Always be positive
		h1 = (float)(Profile[leftINTsupportlocation].elevation + INTHeight - (Profile[TP].elevation + HC));//always left T.P. - right T.P. -> with down hill h1 will be negative.  -> useful when calculate H1L
		From.x = (long)Profile[leftINTsupportlocation].flatdistancefromlanding;
		From.y = (long)( Profile[leftINTsupportlocation].elevation + INTHeight );
	}
	else//heading to the headspar
	{
		d1 = (float)Profile[TP].flatdistancefromlanding;//distance between headspar and current T.P.
		h1 = (float)(Profile[1].elevation + HeadSparHeight - (Profile[TP].elevation + HC));
		From.x = (long)Profile[1].flatdistancefromlanding;
		From.y = (long)( Profile[1].elevation + HeadSparHeight );
	}
		
	To.x = (long) Profile[TP].flatdistancefromlanding;
	To.y = (long)( Profile[TP].elevation + HC );
	


	if (rightINTsupportlocation < TailLocationID)//from an intermediate support
	{
		d2 = (float)(Profile[rightINTsupportlocation].flatdistancefromlanding - Profile[TP].flatdistancefromlanding);//distance between Int. support and current T.P.; right - left
		h2 = (float)((Profile[TP].elevation + HC) - (Profile[rightINTsupportlocation].elevation + INTHeight));//left - right
		From1.x = (long)Profile[rightINTsupportlocation].flatdistancefromlanding;
		From1.y = (long) ( Profile[rightINTsupportlocation].elevation + INTHeight );
	}
	else//from tailspar
	{
		d2 = (float)(Profile[TailLocationID].flatdistancefromlanding - Profile[TP].flatdistancefromlanding);//distance between current T.P. and tailspar
		h2 = (float)((Profile[TP].elevation + HC) - (Profile[TailLocationID].elevation + TailSparHeight) );//left - right
		From1.x = (long)Profile[TailLocationID].flatdistancefromlanding;
		From1.y = (long)( Profile[TailLocationID].elevation + TailSparHeight );
	}
			


	//calculate S
	S = (float)( sqrt(pow(d1,2.0)+pow(h1,2.0)) + sqrt(pow(d2,2.0)+pow(h2,2.0)) );
	
	for (j = 1; j <= TotalSpans; ++j)//add other spans (assumed as a straight line) if there are.
	{
		if (j != SpanNo && CableLengthatEachSpan[j] > 0)
			S += CableLengthatEachSpan[j];//CableLengthatEachSpan[] is global variable and already set in PayloadAnalysis().
	}

	//store the value into global variables
	GlobalD1 = d1;
	GlobalD2 = d2;
	GlobalH1 = h1;
	GlobalH2 = h2;

	return(S);
}




double CCablanView::Radians(double Degree)
{
	double Radian = Degree * 3.141592 / 180.0;
	return (Radian);
}



void CCablanView::OnAnalysesCableways() 
{
	//this sub-routine is almost identical to FindBestLandingLocationWithinBoundary().

	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	OnCablewaysGenerate();

	
	int i, j, k;
	int cs;
	CString tempString;
	
	//user-defined variables
	float MinpayloadbyUser;
	float TailSparHeight;
	int NumberofHeightDevision;

	//counting variable
	int totaliterationpayloadanalysis = 0;



	//for measurign time
	COleDateTime timeStart, timeEnd;
	COleDateTimeSpan spanElapsed;
	double TotalDay, TotalHour, TotalMinute, TotalSecond;
	//end of measuring time
	//saving time
	FILE *outfile_ptr;
	CString tempFileTime = "E:\\thesis\\output\\CableAnalTime.txt";
	outfile_ptr = fopen(tempFileTime, "w");


	//time start
	timeStart = COleDateTime::GetCurrentTime();

	//Sep.9,2002
	int counttemp = 0;

	for (i = 1; i <= pDoc->TotalNumberofLandings; ++i)
	{
		if (pDoc->boolLandingPayloadAnalysis[i] == FALSE)//if this is never done before, or user changed the paramenter.
		{
			pDoc->boolLandingPayloadAnalysis[i] = TRUE;

			cs = pDoc->Landings[i].CableSystem;//cable system for this landing
			MinpayloadbyUser = pDoc->CableSystems[cs].gbUserMinPayload;//design payload

			for (j = 0; j < MAXCABLEWAYS - 1; ++j)//MAXCABLEWAYS -> Total number of cableways per landing is 36 ([0~35])
			{
				//initialize
				pDoc->TailSparTreeHeight[i][j] = pDoc->CableSystems[cs].minspartreeheight;

				//optimize spar tree height-revised on April 30,2002
				NumberofHeightDevision = 3;// (try 4 different heights)???????????????
				for (k = 0; k <= NumberofHeightDevision; ++k)
				{
					//reset boolean variables
					DownHill = FALSE;//determined in GenerateIntermediateSupports()
					HaulBackNeeded = FALSE;//determined in GenerateIntermediateSupports()
					NotEnoughPayload = FALSE;//determined in this subroutine
					SteepChange = FALSE;//determined in GenerateIntermediateSupports()
					PartialOnStream = FALSE;//determined in PayloadAnalysis()
					MoreINTSupportsRequired = FALSE;//determined in GenerateIntermediateSupports()


					TailSparHeight = pDoc->CableSystems[cs].minspartreeheight 
						+ (pDoc->CableSystems[cs].maxspartreeheight - pDoc->CableSystems[cs].minspartreeheight) * ((float)k/(float)NumberofHeightDevision);

					DrawCablewayProfile(i, cs, j);//landing, system, cableway  -- setup Profile[] array

					TailLocationID = TotalNumberofProfileCells;//set tailspar location
					pDoc->Cableways[i][j].taillocationID = TailLocationID;//set tailspar location globally
					pDoc->Cableways[i][j].tailcolumn = Profile[TailLocationID].column;
					pDoc->Cableways[i][j].tailrow = Profile[TailLocationID].row;

					GenerateIntermediateSupports(cs, TailSparHeight);//system//assign true or false to DownHill, HaulBackNeeded, SteepChange, and MoreSupportRequired.
					pDoc->Cableways[i][j].totalnumberofINTs = TotalNumberofINTs;

					PartialOnStream = FALSE;//is determined in PayloadAnalysis()
					++totaliterationpayloadanalysis;
					PayloadAnalysis(cs, i, j, TailSparHeight);//system, landing, cableway, Tailspar height
					pDoc->Payload[i][j] = MinPayload;

					if (MinPayload >= MinpayloadbyUser)
					{
						pDoc->TailSparTreeHeight[i][j] = TailSparHeight;
						break;
					}
				}

				//if still less than user-defined minimum payload, then this is not feasible cable roads.
				if (MinPayload < MinpayloadbyUser)
					NotEnoughPayload = TRUE;

				pDoc->CycleTime[i][j] = CycleTime;//[landingID][CablewayID]


				if (pDoc->CableSystems[cs].Downhill == TRUE)//if this system is available for downhill,
					DownHill = FALSE;//make this always FALSE

				if (pDoc->CableSystems[cs].HaulbackLine == TRUE)//if this system has haulbackline,
					HaulBackNeeded = FALSE;//make this always FALSE


				while (NotEnoughPayload == TRUE || SteepChange == TRUE || DownHill == TRUE || HaulBackNeeded == TRUE || PartialOnStream == TRUE || MoreINTSupportsRequired == TRUE)
				{
					--TailLocationID;
					if (TailLocationID < MINCELLALONGCABLEROAD)//At least 5. not include "="
						break;
				
					pDoc->Cableways[i][j].taillocationID = TailLocationID;
					pDoc->Cableways[i][j].tailcolumn = Profile[TailLocationID].column;
					pDoc->Cableways[i][j].tailrow = Profile[TailLocationID].row;
				
					//initialize
					pDoc->TailSparTreeHeight[i][j] = pDoc->CableSystems[cs].minspartreeheight;

	
					//optimize spar tree height-revised on April 30,2002
					NumberofHeightDevision = 3;// (try 4 different height)???????????????
					for (k = 0; k <= NumberofHeightDevision; ++k)
					{
						//reset boolean variables
						DownHill = FALSE;
						HaulBackNeeded = FALSE;
						NotEnoughPayload = FALSE;
						SteepChange = FALSE;
						PartialOnStream = FALSE;
						MoreINTSupportsRequired = FALSE;//determined in GenerateIntermediateSupports()


						TailSparHeight = pDoc->CableSystems[cs].minspartreeheight 
							+ (pDoc->CableSystems[cs].maxspartreeheight - pDoc->CableSystems[cs].minspartreeheight) * ((float)k/(float)NumberofHeightDevision);


						GenerateIntermediateSupports(cs, TailSparHeight);//system--DowhHill, HaulBackNeeded, and SteepChange are decided here.
						pDoc->Cableways[i][j].totalnumberofINTs = TotalNumberofINTs;

						PartialOnStream = FALSE;//is determined in PayloadAnalysis()
						++totaliterationpayloadanalysis;
						PayloadAnalysis(cs, i, j, TailSparHeight);//system, landing, cableway, tailspar height
						pDoc->Payload[i][j] = MinPayload;

						if (MinPayload < MinpayloadbyUser)
							NotEnoughPayload = TRUE;
						else
						{
							pDoc->TailSparTreeHeight[i][j] = TailSparHeight;
							NotEnoughPayload = FALSE;
							break;
						}
					}

					//if still less than user-defined minimum payload, then this is not feasible cable roads.
					if (MinPayload < MinpayloadbyUser)
						NotEnoughPayload = TRUE;
				

					pDoc->CycleTime[i][j] = CycleTime;

					if (pDoc->CableSystems[cs].Downhill == TRUE)//if this system is available for downhill,
						DownHill = FALSE;//make this always FALSE

					if (pDoc->CableSystems[cs].HaulbackLine == TRUE)//if this system has a haulbackline,
						HaulBackNeeded = FALSE;//make this always FALSE
				}

				//draw with different color so that indentify feasible cableways
				if (NotEnoughPayload == TRUE || SteepChange == TRUE || DownHill == TRUE || HaulBackNeeded == TRUE || PartialOnStream == TRUE || MoreINTSupportsRequired == TRUE) 
				{
					if (SteepChange == TRUE)//Sept.9,2002
						counttemp++;

					DrawEachLine(pDoc->Landings[i].column, pDoc->Landings[i].row, pDoc->Cableways[i][j].tailcolumn, pDoc->Cableways[i][j].tailrow, 10, RGB(100,100,100));
				}
				else
				{
					if (TailSparHeight > pDoc->CableSystems[cs].minspartreeheight)//Spar tree height increased
						DrawEachLine(pDoc->Landings[i].column, pDoc->Landings[i].row, pDoc->Cableways[i][j].tailcolumn, pDoc->Cableways[i][j].tailrow, 10, RGB(200,255,255));		
					else
						DrawEachLine(pDoc->Landings[i].column, pDoc->Landings[i].row, pDoc->Cableways[i][j].tailcolumn, pDoc->Cableways[i][j].tailrow, 10, RGB(200,200,255));		
				}
			}
		
			//writing Tower ID
			tempString.Format("%d", i);
			WriteTextAboveGridCell(pDoc->Landings[i].column, pDoc->Landings[i].row, tempString);
		}
	}



	// measuring time
	// ... perform time-consuming task
	timeEnd = COleDateTime::GetCurrentTime();
	spanElapsed = timeEnd - timeStart;
	TotalDay = spanElapsed.GetDays();
	TotalHour = spanElapsed.GetHours();
	TotalMinute = spanElapsed.GetMinutes();
	TotalSecond = spanElapsed.GetSeconds();
	fprintf(outfile_ptr, "Hour:  %f\n", TotalHour);
	fprintf(outfile_ptr, "Min:  %f\n", TotalMinute);
	fprintf(outfile_ptr, "Sec:  %f\n", TotalSecond);
	fprintf(outfile_ptr, "TotalIteration:  %d\n", totaliterationpayloadanalysis);
	
	//Sept.9,2002
	fprintf(outfile_ptr, "\n\n SteepChordBreak:%d\n", counttemp);

	//end of measuring time
	fclose(outfile_ptr);

}



float CCablanView::FindBestLandingLocationWithinBoundary(int Landingcol, int Landingrow, int CableSystem)//returning total area this landing and cable system can cover
{
	//this sub-routine is called by OnLButtonUp()
	//this sub-routine is almost identical to OnAnalysesCableways().
	//variable arrays [0] used here are for the temporary purpose.
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	int j;
	int cs = CableSystem;
	int tempcol, temprow, tempcellID;//Sept.3,2002
	float totalcoveredarea = 0.;//calculate area by (cable distance * Phi^2 / 36) and accumulate in this variable.
	double eachcoveredarea;
	double diffx, diffy;

	//user-defined variables
	float MinpayloadbyUser;
	float MinSparHeight;

	
	//1. Generate cable ways for this landing
	GenerateCablewaysperLanding(Landingcol, Landingrow, cs);
	//copy back to the current cableway variable.
	memcpy(pDoc->Cableways[0], pDoc->tempCableways, sizeof(pDoc->tempCableways));//Cableways[0] is temporary variable used only here.


	//set temporary landing location for DrawCablewayProfile() subroutine.
	pDoc->Landings[0].CableSystem = cs;
	pDoc->Landings[0].column = Landingcol;
	pDoc->Landings[0].row = Landingrow;


	for (j = 0; j <=35; ++j)
	{
		if (pDoc->Cableways[0][j].tailcolumn <= 0 || pDoc->CableSystems[cs].boolSpecChangebyUser == TRUE)// including 0 for release mode
		{
			//return to initial values.
			pDoc->Cableways[0][j].tailcolumn = pDoc->Cableways[0][j].endcolumn;
			pDoc->Cableways[0][j].tailrow = pDoc->Cableways[0][j].endrow;
			//run profile and keep totalnumberofprofilecells in order to reflect system spec. changes.
			DrawCablewayProfile(0, cs, j);//set Cableways[][].totalnumberofprofilecells
			pDoc->Cableways[0][j].taillocationID = pDoc->Cableways[0][j].totalnumberofprofilecells;
		}
	}



	//2. Payload analyses
	MinpayloadbyUser = pDoc->CableSystems[cs].gbUserMinPayload;
	MinSparHeight = pDoc->CableSystems[cs].minspartreeheight;

	for (j = 0; j < MAXCABLEWAYS - 1; ++j)//MAXCABLEWAYS -> Total number of cableways per landing is 36 ([0~35])
	{
		//reset boolean variables
		DownHill = FALSE;
		HaulBackNeeded = FALSE;
		NotEnoughPayload = FALSE;
		SteepChange = FALSE;
		PartialOnStream = FALSE;

		DrawCablewayProfile(0, cs, j);//landing, system, cableway  -- setup Profile[] array

		TailLocationID = TotalNumberofProfileCells;//set tailspar location
		pDoc->Cableways[0][j].taillocationID = TailLocationID;//set tailspar location globally
		pDoc->Cableways[0][j].tailcolumn = Profile[TailLocationID].column;
		pDoc->Cableways[0][j].tailrow = Profile[TailLocationID].row;

		GenerateIntermediateSupports(cs, MinSparHeight);//system//assign true or false to DownHill, HaulBackNeeded...
		PayloadAnalysis(cs, 0, j, MinSparHeight);//system, landing, cableway
		pDoc->Cableways[0][j].totalnumberofINTs = TotalNumberofINTs;
		
		pDoc->Payload[0][j] = MinPayload;
		if (MinPayload < MinpayloadbyUser)
			NotEnoughPayload = TRUE;

		pDoc->CycleTime[0][j] = CycleTime;


		if (pDoc->CableSystems[cs].Downhill == TRUE)//if this system is available for downhill,
			DownHill = FALSE;//make this always FALSE

		if (pDoc->CableSystems[cs].HaulbackLine == TRUE)//if this system has haulbackline,
			HaulBackNeeded = FALSE;//make this always FALSE


		while (NotEnoughPayload == TRUE || SteepChange == TRUE || DownHill == TRUE || HaulBackNeeded == TRUE || PartialOnStream == TRUE)
		{
			--TailLocationID;
			if (TailLocationID < MINCELLALONGCABLEROAD)//At least 5. not include "="
				break;

			pDoc->Cableways[0][j].taillocationID = TailLocationID;
			pDoc->Cableways[0][j].tailcolumn = Profile[TailLocationID].column;
			pDoc->Cableways[0][j].tailrow = Profile[TailLocationID].row;

			//reset boolean variables
			DownHill = FALSE;
			HaulBackNeeded = FALSE;
			NotEnoughPayload = FALSE;
			SteepChange = FALSE;
			PartialOnStream = FALSE;

			GenerateIntermediateSupports(cs, MinSparHeight);//system
			PayloadAnalysis(cs, 0, j, MinSparHeight);//system, landing, cableway
			pDoc->Cableways[0][j].totalnumberofINTs = TotalNumberofINTs;

			pDoc->Payload[0][j] = MinPayload;
			if (MinPayload < MinpayloadbyUser)
				NotEnoughPayload = TRUE;

			pDoc->CycleTime[0][j] = CycleTime;

			if (pDoc->CableSystems[cs].Downhill == TRUE)//if this system is available for downhill,
				DownHill = FALSE;//make this always FALSE

			if (pDoc->CableSystems[cs].HaulbackLine == TRUE)//if this system has haulbackline,
				HaulBackNeeded = FALSE;//make this always FALSE
		}

		//draw with different color so that identify feasible cableways
		if (NotEnoughPayload == TRUE || SteepChange == TRUE || DownHill == TRUE || HaulBackNeeded == TRUE || PartialOnStream == TRUE) 
			DrawEachLine(pDoc->Landings[0].column, pDoc->Landings[0].row, pDoc->Cableways[0][j].tailcolumn, pDoc->Cableways[0][j].tailrow, 10, RGB(100,100,100));
		else
			DrawEachLine(pDoc->Landings[0].column, pDoc->Landings[0].row, pDoc->Cableways[0][j].tailcolumn, pDoc->Cableways[0][j].tailrow, 10, RGB(200,200,255));


		//Sept.3,2002
		tempcol = pDoc->Cableways[0][j].tailcolumn;
		temprow = pDoc->Cableways[0][j].tailrow;
		tempcellID = pDoc->Cableways[0][j].taillocationID;

		while (pDoc->CellsWithinUnitBoundary[tempcol][temprow] != TRUE && tempcellID > 2)
		{
			--tempcellID;
			
			tempcol = Profile[tempcellID].column;
			temprow = Profile[tempcellID].row;
		}

		if (tempcellID > 2)//Sept.3,2002
		{
			diffx = (double) (pDoc->DTMCoord[Landingcol][Landingrow].x - pDoc->DTMCoord[tempcol][temprow].x);
			diffy = (double) (pDoc->DTMCoord[Landingcol][Landingrow].y - pDoc->DTMCoord[tempcol][temprow].y);

			//diffx = (double) (pDoc->DTMCoord[Landingcol][Landingrow].x - pDoc->DTMCoord[pDoc->Cableways[0][j].tailcolumn][pDoc->Cableways[0][j].tailrow].x);
			//diffy = (double) (pDoc->DTMCoord[Landingcol][Landingrow].y - pDoc->DTMCoord[pDoc->Cableways[0][j].tailcolumn][pDoc->Cableways[0][j].tailrow].y);

			eachcoveredarea = ( pow(diffx, 2.0) + pow(diffy, 2.0) );

			totalcoveredarea += (float)eachcoveredarea;//The criterion for the best landing location is total covered area by the feasible skyline corridors from the landing.
		}
	}

	return(totalcoveredarea);
}



void CCablanView::OnInputdataCablesystem() 
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int i;

	if (pDoc->gbCurrentCableSystem <= 0 || pDoc->gbCurrentCableSystem > MAXCABLESYSTEMS)//if this dialog box gets opened directly.; including 0 for release mode
		pDoc->gbCurrentCableSystem = 1;

	int CurrentSystem = pDoc->gbCurrentCableSystem;//local variable just to simplify coding.

	CCableSystemDataInput dlg;
	
	dlg.CurrentSystem = CurrentSystem;
	memcpy(dlg.CableSystems, pDoc->CableSystems, sizeof(pDoc->CableSystems));


	if (dlg.DoModal() == IDOK)
	{
		pDoc->gbCurrentCableSystem = dlg.CurrentSystem;//save to global variable
		memcpy(pDoc->CableSystems, dlg.CableSystems, sizeof(pDoc->CableSystems));

		//indicating the parameters are changed.
		//change only the landings having the current system.
		for (i = 1; i <= pDoc->TotalNumberofLandings; ++i)
		{
			if (pDoc->CableSystems[pDoc->Landings[i].CableSystem].boolSpecChangebyUser == TRUE)
				pDoc->boolLandingPayloadAnalysis[i] = FALSE;//boolLandingPayloadAnalysis[landingID]
		}
	}


}





void CCablanView::OnInputdataOperationdata() 
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int i;

	if (pDoc->gbCurrentCableSystem <= 0 || pDoc->gbCurrentCableSystem > MAXCABLESYSTEMS)//if this dialog box gets opened directly.; including 0 for release mode
		pDoc->gbCurrentCableSystem = 1;

	int CurrentSystem = pDoc->gbCurrentCableSystem;//local variable just to simplify coding.

	CDialogCableOperationData dlg;

	dlg.m_Density = pDoc->CableSystems[CurrentSystem].gbDensity;
	dlg.m_EngineEfficiency = pDoc->CableSystems[CurrentSystem].gbEngineEfficiency;
	dlg.m_EnginePower = pDoc->CableSystems[CurrentSystem].gbEnginePower;
	dlg.m_HookTime = pDoc->CableSystems[CurrentSystem].gbHookTime;
	dlg.m_LoadFactor = pDoc->CableSystems[CurrentSystem].gbLoadFactor;
	dlg.m_RiggingCosts = pDoc->CableSystems[CurrentSystem].gbRiggingCosts;
	dlg.m_UnhookTime = pDoc->CableSystems[CurrentSystem].gbUnhookTime;
	dlg.m_UnitCost = pDoc->CableSystems[CurrentSystem].gbUnitCost;
	dlg.m_DelayPercentage = pDoc->CableSystems[CurrentSystem].gbDelayPercentage;
	dlg.m_FellingCosts = pDoc->CableSystems[CurrentSystem].gbFellingCosts;
	dlg.m_MoveInCosts = pDoc->CableSystems[CurrentSystem].gbMoveInCosts;
	dlg.m_YarderSetupHour = pDoc->CableSystems[CurrentSystem].gbYarderSetupHour;
	dlg.m_CableRoadChangeHour = pDoc->CableSystems[CurrentSystem].gbCableRoadChangeHour;
	dlg.m_UserMaxPayload = pDoc->CableSystems[CurrentSystem].gbUserMaxPayload;
	dlg.m_UserMinPayload = pDoc->CableSystems[CurrentSystem].gbUserMinPayload;
	dlg.m_InhaulSpeedLimit = pDoc->CableSystems[CurrentSystem].gbInhaulSpeedLimit;
	dlg.m_InhaulSpeedLimitFull = pDoc->CableSystems[CurrentSystem].gbInhaulSpeedLimitFull;
	dlg.m_OuthaulSpeedLimit = pDoc->CableSystems[CurrentSystem].gbOuthaulSpeedLimit;
	dlg.m_LateralYardingSpeed = pDoc->CableSystems[CurrentSystem].gbLateralYardingSpeed;
	dlg.m_PenaltyForBeyondLateralYarding = pDoc->CableSystems[CurrentSystem].gbPenaltyForBeyondLateralYarding;
	dlg.m_MultiplierLYD1 = pDoc->CableSystems[CurrentSystem].gbMultiplierLYD1;
	dlg.m_MultiplierLYD2 = pDoc->CableSystems[CurrentSystem].gbMultiplierLYD2;
	dlg.m_MultiplierLYD3 = pDoc->CableSystems[CurrentSystem].gbMultiplierLYD3;
	dlg.m_GiveUpCellDistance = pDoc->CableSystems[CurrentSystem].gbGiveUpCellDistance;
	
	

	if (dlg.DoModal() == IDOK)
	{
		pDoc->CableSystems[CurrentSystem].gbDensity = dlg.m_Density;
		pDoc->CableSystems[CurrentSystem].gbEngineEfficiency = dlg.m_EngineEfficiency;
		pDoc->CableSystems[CurrentSystem].gbEnginePower = dlg.m_EnginePower;
		pDoc->CableSystems[CurrentSystem].gbHookTime = dlg.m_HookTime;
		pDoc->CableSystems[CurrentSystem].gbLoadFactor = dlg.m_LoadFactor;
		pDoc->CableSystems[CurrentSystem].gbRiggingCosts = dlg.m_RiggingCosts;
		pDoc->CableSystems[CurrentSystem].gbUnhookTime = dlg.m_UnhookTime;
		pDoc->CableSystems[CurrentSystem].gbUnitCost = dlg.m_UnitCost;
		pDoc->CableSystems[CurrentSystem].gbDelayPercentage = dlg.m_DelayPercentage;
		pDoc->CableSystems[CurrentSystem].gbFellingCosts = dlg.m_FellingCosts;
		pDoc->CableSystems[CurrentSystem].gbMoveInCosts = dlg.m_MoveInCosts;
		pDoc->CableSystems[CurrentSystem].gbYarderSetupHour = dlg.m_YarderSetupHour;
		pDoc->CableSystems[CurrentSystem].gbCableRoadChangeHour = dlg.m_CableRoadChangeHour;
		pDoc->CableSystems[CurrentSystem].gbUserMaxPayload = dlg.m_UserMaxPayload;
		pDoc->CableSystems[CurrentSystem].gbUserMinPayload = dlg.m_UserMinPayload;
		pDoc->CableSystems[CurrentSystem].gbInhaulSpeedLimit = dlg.m_InhaulSpeedLimit;
		pDoc->CableSystems[CurrentSystem].gbInhaulSpeedLimitFull = dlg.m_InhaulSpeedLimitFull;
		pDoc->CableSystems[CurrentSystem].gbOuthaulSpeedLimit = dlg.m_OuthaulSpeedLimit;
		pDoc->CableSystems[CurrentSystem].gbLateralYardingSpeed = dlg.m_LateralYardingSpeed;
		pDoc->CableSystems[CurrentSystem].gbPenaltyForBeyondLateralYarding = dlg.m_PenaltyForBeyondLateralYarding;
		pDoc->CableSystems[CurrentSystem].gbMultiplierLYD1 = dlg.m_MultiplierLYD1;
		pDoc->CableSystems[CurrentSystem].gbMultiplierLYD2 = dlg.m_MultiplierLYD2;
		pDoc->CableSystems[CurrentSystem].gbMultiplierLYD3 = dlg.m_MultiplierLYD3;
		pDoc->CableSystems[CurrentSystem].gbGiveUpCellDistance = dlg.m_GiveUpCellDistance;

		//indicating the parameters are changed.
		//change only the landings having the current system.
		for (i = 1; i <= pDoc->TotalNumberofLandings; ++i)
		{
			if (pDoc->Landings[i].CableSystem == CurrentSystem)
				pDoc->boolLandingPayloadAnalysis[i] = FALSE;//boolLandingPayloadAnalysis[landingID]
		}
	}	
}





void CCablanView::OnInputdataRoadcosts() 
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	CDialogRoadCosts dlg;


	dlg.m_HaulingCosts = pDoc->HaulingCosts;
	dlg.m_RGFactor1 = pDoc->RGFactor1;
	dlg.m_RGFactor2 = pDoc->RGFactor2;
	dlg.m_RGFactor3 = pDoc->RGFactor3;
	dlg.m_RGFactor4 = pDoc->RGFactor4;
	dlg.m_RGFactor5 = pDoc->RGFactor5;
	dlg.m_RGFactor6 = pDoc->RGFactor6;
	dlg.m_RGFactor7 = pDoc->RGFactor7;
	dlg.m_RoadCosts = pDoc->RoadCosts;
	dlg.m_SSFactor1 = pDoc->SSFactor1;
	dlg.m_SSFactor2 = pDoc->SSFactor2;
	dlg.m_SSFactor3 = pDoc->SSFactor3;
	dlg.m_SSFactor4 = pDoc->SSFactor4;
	dlg.m_SSFactor5 = pDoc->SSFactor5;
	dlg.m_SSFactor6 = pDoc->SSFactor6;
	dlg.m_SSFactor7 = pDoc->SSFactor7;
	dlg.m_RGLimit = pDoc->RGLimit;
	dlg.m_SSLimit = pDoc->SSLimit;
	dlg.m_MultiplierForRoadsonStream = pDoc->MultiplierForRoadsOnStream;



	if (dlg.DoModal() == IDOK)
	{
		pDoc->HaulingCosts = dlg.m_HaulingCosts;
		pDoc->RGFactor1 = dlg.m_RGFactor1;
		pDoc->RGFactor2 = dlg.m_RGFactor2;
		pDoc->RGFactor3 = dlg.m_RGFactor3;
		pDoc->RGFactor4 = dlg.m_RGFactor4;
		pDoc->RGFactor5 = dlg.m_RGFactor5;
		pDoc->RGFactor6 = dlg.m_RGFactor6;
		pDoc->RGFactor7 = dlg.m_RGFactor7;
		pDoc->RoadCosts = dlg.m_RoadCosts;
		pDoc->SSFactor1 = dlg.m_SSFactor1;
		pDoc->SSFactor2 = dlg.m_SSFactor2;
		pDoc->SSFactor3 = dlg.m_SSFactor3;
		pDoc->SSFactor4 = dlg.m_SSFactor4;
		pDoc->SSFactor5 = dlg.m_SSFactor5;
		pDoc->SSFactor6 = dlg.m_SSFactor6;
		pDoc->SSFactor7 = dlg.m_SSFactor7;
		pDoc->RGLimit = dlg.m_RGLimit;
		pDoc->SSLimit = dlg.m_SSLimit;
		pDoc->MultiplierForRoadsOnStream = dlg.m_MultiplierForRoadsonStream;
	}	
}



void CCablanView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CDialogPathAnalysis dlg;
	CDialogLandingCosts dlgLandingCosts;

	int i,j;
	int k,n, cs;
	int ld;
	int Foundi = -1;
	int Foundj = -1;
	CString tempString, tempString1;

	//local variables for optimizing landing location within a boundary.
	int startcol, startrow, endcol, endrow;
	float maxarea = -1.;
	float coveredarea;
	int keptcol = -1, keptrow = -1;



	pDC = new CClientDC(this);
	OnPrepareDC(pDC);
	pDC->SetMapMode(MM_ISOTROPIC);
	pDC->SetWindowExt(ViewScale, ViewScale);

	pDC->DPtoLP(&point);

	point.x = (long)(point.x + pDoc->DTMminX - cpMargin.x);
	point.y = (long)(point.y + pDoc->DTMmaxY + cpMargin.y);


	//find column
	for (i = 1; i <= pDoc->DTMcolumns; ++i)
	{
		j = 1;
		//take any row within the boundary
		while (pDoc->DTMCoord[i][j].x <= 0)//if < 0, it means beyond the boundary; including 0 for release mode
		{
			++j;
			if (j >= pDoc->DTMrows)
				j = 1;
		}
		if (point.x < pDoc->DTMCoord[i][j].x + pDoc->DTMcellsize/2.0 && point.x > pDoc->DTMCoord[i][j].x - pDoc->DTMcellsize/2.0)
		{
			Foundi = i;
			break;
		}
	}

	//find row
	if (Foundi >= 0)
	{
		for (j = 1; j <= pDoc->DTMrows; ++j)
		{
			if (point.y < pDoc->DTMCoord[Foundi][j].y + pDoc->DTMcellsize/2.0 && point.y > pDoc->DTMCoord[Foundi][j].y - pDoc->DTMcellsize/2.0)
			{
				Foundj = j;
				break;
			}
		}
	}


	pDoc->blLandingBoundarySetting = FALSE;//once mouse button is released, then make this boolean false.

		
	if (Foundi < 0 || Foundj < 0)//Foundi and j is initialiezed as -1
		tempString.Format("Data are not available...                                                  ");
	else
	{
		DrawEachCell(Foundi, Foundj, RGB(0,0,0));//column and row
		//legend
		if (boolVolumeLayerShowing == FALSE)
			tempString.Format("x:%d  y:%d  z:%d  c:%d  r:%d  ", pDoc->DTMCoord[Foundi][Foundj].x, pDoc->DTMCoord[Foundi][Foundj].y, pDoc->DTMCoord[Foundi][Foundj].z, Foundi, Foundj);
		else//now DEM shows up on the screen.
			tempString.Format("x:%d  y:%d  z:%.1f  c:%d  r:%d        ", pDoc->DTMCoord[Foundi][Foundj].x, pDoc->DTMCoord[Foundi][Foundj].y, pDoc->OriginalVolume[Foundi][Foundj], Foundi, Foundj);


		//LANDING-----------------------------------------
		//input i and j into a Landing location
		if (boolAddLandings == TRUE)
		{
			boolAddLandings = FALSE;

			//setting Cable system first.
			if (pDoc->gbCurrentCableSystem < 1)
			{
				//pDoc->TotalNumberofLandings has already got added by 1 in OnLandingsPlaceLandings()
				pDoc->Landings[pDoc->TotalNumberofLandings].CableSystem = 1;//initial cable system
				pDoc->gbCurrentCableSystem = 1;//initial cable system
			}
			else
			{
				pDoc->Landings[pDoc->TotalNumberofLandings].CableSystem = pDoc->gbCurrentCableSystem;//initial cable system
			}
			OnInputdataCablesystem();//input value to pDoc->gbCurrentCableSystem.
			pDoc->Landings[pDoc->TotalNumberofLandings].CableSystem = pDoc->gbCurrentCableSystem;//accept change of Cable system in the CableSystemDataInput dialog box

			if (pDoc->BoundaryEnd.x <= 0 || pDoc->BoundaryEnd.y <= 0 || ( pDoc->BoundaryStart.x == pDoc->BoundaryEnd.x && pDoc->BoundaryEnd.y == pDoc->BoundaryStart.y) )//BoundaryEnd is set in OnMouseMove(). BoundaryStart is set in OnLButtonDown.; including 0 for release mode
			{//if Mouse is not moved with holding left button, then set landing location on the cell.
				//pDoc->TotalNumberofLandings is already increased by one at OnLandingsPlacelanding()
				pDoc->Landings[pDoc->TotalNumberofLandings].column = Foundi;
				pDoc->Landings[pDoc->TotalNumberofLandings].row = Foundj;

				//drawing the selected cell
				DrawEachCell(Foundi, Foundj, RGB(0,0,0));//column and row
				//legend
				tempString.Format("x:%d  y:%d  z:%d", pDoc->DTMCoord[Foundi][Foundj].x, pDoc->DTMCoord[Foundi][Foundj].y, pDoc->DTMCoord[Foundi][Foundj].z);
			}
			else
			{//if BoundaryEnd has a value, which means the user move mouse with holding left button to draw a boundary for landing locations.
				//choose which one is smaller.
				if (pDoc->BoundaryStart.x > pDoc->BoundaryEnd.x)
				{
					startcol = pDoc->BoundaryEnd.x;
					endcol = pDoc->BoundaryStart.x;
				}
				else
				{
					startcol = pDoc->BoundaryStart.x;
					endcol = pDoc->BoundaryEnd.x;
				}

				if (pDoc->BoundaryStart.y > pDoc->BoundaryEnd.y)
				{
					startrow = pDoc->BoundaryEnd.y;
					endrow = pDoc->BoundaryStart.y;
				}
				else
				{
					startrow = pDoc->BoundaryStart.y;
					endrow = pDoc->BoundaryEnd.y;
				}

				for (i = startcol; i <= endcol; ++i)
				{
					for (j = startrow; j <= endrow; ++j)
						DrawEachCell(i, j, RGB(255,255,255));
				}

				//optimize landing locations.	
				//The criterion for the best landing location is total covered area by the feasible skyline corridors from the landing.
				for (i = startcol; i <= endcol; ++i)
				{
					for (j = startrow; j <= endrow; ++j)
					{
						DrawEachCell(i, j, RGB(255,255,255));
						coveredarea = FindBestLandingLocationWithinBoundary(i, j, pDoc->gbCurrentCableSystem);//gbCurrentCableSystem is upadated at OnInputdataCableSystem() which is called by OnLButtonDown().
						if (coveredarea > maxarea)
						{
							maxarea = coveredarea;
							keptcol = i;
							keptrow = j;
						}
					}
				}

				if (keptcol > 0 && keptrow > 0)//already found the best landing location within the boundary.
				{
					//pDoc->TotalNumberofLandings is already increased by one at OnLandingsPlacelanding()
					pDoc->Landings[pDoc->TotalNumberofLandings].column = keptcol;
					pDoc->Landings[pDoc->TotalNumberofLandings].row = keptrow;

					//drawing the selected cell
					DrawEachCell(keptcol, keptrow, RGB(0,0,0));//column and row
					//legend
					tempString.Format("x:%d  y:%d  z:%d", pDoc->DTMCoord[keptcol][keptrow].x, pDoc->DTMCoord[keptcol][keptrow].y, pDoc->DTMCoord[keptcol][keptrow].z);
				}
			}

			//open the LandingCosts Dialog box for inputting landing construction costs.
			memcpy(dlgLandingCosts.LandingCosts, pDoc->LandingCosts, sizeof(pDoc->LandingCosts));
			memcpy(dlgLandingCosts.LandingSharingTowerID, pDoc->LandingSharingTowerID, sizeof(pDoc->LandingSharingTowerID));
			dlgLandingCosts.TotalNumberofLandings = pDoc->TotalNumberofLandings;
			dlgLandingCosts.CurrentLanding = pDoc->TotalNumberofLandings;//current landing ID
			
			if (dlgLandingCosts.DoModal() == IDOK)
			{
				memcpy(pDoc->LandingCosts, dlgLandingCosts.LandingCosts, sizeof(pDoc->LandingCosts));
				memcpy(pDoc->LandingSharingTowerID, dlgLandingCosts.LandingSharingTowerID, sizeof(pDoc->LandingSharingTowerID));
				for (ld = 1; ld <= pDoc->TotalNumberofLandings; ++ld)
				{
					if (pDoc->LandingSharingTowerID[ld] > 0)//if there is any sharing tower,
						//put the same amount of landing costs as the sharing tower has.
						pDoc->LandingCosts[ld] = pDoc->LandingCosts[pDoc->LandingSharingTowerID[ld]];
				}
			}


			//writing Tower ID number
			tempString1.Format("%d", pDoc->TotalNumberofLandings);
			WriteTextAboveGridCell(pDoc->Landings[pDoc->TotalNumberofLandings].column, pDoc->Landings[pDoc->TotalNumberofLandings].row, tempString1);
		}//end of select landing location


		//Select CABLEWAY - Terrain Profile------------------------------------
		if (boolCheckProfile == TRUE)
		{
			for (n = 1; n <= pDoc->TotalNumberofLandings; ++n)
			{
				cs = pDoc->Landings[n].CableSystem;//cable system setting
				for (k = 0; k <= 35; ++k)
				{
					if (i == pDoc->Cableways[n][k].tailcolumn && j == pDoc->Cableways[n][k].tailrow)
					{
						DrawCablewayProfile(n, cs, k);//select cells on profile and keep terrain data into Profile[]
						//open a dialog box for selecting tail spar location and analysing cable system
						dlg.landingNo = n;
						dlg.cablewayNo = k;
						dlg.systemNo = cs;
						memcpy(dlg.Profile , Profile, sizeof(Profile));
						dlg.CableSystems.intermediatesupportheight = pDoc->CableSystems[cs].intermediatesupportheight;
						dlg.CableSystems.maxlateraldist = pDoc->CableSystems[cs].maxlateraldist;
						dlg.CableSystems.maxyarddist = pDoc->CableSystems[cs].maxyarddist;
						dlg.CableSystems.towerheight = pDoc->CableSystems[cs].towerheight;
						dlg.CableSystems.maxspartreeheight = pDoc->CableSystems[cs].maxspartreeheight;
						dlg.CableSystems.minspartreeheight = pDoc->CableSystems[cs].minspartreeheight;
						dlg.TotalNumberofProfileCells = TotalNumberofProfileCells;
						dlg.TailLocationID = pDoc->Cableways[n][k].taillocationID;
						dlg.gbBeta = pDoc->CableSystems[cs].gbBeta;
						dlg.gbCarriageDepth = pDoc->CableSystems[cs].gbCarriageDepth;
						dlg.gbDLog = pDoc->CableSystems[cs].gbDLog;
						dlg.gbL = pDoc->CableSystems[cs].gbL;
						dlg.gbLC = pDoc->CableSystems[cs].gbLC;
						dlg.gbMyu = pDoc->CableSystems[cs].gbMyu;
						dlg.gbTmax = pDoc->CableSystems[cs].gbTmax;
						dlg.gbMmax = pDoc->CableSystems[cs].gbMmax;
						dlg.gbW1 = pDoc->CableSystems[cs].gbW1;
						dlg.gbW3 = pDoc->CableSystems[cs].gbW3;
						dlg.gbW4 = pDoc->CableSystems[cs].gbW4;
						dlg.gbWc = pDoc->CableSystems[cs].gbWc;
						dlg.gbClearance = pDoc->CableSystems[cs].gbClearance;
						dlg.gbAllowableDeflection = pDoc->CableSystems[cs].gbAllowableDeflection;
						dlg.gbMinSlopewithoutHBL = pDoc->CableSystems[cs].gbMinSlopewithoutHBL;
						dlg.gbMaxSlopeChange = pDoc->CableSystems[cs].gbMaxSlopeChange;//maximum allowable change in chord slope between adjacent spans
						dlg.gbMaxINTSupports = pDoc->CableSystems[cs].gbMaxINTSupports;//limit of number of INT supports.

						//operation cost data
						dlg.gbHooktimeperCM = pDoc->CableSystems[cs].gbHookTime;// sec/ton(cubic meter)
						dlg.gbUnhooktimeperCM = pDoc->CableSystems[cs].gbUnhookTime;// sec/ton(cubic meter)
						dlg.gbRiggingCosts = pDoc->CableSystems[cs].gbRiggingCosts;//sec. per intermediate support and tailspar
						dlg.gbUserMinPayload = pDoc->CableSystems[cs].gbUserMinPayload;//2200 lb  -> kg
						dlg.gbDensity = pDoc->CableSystems[cs].gbDensity;//2200 lb / cubic meter  -> kg / cubic meter
						dlg.gbUserMaxPayload = pDoc->CableSystems[cs].gbUserMaxPayload;//6600 lb  -> kg
						dlg.gbLoadFactor = pDoc->CableSystems[cs].gbLoadFactor;//percent of skyline capacity actually used
						dlg.gbCableOperationUnitCost = pDoc->CableSystems[cs].gbUnitCost;//hourly cost of tower operation
						dlg.gbDelayPercentage = pDoc->CableSystems[cs].gbDelayPercentage;
						dlg.gbFellingCosts = pDoc->CableSystems[cs].gbFellingCosts;
						dlg.gbMoveInCosts = pDoc->CableSystems[cs].gbMoveInCosts;//System move-in costs
						dlg.gbYarderSetupHour = pDoc->CableSystems[cs].gbYarderSetupHour;//System install time
						dlg.gbCableRoadChangeHour = pDoc->CableSystems[cs].gbCableRoadChangeHour;//skyline reinstalling time
						dlg.gbEnginePower = pDoc->CableSystems[cs].gbEnginePower;//horse power
						dlg.gbEngineEfficiency = pDoc->CableSystems[cs].gbEngineEfficiency;//65% of engine power
						dlg.gbInhaulSpeedLimit = pDoc->CableSystems[cs].gbInhaulSpeedLimit;//speed limit when partial suspension.. speed should not be greater than this limit.(m/sec)
						dlg.gbInhaulSpeedLimitFull = pDoc->CableSystems[cs].gbInhaulSpeedLimitFull;//speed limit when full suspension.. 
						dlg.gbOuthaulSpeedLimit = pDoc->CableSystems[cs].gbOuthaulSpeedLimit;//speed limit when outhauling..
						dlg.gbLateralYardingSpeed = pDoc->CableSystems[cs].gbLateralYardingSpeed;//average speed for lateral yarding.
						dlg.gbPenaltyForBeyondLateralYarding = pDoc->CableSystems[cs].gbPenaltyForBeyondLateralYarding;
						dlg.gbMultiplierLYD1 = pDoc->CableSystems[cs].gbMultiplierLYD1;
						dlg.gbMultiplierLYD2 = pDoc->CableSystems[cs].gbMultiplierLYD2;
						dlg.gbMultiplierLYD3 = pDoc->CableSystems[cs].gbMultiplierLYD3;
						dlg.gbGiveUpCellDistance = pDoc->CableSystems[cs].gbGiveUpCellDistance;

						//tail spar height
						if (pDoc->TailSparTreeHeight[n][k] >= pDoc->CableSystems[cs].minspartreeheight && pDoc->TailSparTreeHeight[n][k] <= pDoc->CableSystems[cs].maxspartreeheight)
							dlg.SparTreeHeight = pDoc->TailSparTreeHeight[n][k];
						else
							dlg.SparTreeHeight = pDoc->CableSystems[cs].minspartreeheight;

																			
						if (dlg.DoModal() == IDOK)
						{
							k = 35;//get out of for-next statement.
						}
						else
							k = 35;//get out of for-next statement.
					}
				}
			}
		}//end of select Profile

	}//end of Foundi, Foundj



	//set a different mapmode for text output (legend)
	pDC = new CClientDC(this);
	OnPrepareDC(pDC);
	pDC->SetMapMode(MM_TEXT);//just for legend
	CRect WinRect;
	GetClientRect(WinRect);
	//writing guide
	pDC->TextOut(WinRect.left + 50, 3, tempString);


	CScrollView::OnLButtonUp(nFlags, point);
}



/////////////////////////////////////////////////////////////////////////////////////////////////
//ROAD ROUTING///////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////


void CCablanView::GenerateRoadRoutefromExistingRoadtoLandings(int landingNo, CPoint StartingCell, CPoint EndingCell)
{
	//select a route between two cells considering slope and distance from the 'EndingCell'

	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	MaxRoadLength = MAXROADLENGTH;//defined in View
	int NumberofRouteCell = 0;
	int i, j, k;
	int tempstartx, tempendx;
	int tempstarty, tempendy;
	CPoint NextRoute;
	float tempdist;
	float mindist;
	int keptj, keptk;
	

	//select one cell in the current route, and starts from the cell.
	CPoint SecondCell;

	//initialize 
	for (i = 0; i < MaxRoadLength; ++i)
	{
		tempRoadCells[i].x = -1;
		tempRoadCells[i].y = -1;
	}
	
	tempRoadCells[0].x = StartingCell.x;
	tempRoadCells[0].y = StartingCell.y;


	//set the second cell as the closest cell to the existing road.
	
	//keep within the boundary
	if (StartingCell.x - 1 > 0)
		tempstartx = StartingCell.x - 1;
	else
		tempstartx = 1;
	
	if (StartingCell.x + 1 <= pDoc->DTMcolumns)
		tempendx = StartingCell.x + 1;
	else
		tempendx = pDoc->DTMcolumns;

	if (StartingCell.y - 1 > 0)
		tempstarty = StartingCell.y - 1;
	else
		tempstarty = 1;
	if (StartingCell.y + 1 <= pDoc->DTMrows)
		tempendy = StartingCell.y + 1;
	else
		tempendy = pDoc->DTMrows;
		
	//select the closest cell.
	mindist = (float)999999999999.9;
	for (j = tempstartx; j <= tempendx; ++j)
	{
		for (k = tempstarty; k <= tempendy; ++k)
		{
			tempdist = (float)( pow((double)(j - EndingCell.x), 2.0) + pow((double)(k - EndingCell.y), 2.0) );
			if (mindist > tempdist)
			{
				mindist = tempdist;
				keptj = j;
				keptk = k;
			}
		}
	}
	//keep the second road cell
	tempRoadCells[1].x = keptj;
	tempRoadCells[1].y = keptk;
				

	NumberofRouteCell = MaxRoadLength;


	for (j = 2; j < NumberofRouteCell; ++j)
	{
		NextRoute = RandomlyFindingNextCellonRoute(tempRoadCells[j-1], tempRoadCells[j-2], EndingCell);
		if (NextRoute.x == -1 && NextRoute.y == -1)//if all neighborcells have -1 value
		{										   //it means current cell is the very corner cell
			NumberofRouteCell = j - 1;				   //Stop routing on this cell and change Number of Cells
			break;
		}
		else
		{
			for (k = 2; k < j; ++k)
			{
				if (NextRoute.x == tempRoadCells[k].x && NextRoute.y == tempRoadCells[k].y)//if the same cell is selected, then stop at the cell
				{
					NumberofRouteCell = j;
					break;
				}
			}

			if (NextRoute.x == EndingCell.x && NextRoute.y == EndingCell.y)//if the road cell meets the existing cell.
				NumberofRouteCell = j;
			
			tempRoadCells[j].x = NextRoute.x;
			tempRoadCells[j].y = NextRoute.y;
		}
	}


	//in case of route not ending at the EndingCell.
	float slope;
	float intercept;
	int start, ending;
	if (tempRoadCells[NumberofRouteCell].x != EndingCell.x || tempRoadCells[NumberofRouteCell].y != EndingCell.y)
	{
		//draw a straight line to the EndingCell.
		if (tempRoadCells[NumberofRouteCell].x - EndingCell.x != 0)
		{
			slope = (float)(tempRoadCells[NumberofRouteCell].y - EndingCell.y) / (float)(tempRoadCells[NumberofRouteCell].x - EndingCell.x);
			intercept = (float)EndingCell.y - (float)EndingCell.x * slope;
		}
		else
			slope = (float)999999999.9;


		if (fabs(slope) > 1 && fabs(slope) < 10000.0)//select y-axis as an independent variable
		{
			if (tempRoadCells[NumberofRouteCell].y < EndingCell.y)
			{
				start = tempRoadCells[NumberofRouteCell].y;
				ending = EndingCell.y;
			}
			else
			{
				ending = tempRoadCells[NumberofRouteCell].y;
				start = EndingCell.y;
			}
			
			for (j = start + 1; j <= ending; ++j)//changing in y direction
			{
				i = (int)(((float)j - intercept)/slope);//find columne 
				++NumberofRouteCell;
				tempRoadCells[NumberofRouteCell].x = i;
				tempRoadCells[NumberofRouteCell].y = j;
			}
		}
		else if (fabs(slope) > 10000.0)//when slope is infinite ->North and South direction
		{
			if (tempRoadCells[NumberofRouteCell].y < EndingCell.y)
			{
				start = tempRoadCells[NumberofRouteCell].y;
				ending = EndingCell.y;
			}
			else
			{
				ending = tempRoadCells[NumberofRouteCell].y;
				start = EndingCell.y;
			}

			i = EndingCell.x;
			for (j = start + 1; j <= ending; ++j)//changing in y direction
			{
				++NumberofRouteCell;
				tempRoadCells[NumberofRouteCell].x = i;
				tempRoadCells[NumberofRouteCell].y = j;
			}
		}
		else//select x-axis as an independent variable
		{
			if (tempRoadCells[NumberofRouteCell].x < EndingCell.x)
			{
				start = tempRoadCells[NumberofRouteCell].x;
				ending = EndingCell.x;
			}
			else
			{
				ending = tempRoadCells[NumberofRouteCell].x;
				start = EndingCell.x;
			}
			
			for (i = start + 1; i <= ending; ++i)//changing in x direction
			{
				j = (int)((float)i * slope + intercept);//find columne 
				++NumberofRouteCell;
				tempRoadCells[NumberofRouteCell].x = i;
				tempRoadCells[NumberofRouteCell].y = j;
			}
		}
	}

	tempRoadCells[0].x = StartingCell.x;
	tempRoadCells[0].y = StartingCell.y;
	tempRoadCells[NumberofRouteCell+1].x = EndingCell.x;
	tempRoadCells[NumberofRouteCell+1].y = EndingCell.y;


	//input -1 to the rest of route cells up to the maximum number of cell (MaxLength)
	for (j = NumberofRouteCell + 1; j < MaxRoadLength; ++j)
	{
		tempRoadCells[j].x = -1;
		tempRoadCells[j].y = -1;
	}

}


void CCablanView::GenerateRoadRoutefromOnePointtoTheother(int landingNo, CPoint StartingCell, CPoint EndingCell)
{
	//draw a straight line between two selected cells
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	MaxRoadLength = MAXROADLENGTH;//defined in View
	int NumberofRouteCell = 0;
	int i, j;
	

	//initialize 
	for (i = 0; i < MaxRoadLength; ++i)
	{
		tempRoadCells[i].x = -1;
		tempRoadCells[i].y = -1;
	}
	

	//in case of route not ending at the EndingCell.
	float slope;
	float intercept;

	//draw a straight line to the EndingCell.
	if (StartingCell.x - EndingCell.x != 0)
	{
		slope = (float)(StartingCell.y - EndingCell.y) / (float)(StartingCell.x - EndingCell.x);
		intercept = (float)EndingCell.y - (float)EndingCell.x * slope;
	}
	else
		slope = (float)999999999.9;


	if (fabs(slope) > 1 && fabs(slope) < 10000.0)//select y-axis as an independent variable
	{
		if (StartingCell.y < EndingCell.y)
		{
			for (j = StartingCell.y; j <= EndingCell.y; ++j)//changing in y direction
			{
				i = (int)(((float)j - intercept)/slope);//find columne 
				
				tempRoadCells[NumberofRouteCell].x = i;
				tempRoadCells[NumberofRouteCell].y = j;
				++NumberofRouteCell;
			}
		}
		else
		{
			for (j = StartingCell.y; j >= EndingCell.y; --j)//changing in y direction
			{
				i = (int)(((float)j - intercept)/slope);//find columne 
				
				tempRoadCells[NumberofRouteCell].x = i;
				tempRoadCells[NumberofRouteCell].y = j;
				++NumberofRouteCell;
			}
		}
	}
	else if (fabs(slope) > 10000.0)//when slope is infinite ->North and South direction
	{
		if (StartingCell.y < EndingCell.y)
		{
			i = EndingCell.x;
			for (j = StartingCell.y; j <= EndingCell.y; ++j)//changing in y direction
			{
				tempRoadCells[NumberofRouteCell].x = i;
				tempRoadCells[NumberofRouteCell].y = j;
				++NumberofRouteCell;
			}
		}
		else
		{
			i = EndingCell.x;
			for (j = StartingCell.y; j >= EndingCell.y; --j)//changing in y direction
			{
				tempRoadCells[NumberofRouteCell].x = i;
				tempRoadCells[NumberofRouteCell].y = j;
				++NumberofRouteCell;
			}
		}
	}
	else//select x-axis as an independent variable
	{
		if (StartingCell.x < EndingCell.x)
		{
			for (i = StartingCell.x; i <= EndingCell.x; ++i)//changing in x direction
			{
				j = (int)((float)i * slope + intercept);//find columne 
				tempRoadCells[NumberofRouteCell].x = i;
				tempRoadCells[NumberofRouteCell].y = j;
				++NumberofRouteCell;
			}
		}
		else
		{
			for (i = StartingCell.x; i >= EndingCell.x; --i)//changing in x direction
			{
				j = (int)((float)i * slope + intercept);//find columne 
				tempRoadCells[NumberofRouteCell].x = i;
				tempRoadCells[NumberofRouteCell].y = j;
				++NumberofRouteCell;
			}
		}

	}
	

	//input starting cell and ending cell
	//above algorithm might not input exact starting and ending cell because it uses slope and intercept then integerize.
	tempRoadCells[0].x = StartingCell.x;
	tempRoadCells[0].y = StartingCell.y;
	tempRoadCells[NumberofRouteCell-1].x = EndingCell.x;
	tempRoadCells[NumberofRouteCell-1].y = EndingCell.y;



	//input -1 to the rest of route cells up to the maximum number of cell (MaxLength)
	for (j = NumberofRouteCell; j < MaxRoadLength; ++j)
	{
		tempRoadCells[j].x = -1;
		tempRoadCells[j].y = -1;
	}
}




CPoint CCablanView::RandomlyFindingNextCellonRoute(CPoint CurrentCell, CPoint PreviousCell, CPoint TargetCell)
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int i;

	if (CurrentCell.x == -1 || CurrentCell.y == -1)
		return (-1,-1);


	/////////////////////////////////////////////////////////////////////////
	//Choose one of 3 neighbors --> IP angle would be less than 45 degree
	/////////////////////////////////////////////////////////////////////////

	CPoint NeighborCells[3];
	int Select = 0;
	//calculate direction between current and previous cell
	int IncrementX = CurrentCell.x - PreviousCell.x;
	int IncrementY = CurrentCell.y - PreviousCell.y;

	if (IncrementX != 0 && IncrementY == 0)//direction will be North or South ; [i][j]=i represents rows and j represents columns
	{
		NeighborCells[0].x = CurrentCell.x + IncrementX;
		NeighborCells[1].x = CurrentCell.x + IncrementX;
		NeighborCells[2].x = CurrentCell.x + IncrementX;
		NeighborCells[0].y = CurrentCell.y - 1;
		NeighborCells[1].y = CurrentCell.y;
		NeighborCells[2].y = CurrentCell.y + 1;
	}
	else if (IncrementX != 0 && IncrementY != 0)//direction will be NW, NE, SW, SE
	{
		NeighborCells[0].x = CurrentCell.x + IncrementX;
		NeighborCells[1].x = CurrentCell.x + IncrementX;
		NeighborCells[0].y = CurrentCell.y;
		NeighborCells[1].y = CurrentCell.y + IncrementY;
	
		NeighborCells[2].y = CurrentCell.y + IncrementY;
		NeighborCells[2].x = CurrentCell.x;
	}
	else if (IncrementX == 0 && IncrementY != 0)//direction will be East or West
	{
		NeighborCells[0].y = CurrentCell.y + IncrementY;
		NeighborCells[1].y = CurrentCell.y + IncrementY;
		NeighborCells[2].y = CurrentCell.y + IncrementY;
		NeighborCells[0].x = CurrentCell.x - 1;
		NeighborCells[1].x = CurrentCell.x;
		NeighborCells[2].x = CurrentCell.x + 1;
	}




	//evalutate neighbors -> 3 cells
	//calculate slope between two cells
	double slope[3];
	double distance, diffx, diffy, diffelev;
	int tempx, tempy;// just for informing NeghborCells
	int tempx1, tempy1;// just for informing NeghborCells
	


	for (i = 0; i < 3; ++i)
	{
		if (NeighborCells[i].x >= 1 && NeighborCells[i].y >= 1 && NeighborCells[i].x <= pDoc->DTMcolumns && NeighborCells[i].y <= pDoc->DTMrows)//less than min. or bigger than max.
		{
			tempx = NeighborCells[i].x;
			tempy = NeighborCells[i].y;
			tempx1 = CurrentCell.x;
			tempy1 = CurrentCell.y;

			diffx = pDoc->DTMCoord[CurrentCell.x][CurrentCell.y].x - pDoc->DTMCoord[NeighborCells[i].x][NeighborCells[i].y].x;
			diffy = pDoc->DTMCoord[CurrentCell.x][CurrentCell.y].y - pDoc->DTMCoord[NeighborCells[i].x][NeighborCells[i].y].y;

			distance = sqrt(diffx*diffx + diffy*diffy);
			diffelev = fabs(pDoc->DTMCoord[CurrentCell.x][CurrentCell.y].z - pDoc->DTMCoord[NeighborCells[i].x][NeighborCells[i].y].z);
		
			slope[i] = diffelev/distance;//slope is alwasy positive value
			if (slope[i] > 0.30)//maximum gradient available-->feasible 
				slope[i] = -1.0;
		}
		else
			slope[i] = -1.0;//should not be chosen
	}



	//error handler ---> 3 neighborhood cells
	int count = 0;//checking all neighbor cells have -1
	float tempdist[3];
	for (i = 0; i < 3; ++i)
	{
		//if cell is outside of the boundary regardless slope. Slope is negative if cell is beyond the boundary
		if (NeighborCells[i].x <= 0 || NeighborCells[i].y <= 0 || NeighborCells[i].x > pDoc->DTMcolumns || NeighborCells[i].y > pDoc->DTMrows || slope[i] < 0)//including 0 for release mode
		{
			NeighborCells[i].x = -1;
			NeighborCells[i].y = -1;
			count++;//if count is 3, that means all neighbors are infeasible cells
			tempdist[i] = (float)999999999999999.9;
		}
		else
		{
			tempdist[i] = (float)pow((double)(NeighborCells[i].x - TargetCell.x), 2.0) + (float)pow((double)(NeighborCells[i].y - TargetCell.y), 2.0);
			tempdist[i] *= (float)pow( fabs((double)slope[i]), .07);
		}
	}
	

	//verifying if Route is inside of boundary or not
	float mindist = (float)999999999999.9;
	if (count < 3)
	{
		/*
		//randomly select one of three candidates.
		do
		{
			Select = (rand() % 3);
		}
		while (NeighborCells[Select].x == -1 || NeighborCells[Select].y == -1 || slope[Select] < 0.0);
		*/

		//select the closest cell from the target.
		for (i = 0; i < 3; ++i)
		{
			if (mindist > tempdist[i])
			{
				mindist = tempdist[i];
				Select = i;
			}
		}

		return(NeighborCells[Select]);
	}
	else
		return(-1, -1);//all neighbors are outside boundary or exceed maximum gradient

}


/*
void CCablanView::SimulatedAnnealingProcessforTruckRoadRouting(int landingNo)
{
	//initialize the parameters
	double OBJValue = 0.0;
	double Current_Sol = 0.0;
	double Best_Sol = 0.0;
	double Temperature = 1000000.0;
	double MinTemperature = 10.0;
	double CoolingRate = 0.95;
	int Iteration = 200;
	double randomprobability = 0.0;
	double valuebytemperature = 0.0;
	int Count = 0;
	
	CPoint PreviousTruckRoadRoute[1500];//for keeping the previous one if a new one is not better one.

	//saving in file
//	FILE *outfile_ptr;
//	CString csOutFileName = "e:\\Temp\\SATruck.txt";

//	outfile_ptr = fopen(csOutFileName, "w");

	//set an initial solution
	HavingCurrentRoute = FALSE;//this works in RandomlyGenerateRoadRoute()
	RandomlyGenerateTruckRoadRoute(MaxTruckRoadLength);
	OBJValue = TruckRoadGenerateTransportationCostSpreading();//call sub-routine
	TruckRoadCost = TruckRoadCalculateRoadConstructionCost(TruckRoadUnitCost);//call sub-routine
	OBJValue += TruckRoadCost;
	Current_Sol = OBJValue;
	Best_Sol = OBJValue;
	memcpy(BestTruckRoadRoute, TruckRoadRoute, sizeof(TruckRoadRoute));
	memcpy(PreviousTruckRoadRoute, TruckRoadRoute, sizeof(TruckRoadRoute));
	HavingCurrentRoute = TRUE;//this works in RandomlyGenerateRoadRoute()	

	//writing initial population
//	fprintf(outfile_ptr, "ImFixed: %lf\n", ImprovedFixedCost);
//	fprintf(outfile_ptr, "ImVariable: %lf\n", ImprovedVariableCost);
//	fprintf(outfile_ptr, "TruckRoadCost: %lf\n", TruckRoadCost);
//	fprintf(outfile_ptr, "OBJValue: %lf\n", OBJValue);
//	fprintf(outfile_ptr, "Totallength: %lf\n\n", TotalTruckRoadLength);


	

	
	//Simulated Annealing Process
	do
	{
		do
		{
			OBJValue = 0.0;
			RandomlyGenerateTruckRoadRoute(MaxTruckRoadLength);
			OBJValue = TruckRoadGenerateTransportationCostSpreading();//call sub-routine
			TruckRoadCost = TruckRoadCalculateRoadConstructionCost(TruckRoadUnitCost);//call sub-routine
			OBJValue += TruckRoadCost;
//			fprintf(outfile_ptr, "ImFixed: %lf\n", ImprovedFixedCost);
//			fprintf(outfile_ptr, "ImVariable: %lf\n", ImprovedVariableCost);
//			fprintf(outfile_ptr, "TruckRoadCost: %lf\n", TruckRoadCost);
//			fprintf(outfile_ptr, "OBJValue: %lf\n", OBJValue);
//			fprintf(outfile_ptr, "Totallength: %lf\n\n", TotalTruckRoadLength);
	
			if (OBJValue < Current_Sol)//minimizing cost
			{
				Current_Sol = OBJValue;
				memcpy(PreviousTruckRoadRoute, TruckRoadRoute, sizeof(TruckRoadRoute));//change the current route
				//keep Globalbest 
				if (OBJValue < Best_Sol)
				{
					Best_Sol = OBJValue;
					memcpy(BestTruckRoadRoute, TruckRoadRoute, sizeof(TruckRoadRoute));
					//redrawing
					OnViewTerrain();
					DrawTruckRoadRoute();
				}
			}
			else if (OBJValue > Current_Sol)
			{
				randomprobability = (double) rand() / RAND_MAX;
				valuebytemperature = exp( (Current_Sol - OBJValue) / Temperature );
				if (randomprobability < valuebytemperature)
				{
					Current_Sol = OBJValue;
					memcpy(PreviousTruckRoadRoute, TruckRoadRoute, sizeof(TruckRoadRoute));
				}
				else
					memcpy(TruckRoadRoute, PreviousTruckRoadRoute, sizeof(TruckRoadRoute));//keep old one.
			}	
		
			Count++;
		}	
		while (Count <= Iteration);
	
		Temperature = Temperature * CoolingRate;
		Count = 0;
	}
	while (Temperature > MinTemperature);

	//drawing the result
	memcpy(TruckRoadRoute, BestTruckRoadRoute, sizeof(TruckRoadRoute));
	DrawTruckRoadRoute();
//	fclose(outfile_ptr);
}

*/




void CCablanView::DrawRoadRoute(int landingNo)
{
/*
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	//drawing
	CPoint DrawFrom, DrawTo;
	int i = 0;//start from 0 which is landing location.
	int col, row;

	while (pDoc->AccessRoadtoLanding[landingNo][i].x >= 0 && pDoc->AccessRoadtoLanding[landingNo][i].x <= pDoc->DTMcolumns && pDoc->AccessRoadtoLanding[landingNo][i].y >= 0 && pDoc->AccessRoadtoLanding[landingNo][i].y <= pDoc->DTMrows)
	{
		col = pDoc->AccessRoadtoLanding[landingNo][i].x;
		row = pDoc->AccessRoadtoLanding[landingNo][i].y;

		DrawFrom.x = (long)(pDoc->DTMCoord[col][row].x - pDoc->DTMminX - (pDoc->DTMcellsize/2.0) + cpMargin.x);// (CellHeight/2) integer problem??
		DrawFrom.y = (long)(pDoc->DTMCoord[col][row].y - pDoc->DTMmaxY - (pDoc->DTMcellsize/2.0) - cpMargin.y);
		DrawTo.x = (long)(pDoc->DTMCoord[col][row].x - pDoc->DTMminX + (pDoc->DTMcellsize/2.0) + cpMargin.x);
		DrawTo.y = (long)(pDoc->DTMCoord[col][row].y - pDoc->DTMmaxY + (pDoc->DTMcellsize/2.0) - cpMargin.y);
			
		// drawing
		CRect rcSquare(DrawFrom.x, DrawFrom.y, DrawTo.x, DrawTo.y);
			
		pDC->FillSolidRect(rcSquare, RGB(0,0,0));

		if (i == MaxRoadLength)//MaxRoadLength is defined in View "GenerateRoadRoutefromExistingRoadtoLandings()"
			break;

		++i;
	}


	//drawing existing road
	col = ExistingRoad.x;
	row = ExistingRoad.y;

	DrawFrom.x = (long)(pDoc->DTMCoord[col][row].x - pDoc->DTMminX - (pDoc->DTMcellsize/2.0) + cpMargin.x);// (CellHeight/2) integer problem??
	DrawFrom.y = (long)(pDoc->DTMCoord[col][row].y - pDoc->DTMmaxY - (pDoc->DTMcellsize/2.0) - cpMargin.y);
	DrawTo.x = (long)(pDoc->DTMCoord[col][row].x - pDoc->DTMminX + (pDoc->DTMcellsize/2.0) + cpMargin.x);
	DrawTo.y = (long)(pDoc->DTMCoord[col][row].y - pDoc->DTMmaxY + (pDoc->DTMcellsize/2.0) - cpMargin.y);
			
	// drawing
	CRect rcSquare(DrawFrom.x, DrawFrom.y, DrawTo.x, DrawTo.y);
			
	pDC->FillSolidRect(rcSquare, RGB(255,0,0));


	//drawing user dragging vertices
	for (i = 1; i <= NumberofSelectedRoadCellsbyUser; ++i)//SelectedRoadCellsbyUser[] starts from 1.
	{
		col = SelectedRoadCellsbyUser[i].x;
		row = SelectedRoadCellsbyUser[i].y;

		DrawFrom.x = (long)(pDoc->DTMCoord[col][row].x - pDoc->DTMminX - (pDoc->DTMcellsize/2.0) + cpMargin.x);// (CellHeight/2) integer problem??
		DrawFrom.y = (long)(pDoc->DTMCoord[col][row].y - pDoc->DTMmaxY - (pDoc->DTMcellsize/2.0) - cpMargin.y);
		DrawTo.x = (long)(pDoc->DTMCoord[col][row].x - pDoc->DTMminX + (pDoc->DTMcellsize/2.0) + cpMargin.x);
		DrawTo.y = (long)(pDoc->DTMCoord[col][row].y - pDoc->DTMmaxY + (pDoc->DTMcellsize/2.0) - cpMargin.y);
		CRect rcSquare(DrawFrom.x, DrawFrom.y, DrawTo.x, DrawTo.y);
		pDC->FillSolidRect(rcSquare, RGB(0,255,0));
	}
*/

}




////////////////////////////////////////////////////////////////////////////////////

void CCablanView::OnNetworkRoadsDrawroads() 
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int i;
	//clean up the screen and draw DTM
	DrawDTM();

	for (i = 1; i <= pDoc->TotalNumberofLandings; ++i)
		DrawRoadRoute(i);
	
}

void CCablanView::OnNetworkRoadsGenerate() 
{
/*
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	blRoadRoutingbyUser = TRUE;
	int i, j;
	CPoint StartingCell, EndingCell;

	for (i = 1; i <= pDoc->TotalNumberofLandings; ++i)
	{
		SelectedRoadCellsbyUser[1].x = pDoc->Landings[i].column;
		SelectedRoadCellsbyUser[1].y = pDoc->Landings[i].row;

		gbLandingNumber = i;
		StartingCell.x = pDoc->Landings[i].column;
		StartingCell.y = pDoc->Landings[i].row;
		EndingCell.x = ExistingRoad.x;
		EndingCell.y = ExistingRoad.y;

		pDoc->AccessRoadtoLanding[i][0].x = StartingCell.x;
		pDoc->AccessRoadtoLanding[i][0].y = StartingCell.y;

		GenerateRoadRoutefromExistingRoadtoLandings(i, StartingCell, EndingCell);
		
		j = 1;
		while (tempRoadCells[j].x > 0 && tempRoadCells[j].y > 0)
		{
			pDoc->AccessRoadtoLanding[i][j].x = tempRoadCells[j].x;
			pDoc->AccessRoadtoLanding[i][j].y = tempRoadCells[j].y;	
			++j;
		}
		
		//delete after the route meets ExistingRoad location.
		if (pDoc->AccessRoadtoLanding[i][j-1].x == ExistingRoad.x && pDoc->AccessRoadtoLanding[i][j-1].y == ExistingRoad.y)
		{
			while (pDoc->AccessRoadtoLanding[i][j].x > 0 || pDoc->AccessRoadtoLanding[i][j].y > 0)//1) 'or' instead of 'and' like above. 2) 'j' instead of 'j-1' like above.
			{
				pDoc->AccessRoadtoLanding[i][j].x = -1;
				pDoc->AccessRoadtoLanding[i][j].y = -1;
				++j;
			}
		}

		NumberofSelectedRoadCellsbyUser = 2;//initialize variable in order to disable all SelectedRoadCellsbyUser[];
		//clean up the screen and draw DTM
		DrawDTM();
		DrawRoadRoute(i);
	}	
*/
}

/////////////////////////////////////////////////////////////////////////////////











///////////////////////////////////////////////////////////////////////////////////////
//through Logging system to Landings
///////////////////////////////////////////////////////////////////////////////////////

void CCablanView::GenerateCableSystemNetwork(float RoadVariableCostsperLanding[], float RoadFixedCostsperLanding[], int Iteration)
{

	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	int i, j;
	int k, l;
	int nolinks;//counting total number of links
	int cablesystemID;
	double x1, y1, z1, x, y, z, dist, tempdist;//dist is from each cell to the cable corridor.
	double tempx, tempy, tempz;
	int temprow, tempcol;
	double distfromheadspar;//distance from headspar to the carriage location for lateral yarding.//not used currently
	double distfromheadtotail;//distance from headspar to the tail spar//not used currently
	double m, b;//slope and intercept
	CString tempString1, tempString2;
	float variable, fixed;
	float hookingtime;//hooking and unhooking
	float lateralyardingtime;
	float yardingtime;
	float totalyardingtime;//will include lateral yarding time and average yarding tim (Cycletime).
	int actuallandingID;
	bool feasibility;
	int minX, maxX, minY, maxY, tempint, landingX, landingY, landingZ, tailX, tailY, tailZ;
	double tempshortestdistfromanycables;//for the cells not covered by any cableways.
	int tempCable, tempCablesystem, tempLanding;//for uncovered cell--keeping cable id and landing id having the shortest distance from the cell.
	bool LinktoSystem[MAXCABLEWAYS][MAXCABLESYSTEMS][MAXLANDINGS];
	bool LinktoLanding[MAXCABLESYSTEMS][MAXLANDINGS];//in order to add only one link from each cableway.
	float RiggingCosts;//sec. per intermediate support and tailspar

	int NumberofTripsNeeded;//how many trips of carriage is required for yarding one grid cell

	short CarriagePosition;//ID number along the cells under the corridor-position for a volume cell.
	short tempCarriagePosition;
	bool boolExceedMaxLateral;//if lateral yarding distance exceeds the maximum number.

	float Penalty;//penalty for over lateryarding, beyond external yarding distance... Penalty times cycletime.
	float MultiplierLYD;//non-linear equation for estimating LYD time.


	//checking fish stream along the lateral yarding road//Sep.12,2002
	int lateralsmalli, lateralsmallj;
	int lateralbigi, lateralbigj;
	int laterali, lateralj;
	double lateralslope, lateralintercept;
	double lateralcellx, lateralcelly;//coordinates of intercepting point to the lateral yarding road
	double lateralcellx1, lateralcelly1;//coordinates of a grid cell.
	double lateraldist;



	//counting variables for the application examples in the thesis.

	//temporary routine for just testing...
	int totalFeasibleCableroads = 0;
	int sumofcableroadlength = 0;
	float averageofcableroadlength;
	int cableroads5to10 = 0;
	int cableroads11to20 = 0;
	int cableroads21to30 = 0;
	int cableroads31to40 = 0;
	int cableroads41to50 = 0;
	int cableroads51to60 = 0;
	
	int intermediatesupport1 = 0;
	int intermediatesupport2 = 0;
	int intermediatesupport3 = 0;


	for (k = 1; k <= pDoc->TotalNumberofLandings; ++k)
	{
		for (l = 0; l < MAXCABLEWAYS-1; ++l)//from 0 to 35.
		{
			//??????????????????????????????????????????????
			if (pDoc->Cableways[k][l].taillocationID > MINCELLALONGCABLEROAD)//if the cable road is shorter than 5*grid cell size, then this cable corridor is not feasible.
			{
				++totalFeasibleCableroads;
				sumofcableroadlength += pDoc->Cableways[k][l].taillocationID;
				if (pDoc->Cableways[k][l].taillocationID < 11)
					++cableroads5to10;
				else if (pDoc->Cableways[k][l].taillocationID < 21)
					++cableroads11to20;
				else if (pDoc->Cableways[k][l].taillocationID < 31)
					++cableroads21to30;
				else if (pDoc->Cableways[k][l].taillocationID < 41)
					++cableroads31to40;
				else if (pDoc->Cableways[k][l].taillocationID < 51)
					++cableroads41to50;
				else 
					++cableroads51to60;
				if (pDoc->Cableways[k][l].totalnumberofINTs == 1)
					++intermediatesupport1;
				else if (pDoc->Cableways[k][l].totalnumberofINTs == 2)
					++intermediatesupport2;
				else if (pDoc->Cableways[k][l].totalnumberofINTs == 3)
					++intermediatesupport3;
			}			
		}
	}

	averageofcableroadlength = (float)sumofcableroadlength / (float)totalFeasibleCableroads;

	//end of temporary routine for just testing



	
if (Iteration <= 1)//generate these links only for the first iteration.
{

	//saving in file
	FILE *outfile_ptr;
	CString csOutFileName = "e:\\thesis\\output\\cablematrix.txt";

	outfile_ptr = fopen(csOutFileName, "w");


	//initializing booleans for Cell. If grid cell is covered by any of cableways, then it gets TRUE.
	memset(pDoc->bCoveredCell, 0, sizeof(pDoc->bCoveredCell));//FALSE
	memset(LinktoSystem, 0, sizeof(LinktoSystem));//FALSE
	memset(LinktoLanding, 0, sizeof(LinktoLanding));//FALSE


	pDoc->TotalNumberofLinks = 0;
	nolinks = 0;



	////////////////////////////////////////////////////////////////////////////////
	//Seperate

	for (j = pDoc->UnitBoundary.startRow; j <= pDoc->UnitBoundary.endRow; ++j)
	{
		for (i = pDoc->UnitBoundary.startCol; i <= pDoc->UnitBoundary.endCol; ++i)
		{
			if (pDoc->CellsWithinUnitBoundary[i][j] == TRUE)
			{
				//initialize variable
				tempshortestdistfromanycables = 1000000000000.0;//large value for keeping minimum value.
				boolExceedMaxLateral = FALSE;
	
				if (pDoc->Volume[i][j] > 0)//timber to be cut is existing
				{
					for (k = 1; k <= pDoc->TotalNumberofLandings; ++k)
					{
						for (l = 0; l < MAXCABLEWAYS-1; ++l)//from 0 to 35.
						{
							//??????????????????????????????????????????????
							if (pDoc->Cableways[k][l].taillocationID > MINCELLALONGCABLEROAD)//if the cable road is shorter than 5*grid cell size, then this cable corridor is not feasible.
							{
								//measuring distance from the cell to each cableway
								//(x1,y1); log pickup point
								//(x,y); carriage location along the cable road
								x1 = pDoc->DTMCoord[i][j].x;//cell coordinate
								y1 = pDoc->DTMCoord[i][j].y;
								z1 = pDoc->DTMCoord[i][j].z;
								m = pDoc->Cableways[k][l].slope;//skyline corridor linear equation
								b = pDoc->Cableways[k][l].intercept;
								//find coordinates of the intercepting point (x, y).
								if (fabs(m) < 0.0001)//when l = 9 and 27--look at GenerateCablewaysperLanding()
								{
									x = x1;
									y = b;
								}
								else if (m > 100000.0)//when l = 1 and 18
								{
									y = y1;
									x = pDoc->DTMCoord[pDoc->Landings[k].column][pDoc->Landings[k].row].x;
								}
								else
								{
									x = (y1 + 1.0/m*x1 - b)*m/(pow(m,2.0)+1);//intercepting point by the perpendicular line from the cell.
									y = m*x + b;
								}

								//find cell address in row and col.
								tempcol = FindCellAddressbasedonCoordinates(x, y).x;
								temprow = FindCellAddressbasedonCoordinates(x, y).y;
								z = pDoc->DTMCoord[tempcol][temprow].z;//elevation of this intercepting point.

								//horizontal distance
								dist = sqrt(pow(x1-x, 2.0) + pow(y1-y, 2.0));
								//slope distance
								dist = sqrt(pow(dist, 2.0) + pow(z1-z, 2.0));

								//horizontal distance-this distance is just for finding out CarriagePosition on the corridor.
								distfromheadspar = sqrt(pow(x-pDoc->DTMCoord[pDoc->Landings[k].column][pDoc->Landings[k].row].x, 2.0) + pow(y-pDoc->DTMCoord[pDoc->Landings[k].column][pDoc->Landings[k].row].y, 2.0));
	
	
								cablesystemID = pDoc->Landings[k].CableSystem;

								//checking whether x and y fall within the cable length.
								feasibility = TRUE;
								maxX = pDoc->DTMCoord[pDoc->Cableways[k][l].tailcolumn][pDoc->Cableways[k][l].tailrow].x;
								minX = pDoc->DTMCoord[pDoc->Landings[k].column][pDoc->Landings[k].row].x;
							
								if (minX > maxX)
								{
									tempint = maxX;
									maxX = minX;
									minX = tempint;
								}
								maxY = pDoc->DTMCoord[pDoc->Cableways[k][l].tailcolumn][pDoc->Cableways[k][l].tailrow].y;
								minY = pDoc->DTMCoord[pDoc->Landings[k].column][pDoc->Landings[k].row].y;
								if (minY > maxY)
								{
									tempint = maxY;
									maxY = minY;
									minY = tempint;
								}
						
								if (x > maxX || x < minX)
									feasibility = FALSE;
		
								if (y > maxY || y < minY)
									feasibility = FALSE;
	
								if (dist > pDoc->CableSystems[cablesystemID].maxlateraldist)//maximum lateral yarding distance.
								{
									feasibility = FALSE;
									boolExceedMaxLateral = TRUE;
								}


								//calculate distance from head to tail.-horizontal distance. This is only for finding CarriagePosition.
								distfromheadtotail = sqrt(pow(maxX-minX, 2.0) + pow(maxY-minY, 2.0));


								//checking payload whether this volume can be carried up to landing or not.
								//finding where the carriage location would be along the skyline corridor - proportional location 
								CarriagePosition = (short)((double)pDoc->Cableways[k][l].taillocationID*distfromheadspar/distfromheadtotail);
								if (CarriagePosition >= pDoc->Cableways[k][l].taillocationID)
									CarriagePosition = pDoc->Cableways[k][l].taillocationID - 1;
								else if (CarriagePosition < MINCELLALONGCABLEROAD)
									CarriagePosition = 1;//directly to Landing
								
								
								//keeps shortest distance from the current cable way to the current cell for not covered cells (below).
								if (feasibility == FALSE)
								{
									landingX = pDoc->DTMCoord[pDoc->Landings[k].column][pDoc->Landings[k].row].x;
									landingY = pDoc->DTMCoord[pDoc->Landings[k].column][pDoc->Landings[k].row].y;
									landingZ = pDoc->DTMCoord[pDoc->Landings[k].column][pDoc->Landings[k].row].z;
									tailX = pDoc->DTMCoord[pDoc->Cableways[k][l].tailcolumn][pDoc->Cableways[k][l].tailrow].x;
									tailY = pDoc->DTMCoord[pDoc->Cableways[k][l].tailcolumn][pDoc->Cableways[k][l].tailrow].y;
									tailZ = pDoc->DTMCoord[pDoc->Cableways[k][l].tailcolumn][pDoc->Cableways[k][l].tailrow].z;
									
									tempshortestdistfromanycables = 1000000000000.0;//large value for keeping minimum value.
							
									//distance from landing
									//horizontal
									tempdist = sqrt(pow(x1-landingX, 2.0) + pow(y1-landingY, 2.0));
									//slope distance
									tempdist = sqrt(pow(tempdist, 2.0) + pow(z1-landingZ, 2.0));

									if (tempshortestdistfromanycables > tempdist)
									{
										tempshortestdistfromanycables = tempdist;
										tempCable = l;
										tempCablesystem = cablesystemID;
										tempLanding = k;
										tempCarriagePosition = 1;//directly to Landing
										tempx = landingX;
										tempy = landingY;
										tempz = landingZ;
									}
							
									//distance from tail
									//horizontal
									tempdist = sqrt(pow(x1-tailX, 2.0) + pow(y1-tailY, 2.0));
									//slope distance
									tempdist = sqrt(pow(tempdist, 2.0) + pow(z1-tailZ, 2.0));

									if (tempshortestdistfromanycables > tempdist)
									{
										tempshortestdistfromanycables = tempdist;
										tempCable = l;
										tempCablesystem = cablesystemID;
										tempLanding = k;
										tempCarriagePosition = pDoc->Cableways[k][l].taillocationID - 1;
										tempx = tailX;
										tempy = tailY;
										tempz = tailZ;
									}

									if (x <= maxX && x >= minX && y <= maxY && y >= minY)//current cell is located between landing and tail spar but further than lateral yarding distance.
									{
										if (tempshortestdistfromanycables > dist)//dist is the distance from the cable way to the current cell with right angle.
										{
											tempshortestdistfromanycables = dist;
											tempCable = l;
											tempCablesystem = cablesystemID;
											tempLanding = k;
											tempCarriagePosition = CarriagePosition;
											tempx = x;
											tempy = y;
											tempz = z;
										}
									}
								}



								//costs
								//1. calculate the number of carriage cycle required for yarding a grid cell using Payload(maximum load for this cable road) and volume amount in the cell.
								//2. calculate one carriage cycle costs including lateral yarding costs
								//2. fixed cost will be one carriage cycle costs multiplied by the number of trips.
								if (feasibility == TRUE)//the current cell is in within the cable length and lateral yarding distance.
									Penalty = 1.0;//no penalty
								else
								{
									Penalty = pDoc->CableSystems[cablesystemID].gbPenaltyForBeyondLateralYarding;
									dist = tempshortestdistfromanycables;//shortest distance among perpendicular distance, distances from tail and head.
									CarriagePosition = tempCarriagePosition;
									x = tempx;
									y = tempy;
									z = tempz;
									if (dist > pDoc->CableSystems[cablesystemID].gbGiveUpCellDistance)//farther than this distance..., then give up this cell.
										Penalty = -1.;
								}

//**************************************************************
								//Sep.12,2002
								//slope and intercept of the line between log pickup point and carriage position.
								if (Penalty > 0.0)
								{
									if ((x-x1) != 0.)
										lateralslope = (y-y1)/(x-x1);
									else
										lateralslope = 100000000000000.;
	
									lateralintercept = y - lateralslope * x;

	
									lateralsmalli = i;
									lateralbigi = FindCellAddressbasedonCoordinates(x, y).x;
									if (lateralsmalli > lateralbigi)
									{
										lateralsmalli = lateralbigi;
										lateralbigi = i;
									}

									lateralsmallj = j;
									lateralbigj = FindCellAddressbasedonCoordinates(x, y).y;
									if (lateralsmallj > lateralbigj)
									{
										lateralsmallj = lateralbigj;
										lateralbigj = j;
									}

									//check all grid cells within the rectangle of (x,y, x1, y1)
									for (laterali = lateralsmalli; laterali <= lateralbigi; ++laterali)
									{
										for (lateralj = lateralsmallj; lateralj <= lateralbigj; ++lateralj)
										{
											if (pDoc->FishStreamBuffer[laterali][lateralj] > 0)
											{
												//calculate the distance from the grid cell to the lateral yarding road.
												//find coordinates of the intercepting point (lateralcellx, lateralcelly).
												lateralcellx1 = pDoc->DTMCoord[laterali][lateralj].x;
												lateralcelly1 = pDoc->DTMCoord[laterali][lateralj].y;

												if (fabs(lateralslope) < 0.0001)
												{
													lateralcellx = lateralcellx1;
													lateralcelly = y;
												}
												else if (lateralslope > 100000.0)
												{
													lateralcelly = lateralcelly1;
													lateralcellx = x;
												}
												else
												{
													lateralcellx = (lateralcelly1 + 1.0/lateralslope*lateralcellx1 - lateralintercept)*lateralslope/(pow(lateralslope,2.0)+1);//intercepting point by the perpendicular line from the cell.
													lateralcelly = lateralslope*lateralcellx + lateralintercept;
												}
	
												//horizontal distance
												lateraldist = sqrt(pow(lateralcellx1-lateralcellx, 2.0) + pow(lateralcelly1-lateralcelly, 2.0));
												
												if (lateraldist < pDoc->DTMcellsize)
												{
													Penalty  = -1.0;
													//break
													laterali = lateralbigi;
													lateralj = lateralbigj;
												}
											}
										}
									}
								}//end of Penaly > 0.0
///*************************************************************

								if (Penalty > 0.0)
								{
									//setup network
									//writing to file; cell - system - landing
									//link from the cell to system
									tempString1.Format("C%dR%d", i, j);//column and row
									tempString2.Format("C%dS%dL%d", l, cablesystemID, k);//cableway, system, landing

									//check which one is smaller between UserDefined maximum payload and max payload from payloadanalysis.
									if (pDoc->PayloadAtEachTP[k][l][CarriagePosition] < pDoc->CableSystems[cablesystemID].gbUserMaxPayload)
										NumberofTripsNeeded = (int)(pDoc->Volume[i][j]*pDoc->CableSystems[cablesystemID].gbDensity)  / (int)(pDoc->PayloadAtEachTP[k][l][CarriagePosition]);//how many carrige trips are required to yard this cell; unit: kg.
									else
										NumberofTripsNeeded = (int)(pDoc->Volume[i][j]*pDoc->CableSystems[cablesystemID].gbDensity)  / (int)(pDoc->CableSystems[cablesystemID].gbUserMaxPayload);
								
									//add 1 to NumberofTripsNeeded.
									++NumberofTripsNeeded;

									//estimating yarding time for this volume cell
									hookingtime = pDoc->Volume[i][j] * (float)(pDoc->CableSystems[cablesystemID].gbHookTime + pDoc->CableSystems[cablesystemID].gbUnhookTime);//m3 * sec/m3 = sec
									//applying non-linear time equation for LYD
									MultiplierLYD = 1.0;
									if (dist < 0.333 * pDoc->CableSystems[cablesystemID].maxlateraldist)
										MultiplierLYD = pDoc->CableSystems[cablesystemID].gbMultiplierLYD1;
									else if (dist >= 0.333 * pDoc->CableSystems[cablesystemID].maxlateraldist && dist < 0.667 * pDoc->CableSystems[cablesystemID].maxlateraldist)
										MultiplierLYD = pDoc->CableSystems[cablesystemID].gbMultiplierLYD2;
									else 
										MultiplierLYD = pDoc->CableSystems[cablesystemID].gbMultiplierLYD3;
									
									//applying Penalty
									lateralyardingtime = (float)(Penalty * dist) / pDoc->CableSystems[cablesystemID].gbLateralYardingSpeed;//include later yarding time
									lateralyardingtime *= MultiplierLYD;
									lateralyardingtime *= (float)NumberofTripsNeeded;
									yardingtime = pDoc->CycleTimeAtEachTP[k][l][CarriagePosition];//cycle time does not include hooking and unhooking time. -- See CalculateUnitCosts()
									yardingtime *= (float)NumberofTripsNeeded;
								
									totalyardingtime = hookingtime + lateralyardingtime + yardingtime;
									//taking delay time into account - July 4, 2002
									totalyardingtime += totalyardingtime * (float)(pDoc->CableSystems[cablesystemID].gbDelayPercentage/(float)100.0);
								
									//variable cost will be the unit which is per unit volume (m3).
									variable = (float) (totalyardingtime / 3600. * pDoc->CableSystems[cablesystemID].gbUnitCost ) / (float)(pDoc->Volume[i][j]);//totalyardingtime(sec)/3600 * System Unitcosts($/hr);
									fixed = 0.;

									fprintf(outfile_ptr, "%s	%s	%10.2f	%10.2f\n", tempString1, tempString2, variable, fixed);//yarding cost. currently using distance. no fixed costs.
									++nolinks;
									pDoc->ConvertfromCStringToChar(pDoc->Links[nolinks].FromNode, tempString1);
									pDoc->ConvertfromCStringToChar(pDoc->Links[nolinks].ToNode, tempString2);
									pDoc->Links[nolinks].VariableCost = variable;
									pDoc->Links[nolinks].FixedCost = fixed;

									//link from the grid cell to system (skyline reinstalling costs occurs as a fixed cost + INT supports rigging costs)
									//fixed costs per corridor include reinstalling costs and rigging intermediate supports costs.
									if (LinktoSystem[l][cablesystemID][k] == FALSE)
									{
										LinktoSystem[l][cablesystemID][k] = TRUE;
										tempString1 = tempString2;//cableway, system, landing
										tempString2.Format("S%dL%d", cablesystemID, k);//landings
										variable = 0.;
										fixed = pDoc->CableSystems[cablesystemID].gbCableRoadChangeHour * pDoc->CableSystems[cablesystemID].gbUnitCost;//skyline corridor reinstalling costs

										//counting number of intermediate spars
										if (pDoc->Cableways[k][l].totalnumberofINTs > 0)
										{
											RiggingCosts = pDoc->CableSystems[cablesystemID].gbRiggingCosts * pDoc->Cableways[k][l].totalnumberofINTs;//not including tail spar... assuming tail spar setting costs are included in cable reinstalling costs.
											fixed += RiggingCosts;
										}
	
										fprintf(outfile_ptr, "%s	%s	%10.2f	%10.2f\n", tempString1, tempString2, variable, fixed);//landing + system costs. currently 0. no variable costs.
										++nolinks;
										pDoc->ConvertfromCStringToChar(pDoc->Links[nolinks].FromNode, tempString1);
										pDoc->ConvertfromCStringToChar(pDoc->Links[nolinks].ToNode, tempString2);
										pDoc->Links[nolinks].VariableCost = variable;
										pDoc->Links[nolinks].FixedCost = fixed;

	
										//link from the system to landing (tower location) -- system move-in costs occur.
										if (LinktoLanding[cablesystemID][k] == FALSE)
										{
											LinktoLanding[cablesystemID][k] = TRUE;
											tempString1 = tempString2;//system, landing
	
											if (pDoc->LandingSharingTowerID[k] <= 0)
												actuallandingID = k;
											else
											{
												actuallandingID = pDoc->LandingSharingTowerID[k];
												//in case that this actuallandingID also shares with other landing.
												while (pDoc->LandingSharingTowerID[actuallandingID] > 0)
													actuallandingID = pDoc->LandingSharingTowerID[actuallandingID];
											}
	
											tempString2.Format("L%d", actuallandingID);//landings
											variable = 0.;
											fixed = pDoc->CableSystems[cablesystemID].gbMoveInCosts;//system move-in costs
											fixed += pDoc->CableSystems[cablesystemID].gbYarderSetupHour * pDoc->CableSystems[cablesystemID].gbUnitCost;//add intial yarder setup costs
											fprintf(outfile_ptr, "%s	%s	%10.2f	%10.2f\n", tempString1, tempString2, variable, fixed);//landing + system costs. currently 0. no variable costs.
											++nolinks;
											pDoc->ConvertfromCStringToChar(pDoc->Links[nolinks].FromNode, tempString1);
											pDoc->ConvertfromCStringToChar(pDoc->Links[nolinks].ToNode, tempString2);
											pDoc->Links[nolinks].VariableCost = variable;
											pDoc->Links[nolinks].FixedCost = fixed;
										}
									}
									pDoc->bCoveredCell[i][j] = TRUE;//put TRUE in the global variable indicating this cell is covered by any of cableways.
								}//if Penalty > 0
							}//end of if (pDoc->Cableways[k][l].taillocationID > 5)
						}//end of l
					}//end of k



					if (pDoc->bCoveredCell[i][j] != TRUE)//never had a link for this cell---give up and connect directly to the destination.
					{
						//give-up cells--put -1 in the volume
						pDoc->Volume[i][j] = -1.;
					}
				}//end of if (Volume > 0)
			}//end of if (Cellwithin == TRUE)	
		}//end of i
	}//end of j
		


	//store the results in the memory--this link data do not change.
	pDoc->CableTotalNumberofLinks = nolinks;
	memcpy(pDoc->CableLinks, pDoc->Links, sizeof(pDoc->Links));
	
	
	///////////////////////////////////////////////////
	//links from each landing to dummy destination -- input landing costs for actual landing locations
	///////////////////////////////////////////////////
	for (k = 1; k <= pDoc->TotalNumberofLandings; ++k)
	{
		if (pDoc->LandingSharingTowerID[k] <= 0)//when this is an actual landing.
		{
			tempString1.Format("L%d", k);//landing ID matches with k -- see GenerateRoadSaleData()
			tempString2.Format("DummyDest1");
			variable = RoadVariableCostsperLanding[k];
			fixed = RoadFixedCostsperLanding[k] + pDoc->LandingCosts[k];

			fprintf(outfile_ptr, "%s	%s	%10.2f	%10.2f\n", tempString1, tempString2, variable, fixed);
			++nolinks;
			pDoc->ConvertfromCStringToChar(pDoc->Links[nolinks].FromNode, tempString1);
			pDoc->ConvertfromCStringToChar(pDoc->Links[nolinks].ToNode, tempString2);
			pDoc->Links[nolinks].VariableCost = variable;
			pDoc->Links[nolinks].FixedCost = fixed;	
		}
	}


	pDoc->TotalNumberofLinks = nolinks;
	fclose(outfile_ptr);
}
else
{
	//restore back to Links for running network algorithm
	nolinks = pDoc->CableTotalNumberofLinks;
	memcpy(pDoc->Links, pDoc->CableLinks, sizeof(pDoc->Links));
	

	///////////////////////////////////////////////////
	//links from each landing to dummy destination -- input landing costs for actual landing locations
	///////////////////////////////////////////////////
	for (k = 1; k <= pDoc->TotalNumberofLandings; ++k)
	{
		if (pDoc->LandingSharingTowerID[k] <= 0)//when this is an actual landing.
		{
			tempString1.Format("L%d", k);//landing ID matches with k -- see GenerateRoadSaleData()
			tempString2.Format("DummyDest1");
			variable = RoadVariableCostsperLanding[k];
			//Sep.15,2002
			if (Iteration > 10)//max repetitions????????????????
				Iteration = 1;//to calculate actual costs

			fixed = RoadFixedCostsperLanding[k] + pDoc->LandingCosts[k] * (float)Iteration;//Sep.15,2002

//			fprintf(outfile_ptr, "%s	%s	%10.2f	%10.2f\n", tempString1, tempString2, variable, fixed);
			++nolinks;
			pDoc->ConvertfromCStringToChar(pDoc->Links[nolinks].FromNode, tempString1);
			pDoc->ConvertfromCStringToChar(pDoc->Links[nolinks].ToNode, tempString2);
			pDoc->Links[nolinks].VariableCost = variable;
			pDoc->Links[nolinks].FixedCost = fixed;	
		}
	}

	pDoc->TotalNumberofLinks = nolinks;
}

}


void CCablanView::GenerateCableSystemNetworkAllAtOnce()
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	int i, j;
	int k, l;
	int nolinks;//counting total number of links
	int cablesystemID;
	double x1, y1, z1, x, y, z, dist, tempdist;//dist is from each cell to the cable corridor.
	int temprow, tempcol;
	double distfromheadspar;//distance from headspar to the carriage location for lateral yarding.//not used currently
	double distfromheadtotail;//distance from headspar to the tail spar//not used currently
	double m, b;//slope and intercept
	CString tempString1, tempString2;
	float variable, fixed;
	float hookingtime;//hooking and unhooking
	float lateralyardingtime;
	float yardingtime;
	float totalyardingtime;//will include lateral yarding time and average yarding tim (Cycletime).
	int actuallandingID;
	bool feasibility;
	int minX, maxX, minY, maxY, tempint, landingX, landingY, landingZ, tailX, tailY, tailZ;
	double tempshortestdistfromanycables;//for the cells not covered by any cableways.
	int tempCable, tempCablesystem, tempLanding;//for uncovered cell--keeping cable id and landing id having the shortest distance from the cell.
	bool LinktoSystem[MAXCABLEWAYS][MAXCABLESYSTEMS][MAXLANDINGS];
	bool LinktoLanding[MAXCABLESYSTEMS][MAXLANDINGS];//in order to add only one link from each cableway.
	float RiggingCosts;//sec. per intermediate support and tailspar

	int NumberofTripsNeeded;//how many trips of carriage is required for yarding one grid cell

	short CarriagePosition;//ID number along the cells under the corridor-position for a volume cell.
	short tempCarriagePosition;
	bool boolExceedMaxLateral;//if lateral yarding distance exceeds the maximum number.

	float Penalty;//penalty for over lateryarding, beyond external yarding distance... Penalty times cycletime.
	float MultiplierLYD;//non-linear equation for estimating LYD time.



	//counting variables for the application examples in the thesis.

	//temporary routine for just testing...
	int totalFeasibleCableroads = 0;
	int sumofcableroadlength = 0;
	float averageofcableroadlength;
	int cableroads5to10 = 0;
	int cableroads11to20 = 0;
	int cableroads21to30 = 0;
	int cableroads31to40 = 0;
	int cableroads41to50 = 0;
	int cableroads51to60 = 0;
	
	int intermediatesupport1 = 0;
	int intermediatesupport2 = 0;
	int intermediatesupport3 = 0;


	for (k = 1; k <= pDoc->TotalNumberofLandings; ++k)
	{
		for (l = 0; l < MAXCABLEWAYS-1; ++l)//from 0 to 35.
		{
			//??????????????????????????????????????????????
			if (pDoc->Cableways[k][l].taillocationID > MINCELLALONGCABLEROAD)//if the cable road is shorter than 5*grid cell size, then this cable corridor is not feasible.
			{
				++totalFeasibleCableroads;
				sumofcableroadlength += pDoc->Cableways[k][l].taillocationID;
				if (pDoc->Cableways[k][l].taillocationID < 11)
					++cableroads5to10;
				else if (pDoc->Cableways[k][l].taillocationID < 21)
					++cableroads11to20;
				else if (pDoc->Cableways[k][l].taillocationID < 31)
					++cableroads21to30;
				else if (pDoc->Cableways[k][l].taillocationID < 41)
					++cableroads31to40;
				else if (pDoc->Cableways[k][l].taillocationID < 51)
					++cableroads41to50;
				else 
					++cableroads51to60;
				if (pDoc->Cableways[k][l].totalnumberofINTs == 1)
					++intermediatesupport1;
				else if (pDoc->Cableways[k][l].totalnumberofINTs == 2)
					++intermediatesupport2;
				else if (pDoc->Cableways[k][l].totalnumberofINTs == 3)
					++intermediatesupport3;
			}			
		}
	}

	averageofcableroadlength = (float)sumofcableroadlength / (float)totalFeasibleCableroads;

	//end of temporary routine for just testing



	//saving in file
	FILE *outfile_ptr;
	CString csOutFileName = "e:\\thesis\\output\\cablematrix.txt";

	outfile_ptr = fopen(csOutFileName, "w");


	//initializing booleans for Cell. If grid cell is covered by any of cableways, then it gets TRUE.
	memset(pDoc->bCoveredCell, 0, sizeof(pDoc->bCoveredCell));//FALSE
	memset(LinktoSystem, 0, sizeof(LinktoSystem));//FALSE
	memset(LinktoLanding, 0, sizeof(LinktoLanding));//FALSE


	pDoc->TotalNumberofLinks = 0;
	nolinks = 0;



	////////////////////////////////////////////////////////////////////////////////
	//Seperate

	for (j = pDoc->UnitBoundary.startRow; j <= pDoc->UnitBoundary.endRow; ++j)
	{
		for (i = pDoc->UnitBoundary.startCol; i <= pDoc->UnitBoundary.endCol; ++i)
		{
			if (pDoc->CellsWithinUnitBoundary[i][j] == TRUE)
			{
				//initialize variable
				tempshortestdistfromanycables = 1000000000000.0;//large value for keeping minimum value.
				boolExceedMaxLateral = FALSE;
	
				if (pDoc->Volume[i][j] > 0)//timber to be cut is existing
				{
					for (k = 1; k <= pDoc->TotalNumberofLandings; ++k)
					{
						for (l = 0; l < MAXCABLEWAYS-1; ++l)//from 0 to 35.
						{
							//??????????????????????????????????????????????
							if (pDoc->Cableways[k][l].taillocationID > MINCELLALONGCABLEROAD)//if the cable road is shorter than 5*grid cell size, then this cable corridor is not feasible.
							{
								//measuring distance from the cell to each cableway
								x1 = pDoc->DTMCoord[i][j].x;//cell coordinate
								y1 = pDoc->DTMCoord[i][j].y;
								z1 = pDoc->DTMCoord[i][j].z;
								m = pDoc->Cableways[k][l].slope;//skyline corridor linear equation
								b = pDoc->Cableways[k][l].intercept;
								//find coordinates of the intercepting point (x, y).
								if (fabs(m) < 0.0001)//when l = 9 and 27--look at GenerateCablewaysperLanding()
								{
									x = x1;
									y = b;
								}
								else if (m > 100000.0)//when l = 1 and 18
								{
									y = y1;
									x = pDoc->DTMCoord[pDoc->Landings[k].column][pDoc->Landings[k].row].x;
								}
								else
								{
									x = (y1 + 1.0/m*x1 - b)*m/(pow(m,2.0)+1);//intercepting point by the perpendicular line from the cell.
									y = m*x + b;
								}

								//find cell address in row and col.
								tempcol = FindCellAddressbasedonCoordinates(x, y).x;
								temprow = FindCellAddressbasedonCoordinates(x, y).y;
								z = pDoc->DTMCoord[tempcol][temprow].z;//elevation of this intercepting point.

								//horizontal distance
								dist = sqrt(pow(x1-x, 2.0) + pow(y1-y, 2.0));
								//slope distance
								dist = sqrt(pow(dist, 2.0) + pow(z1-z, 2.0));

								//horizontal distance-this distance is just for finding out CarriagePosition on the corridor.
								distfromheadspar = sqrt(pow(x-pDoc->DTMCoord[pDoc->Landings[k].column][pDoc->Landings[k].row].x, 2.0) + pow(y-pDoc->DTMCoord[pDoc->Landings[k].column][pDoc->Landings[k].row].y, 2.0));
	
	
								cablesystemID = pDoc->Landings[k].CableSystem;

								//checking whether x and y fall within the cable length.
								feasibility = TRUE;
								maxX = pDoc->DTMCoord[pDoc->Cableways[k][l].tailcolumn][pDoc->Cableways[k][l].tailrow].x;
								minX = pDoc->DTMCoord[pDoc->Landings[k].column][pDoc->Landings[k].row].x;
							
								if (minX > maxX)
								{
									tempint = maxX;
									maxX = minX;
									minX = tempint;
								}
								maxY = pDoc->DTMCoord[pDoc->Cableways[k][l].tailcolumn][pDoc->Cableways[k][l].tailrow].y;
								minY = pDoc->DTMCoord[pDoc->Landings[k].column][pDoc->Landings[k].row].y;
								if (minY > maxY)
								{
									tempint = maxY;
									maxY = minY;
									minY = tempint;
								}
						
								if (x > maxX || x < minX)
									feasibility = FALSE;
		
								if (y > maxY || y < minY)
									feasibility = FALSE;
	
								if (dist > pDoc->CableSystems[cablesystemID].maxlateraldist)//maximum lateral yarding distance.
								{
									feasibility = FALSE;
									boolExceedMaxLateral = TRUE;
								}


								//calculate distance from head to tail.-horizontal distance. This is only for finding CarriagePosition.
								distfromheadtotail = sqrt(pow(maxX-minX, 2.0) + pow(maxY-minY, 2.0));


								//checking payload whether this volume can be carried up to landing or not.
								//finding where the carriage location would be along the skyline corridor - proportional location 
								CarriagePosition = (short)((double)pDoc->Cableways[k][l].taillocationID*distfromheadspar/distfromheadtotail);
								if (CarriagePosition >= pDoc->Cableways[k][l].taillocationID)
									CarriagePosition = pDoc->Cableways[k][l].taillocationID - 1;
								else if (CarriagePosition < MINCELLALONGCABLEROAD)
									CarriagePosition = 1;//directly to Landing
								
								
								//keeps shortest distance from the current cable way to the current cell for not covered cells (below).
								if (feasibility == FALSE)
								{
									landingX = pDoc->DTMCoord[pDoc->Landings[k].column][pDoc->Landings[k].row].x;
									landingY = pDoc->DTMCoord[pDoc->Landings[k].column][pDoc->Landings[k].row].y;
									landingZ = pDoc->DTMCoord[pDoc->Landings[k].column][pDoc->Landings[k].row].z;
									tailX = pDoc->DTMCoord[pDoc->Cableways[k][l].tailcolumn][pDoc->Cableways[k][l].tailrow].x;
									tailY = pDoc->DTMCoord[pDoc->Cableways[k][l].tailcolumn][pDoc->Cableways[k][l].tailrow].y;
									tailZ = pDoc->DTMCoord[pDoc->Cableways[k][l].tailcolumn][pDoc->Cableways[k][l].tailrow].z;
									
									tempshortestdistfromanycables = 1000000000000.0;//large value for keeping minimum value.
							
									//distance from landing
									//horizontal
									tempdist = sqrt(pow(x1-landingX, 2.0) + pow(y1-landingY, 2.0));
									//slope distance
									tempdist = sqrt(pow(tempdist, 2.0) + pow(z1-landingZ, 2.0));

									if (tempshortestdistfromanycables > tempdist)
									{
										tempshortestdistfromanycables = tempdist;
										tempCable = l;
										tempCablesystem = cablesystemID;
										tempLanding = k;
										tempCarriagePosition = 1;//directly to Landing
									}
							
									//distance from tail
									//horizontal
									tempdist = sqrt(pow(x1-tailX, 2.0) + pow(y1-tailY, 2.0));
									//slope distance
									tempdist = sqrt(pow(tempdist, 2.0) + pow(z1-tailZ, 2.0));

									if (tempshortestdistfromanycables > tempdist)
									{
										tempshortestdistfromanycables = tempdist;
										tempCable = l;
										tempCablesystem = cablesystemID;
										tempLanding = k;
										tempCarriagePosition = pDoc->Cableways[k][l].taillocationID - 1;
									}

									if (x <= maxX && x >= minX && y <= maxY && y >= minY)//current cell is located between landing and tail spar but further than lateral yarding distance.
									{
										if (tempshortestdistfromanycables > dist)//dist is the distance from the cable way to the current cell with right angle.
										{
											tempshortestdistfromanycables = dist;
											tempCable = l;
											tempCablesystem = cablesystemID;
											tempLanding = k;
											tempCarriagePosition = CarriagePosition;
										}
									}
								}



								//costs
								//1. calculate the number of carriage cycle required for yarding a grid cell using Payload(maximum load for this cable road) and volume amount in the cell.
								//2. calculate one carriage cycle costs including lateral yarding costs
								//2. fixed cost will be one carriage cycle costs multiplied by the number of trips.
								if (feasibility == TRUE)//the current cell is in within the cable length and lateral yarding distance.
									Penalty = 1.0;//no penalty
								else
								{
									Penalty = pDoc->CableSystems[cablesystemID].gbPenaltyForBeyondLateralYarding;
									dist = tempshortestdistfromanycables;//shortest distance among perpendicular distance, distances from tail and head.
									CarriagePosition = tempCarriagePosition;
									if (dist > pDoc->CableSystems[cablesystemID].gbGiveUpCellDistance)//farther than this distance..., then give up this cell.
										Penalty = -1.;
								}


								if (Penalty > 0.0)
								{
									//setup network
									//writing to file; cell - system - landing
									//link from the cell to system
									tempString1.Format("VC%dR%d", i, j);//column and row
									tempString2.Format("C%dS%dL%d", l, cablesystemID, k);//cableway, system, landing

									//check which one is smaller between UserDefined maximum payload and max payload from payloadanalysis.
									if (pDoc->PayloadAtEachTP[k][l][CarriagePosition] < pDoc->CableSystems[cablesystemID].gbUserMaxPayload)
										NumberofTripsNeeded = (int)(pDoc->Volume[i][j]*pDoc->CableSystems[cablesystemID].gbDensity)  / (int)(pDoc->PayloadAtEachTP[k][l][CarriagePosition]);//how many carrige trips are required to yard this cell; unit: kg.
									else
										NumberofTripsNeeded = (int)(pDoc->Volume[i][j]*pDoc->CableSystems[cablesystemID].gbDensity)  / (int)(pDoc->CableSystems[cablesystemID].gbUserMaxPayload);
								
									//add 1 to NumberofTripsNeeded.
									++NumberofTripsNeeded;

									//estimating yarding time for this volume cell
									hookingtime = pDoc->Volume[i][j] * (float)(pDoc->CableSystems[cablesystemID].gbHookTime + pDoc->CableSystems[cablesystemID].gbUnhookTime);//m3 * sec/m3 = sec
									//applying non-linear time equation for LYD
									MultiplierLYD = 1.0;
									if (dist < 0.333 * pDoc->CableSystems[cablesystemID].maxlateraldist)
										MultiplierLYD = pDoc->CableSystems[cablesystemID].gbMultiplierLYD1;
									else if (dist >= 0.333 * pDoc->CableSystems[cablesystemID].maxlateraldist && dist < 0.667 * pDoc->CableSystems[cablesystemID].maxlateraldist)
										MultiplierLYD = pDoc->CableSystems[cablesystemID].gbMultiplierLYD2;
									else 
										MultiplierLYD = pDoc->CableSystems[cablesystemID].gbMultiplierLYD3;
									
									//applying Penalty
									lateralyardingtime = (float)(Penalty * dist) / pDoc->CableSystems[cablesystemID].gbLateralYardingSpeed;//include later yarding time
									lateralyardingtime *= MultiplierLYD;
									lateralyardingtime *= (float)NumberofTripsNeeded;
									yardingtime = pDoc->CycleTimeAtEachTP[k][l][CarriagePosition];//cycle time does not include hooking and unhooking time. -- See CalculateUnitCosts()
									yardingtime *= (float)NumberofTripsNeeded;
								
									totalyardingtime = hookingtime + lateralyardingtime + yardingtime;
									//taking delay time into account - July 4, 2002
									totalyardingtime += totalyardingtime * (float)(pDoc->CableSystems[cablesystemID].gbDelayPercentage/(float)100.0);
								
									//variable cost will be the unit which is per unit volume (m3).
									variable = (float) (totalyardingtime / 3600. * pDoc->CableSystems[cablesystemID].gbUnitCost ) / (float)(pDoc->Volume[i][j]);//totalyardingtime(sec)/3600 * System Unitcosts($/hr);
									fixed = 0.;

									fprintf(outfile_ptr, "%s	%s	%10.2f	%10.2f\n", tempString1, tempString2, variable, fixed);//yarding cost. currently using distance. no fixed costs.
									++nolinks;
									pDoc->ConvertfromCStringToChar(pDoc->Links[nolinks].FromNode, tempString1);
									pDoc->ConvertfromCStringToChar(pDoc->Links[nolinks].ToNode, tempString2);
									pDoc->Links[nolinks].VariableCost = variable;
									pDoc->Links[nolinks].FixedCost = fixed;

									//link from the grid cell to system (skyline reinstalling costs occurs as a fixed cost + INT supports rigging costs)
									//fixed costs per corridor include reinstalling costs and rigging intermediate supports costs.
									if (LinktoSystem[l][cablesystemID][k] == FALSE)
									{
										LinktoSystem[l][cablesystemID][k] = TRUE;
										tempString1 = tempString2;//cableway, system, landing
										tempString2.Format("S%dL%d", cablesystemID, k);//landings
										variable = 0.;
										fixed = pDoc->CableSystems[cablesystemID].gbCableRoadChangeHour * pDoc->CableSystems[cablesystemID].gbUnitCost;//skyline corridor reinstalling costs

										//counting number of intermediate spars
										if (pDoc->Cableways[k][l].totalnumberofINTs > 0)
										{
											RiggingCosts = pDoc->CableSystems[cablesystemID].gbRiggingCosts * pDoc->Cableways[k][l].totalnumberofINTs;//not including tail spar... assuming tail spar setting costs are included in cable reinstalling costs.
											fixed += RiggingCosts;
										}
	
										fprintf(outfile_ptr, "%s	%s	%10.2f	%10.2f\n", tempString1, tempString2, variable, fixed);//landing + system costs. currently 0. no variable costs.
										++nolinks;
										pDoc->ConvertfromCStringToChar(pDoc->Links[nolinks].FromNode, tempString1);
										pDoc->ConvertfromCStringToChar(pDoc->Links[nolinks].ToNode, tempString2);
										pDoc->Links[nolinks].VariableCost = variable;
										pDoc->Links[nolinks].FixedCost = fixed;

	
										//link from the system to landing (tower location) -- system move-in costs occur.
										if (LinktoLanding[cablesystemID][k] == FALSE)
										{
											LinktoLanding[cablesystemID][k] = TRUE;
											tempString1 = tempString2;//system, landing
	
											if (pDoc->LandingSharingTowerID[k] <= 0)
												actuallandingID = k;
											else
											{
												actuallandingID = pDoc->LandingSharingTowerID[k];
												//in case that this actuallandingID also shares with other landing.
												while (pDoc->LandingSharingTowerID[actuallandingID] > 0)
													actuallandingID = pDoc->LandingSharingTowerID[actuallandingID];
											}
	
											tempString2.Format("L%d", actuallandingID);//landings
											variable = 0.;
											fixed = pDoc->CableSystems[cablesystemID].gbMoveInCosts;//system move-in costs
											fixed += pDoc->CableSystems[cablesystemID].gbYarderSetupHour * pDoc->CableSystems[cablesystemID].gbUnitCost;//add intial yarder setup costs
											fprintf(outfile_ptr, "%s	%s	%10.2f	%10.2f\n", tempString1, tempString2, variable, fixed);//landing + system costs. currently 0. no variable costs.
											++nolinks;
											pDoc->ConvertfromCStringToChar(pDoc->Links[nolinks].FromNode, tempString1);
											pDoc->ConvertfromCStringToChar(pDoc->Links[nolinks].ToNode, tempString2);
											pDoc->Links[nolinks].VariableCost = variable;
											pDoc->Links[nolinks].FixedCost = fixed;
										}
									}
									pDoc->bCoveredCell[i][j] = TRUE;//put TRUE in the global variable indicating this cell is covered by any of cableways.
								}//if Penalty > 0
							}//end of if (pDoc->Cableways[k][l].taillocationID > 5)
						}//end of l
					}//end of k



					if (pDoc->bCoveredCell[i][j] != TRUE)//never had a link for this cell---give up and connect directly to the destination.
					{
						//give-up cells--put -1 in the volume
						pDoc->Volume[i][j] = -1.;
					}
				}//end of if (Volume > 0)
			}//end of if (Cellwithin == TRUE)	
		}//end of i
	}//end of j
		


	///////////////////////////////////////////////////
	//links from each landing to dummy destination -- input landing costs for actual landing locations
	///////////////////////////////////////////////////
	for (k = 1; k <= pDoc->TotalNumberofLandings; ++k)
	{
		if (pDoc->LandingSharingTowerID[k] <= 0)//when this is an actual landing.
		{
			tempString1.Format("L%d", k);//landing ID matches with k -- see GenerateRoadSaleData()
			tempString2.Format("C%dR%d", pDoc->Landings[k].column, pDoc->Landings[k].row);
			variable = 0.;
			fixed = pDoc->LandingCosts[k];
			fprintf(outfile_ptr, "%s	%s	%10.2f	%10.2f\n", tempString1, tempString2, variable, fixed);
			++nolinks;
			pDoc->ConvertfromCStringToChar(pDoc->Links[nolinks].FromNode, tempString1);
			pDoc->ConvertfromCStringToChar(pDoc->Links[nolinks].ToNode, tempString2);
			pDoc->Links[nolinks].VariableCost = variable;
			pDoc->Links[nolinks].FixedCost = fixed;	
		}
	}

	pDoc->TotalNumberofLinks = nolinks;

	fclose(outfile_ptr);

}


void CCablanView::GenerateCableSystemSaleData(int Iteration)
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int i, j;
	int nosales;//total number of sales
	float volume;
	short year;
	CString tempString1, tempString2;


if (Iteration <= 1)
{
	//saving in file
	FILE *outfile_ptr;
	CString csOutFileName = "e:\\thesis\\output\\salecablematrix.txt";

	outfile_ptr = fopen(csOutFileName, "w");

	//initializing
	pDoc->TotalNumberofSales = 0;
	nosales = 0;


	for (j = pDoc->UnitBoundary.startRow; j <= pDoc->UnitBoundary.endRow; ++j)
	{
		for(i = pDoc->UnitBoundary.startCol; i <= pDoc->UnitBoundary.endCol; ++i)
		{	
			if (pDoc->CellsWithinUnitBoundary[i][j] == TRUE)
			{
				if (pDoc->Volume[i][j] > 0)//timber to be cut is existing
				{	
					//writing to file
					tempString1.Format("C%dR%d", i, j);
				
			
					//temporary destination (from Cell to Landings)
					tempString2.Format("DummyDest1");
					volume = pDoc->Volume[i][j];
					year = 0;
					fprintf(outfile_ptr, "%s	%s	%10.2f	%d\n", tempString1, tempString2, volume, year);
					++nosales;
					pDoc->ConvertfromCStringToChar(pDoc->Sales[nosales].EntryNode, tempString1);
					pDoc->ConvertfromCStringToChar(pDoc->Sales[nosales].DestNode, tempString2);
					pDoc->Sales[nosales].Volume = volume;
					pDoc->Sales[nosales].Year = year;	
				}
			}
		}
	}


	pDoc->TotalNumberofSales = nosales;

	fclose(outfile_ptr);

	//store the results in the memory
	memcpy(pDoc->CableSales, pDoc->Sales, sizeof(pDoc->Sales));
	pDoc->CableTotalNumberofSales = pDoc->TotalNumberofSales;
}
else
{
	//store the results in the memory
	memcpy(pDoc->Sales, pDoc->CableSales, sizeof(pDoc->Sales));
	pDoc->TotalNumberofSales = pDoc->CableTotalNumberofSales;
}

}


void CCablanView::GenerateCableSystemSaleDataAllAtOnce()
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int i, j;
	int nosales;//total number of sales
	float volume;
	short year;
	CString tempString1, tempString2;

	//saving in file
	FILE *outfile_ptr;
	CString csOutFileName = "e:\\thesis\\output\\salecablematrix.txt";

	outfile_ptr = fopen(csOutFileName, "w");

	//initializing
	pDoc->TotalNumberofSales = 0;
	nosales = 0;


////////////////////////////////////////////////////////////////////////
//all at once
////////////////////////////////////////////////////////////////////////


	for (j = 1; j <= pDoc->DTMrows; ++j)
	{
		for(i = 1; i <= pDoc->DTMcolumns; ++i)
		{
			if (pDoc->Volume[i][j] > 0)//timber to be cut is existing
			{	
				//writing to file
				tempString1.Format("VC%dR%d", i, j);
				
				
				//temporary destination (from Cell to Landings)
				tempString2.Format("C80R80");
				volume = pDoc->Volume[i][j];
				year = 0;
				fprintf(outfile_ptr, "%s	%s	%10.2f	%d\n", tempString1, tempString2, volume, year);
				++nosales;
				pDoc->ConvertfromCStringToChar(pDoc->Sales[nosales].EntryNode, tempString1);
				pDoc->ConvertfromCStringToChar(pDoc->Sales[nosales].DestNode, tempString2);
				pDoc->Sales[nosales].Volume = volume;
				pDoc->Sales[nosales].Year = year;	
			}
		}
	}


	pDoc->TotalNumberofSales = nosales;

	fclose(outfile_ptr);
	
}





///////////////////////////////////////////////////////////////////////////////////////////////////
//Road Matrix
///////////////////////////////////////////////////////////////////////////////////////////////////

void CCablanView::GenerateRoadNetwork(int Iteration)
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int i, j;
	int gotoi, gotoj;
	double dist, slope, sidedist, sideslope;
	double sqrt_two = pow(2.0, 0.5);
	CString tempString1, tempString2;
	float variable, fixed;
	bool LinkAvailability[MAXCOLUMNS][MAXROWS];
	int totalroadcells = 0;
	int totalexistingroadcells = 0;
	

if (Iteration <= 1)
{
	//saving in file
	FILE *outfile_ptr;
	CString csOutFileName = "e:\\thesis\\output\\roadmatrix.txt";

	outfile_ptr = fopen(csOutFileName, "w");


	//initializing
	pDoc->TotalNumberofLinks = 0;
	memset(LinkAvailability, 0, sizeof(LinkAvailability));

	//by using LinkAvailability[gotoi][gotoj], most of cells need generate links only for from East to SouthWest --> Save time.
	//???????????????????????????????????????????????????????????????????????????????
	//unit boundary for road network must be bigger or equal to the harvest unit boundary and need to be defined by the user.
	//current boudary is the rectangular boundary of the harvest unit boundary.
	for (j = pDoc->UnitBoundary.startRow + 1; j <= pDoc->UnitBoundary.endRow - 1; ++j)
	{
		for (i = pDoc->UnitBoundary.startCol + 1; i <= pDoc->UnitBoundary.endCol - 1; ++i)
		{
			if (pDoc->DTMCoord[i][j].z > 0)//data are existing
			{
				++totalroadcells;//counting total cells considered.

				//1. NorthWest
				gotoi = i-1;
				gotoj = j-1;

				if (LinkAvailability[gotoi][gotoj] != TRUE)// if all links toward to and from this cell are already generated, then this bool variable has TRUE value. --> Skip this procedure.
				{
					//forward direction looks at only two consecutive cells.
					dist = pDoc->DTMcellsize * sqrt_two;//sqr(2)*cellsize
					slope = fabs((double)pDoc->DTMCoord[gotoi][gotoj].z - (double)pDoc->DTMCoord[i][j].z) / dist;
					//side slope is calculated from two side cells of the front cell (cross three consecutive cells). Cell distance will be distance of two cells.
					sidedist = dist * 2;
					sideslope = fabs((double)pDoc->DTMCoord[gotoi+1][gotoj-1].z - (double)pDoc->DTMCoord[gotoi-1][gotoj+1].z) / sidedist;

//slope limitation~~~??????????????????????????????					
					if (slope < pDoc->RGLimit && sideslope < pDoc->SSLimit)
					{
						variable = VariableCostsRoadCell(dist, slope);
						fixed = FixedCostsRoadCell(dist, slope, sidedist, sideslope);
						if (pDoc->FishStreamBuffer[gotoi][gotoj] > 0)
							fixed *= pDoc->MultiplierForRoadsOnStream;

						//writing to file -- generate two links (two ways)
						GenerateRoadNetwork_WritingtoFile(i, j, gotoi, gotoj, variable, fixed, outfile_ptr, (float)dist, (float)slope, (float)sideslope);
					}
				}
				

				//2. North
				gotoi = i;
				gotoj = j-1;
				
				if (LinkAvailability[gotoi][gotoj] != TRUE)
				{
					dist = pDoc->DTMcellsize;
					slope = fabs((double)pDoc->DTMCoord[gotoi][gotoj].z - (double)pDoc->DTMCoord[i][j].z) / dist;
					sidedist = dist * 2;
					sideslope = fabs((double)pDoc->DTMCoord[gotoi+1][gotoj].z - (double)pDoc->DTMCoord[gotoi-1][gotoj].z) / sidedist;
				
					if (slope < pDoc->RGLimit && sideslope < pDoc->SSLimit)
					{
						variable = VariableCostsRoadCell(dist, slope);
						fixed = FixedCostsRoadCell(dist, slope, sidedist, sideslope);
						if (pDoc->FishStreamBuffer[gotoi][gotoj] > 0)
							fixed *= pDoc->MultiplierForRoadsOnStream;

						//writing to file
						GenerateRoadNetwork_WritingtoFile(i, j, gotoi, gotoj, variable, fixed, outfile_ptr, (float)dist, (float)slope, (float)sideslope);
					}
				}
				

				//3. NorthEast
				gotoi = i+1;
				gotoj = j-1;
				
				if (LinkAvailability[gotoi][gotoj] != TRUE)
				{
					dist = pDoc->DTMcellsize * sqrt_two;//sqr(2)*cellsize
					slope = fabs((double)pDoc->DTMCoord[gotoi][gotoj].z - (double)pDoc->DTMCoord[i][j].z) / dist;
					sidedist = dist * 2;
					sideslope = fabs((double)pDoc->DTMCoord[gotoi-1][gotoj-1].z - (double)pDoc->DTMCoord[gotoi+1][gotoj+1].z) / sidedist;

					if (slope < pDoc->RGLimit && sideslope < pDoc->SSLimit)
					{
						variable = VariableCostsRoadCell(dist, slope);
						fixed = FixedCostsRoadCell(dist, slope, sidedist, sideslope);
						if (pDoc->FishStreamBuffer[gotoi][gotoj] > 0)
							fixed *= pDoc->MultiplierForRoadsOnStream;

						//writing to file
						GenerateRoadNetwork_WritingtoFile(i, j, gotoi, gotoj, variable, fixed, outfile_ptr, (float)dist, (float)slope, (float)sideslope);
					}
				}
				


				//4. East
				gotoi = i+1;
				gotoj = j;
				
				if (LinkAvailability[gotoi][gotoj] != TRUE)
				{
					dist = pDoc->DTMcellsize;
					slope = fabs((double)pDoc->DTMCoord[gotoi][gotoj].z - (double)pDoc->DTMCoord[i][j].z) / dist;
					sidedist = dist * 2;
					sideslope = fabs((double)pDoc->DTMCoord[gotoi][gotoj-1].z - (double)pDoc->DTMCoord[gotoi][gotoj+1].z) / sidedist;

					if (slope < pDoc->RGLimit && sideslope < pDoc->SSLimit)
					{
						variable = VariableCostsRoadCell(dist, slope);
						fixed = FixedCostsRoadCell(dist, slope, sidedist, sideslope);
						if (pDoc->FishStreamBuffer[gotoi][gotoj] > 0)
							fixed *= pDoc->MultiplierForRoadsOnStream;

						//writing to file
						GenerateRoadNetwork_WritingtoFile(i, j, gotoi, gotoj, variable, fixed, outfile_ptr, (float)dist, (float)slope, (float)sideslope);
					}
				}
				

				//5. SouthEast
				gotoi = i+1;
				gotoj = j+1;
				
				if (LinkAvailability[gotoi][gotoj] != TRUE)
				{
					dist = pDoc->DTMcellsize * sqrt_two;//sqr(2)*cellsize
					slope = fabs((double)pDoc->DTMCoord[gotoi][gotoj].z - (double)pDoc->DTMCoord[i][j].z) / dist;
					sidedist = dist * 2;
					sideslope = fabs((double)pDoc->DTMCoord[gotoi-1][gotoj+1].z - (double)pDoc->DTMCoord[gotoi+1][gotoj-1].z) / sidedist;

					if (slope < pDoc->RGLimit && sideslope < pDoc->SSLimit)
					{
						variable = VariableCostsRoadCell(dist, slope);
						fixed = FixedCostsRoadCell(dist, slope, sidedist, sideslope);
						if (pDoc->FishStreamBuffer[gotoi][gotoj] > 0)
							fixed *= pDoc->MultiplierForRoadsOnStream;

						//writing to file
						GenerateRoadNetwork_WritingtoFile(i, j, gotoi, gotoj, variable, fixed, outfile_ptr, (float)dist, (float)slope, (float)sideslope);
					}
				}
				


				//6. South
				gotoi = i;
				gotoj = j+1;
				
				if (LinkAvailability[gotoi][gotoj] != TRUE)
				{
					dist = pDoc->DTMcellsize;
					slope = fabs((double)pDoc->DTMCoord[gotoi][gotoj].z - (double)pDoc->DTMCoord[i][j].z) / dist;
					sidedist = dist * 2;
					sideslope = fabs((double)pDoc->DTMCoord[gotoi-1][gotoj].z - (double)pDoc->DTMCoord[gotoi+1][gotoj].z) / sidedist;

					if (slope < pDoc->RGLimit && sideslope < pDoc->SSLimit)
					{
						variable = VariableCostsRoadCell(dist, slope);
						fixed = FixedCostsRoadCell(dist, slope, sidedist, sideslope);
						if (pDoc->FishStreamBuffer[gotoi][gotoj] > 0)
							fixed *= pDoc->MultiplierForRoadsOnStream;

						//writing to file
						GenerateRoadNetwork_WritingtoFile(i, j, gotoi, gotoj, variable, fixed, outfile_ptr, (float)dist, (float)slope, (float)sideslope);
					}
				}
				


				//7. SouthWest
				gotoi = i-1;
				gotoj = j+1;
				
				if (LinkAvailability[gotoi][gotoj] != TRUE)
				{
					dist = pDoc->DTMcellsize * sqrt_two;//sqr(2)*cellsize
					slope = fabs((double)pDoc->DTMCoord[gotoi][gotoj].z - (double)pDoc->DTMCoord[i][j].z) / dist;
					sidedist = dist * 2;
					sideslope = fabs((double)pDoc->DTMCoord[gotoi-1][gotoj-1].z - (double)pDoc->DTMCoord[gotoi+1][gotoj+1].z) / sidedist;

					if (slope < pDoc->RGLimit && sideslope < pDoc->SSLimit)
					{
						variable = VariableCostsRoadCell(dist, slope);
						fixed = FixedCostsRoadCell(dist, slope, sidedist, sideslope);
						if (pDoc->FishStreamBuffer[gotoi][gotoj] > 0)
							fixed *= pDoc->MultiplierForRoadsOnStream;

						//writing to file
						GenerateRoadNetwork_WritingtoFile(i, j, gotoi, gotoj, variable, fixed, outfile_ptr, (float)dist, (float)slope, (float)sideslope);
					}
				}
				


				//8. West
				gotoi = i-1;
				gotoj = j;
				
				if (LinkAvailability[gotoi][gotoj] != TRUE)
				{
					dist = pDoc->DTMcellsize;
					slope = fabs((double)pDoc->DTMCoord[gotoi][gotoj].z - (double)pDoc->DTMCoord[i][j].z) / dist;
					sidedist = dist * 2;
					sideslope = fabs((double)pDoc->DTMCoord[gotoi][gotoj-1].z - (double)pDoc->DTMCoord[gotoi][gotoj+1].z) / sidedist;

					if (slope < pDoc->RGLimit && sideslope < pDoc->SSLimit)
					{
						variable = VariableCostsRoadCell(dist, slope);
						fixed = FixedCostsRoadCell(dist, slope, sidedist, sideslope);
						if (pDoc->FishStreamBuffer[gotoi][gotoj] > 0)
							fixed *= pDoc->MultiplierForRoadsOnStream;

						//writing to file
						GenerateRoadNetwork_WritingtoFile(i, j, gotoi, gotoj, variable, fixed, outfile_ptr, (float)dist, (float)slope, (float)sideslope);
					}
				}
				

				LinkAvailability[i][j] = TRUE;
			}
		}
	}


	//add links from TimberExits to SuperExit
	for (i = 1; i <= pDoc->TotalNumberofTimberExits; ++i)
	{
		//one way
		++pDoc->TotalNumberofLinks;
		//writing to file
		tempString1.Format("C%dR%d", pDoc->TimberExits[i].column, pDoc->TimberExits[i].row);
		tempString2 = "SuperExit";
		fprintf(outfile_ptr, "%s	%s	%10.2f	%10.2f\n", tempString1, tempString2, 0.0, 0.0);
		//writing to memory
		pDoc->ConvertfromCStringToChar(pDoc->Links[pDoc->TotalNumberofLinks].FromNode, tempString1);
		pDoc->ConvertfromCStringToChar(pDoc->Links[pDoc->TotalNumberofLinks].ToNode, tempString2);
		pDoc->Links[pDoc->TotalNumberofLinks].VariableCost = 0.0;
		pDoc->Links[pDoc->TotalNumberofLinks].FixedCost = 0.0;	
	}


	
	//add links from existing roads to SuperExit

//	//include only existing roads within the planning area
//	for (j = pDoc->UnitBoundary.startRow + 1; j <= pDoc->UnitBoundary.endRow - 1; ++j)
//	{
//		for (i = pDoc->UnitBoundary.startCol + 1; i <= pDoc->UnitBoundary.endCol - 1; ++i)
//		{

	for (j = 1; j <= pDoc->DTMrows; ++j)
	{
		for(i = 1; i <= pDoc->DTMcolumns; ++i)
		{
			if (pDoc->ExistingRoads[i][j] > 0)
			{
				++totalexistingroadcells;
				//one way
				++pDoc->TotalNumberofLinks;
				//writing to file
				tempString1.Format("C%dR%d", i, j);
				tempString2 = "SuperExit";
				fprintf(outfile_ptr, "%s	%s	%10.2f	%10.2f\n", tempString1, tempString2, 0.0, 0.0);
				//writing to memory
				pDoc->ConvertfromCStringToChar(pDoc->Links[pDoc->TotalNumberofLinks].FromNode, tempString1);
				pDoc->ConvertfromCStringToChar(pDoc->Links[pDoc->TotalNumberofLinks].ToNode, tempString2);
				pDoc->Links[pDoc->TotalNumberofLinks].VariableCost = 0.0;
				pDoc->Links[pDoc->TotalNumberofLinks].FixedCost = 0.0;
			}
		}
	}


	fclose(outfile_ptr);


	//store links to the memory
	memcpy(pDoc->RoadLinks, pDoc->Links, sizeof(pDoc->Links));
	pDoc->RoadTotalNumberofLinks = pDoc->TotalNumberofLinks;


}
else
{
	//restore links to the memory
	memcpy(pDoc->Links, pDoc->RoadLinks, sizeof(pDoc->Links));
	pDoc->TotalNumberofLinks = pDoc->RoadTotalNumberofLinks;
}




}


void CCablanView::GenerateRoadNetwork_WritingtoFile(int fromcol, int fromrow, int tocol, int torow, float variable, float fixed, FILE *outfile_ptr, float dist, float slope, float sideslope)
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CString tempString1, tempString2;
	
	//one way
	++pDoc->TotalNumberofLinks;
	//writing to file
	tempString1.Format("C%dR%d", fromcol, fromrow);
	tempString2.Format("C%dR%d", tocol, torow);
	fprintf(outfile_ptr, "%s	%s	%10.2f	%10.2f\n", tempString1, tempString2, variable, fixed);
	//writing to memory
	pDoc->ConvertfromCStringToChar(pDoc->Links[pDoc->TotalNumberofLinks].FromNode, tempString1);
	pDoc->ConvertfromCStringToChar(pDoc->Links[pDoc->TotalNumberofLinks].ToNode, tempString2);
	pDoc->Links[pDoc->TotalNumberofLinks].VariableCost = variable;
	pDoc->Links[pDoc->TotalNumberofLinks].FixedCost = fixed;
	pDoc->tempDist[pDoc->TotalNumberofLinks] = dist;
	pDoc->tempSlope[pDoc->TotalNumberofLinks] = slope;
	pDoc->tempSideSlope[pDoc->TotalNumberofLinks] = sideslope;


	//the other way
	++pDoc->TotalNumberofLinks;
	//writing to file
	fprintf(outfile_ptr, "%s	%s	%10.2f	%10.2f\n", tempString2, tempString1, variable, fixed);
	//writing to memory
	pDoc->ConvertfromCStringToChar(pDoc->Links[pDoc->TotalNumberofLinks].FromNode, tempString2);
	pDoc->ConvertfromCStringToChar(pDoc->Links[pDoc->TotalNumberofLinks].ToNode, tempString1);
	pDoc->Links[pDoc->TotalNumberofLinks].VariableCost = variable;
	pDoc->Links[pDoc->TotalNumberofLinks].FixedCost = fixed;
	pDoc->tempDist[pDoc->TotalNumberofLinks] = dist;
	pDoc->tempSlope[pDoc->TotalNumberofLinks] = slope;
	pDoc->tempSideSlope[pDoc->TotalNumberofLinks] = sideslope;

}







void CCablanView::GenerateRoadNetworkAllAtOnce()
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int i, j;
	int gotoi, gotoj;
	double dist, slope, sidedist, sideslope;
	double sqrt_two = pow(2.0, 0.5);
	CString tempString1, tempString2;
	float variable, fixed;
	bool LinkAvailability[MAXCOLUMNS][MAXROWS];

	//saving in file
	FILE *outfile_ptr;
	CString csOutFileName = "e:\\thesis\\output\\roadmatrix.txt";

	outfile_ptr = fopen(csOutFileName, "w");


	//initializing
	pDoc->TotalNumberofLinks;//continued

	memset(LinkAvailability, 0, sizeof(LinkAvailability));

	//by using LinkAvailability[gotoi][gotoj], most of cells need generate links only for from East to SouthWest --> Save time.
	//???????????????????????????????????????????????????????????????????????????????
	//unit boundary for road network must be bigger or equal to the harvest unit boundary and need to be defined by the user.
	//current boudary is the rectangular boundary of the harvest unit boundary.
	for (j = pDoc->UnitBoundary.startRow + 1; j <= pDoc->UnitBoundary.endRow - 1; ++j)
	{
		for (i = pDoc->UnitBoundary.startCol + 1; i <= pDoc->UnitBoundary.endCol - 1; ++i)
		{
			if (pDoc->DTMCoord[i][j].z > 0)//data are existing
			{
				//1. NorthWest
				gotoi = i-1;
				gotoj = j-1;

				if (LinkAvailability[gotoi][gotoj] != TRUE)// if all links toward to and from this cell are already generated, then this bool variable has TRUE value. --> Skip this procedure.
				{
					//forward direction looks at only two consecutive cells.
					dist = pDoc->DTMcellsize * sqrt_two;//sqr(2)*cellsize
					slope = fabs((double)pDoc->DTMCoord[gotoi][gotoj].z - (double)pDoc->DTMCoord[i][j].z) / dist;
					//side slope is calculated from two side cells of the front cell (cross three consecutive cells). Cell distance will be distance of two cells.
					sidedist = dist * 2;
					sideslope = fabs((double)pDoc->DTMCoord[gotoi+1][gotoj-1].z - (double)pDoc->DTMCoord[gotoi-1][gotoj+1].z) / sidedist;

//slope limitation~~~??????????????????????????????					
					if (slope < pDoc->RGLimit && sideslope < pDoc->SSLimit)
					{
						variable = VariableCostsRoadCell(dist, slope);
						fixed = FixedCostsRoadCell(dist, slope, sidedist, sideslope);
						if (pDoc->FishStreamBuffer[gotoi][gotoj] > 0)
							fixed *= pDoc->MultiplierForRoadsOnStream;

						//writing to file -- generate two links (two ways)
						GenerateRoadNetwork_WritingtoFile(i, j, gotoi, gotoj, variable, fixed, outfile_ptr, (float)dist, (float)slope, (float)sideslope);
					}
				}
				


				//2. North
				gotoi = i;
				gotoj = j-1;
				
				if (LinkAvailability[gotoi][gotoj] != TRUE)
				{
					dist = pDoc->DTMcellsize;
					slope = fabs((double)pDoc->DTMCoord[gotoi][gotoj].z - (double)pDoc->DTMCoord[i][j].z) / dist;
					sidedist = dist * 2;
					sideslope = fabs((double)pDoc->DTMCoord[gotoi+1][gotoj].z - (double)pDoc->DTMCoord[gotoi-1][gotoj].z) / sidedist;
				
					if (slope < pDoc->RGLimit && sideslope < pDoc->SSLimit)
					{
						variable = VariableCostsRoadCell(dist, slope);
						fixed = FixedCostsRoadCell(dist, slope, sidedist, sideslope);
						if (pDoc->FishStreamBuffer[gotoi][gotoj] > 0)
							fixed *= pDoc->MultiplierForRoadsOnStream;

						//writing to file
						GenerateRoadNetwork_WritingtoFile(i, j, gotoi, gotoj, variable, fixed, outfile_ptr, (float)dist, (float)slope, (float)sideslope);
					}
				}
				

				//3. NorthEast
				gotoi = i+1;
				gotoj = j-1;
				
				if (LinkAvailability[gotoi][gotoj] != TRUE)
				{
					dist = pDoc->DTMcellsize * sqrt_two;//sqr(2)*cellsize
					slope = fabs((double)pDoc->DTMCoord[gotoi][gotoj].z - (double)pDoc->DTMCoord[i][j].z) / dist;
					sidedist = dist * 2;
					sideslope = fabs((double)pDoc->DTMCoord[gotoi-1][gotoj-1].z - (double)pDoc->DTMCoord[gotoi+1][gotoj+1].z) / sidedist;

					if (slope < pDoc->RGLimit && sideslope < pDoc->SSLimit)
					{
						variable = VariableCostsRoadCell(dist, slope);
						fixed = FixedCostsRoadCell(dist, slope, sidedist, sideslope);
						if (pDoc->FishStreamBuffer[gotoi][gotoj] > 0)
							fixed *= pDoc->MultiplierForRoadsOnStream;

						//writing to file
						GenerateRoadNetwork_WritingtoFile(i, j, gotoi, gotoj, variable, fixed, outfile_ptr, (float)dist, (float)slope, (float)sideslope);
					}
				}
				


				//4. East
				gotoi = i+1;
				gotoj = j;
				
				if (LinkAvailability[gotoi][gotoj] != TRUE)
				{
					dist = pDoc->DTMcellsize;
					slope = fabs((double)pDoc->DTMCoord[gotoi][gotoj].z - (double)pDoc->DTMCoord[i][j].z) / dist;
					sidedist = dist * 2;
					sideslope = fabs((double)pDoc->DTMCoord[gotoi][gotoj-1].z - (double)pDoc->DTMCoord[gotoi][gotoj+1].z) / sidedist;

					if (slope < pDoc->RGLimit && sideslope < pDoc->SSLimit)
					{
						variable = VariableCostsRoadCell(dist, slope);
						fixed = FixedCostsRoadCell(dist, slope, sidedist, sideslope);
						if (pDoc->FishStreamBuffer[gotoi][gotoj] > 0)
							fixed *= pDoc->MultiplierForRoadsOnStream;

						//writing to file
						GenerateRoadNetwork_WritingtoFile(i, j, gotoi, gotoj, variable, fixed, outfile_ptr, (float)dist, (float)slope, (float)sideslope);
					}
				}
				

				//5. SouthEast
				gotoi = i+1;
				gotoj = j+1;
				
				if (LinkAvailability[gotoi][gotoj] != TRUE)
				{
					dist = pDoc->DTMcellsize * sqrt_two;//sqr(2)*cellsize
					slope = fabs((double)pDoc->DTMCoord[gotoi][gotoj].z - (double)pDoc->DTMCoord[i][j].z) / dist;
					sidedist = dist * 2;
					sideslope = fabs((double)pDoc->DTMCoord[gotoi-1][gotoj+1].z - (double)pDoc->DTMCoord[gotoi+1][gotoj-1].z) / sidedist;

					if (slope < pDoc->RGLimit && sideslope < pDoc->SSLimit)
					{
						variable = VariableCostsRoadCell(dist, slope);
						fixed = FixedCostsRoadCell(dist, slope, sidedist, sideslope);
						if (pDoc->FishStreamBuffer[gotoi][gotoj] > 0)
							fixed *= pDoc->MultiplierForRoadsOnStream;

						//writing to file
						GenerateRoadNetwork_WritingtoFile(i, j, gotoi, gotoj, variable, fixed, outfile_ptr, (float)dist, (float)slope, (float)sideslope);
					}
				}
				


				//6. South
				gotoi = i;
				gotoj = j+1;
				
				if (LinkAvailability[gotoi][gotoj] != TRUE)
				{
					dist = pDoc->DTMcellsize;
					slope = fabs((double)pDoc->DTMCoord[gotoi][gotoj].z - (double)pDoc->DTMCoord[i][j].z) / dist;
					sidedist = dist * 2;
					sideslope = fabs((double)pDoc->DTMCoord[gotoi-1][gotoj].z - (double)pDoc->DTMCoord[gotoi+1][gotoj].z) / sidedist;

					if (slope < pDoc->RGLimit && sideslope < pDoc->SSLimit)
					{
						variable = VariableCostsRoadCell(dist, slope);
						fixed = FixedCostsRoadCell(dist, slope, sidedist, sideslope);
						if (pDoc->FishStreamBuffer[gotoi][gotoj] > 0)
							fixed *= pDoc->MultiplierForRoadsOnStream;

						//writing to file
						GenerateRoadNetwork_WritingtoFile(i, j, gotoi, gotoj, variable, fixed, outfile_ptr, (float)dist, (float)slope, (float)sideslope);
					}
				}
				


				//7. SouthWest
				gotoi = i-1;
				gotoj = j+1;
				
				if (LinkAvailability[gotoi][gotoj] != TRUE)
				{
					dist = pDoc->DTMcellsize * sqrt_two;//sqr(2)*cellsize
					slope = fabs((double)pDoc->DTMCoord[gotoi][gotoj].z - (double)pDoc->DTMCoord[i][j].z) / dist;
					sidedist = dist * 2;
					sideslope = fabs((double)pDoc->DTMCoord[gotoi-1][gotoj-1].z - (double)pDoc->DTMCoord[gotoi+1][gotoj+1].z) / sidedist;

					if (slope < pDoc->RGLimit && sideslope < pDoc->SSLimit)
					{
						variable = VariableCostsRoadCell(dist, slope);
						fixed = FixedCostsRoadCell(dist, slope, sidedist, sideslope);
						if (pDoc->FishStreamBuffer[gotoi][gotoj] > 0)
							fixed *= pDoc->MultiplierForRoadsOnStream;

						//writing to file
						GenerateRoadNetwork_WritingtoFile(i, j, gotoi, gotoj, variable, fixed, outfile_ptr, (float)dist, (float)slope, (float)sideslope);
					}
				}
				


				//8. West
				gotoi = i-1;
				gotoj = j;
				
				if (LinkAvailability[gotoi][gotoj] != TRUE)
				{
					dist = pDoc->DTMcellsize;
					slope = fabs((double)pDoc->DTMCoord[gotoi][gotoj].z - (double)pDoc->DTMCoord[i][j].z) / dist;
					sidedist = dist * 2;
					sideslope = fabs((double)pDoc->DTMCoord[gotoi][gotoj-1].z - (double)pDoc->DTMCoord[gotoi][gotoj+1].z) / sidedist;

					if (slope < pDoc->RGLimit && sideslope < pDoc->SSLimit)
					{
						variable = VariableCostsRoadCell(dist, slope);
						fixed = FixedCostsRoadCell(dist, slope, sidedist, sideslope);
						if (pDoc->FishStreamBuffer[gotoi][gotoj] > 0)
							fixed *= pDoc->MultiplierForRoadsOnStream;

						//writing to file
						GenerateRoadNetwork_WritingtoFile(i, j, gotoi, gotoj, variable, fixed, outfile_ptr, (float)dist, (float)slope, (float)sideslope);
					}
				}
				

				LinkAvailability[i][j] = TRUE;
			}
		}
	}


	//add links from TimberExits to SuperExit
	for (i = 1; i <= pDoc->TotalNumberofTimberExits; ++i)
	{
		//one way
		++pDoc->TotalNumberofLinks;
		//writing to file
		tempString1.Format("C%dR%d", pDoc->TimberExits[i].column, pDoc->TimberExits[i].row);
		tempString2 = "SuperExit";
		fprintf(outfile_ptr, "%s	%s	%10.2f	%10.2f\n", tempString1, tempString2, 0.0, 0.0);
		//writing to memory
		pDoc->ConvertfromCStringToChar(pDoc->Links[pDoc->TotalNumberofLinks].FromNode, tempString1);
		pDoc->ConvertfromCStringToChar(pDoc->Links[pDoc->TotalNumberofLinks].ToNode, tempString2);
		pDoc->Links[pDoc->TotalNumberofLinks].VariableCost = 0.0;
		pDoc->Links[pDoc->TotalNumberofLinks].FixedCost = 0.0;	
	}


	//add links from existing roads to SuperExit
	for (j = 1; j <= pDoc->DTMrows; ++j)
	{
		for(i = 1; i <= pDoc->DTMcolumns; ++i)
		{
			if (pDoc->ExistingRoads[i][j] > 0)
			{
				//one way
				++pDoc->TotalNumberofLinks;
				//writing to file
				tempString1.Format("C%dR%d", i, j);
				tempString2 = "SuperExit";
				fprintf(outfile_ptr, "%s	%s	%10.2f	%10.2f\n", tempString1, tempString2, 0.0, 0.0);
				//writing to memory
				pDoc->ConvertfromCStringToChar(pDoc->Links[pDoc->TotalNumberofLinks].FromNode, tempString1);
				pDoc->ConvertfromCStringToChar(pDoc->Links[pDoc->TotalNumberofLinks].ToNode, tempString2);
				pDoc->Links[pDoc->TotalNumberofLinks].VariableCost = 0.0;
				pDoc->Links[pDoc->TotalNumberofLinks].FixedCost = 0.0;
			}
		}
	}


	fclose(outfile_ptr);

}



float CCablanView::VariableCostsRoadCell(double dist, double slope)
{
	//transportation costs (hauling costs)

	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	double slopedistance;
	float variable;

	slopedistance = sqrt( pow(dist, 2.0) + pow(dist * slope, 2.0));

	variable = (float)(slopedistance * (pDoc->HaulingCosts / 1000.)); //HaulingCosts is in KM, while distance is in meter.

	return (variable);
}



float CCablanView::FixedCostsRoadCell(double dist, double slope, double sidedist, double sideslope)
{
	//road construction costs 

	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	float fixed;
	float slopefactor, sideslopefactor, totalfactor;

	if (slope < 0.05)
		slopefactor = pDoc->RGFactor1;
	else if (slope < 0.1)
		slopefactor = pDoc->RGFactor2;
	else if (slope < 0.15)
		slopefactor = pDoc->RGFactor3;
	else if (slope < 0.2)
		slopefactor = pDoc->RGFactor4;
	else if (slope < 0.25)
		slopefactor = pDoc->RGFactor5;
	else if (slope < 0.3)
		slopefactor = pDoc->RGFactor6;
	else 
		slopefactor = pDoc->RGFactor7;


	if (sideslope < 0.15)
		sideslopefactor = pDoc->SSFactor1;
	else if (sideslope < 0.3)
		sideslopefactor = pDoc->SSFactor2;
	else if (sideslope < 0.45)
		sideslopefactor = pDoc->SSFactor3;
	else if (sideslope < 0.6)
		sideslopefactor = pDoc->SSFactor4;
	else if (sideslope < 0.75)
		sideslopefactor = pDoc->SSFactor5;
	else if (sideslope < 0.9)
		sideslopefactor = pDoc->SSFactor6;
	else 
		sideslopefactor = pDoc->SSFactor7;

	
	totalfactor = slopefactor * sideslopefactor;


	//Construction cost is in KM, while distance is in meter.
	fixed = (float)(dist * (pDoc->RoadCosts / 1000.) * totalfactor);
	

	return (fixed);
}



void CCablanView::GenerateRoadSaleData(float LandingVolume[], int Iteration)
{
	//does not use Iteration-->this links data are generated everytime when this subroutine is called.
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int k;
	int nosales;//total number of sales
	float volume;
	short year;
	CString tempString1, tempString2;
	int actuallanding;

	//initializing
	pDoc->TotalNumberofSales = 0;
	nosales = 0;

	FILE *outfile_ptr;
	CString csOutFileName;

	////////////////////////////////////////////////////
	//generate sale file for road network (from landings to final destination)
	csOutFileName = "e:\\thesis\\output\\saleroadmatrix.txt";
	outfile_ptr = fopen(csOutFileName, "w");

	//recalculate volume at landing for some towers sharing the same landing.
	for (k = 1; k <= pDoc->TotalNumberofLandings; ++k)
	{
		if (LandingVolume[k] > 0)
		{
			if (pDoc->LandingSharingTowerID[k] > 0)
			{
				actuallanding = pDoc->LandingSharingTowerID[k];
				//in case that this actuallandingID also shares with other landing.
				while (pDoc->LandingSharingTowerID[actuallanding] > 0)
					actuallanding = pDoc->LandingSharingTowerID[actuallanding];

				LandingVolume[actuallanding] += LandingVolume[k];
				LandingVolume[k] = 0.;
			}
		}
	}

	
	for (k = 1; k <= pDoc->TotalNumberofLandings; ++k)
	{
		if (LandingVolume[k] > 0)//if there is volume into the landing (only actual landings have volume --see above)
		{
			tempString1.Format("C%dR%d", pDoc->Landings[k].column, pDoc->Landings[k].row);//ColRow format
			//final destination
			tempString2 = "SuperExit";
			volume = LandingVolume[k];//Order of LandingVolume[] matches with that of k.
			year = 0;
			//writing to file
			fprintf(outfile_ptr, "%s	%s	%10.2f	%d\n", tempString1, tempString2, volume, year);

			//writing to memory
			++nosales;
			pDoc->ConvertfromCStringToChar(pDoc->Sales[nosales].EntryNode, tempString1);
			pDoc->ConvertfromCStringToChar(pDoc->Sales[nosales].DestNode, tempString2);
			pDoc->Sales[nosales].Volume = volume;
			pDoc->Sales[nosales].Year = year;	
		}
	}

	fclose(outfile_ptr);
	pDoc->TotalNumberofSales = nosales;

}








//*****************************************************************************************************
//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
//NETWORK ANALYSIS	
///////////////////////////////////////////////////////////////////////////////////////////////////////
//*****************************************************************************************************


void CCablanView::OnNetworkCablesystemsGeneratenetwork() 
{

}


void CCablanView::OnNetworkRoadsGeneratenetwork() 
{
	
	
}





void CCablanView::RunShortestPathHeuristic()
{
	///////////////////////////////////////////////////////////
	//This subroutine is for
	//1. running ShortestPathAlgorithm() with original volume set
	////////////////////////////////////////////////////////////////////////////
	// This routine calls DataManager()-->Reset all variable and Restart
	// This routine calls ShortestPathRun()

	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);


	int i;
	CString tempString;


	/////////////////////////////////////////
	//preparing to run problem solving algorithms
	/////////////////////////////////////////
	//rearrange everything
	//AFX_MANAGE_STATE(AfxGetStaticModuleState());
	pDoc->DataManager();//sorting and numbering user's input data

	

	//restore original volume
	pDoc->IterationNo = 0;
	pDoc->BestIterationNo = 0;
	pDoc->TotalCostfromShortestPathAlgorithm = (float)9999999999999999999.0;
	for (i = 1; i <= pDoc->TotalNumberofSales; ++i)
		pDoc->TemporaryVolume[i] = pDoc->Sales[i].Volume;


	//checking whether all nodes in the sale file are in the link file or not
	bool Error = FALSE;
	for (i = 1; i <= pDoc->TotalNumberofSales; ++i)
	{
		if((pDoc->Entry[i] == 0 || pDoc->Destination[i] == 0) && pDoc->SaleExclusion[i] != TRUE)
		{
			Error = TRUE;
			AfxMessageBox("Some sale data are not appropriate. Please check your sale data.", MB_ICONSTOP);
			return;
		}
	}
	
	

	////////////////////////////////////////////////
	//end of preparing data
	///////////////////////////////////////////////


	if (Error != TRUE && pDoc->TotalNumberofSales > 0)
	{
		pDoc->PreparingDataforShortestPath();
		pDoc->IsFirstIterationDone = TRUE;//should be placed before ShortestPathRun(), since it could be changed in ShortestPathRun()

		pDoc->ShortestPathRun();
	}		
}


void CCablanView::OnNetworkOptimizing() 
{
	////////////////////////////////////////////////////////////////////////////////////
	//Optimize cable system, landing, and road simultaneously.
	//1. Optimize cable system and landing location with zero road connection costs.
	//2. Take out timber harvest volume at each landing and put them into road network
	//3. Optimize road network
	//4. Take out road costs for each landing, and go back to step 1.
	//5. Repeat step 1 through step 4 until the same answer comes out.
	////////////////////////////////////////////////////////////////////////////////////

	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int i, j, k, iteration;
	int bestiteration;
	float BestSolutionCable;//keep total costs from the best solution
	float BestSolutionRoad;
	float BestSolutionTotal;
	CString tempString;
	float PreviousSolution;
	float CurrentSolution;
	double seconds = 0.0;


	//set Unit boundary
	if (pDoc->UnitBoundary.startCol <= 0)//; including 0 for release mode
	{
		AfxMessageBox("You need to designate harvest unit boundaries.", MB_ICONEXCLAMATION);
	}
	

	//check whether the timber exit(s) is already input.
	bool tempExit = TRUE;
	if (pDoc->TimberExits[1].column <= 0 || pDoc->TimberExits[1].row <= 0)//; including 0 for release mode
	{
		tempExit = FALSE;
		for (j = 1; j <= pDoc->DTMrows; ++j)
		{
			for(i = 1; i <= pDoc->DTMcolumns; ++i)
			{
				if (pDoc->ExistingRoads[i][j] > 0)
				{
					j = pDoc->DTMrows;
					tempExit = TRUE;
					break;
				}
			}
		}
	}
	
	if (tempExit == FALSE)
	{
		AfxMessageBox("You have not placed timber exit location(s).", MB_ICONEXCLAMATION);
		return;
	}


	//for measurign time
	COleDateTime timeStart, timeEnd;
	COleDateTimeSpan spanElapsed;
	double TotalDay, TotalHour, TotalMinute, TotalSecond;
	//end of measuring time
	//saving time
	FILE *outfile_ptr;
	CString tempFileTime = "E:\\thesis\\output\\T&Results.txt";
	outfile_ptr = fopen(tempFileTime, "w");


	//initializing road costs for each landing
	CurrentSolution = 0.;
	BestSolutionTotal = (float)9999999999999999999999.;
	iteration = 0;
	for (i = 1; i <= pDoc->TotalNumberofLandings; ++i)
	{
		RoadVariableCostsperLanding[i] = 0.;
		RoadFixedCostsperLanding[i] = 0.;
	}


	//saving the best solution as the file
	FILE *infile1_ptr, *outfile1_ptr;
	CString tempString1, tempString2;
	int ch;


	//time start
	timeStart = COleDateTime::GetCurrentTime();

	do
	{
		++iteration;
		PreviousSolution = CurrentSolution;
		//Step 1.
		GenerateCableSystemNetwork(RoadVariableCostsperLanding, RoadFixedCostsperLanding, iteration);
		GenerateCableSystemSaleData(iteration);
		tempString = "e:\\thesis\\output\\cablepath.txt";//define file name for keeping the best route
		pDoc->ChangecsFileName(tempString);
		
		tempString2.Format("%d.txt", iteration);
		pDoc->csTempOutputFile = "e:\\thesis\\output\\cableoptimizing";//keep recording of each iteration 
		pDoc->csTempOutputFile += tempString2;
		RunShortestPathHeuristic();
		BestSolutionCable = pDoc->TotalCostfromShortestPathAlgorithm;
		DrawDTM();
		DrawCablePath(pDoc->csFileName);//draw output on screen and save data to Results variables.


		//Step 2.
		//keep label for landing nodes
		for (k = 1; k <= pDoc->TotalNumberofLandings; ++k)
		{
			LandingVolume[k] = FindVolumeatLanding(k);
		}

	
		//Step 3.
		GenerateRoadNetwork(iteration);
		GenerateRoadSaleData(LandingVolume, iteration);
		pDoc->csFileName = "E:\\thesis\\output\\roadpath.txt";//define file name for keeping the best route
		
		tempString2.Format("%d.txt", iteration);
		pDoc->csTempOutputFile = "e:\\thesis\\output\\roadoptimizing";//keep recording of each iteration 
		pDoc->csTempOutputFile += tempString2;
		RunShortestPathHeuristic();
		BestSolutionRoad = pDoc->TotalCostfromShortestPathAlgorithm;
		DrawRoadPath(pDoc->csFileName);//draw output on screen and save data to Results variables.


		CalculateVariableandFixedCostsperSale();//fill RoadVariableCostsperLanding[] and RoadFixedCostsperLanding[] variables;

		CurrentSolution = BestSolutionCable + BestSolutionRoad;

		//without road part in cable network.
		CurrentSolution = RecalculateTotalVariableandFixedCostsfromEachCelltoDestination(iteration);//this subroutine calls DrawCablePath() nad DrawRoadPath() which fill the result variables in. --> we can save the results after this subroutine.

		//keep the best solution
		if (BestSolutionTotal > CurrentSolution)
		{
			bestiteration = iteration;
			BestSolutionTotal = CurrentSolution;
			//file copy
			tempString1 = "e:\\thesis\\output\\cablepath.txt";//the best path at each optimization process-subject to change after the best is found.
			infile1_ptr = fopen(tempString1, "rb");
			tempString1 = "e:\\thesis\\output\\bestcablepath.txt";//the best path after including road 
			outfile1_ptr = fopen(tempString1, "w");
			while ( (ch = fgetc(infile1_ptr) ) != EOF )
				fputc(ch, outfile1_ptr);

			fclose(infile1_ptr);
			fclose(outfile1_ptr);

			//file copy
			tempString1 = "e:\\thesis\\output\\roadpath.txt";
			infile1_ptr = fopen(tempString1, "rb");
			tempString1 = "e:\\thesis\\output\\bestroadpath.txt";
			outfile1_ptr = fopen(tempString1, "w");
			while ( (ch = fgetc(infile1_ptr) ) != EOF )
				fputc(ch, outfile1_ptr);

			fclose(infile1_ptr);
			fclose(outfile1_ptr);

			SaveResultstoFile();//save all results file for drawing the results.
		}

		// measuring time
		// ... perform time-consuming task
		timeEnd = COleDateTime::GetCurrentTime();
		spanElapsed = timeEnd - timeStart;
		TotalDay = spanElapsed.GetDays();
		TotalHour = spanElapsed.GetHours();
		TotalMinute = spanElapsed.GetMinutes();
		TotalSecond = spanElapsed.GetSeconds();
		
		fprintf(outfile_ptr, "Iteration %d\n", iteration);
//		fprintf(outfile_ptr, "Day %f\n", TotalDay);
//		fprintf(outfile_ptr, "Hour %f\n", TotalHour);
//		fprintf(outfile_ptr, "Minute %f\n", TotalMinute);
//		fprintf(outfile_ptr, "Second %f\n", TotalSecond);
		fprintf(outfile_ptr, "Hour Min Sec %f %f %f \n", TotalHour, TotalMinute, TotalSecond);
		fprintf(outfile_ptr, "Cab Road Total %f %f %f\n", BestSolutionCable, BestSolutionRoad, CurrentSolution);//CurrentSolution is overall costs. BestSolutionCable includes road cost as well.
		

//		fprintf(outfile_ptr, "Diff %f\n", CurrentSolution - PreviousSolution);
//		fprintf(outfile_ptr, "Road %f\n", BestSolutionRoad);
//		fprintf(outfile_ptr, "Total %f\n\n", CurrentSolution);
		//end of measuring time

	} while (fabs(PreviousSolution - CurrentSolution) > 0.001 * CurrentSolution && iteration < 10);//iteration????????????????????????


	fclose(outfile_ptr);


	//Step 4  -- recalculate the best solution from the each volume cell to the final destination.
	//		  -- write reports using CableReports(), RoadReports(), and "T&ResultsSep.txt"
	CurrentSolution = RecalculateBestTotalVariableandFixedCostsfromEachCelltoDestination(bestiteration);
	SaveResultstoFile();

//	OnNetworkOptimizingallatonce();

}



float CCablanView::FindVolumeatLanding(int LandingID)
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int i;
	CString tempString;
	char tempChar[MAX_CHAR];
	float Volume;


	tempString.Format("L%d", LandingID);//Landing (L1, L2, L3....)
	pDoc->ConvertfromCStringToChar(tempChar, tempString);

	for (i = 1; i <= pDoc->TotalNumberofLinks; ++i)
	{
		//finding right link number
		if (strcmp(pDoc->Links[i].FromNode, tempChar) == 0)//indicating Landing (L1, L2, L3....)
		{
			Volume = pDoc->AccumulatedVolumeperLink[i];
			break;
		}
	}

	return (Volume);

}




float CCablanView::RecalculateTotalVariableandFixedCostsfromEachCelltoDestination(int Iteration)
{
	//FINAL REPORTS///////////////////////
	//read from the best path files (bestcablepath.txt and bestroadpath.txt)
	//recalculate overall costs and write the results to "T&ResultsSep.txt".
	//draw the best results and assign the values into result variables using DrawCablePath() and DrawRoadPath().
	
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);


	int i, j;
	int totalnumberofsales;
	int Method;
	int SaleNumber, NumberofLinksinPath;
	float Volume;
	int beginNode, endNode, tempLinkID, tempNodeID;
	float totalvariable = 0.;
	float totalfixed = 0.;
	CString tempString;
	float cablevariable = 0.;
	float cablefixed = 0.;
	float roadvariable = 0.;
	float roadfixed = 0.;
	CString FileName;

	float tempRoadVariableCostsperLanding[MAXLANDINGS];//data transfered from road network to cable system network
	float tempRoadFixedCostsperLanding[MAXLANDINGS];//data transfered from road network to cable system network


	//store in temporary variables
	memcpy(tempRoadVariableCostsperLanding, RoadVariableCostsperLanding, sizeof(tempRoadVariableCostsperLanding));
	memcpy(tempRoadFixedCostsperLanding, RoadFixedCostsperLanding, sizeof(tempRoadFixedCostsperLanding));


	endNode = 0;
	
	//initializing road costs for each landing
	for (i = 1; i <= pDoc->TotalNumberofLandings; ++i)
	{
		RoadVariableCostsperLanding[i] = 0.;
		RoadFixedCostsperLanding[i] = 0.;
	}

	GenerateCableSystemNetwork(RoadVariableCostsperLanding, RoadFixedCostsperLanding, 100);//skip the generating all links, just generate links from landing to dummy destination with 0 road costs.
	GenerateCableSystemSaleData(100);//?????????????????
	pDoc->DataManager();//resorting

	//initializing
	for (i = 1; i <= pDoc->TotalNumberofLinks; ++i)
		pDoc->AccumulatedVolumeperLink[i] = 0.0;//initializing--> this will contain orginal fixed sale volume (Sales[].Volume)
	


	/////////////////////////////////////////////////////////////////////
	//Cable part
	/////////////////////////////////////////////////////////////////////

	FILE* infile_ptr;
	FileName = "E:\\thesis\\output\\cablepath.txt";//define file name for keeping the best route
	infile_ptr = fopen(FileName, "rb");//csFileName is set at pDoc->SaveResultsToFile()

	//read total number of sales including excluded sales
	fscanf(infile_ptr, "%d", &Method);//which algorithm found this solution->Global variable
	fscanf(infile_ptr, "%d", &totalnumberofsales);//local temporary variable -> just for skipping this data in file


	//First; count volume on AccumulatedVolumeperLink
	for (i = 1; i <= totalnumberofsales; ++i)
	{
		if (pDoc->SaleExclusion[i] != TRUE)
		{
			//read result for each sale
			fscanf(infile_ptr, "%d", &SaleNumber);
			fscanf(infile_ptr, "%f", &Volume);
			fscanf(infile_ptr, "%d", &NumberofLinksinPath);

			if (NumberofLinksinPath <= 0)
			{
				AfxMessageBox("One of the sales could not be found.", MB_ICONEXCLAMATION);
				break;
			}
			else
			{
				//put volume into AccumulatedVolumeperLink[]
				for (j = 1; j <= NumberofLinksinPath; ++j)
				{
					beginNode = endNode;
					fscanf(infile_ptr, "%d", &tempNodeID);//right order, not in reverse order
					endNode = tempNodeID;
					
					//find link ID
					if (j >= 2)//from the second node
					{
						tempLinkID = pDoc->FindLinkNumberbetweenTwoNodes(beginNode, endNode);
						pDoc->AccumulatedVolumeperLink[tempLinkID] += Volume;
					}
				}
			}
		}
	}

	
	//calculate fixed and variable costs
	for (i = 1; i <= pDoc->TotalNumberofLinks; ++i)
	{
		if (pDoc->AccumulatedVolumeperLink[i] > 0)
		{
			cablefixed += pDoc->Links[i].FixedCost;
			cablevariable += pDoc->Links[i].VariableCost * pDoc->AccumulatedVolumeperLink[i];
		}
	}


	fclose(infile_ptr);//close file

	
	DrawCablePath(FileName);//draw and fill the result variables.
//	CableReports(FileName);//write textfile with node name.



	//Step 2.
	//keep label for landing nodes
	for (i = 1; i <= pDoc->TotalNumberofLandings; ++i)
		LandingVolume[i] = FindVolumeatLanding(i);
	



	//////////////////////////////////////////////////
	//road part
	//////////////////////////////////////////////////

	GenerateRoadNetwork(10);//10 is just number bigger than 0..--> see GenerateRoadNetwork();
	GenerateRoadSaleData(LandingVolume, 10);
	pDoc->DataManager();//resorting	
	

	//initializing
	for (i = 1; i <= pDoc->TotalNumberofLinks; ++i)
		pDoc->AccumulatedVolumeperLink[i] = 0.0;//initializing--> this will contain orginal fixed sale volume (Sales[].Volume)



	FileName = "E:\\thesis\\output\\roadpath.txt";//define file name for keeping the best route
	infile_ptr = fopen(FileName, "rb");//csFileName is set at pDoc->SaveResultsToFile()

	
	//read total number of sales including excluded sales
	fscanf(infile_ptr, "%d", &Method);//which algorithm found this solution->Global variable
	fscanf(infile_ptr, "%d", &totalnumberofsales);//local temporary variable -> just for skipping this data in file


	//First; count volume on AccumulatedVolumeperLink
	for (i = 1; i <= totalnumberofsales; ++i)
	{
		if (pDoc->SaleExclusion[i] != TRUE)
		{
			//read result for each sale
			fscanf(infile_ptr, "%d", &SaleNumber);
			fscanf(infile_ptr, "%f", &Volume);
			fscanf(infile_ptr, "%d", &NumberofLinksinPath);

			if (NumberofLinksinPath <= 0)
			{
				AfxMessageBox("One of the sales could not be found.", MB_ICONEXCLAMATION);
				break;
			}
			else
			{
				for (j = 1; j <= NumberofLinksinPath; ++j)
				{
					beginNode = endNode;
					fscanf(infile_ptr, "%d", &tempNodeID);//right order, not in reverse order
					endNode = tempNodeID;
					
					//find link ID
					if (j >= 2)//from the second node
					{
						tempLinkID = pDoc->FindLinkNumberbetweenTwoNodes(beginNode, endNode);
						pDoc->AccumulatedVolumeperLink[tempLinkID] += Volume;
					}
				}
			}
		}
	}


	//calculate fixed and variable costs
	for (i = 1; i <= pDoc->TotalNumberofLinks; ++i)
	{
		if (pDoc->AccumulatedVolumeperLink[i] > 0)
		{
			roadfixed += pDoc->Links[i].FixedCost;
			roadvariable += pDoc->Links[i].VariableCost * pDoc->AccumulatedVolumeperLink[i];
		}
	}


	totalvariable = cablevariable + roadvariable;
	totalfixed = cablefixed + roadfixed;


	fclose(infile_ptr);//close file
	
	DrawRoadPath(FileName);//draw and fill the result variables.
//	RoadReports(FileName);//write textfile with node name.


	//output file for summary
	FILE *outfile_ptr;
	CString tempFile1;
	tempFile1.Format("E:\\thesis\\output\\T&ResultsSep%d.txt", Iteration);
	outfile_ptr = fopen(tempFile1, "w");

	fprintf(outfile_ptr, "Iteration: %d\n", Iteration);
	fprintf(outfile_ptr, "CableV: %f	CableF: %f\n", cablevariable, cablefixed);
	fprintf(outfile_ptr, "RoadV: %f RoadF: %f\n", roadvariable, roadfixed);
	fprintf(outfile_ptr, "TotalV: %f TotalF: %f\n", totalvariable, totalfixed);
	fprintf(outfile_ptr, "TotalCable: %f TotalRoad: %f\n", cablevariable + cablefixed, roadvariable + roadfixed);
	fprintf(outfile_ptr, "OverallTotal: %f\n", totalvariable + totalfixed);
	fclose(outfile_ptr);


	//restore back to original variables
	memcpy(RoadVariableCostsperLanding, tempRoadVariableCostsperLanding, sizeof(tempRoadVariableCostsperLanding));
	memcpy(RoadFixedCostsperLanding, tempRoadFixedCostsperLanding, sizeof(tempRoadFixedCostsperLanding));


	return(totalvariable + totalfixed);


}

float CCablanView::RecalculateBestTotalVariableandFixedCostsfromEachCelltoDestination(int Iteration)
{
	//FINAL REPORTS///////////////////////
	//read from the best path files (bestcablepath.txt and bestroadpath.txt)
	//recalculate overall costs and write the results to "T&ResultsSep.txt".
	//draw the best results and assign the values into result variables using DrawCablePath() and DrawRoadPath().
	//write reports to the file with node name using CableReports() and RoadReports()
	
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);


	int i, j;
	int totalnumberofsales;
	int Method;
	int SaleNumber, NumberofLinksinPath;
	float Volume;
	int beginNode, endNode, tempLinkID, tempNodeID;
	float totalvariable = 0.;
	float totalfixed = 0.;
	CString tempString;
	float cablevariable = 0.;
	float cablefixed = 0.;
	float roadvariable = 0.;
	float roadfixed = 0.;
	CString FileName;

	float tempRoadVariableCostsperLanding[MAXLANDINGS];//data transfered from road network to cable system network
	float tempRoadFixedCostsperLanding[MAXLANDINGS];//data transfered from road network to cable system network


	//store in temporary variables
	memcpy(tempRoadVariableCostsperLanding, RoadVariableCostsperLanding, sizeof(tempRoadVariableCostsperLanding));
	memcpy(tempRoadFixedCostsperLanding, RoadFixedCostsperLanding, sizeof(tempRoadFixedCostsperLanding));


	endNode = 0;
	
	//initializing road costs for each landing
	for (i = 1; i <= pDoc->TotalNumberofLandings; ++i)
	{
		RoadVariableCostsperLanding[i] = 0.;
		RoadFixedCostsperLanding[i] = 0.;
	}

	GenerateCableSystemNetwork(RoadVariableCostsperLanding, RoadFixedCostsperLanding, 100);//skip the generating all links, just generate links from landing to dummy destination with 0 road costs.
	GenerateCableSystemSaleData(100);//???????????????
	pDoc->DataManager();//resorting

	//initializing
	for (i = 1; i <= pDoc->TotalNumberofLinks; ++i)
		pDoc->AccumulatedVolumeperLink[i] = 0.0;//initializing--> this will contain orginal fixed sale volume (Sales[].Volume)
	


	/////////////////////////////////////////////////////////////////////
	//Cable part
	/////////////////////////////////////////////////////////////////////

	FILE* infile_ptr;
	FileName = "E:\\thesis\\output\\bestcablepath.txt";//define file name for keeping the best route
	infile_ptr = fopen(FileName, "rb");//csFileName is set at pDoc->SaveResultsToFile()

	//read total number of sales including excluded sales
	fscanf(infile_ptr, "%d", &Method);//which algorithm found this solution->Global variable
	fscanf(infile_ptr, "%d", &totalnumberofsales);//local temporary variable -> just for skipping this data in file


	//First; count volume on AccumulatedVolumeperLink
	for (i = 1; i <= totalnumberofsales; ++i)
	{
		if (pDoc->SaleExclusion[i] != TRUE)
		{
			//read result for each sale
			fscanf(infile_ptr, "%d", &SaleNumber);
			fscanf(infile_ptr, "%f", &Volume);
			fscanf(infile_ptr, "%d", &NumberofLinksinPath);

			if (NumberofLinksinPath <= 0)
			{
				AfxMessageBox("One of the sales could not be found.", MB_ICONEXCLAMATION);
				break;
			}
			else
			{
				//put volume into AccumulatedVolumeperLink[]
				for (j = 1; j <= NumberofLinksinPath; ++j)
				{
					beginNode = endNode;
					fscanf(infile_ptr, "%d", &tempNodeID);//right order, not in reverse order
					endNode = tempNodeID;
					
					//find link ID
					if (j >= 2)//from the second node
					{
						tempLinkID = pDoc->FindLinkNumberbetweenTwoNodes(beginNode, endNode);
						pDoc->AccumulatedVolumeperLink[tempLinkID] += Volume;
					}
				}
			}
		}
	}

	
	//calculate fixed and variable costs
	for (i = 1; i <= pDoc->TotalNumberofLinks; ++i)
	{
		if (pDoc->AccumulatedVolumeperLink[i] > 0)
		{
			cablefixed += pDoc->Links[i].FixedCost;
			cablevariable += pDoc->Links[i].VariableCost * pDoc->AccumulatedVolumeperLink[i];
		}
	}


	fclose(infile_ptr);//close file

	
	DrawCablePath(FileName);//draw and fill the result variables.
	CableReports(FileName);//write textfile with node name.



	//Step 2.
	//keep label for landing nodes
	for (i = 1; i <= pDoc->TotalNumberofLandings; ++i)
		LandingVolume[i] = FindVolumeatLanding(i);
	



	//////////////////////////////////////////////////
	//road part
	//////////////////////////////////////////////////

	GenerateRoadNetwork(10);
	GenerateRoadSaleData(LandingVolume, 10);
	pDoc->DataManager();//resorting	
	

	//initializing
	for (i = 1; i <= pDoc->TotalNumberofLinks; ++i)
		pDoc->AccumulatedVolumeperLink[i] = 0.0;//initializing--> this will contain orginal fixed sale volume (Sales[].Volume)



	FileName = "E:\\thesis\\output\\bestroadpath.txt";//define file name for keeping the best route
	infile_ptr = fopen(FileName, "rb");//csFileName is set at pDoc->SaveResultsToFile()

	
	//read total number of sales including excluded sales
	fscanf(infile_ptr, "%d", &Method);//which algorithm found this solution->Global variable
	fscanf(infile_ptr, "%d", &totalnumberofsales);//local temporary variable -> just for skipping this data in file


	//First; count volume on AccumulatedVolumeperLink
	for (i = 1; i <= totalnumberofsales; ++i)
	{
		if (pDoc->SaleExclusion[i] != TRUE)
		{
			//read result for each sale
			fscanf(infile_ptr, "%d", &SaleNumber);
			fscanf(infile_ptr, "%f", &Volume);
			fscanf(infile_ptr, "%d", &NumberofLinksinPath);

			if (NumberofLinksinPath <= 0)
			{
				AfxMessageBox("One of the sales could not be found.", MB_ICONEXCLAMATION);
				break;
			}
			else
			{
				for (j = 1; j <= NumberofLinksinPath; ++j)
				{
					beginNode = endNode;
					fscanf(infile_ptr, "%d", &tempNodeID);//right order, not in reverse order
					endNode = tempNodeID;
					
					//find link ID
					if (j >= 2)//from the second node
					{
						tempLinkID = pDoc->FindLinkNumberbetweenTwoNodes(beginNode, endNode);
						pDoc->AccumulatedVolumeperLink[tempLinkID] += Volume;
					}
				}
			}
		}
	}


	//calculate fixed and variable costs
	for (i = 1; i <= pDoc->TotalNumberofLinks; ++i)
	{
		if (pDoc->AccumulatedVolumeperLink[i] > 0)
		{
			roadfixed += pDoc->Links[i].FixedCost;
			roadvariable += pDoc->Links[i].VariableCost * pDoc->AccumulatedVolumeperLink[i];
		}
	}


	totalvariable = cablevariable + roadvariable;
	totalfixed = cablefixed + roadfixed;


	fclose(infile_ptr);//close file
	
	DrawRoadPath(FileName);//draw and fill the result variables.
	RoadReports(FileName);//write textfile with node name.


	//output file for summary
	FILE *outfile_ptr;
	CString tempFile1;
	tempFile1.Format("E:\\thesis\\output\\T&ResultsSepFinal.txt");
	outfile_ptr = fopen(tempFile1, "w");

	fprintf(outfile_ptr, "Best Iteration: %d\n", Iteration);
	fprintf(outfile_ptr, "CableV: %f	CableF: %f\n", cablevariable, cablefixed);
	fprintf(outfile_ptr, "RoadV: %f RoadF: %f\n", roadvariable, roadfixed);
	fprintf(outfile_ptr, "TotalV: %f TotalF: %f\n", totalvariable, totalfixed);
	fprintf(outfile_ptr, "OverallTotal: %f\n", totalvariable + totalfixed);
	fclose(outfile_ptr);


	//restore back to original variables
	memcpy(RoadVariableCostsperLanding, tempRoadVariableCostsperLanding, sizeof(tempRoadVariableCostsperLanding));
	memcpy(RoadFixedCostsperLanding, tempRoadFixedCostsperLanding, sizeof(tempRoadFixedCostsperLanding));


	return(totalvariable + totalfixed);


}









void CCablanView::CalculateVariableandFixedCostsperSale()
{
	//this subroutine should be called right after run ShortestPathHeuristic(), since it uses pDoc->SaleExclusion and other information.
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	int i, j, k;
	int tempNodeID;//keeps node ID on the best path for a sale.
	int beginNode, endNode;
	int tempLinkID;//keeps link ID on the best path for a sale.
	float totalvariable = 0.0;
	float totalfixed = 0.0;
	CString tempString;
	char tempChar[MAX_CHAR];
	endNode = 0;
		
	short Method;
	int totalnumberofsales;
	int SaleNumber;
	float Volume;
	int NumberofLinksinPath;
	FILE* infile_ptr;


	infile_ptr = fopen(pDoc->csFileName, "rb");//csFileName is set at pDoc->SaveResultsToFile()

	//read total number of sales including excluded sales
	fscanf(infile_ptr, "%d", &Method);//which algorithem found this solution->Global variable
	fscanf(infile_ptr, "%d", &totalnumberofsales);//local temporary variable -> just for skipping this data in file

	for (i = 1; i <= totalnumberofsales; ++i)
	{
		totalvariable = 0.0;
		totalfixed = 0.0;
		if (pDoc->SaleExclusion[i] != TRUE)
		{
			//read result for each sale
			fscanf(infile_ptr, "%d", &SaleNumber);
			fscanf(infile_ptr, "%f", &Volume);
			fscanf(infile_ptr, "%d", &NumberofLinksinPath);

			if (NumberofLinksinPath <= 0)
			{
				AfxMessageBox("One of the sales could not be found.", MB_ICONEXCLAMATION);
				break;
			}
			else
			{
				for (j = 1; j <= NumberofLinksinPath; ++j)
				{
					beginNode = endNode;
					fscanf(infile_ptr, "%d", &tempNodeID);//right order, not in reverse order
					endNode = tempNodeID;
					
					//find link ID
					if (j >= 2)//from the second node
					{
						tempLinkID = pDoc->FindLinkNumberbetweenTwoNodes(beginNode, endNode);
						totalvariable += pDoc->Links[tempLinkID].VariableCost;
						if (pDoc->AccumulatedVolumeperLink[tempLinkID] > 0)
							totalfixed += pDoc->Links[tempLinkID].FixedCost * (Volume / pDoc->AccumulatedVolumeperLink[tempLinkID]);//equivalent variable costs after calculating proportion of fixed costs.
					}
				}
			}
		}
		
		//find right landing ID
		for (k = 1; k <= pDoc->TotalNumberofLandings; ++k)
		{
			tempString.Format("C%dR%d", pDoc->Landings[k].column, pDoc->Landings[k].row);//ColRow format
			pDoc->ConvertfromCStringToChar(tempChar, tempString);
			//finding right link number
			if (strcmp(pDoc->Sales[i].EntryNode, tempChar) == 0)//if sale number is match with landing ID
			{
				RoadVariableCostsperLanding[k] = totalvariable;//keep variable costs for this landing (road costs)
				RoadFixedCostsperLanding[k] = totalfixed;
				break;
			}
		}
	}

	fclose(infile_ptr);//close file

}

void CCablanView::OnNetworkOptimizingallatonce() 
{
	////////////////////////////////////////////////////////////////////////////////////
	//Optimize cable system, landing, and road simultaneously.
	//1. Optimize cable system and landing location with zero road connection costs.
	//2. Take out timber harvest volume at each landing and put them into road network
	//3. Optimize road network
	//4. Take out road costs for each landing, and go back to step 1.
	//5. Repeat step 1 through step 4 until the same answer comes out.
	////////////////////////////////////////////////////////////////////////////////////

	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CString tempString;
	float CurrentSolution;


	//for measuring time
	COleDateTime timeStart, timeEnd;
	COleDateTimeSpan spanElapsed;
	double TotalDay, TotalHour, TotalMinute, TotalSecond;
	//end of measuring time
	//saving time
	FILE *outfile_ptr;
	CString tempFileTime = "E:\\thesis\\output\\T&ResultsAllAtOnce.txt";
	outfile_ptr = fopen(tempFileTime, "w");

	
	//all at once

	//initializing road costs for each landing
	CurrentSolution = 0.;


	//time start
	timeStart = COleDateTime::GetCurrentTime();
		
		
	
	//Step 1.
	GenerateCableSystemNetworkAllAtOnce();
	GenerateRoadNetworkAllAtOnce();
	GenerateCableSystemSaleDataAllAtOnce();
	pDoc->csFileName = "e:\\thesis\\output\\allpath.txt";//define file name for keeping the best route
	RunShortestPathHeuristic();
	CurrentSolution = pDoc->TotalCostfromShortestPathAlgorithm;
	DrawAllPath();



	// measuring time
	// ... perform time-consuming task
	timeEnd = COleDateTime::GetCurrentTime();
	spanElapsed = timeEnd - timeStart;
	TotalDay = spanElapsed.GetDays();
	TotalHour = spanElapsed.GetHours();
	TotalMinute = spanElapsed.GetMinutes();
	TotalSecond = spanElapsed.GetSeconds();
		
	fprintf(outfile_ptr, "Hour Min Sec Total %f %f %f %f\n", TotalHour, TotalMinute, TotalSecond, CurrentSolution);
	//end of measuring time

	fclose(outfile_ptr);

	
}




void CCablanView::DrawCablePath(CString FileName)
{
	//drawing path from each volume cell to landing after optimized.
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pDC = new CClientDC(this);
	OnPrepareDC(pDC);

	pDC->SetMapMode(MM_ISOTROPIC);
	pDC->SetWindowExt(ViewScale, ViewScale);


	int i, j, k;
	int totalnumberofsales;
	int Method;
	int SaleNumber, NumberofLinksinPath;
	float Volume;
	int tempNodeID;
	int column, row, cable, system, landing;
	CString tempString;
	double x, y, x1, y1, m, b, dist;
	int maxX, maxY, minX, minY;
	bool feasibility;
	int landingX, landingY, tailX, tailY;
	int tempint;
	double tempdist, tempshortestdistfromanycables;
	double tempx, tempy;


	penColor[2].CreatePen(PS_SOLID, 5, RGB(150,150,150));
	CPen *pOldPen = pDC->SelectObject(&penColor[2]);


	//initializing for result variables
	TotalNumberofFinalCableways = 0;
	for (i = 0; i < MAXLANDINGS * MAXCABLESYSTEMS * MAXCABLEWAYS; ++i)
	{
		CableResults[i].cableway = 0;
		CableResults[i].landing = 0;
		CableResults[i].system = 0;
	}
	for (i = 0; i < MAXCOLUMNS; ++i)
	{
		for (j = 0; j < MAXROWS; ++j)
		{
			FromCellToCableResults[i][j].x = 0;
			FromCellToCableResults[i][j].y = 0;
		}
	}

	

	//repaint with white brush
	for (j = pDoc->UnitBoundary.startRow; j <= pDoc->UnitBoundary.endRow; ++j)
	{
		for(i = pDoc->UnitBoundary.startCol; i <= pDoc->UnitBoundary.endCol; ++i)
		{
			if (pDoc->CellsWithinUnitBoundary[i][j] == TRUE)
				DrawEachCell(i, j, RGB(255,255,255));
		}
	}


	FILE* infile_ptr;
	infile_ptr = fopen(FileName, "rb");

	//read total number of sales including excluded sales
	fscanf(infile_ptr, "%d", &Method);//which algorithm found this solution->Global variable
	fscanf(infile_ptr, "%d", &totalnumberofsales);//local temporary variable -> just for skipping this data in file

	for (i = 1; i <= totalnumberofsales; ++i)
	{
		if (pDoc->SaleExclusion[i] != TRUE)
		{
			//read result for each sale
			fscanf(infile_ptr, "%d", &SaleNumber);
			fscanf(infile_ptr, "%f", &Volume);
			fscanf(infile_ptr, "%d", &NumberofLinksinPath);

			if (NumberofLinksinPath <= 0)
			{
				AfxMessageBox("One of the sales could not be found.", MB_ICONEXCLAMATION);
				break;
			}
			else
			{
				for (j = 1; j <= NumberofLinksinPath; ++j)//NumberofLinksinPath has to be 4 (origin cell(C#R#), system(C#S#L#), landing(L#), and dummy destination). 
				{
					fscanf(infile_ptr, "%d", &tempNodeID);//right order, not in reverse order
					
					if (pDoc->Label_NumberToString[tempNodeID][0] == 'C')
					{
						k = 1;//skip the first letter which is 'C'
						tempString = "";
						while (pDoc->Label_NumberToString[tempNodeID][k] != 'R' && pDoc->Label_NumberToString[tempNodeID][k] != 'S')//Row or System
						{
							tempString += pDoc->Label_NumberToString[tempNodeID][k];
							++k;
						}

						if (pDoc->Label_NumberToString[tempNodeID][k] == 'R')
						{
							column = atoi(tempString);
	
							tempString = "";
							++k;
							while (k < MAX_CHAR)
							{
								tempString += pDoc->Label_NumberToString[tempNodeID][k];
								++k;
							}
							row = atoi(tempString);
						}
						else//system and landing
						{
							cable = atoi(tempString);
								
							tempString = "";
							++k;
							while (pDoc->Label_NumberToString[tempNodeID][k] != 'L')
							{
								tempString += pDoc->Label_NumberToString[tempNodeID][k];
								++k;
							}
							system = atoi(tempString);

							tempString = "";
							++k;							
							while (k < MAX_CHAR)
							{
								tempString += pDoc->Label_NumberToString[tempNodeID][k];
								++k;
							}
							landing = atoi(tempString);
						}
					}
					else//landing
					{
						//we can skip landing number since we know the ID number in above.
					}
				}

				//keep data in result variables
				++TotalNumberofFinalCableways;
				CableResults[TotalNumberofFinalCableways].cableway = cable;
				CableResults[TotalNumberofFinalCableways].landing = landing;
				CableResults[TotalNumberofFinalCableways].system = system;
			}
		}


		//drawing line from the cell to the cable way.
		//measuring distance from the cell to the cableway
		//this part is identical to the part of GenerateCableSystemNetwork
		x1 = pDoc->DTMCoord[column][row].x;//cell coordinate
		y1 = pDoc->DTMCoord[column][row].y;
		m = pDoc->Cableways[landing][cable].slope;
		b = pDoc->Cableways[landing][cable].intercept;

		if (fabs(m) < 0.0001)
		{
			x = x1;
			y = b;
		}
		else if (m > 100000.0)
		{
			y = y1;
			x = pDoc->DTMCoord[pDoc->Landings[landing].column][pDoc->Landings[landing].row].x;
		}
		else
		{
			x = (y1 + 1.0/m*x1 - b)*m/(pow(m,2.0)+1);
			y = m*x + b;
		}

		//x and y selected above may not be within the cable road boundary. Considering only slope may result in finding a point which is located in opposite direction of the cable road.
		//if x and y is not within the cable road boundary, then find the closest point from the cable road.
		//also look as GenerateCableSystemNetwork()
		dist = sqrt(pow(x1-x, 2.0) + pow(y1-y, 2.0));
		maxX = pDoc->DTMCoord[pDoc->Cableways[landing][cable].tailcolumn][pDoc->Cableways[landing][cable].tailrow].x;
		minX = pDoc->DTMCoord[pDoc->Landings[landing].column][pDoc->Landings[landing].row].x;

		if (minX > maxX)
		{
			tempint = maxX;
			maxX = minX;
			minX = tempint;
		}
		maxY = pDoc->DTMCoord[pDoc->Cableways[landing][cable].tailcolumn][pDoc->Cableways[landing][cable].tailrow].y;
		minY = pDoc->DTMCoord[pDoc->Landings[landing].column][pDoc->Landings[landing].row].y;
		if (minY > maxY)
		{
			tempint = maxY;
			maxY = minY;
			minY = tempint;
		}
		
		feasibility = TRUE;
		if (x > maxX || x < minX)
			feasibility = FALSE;

		if (y > maxY || y < minY)
			feasibility = FALSE;
		
		if (dist > pDoc->CableSystems[system].maxlateraldist)//maximum lateral yarding distance.
			feasibility = FALSE;


		tempx = x;
		tempy = y;
		landingX = pDoc->DTMCoord[pDoc->Landings[landing].column][pDoc->Landings[landing].row].x;
		landingY = pDoc->DTMCoord[pDoc->Landings[landing].column][pDoc->Landings[landing].row].y;
		tailX = pDoc->DTMCoord[pDoc->Cableways[landing][cable].tailcolumn][pDoc->Cableways[landing][cable].tailrow].x;
		tailY = pDoc->DTMCoord[pDoc->Cableways[landing][cable].tailcolumn][pDoc->Cableways[landing][cable].tailrow].y;

		//keeps shortest distance from the current cable way to the current cell for not covered cells.
		if (feasibility == FALSE)
		{
			tempshortestdistfromanycables = 99999999999999.9;
			
			//distance from head.
			tempdist = sqrt(pow(x1-landingX, 2.0) + pow(y1-landingY, 2.0));
			if (tempshortestdistfromanycables > tempdist)
			{
				tempshortestdistfromanycables = tempdist;
				x = landingX;
				y = landingY;				
			}
							
			//distance from tail.
			tempdist = sqrt(pow(x1-tailX, 2.0) + pow(y1-tailY, 2.0));
			if (tempshortestdistfromanycables > tempdist)
			{
				tempshortestdistfromanycables = tempdist;
				x = tailX;
				y = tailY;
			}

			if (tempx <= maxX && tempx >= minX && tempy <= maxY && tempy >= minY)//current cell is located between landing and tail spar but further than lateral yarding distance.
			{
				if (tempshortestdistfromanycables > dist)//dist is the distance from the cable way to the current cell with right angle.
				{
					x = tempx;
					y = tempy;//restore
				}
			}
		}

		//keep point coordinates on which the cell is connected to cableway.
		FromCellToCableResults[column][row].x = (int)x;
		FromCellToCableResults[column][row].y = (int)y;
		

		//draw the timber cell
		DrawEachCell(column, row, RGB(200,200,200));
		//draw a lateral yarding direction (line-cable way)
		DrawFrom.x = (long)(x1 - pDoc->DTMminX + cpMargin.x);
		DrawFrom.y = (long)(y1 - pDoc->DTMmaxY - cpMargin.y);
		DrawTo.x = (long)(x - pDoc->DTMminX + cpMargin.x);
		DrawTo.y = (long)(y - pDoc->DTMmaxY - cpMargin.y);

		pDC->MoveTo(DrawFrom);
		pDC->LineTo(DrawTo);
		
		//Draw cable corridors--different color and width
		DrawEachLine(pDoc->Landings[landing].column, pDoc->Landings[landing].row, pDoc->Cableways[landing][cable].tailcolumn, pDoc->Cableways[landing][cable].tailrow, 7, RGB(50,0,50));
	}

	fclose(infile_ptr);//close file


	pDC->SelectObject(pOldPen);

	//delete CPen Object
	penColor[2].DeleteObject();


	//give-up cells -- display them in red color
	//also look at OnResultsshowtimbercells()
	for (j = 1; j <= pDoc->DTMrows; ++j)
	{
		for (i = 1; i <= pDoc->DTMcolumns; ++i)
		{
			if (pDoc->Volume[i][j] == -1.)
				DrawEachCell(i, j, RGB(255,0,0));
		}
	}

}



void CCablanView::CableReports(CString FileName)
{
	//this subroutine has to be called while the link data are still in memory.
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	//File open to read
	int i, j;
	int TotalNumberofSales;
	int Method;
	int SaleNumber;
	int NumberofLinksinPath;
	float Volume;
	int tempNodeID;
	FILE *infile_ptr;
	FILE *outfile_ptr;
	CString tempString;
	
	double TotalVariableCosts;
	double TotalFixedCosts;
	double TotalVariableandFixedCosts;
	
	infile_ptr = fopen(FileName, "rb");

	tempString = "E:\\thesis\\output\\cablereport.txt";
	outfile_ptr = fopen(tempString, "w");

	fscanf(infile_ptr, "%d", &Method);//which algorithem found this solution
	fscanf(infile_ptr, "%d", &TotalNumberofSales);//read Sale Number

	//read result for each sale
	for (i = 1; i <= TotalNumberofSales; ++i)
	{
		if (pDoc->SaleExclusion[i] != TRUE)
		{
			fscanf(infile_ptr, "%d", &SaleNumber);
			fscanf(infile_ptr, "%f", &Volume);
			fscanf(infile_ptr, "%d", &NumberofLinksinPath);
						
			for (j = 1; j <= NumberofLinksinPath; ++j)
			{
				fscanf(infile_ptr, "%d", &tempNodeID);//FinalPath[] contains node number instead of link number 
				fprintf(outfile_ptr, "%s-", pDoc->Label_NumberToString[tempNodeID]);
			}
			fprintf(outfile_ptr, "%f\n", Volume);
		}
	}

	fscanf(infile_ptr, "%lf", &TotalVariableCosts);
	fscanf(infile_ptr, "%lf", &TotalFixedCosts);
	fscanf(infile_ptr, "%lf", &TotalVariableandFixedCosts);

	fclose(infile_ptr);
	fclose(outfile_ptr);
}





void CCablanView::DrawRoadPath(CString FileName)
{

	//drawing path from each volume cell to landing after optimized.
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pDC = new CClientDC(this);
	OnPrepareDC(pDC);

	pDC->SetMapMode(MM_ISOTROPIC);
	pDC->SetWindowExt(ViewScale, ViewScale);



	int i, j, k;
	int totalnumberofsales;
	int Method;
	int SaleNumber, NumberofLinksinPath;
	float Volume;
	int tempNodeID;
	CString tempString;
	int column, row;

	//initializing result variables
	TotalNumberofFinalRoadCells = 0;
	for (i = 0; i < MAXROADLENGTH; ++i)
	{
		RoadCellResults[i].col = 0;
		RoadCellResults[i].row = 0;
		RoadCellResults[i].nodeno = 0;
	}

	TotalNumberofFinalLandings = 0;
	for (i = 0; i < MAXLANDINGS; ++i)
	{
		LandingLocationResults[i].col = 0;
		LandingLocationResults[i].row = 0;
		LandingLocationResults[i].nodeno = 0;
	}


	FILE* infile_ptr;
	infile_ptr = fopen(FileName, "rb");//csFileName is set at pDoc->SaveResultsToFile()

	//read total number of sales including excluded sales
	fscanf(infile_ptr, "%d", &Method);//which algorithm found this solution->Global variable
	fscanf(infile_ptr, "%d", &totalnumberofsales);//local temporary variable -> just for skipping this data in file

	for (i = 1; i <= totalnumberofsales; ++i)
	{
		if (pDoc->SaleExclusion[i] != TRUE)
		{
			//read result for each sale
			fscanf(infile_ptr, "%d", &SaleNumber);
			fscanf(infile_ptr, "%f", &Volume);
			fscanf(infile_ptr, "%d", &NumberofLinksinPath);

			if (NumberofLinksinPath <= 0)
			{
				AfxMessageBox("One of the sales could not be found.", MB_ICONEXCLAMATION);
				break;
			}
			else
			{
				for (j = 1; j <= NumberofLinksinPath; ++j)//NumberofLinksinPath has to be 4 (origin cell(C#R#), system(C#S#L#), landing(L#), and dummy destination). 
				{
					fscanf(infile_ptr, "%d", &tempNodeID);//right order, not in reverse order
					
					if (pDoc->Label_NumberToString[tempNodeID][0] == 'C')
					{
						k = 1;//skip the first letter which is 'C'
						tempString = "";
						while (pDoc->Label_NumberToString[tempNodeID][k] != 'R')//Row or System
						{
							tempString += pDoc->Label_NumberToString[tempNodeID][k];
							++k;
						}

						if (pDoc->Label_NumberToString[tempNodeID][k] == 'R')
						{
							column = atoi(tempString);
	
							tempString = "";
							++k;
							while (k < MAX_CHAR)
							{
								tempString += pDoc->Label_NumberToString[tempNodeID][k];
								++k;
							}
							row = atoi(tempString);
						}
					}					
					DrawEachCell(column, row, RGB(150,0,150));
					
					//keep road cells in results variables
					++TotalNumberofFinalRoadCells;//starts from 1
					RoadCellResults[TotalNumberofFinalRoadCells].col = column;
					RoadCellResults[TotalNumberofFinalRoadCells].row = row;
					RoadCellResults[TotalNumberofFinalRoadCells].nodeno = tempNodeID;

					if (j == 1)//the first cell is landing location.
					{
						++TotalNumberofFinalLandings;//starts from 1
						LandingLocationResults[TotalNumberofFinalLandings].col = column;
						LandingLocationResults[TotalNumberofFinalLandings].row = row;
						LandingLocationResults[TotalNumberofFinalLandings].nodeno = tempNodeID;
					}					
				}
			}
		}
	}

	fclose(infile_ptr);//close file

}


void CCablanView::RoadReports(CString FileName)
{
	//this subroutine has to be called while the link data are still in memory.
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	//File open to read
	int i, j;
	int TotalNumberofSales;
	int Method;
	int SaleNumber;
	int NumberofLinksinPath;
	float Volume;
	int tempNodeID;
	FILE *infile_ptr;
	FILE *outfile_ptr;
	CString tempString;
	
	double TotalVariableCosts;
	double TotalFixedCosts;
	double TotalVariableandFixedCosts;
	
	infile_ptr = fopen(FileName, "rb");

	tempString = "E:\\thesis\\output\\roadreport.txt";
	outfile_ptr = fopen(tempString, "w");

	fscanf(infile_ptr, "%d", &Method);//which algorithem found this solution
	fscanf(infile_ptr, "%d", &TotalNumberofSales);//read Sale Number

	//read result for each sale
	for (i = 1; i <= TotalNumberofSales; ++i)
	{
		if (pDoc->SaleExclusion[i] != TRUE)
		{
			fscanf(infile_ptr, "%d", &SaleNumber);
			fscanf(infile_ptr, "%f", &Volume);
			fscanf(infile_ptr, "%d", &NumberofLinksinPath);
			fprintf(outfile_ptr, "Sale:	%d\n", SaleNumber);
			fprintf(outfile_ptr, "Volume:	%f\n", Volume);
			fprintf(outfile_ptr, "NumberofCells:	%d\n", NumberofLinksinPath);
			
			for (j = 1; j <= NumberofLinksinPath; ++j)
			{
				fscanf(infile_ptr, "%d", &tempNodeID);//FinalPath[] contains node number instead of link number 
				fprintf(outfile_ptr, "%s-", pDoc->Label_NumberToString[tempNodeID]);
			}
			fprintf(outfile_ptr, "\n");
		}
	}

	fscanf(infile_ptr, "%lf", &TotalVariableCosts);
	fscanf(infile_ptr, "%lf", &TotalFixedCosts);
	fscanf(infile_ptr, "%lf", &TotalVariableandFixedCosts);

	fclose(infile_ptr);
	fclose(outfile_ptr);
}




void CCablanView::DrawAllPath()
{
	//drawing path from each volume cell to final destination.
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int i, j, k;
	int totalnumberofsales;
	int Method;
	int SaleNumber, NumberofLinksinPath;
	float Volume;
	int tempNodeID;
	int column, row, cable, system, landing;
	int rcolumn, rrow;//roadcells
	CString tempString;
	double x, y, x1, y1, m, b, dist;
	int maxX, maxY, minX, minY;
	bool feasibility;
	int landingX, landingY, tailX, tailY;
	int tempint;
	double tempdist, tempshortestdistfromanycables;
	double tempx, tempy;


	penColor[1].CreatePen(PS_SOLID, 5, RGB(0,0,0));
	CPen *pOldPen = pDC->SelectObject(&penColor[1]);


	//repaint with white brush
	for (j = 1; j <= pDoc->DTMrows; ++j)
	{
		for(i = 1; i <= pDoc->DTMcolumns; ++i)
		{
			DrawEachCell(i, j, RGB(255,255,255));
		}
	}




	FILE* infile_ptr;
	pDoc->csFileName = "E:\\thesis\\output\\allpath.txt";//define file name for keeping the best route
	infile_ptr = fopen(pDoc->csFileName, "rb");//csFileName is set at pDoc->SaveResultsToFile()

	//read total number of sales including excluded sales
	fscanf(infile_ptr, "%d", &Method);//which algorithm found this solution->Global variable
	fscanf(infile_ptr, "%d", &totalnumberofsales);//local temporary variable -> just for skipping this data in file

	for (i = 1; i <= totalnumberofsales; ++i)
	{
		if (pDoc->SaleExclusion[i] != TRUE)
		{
			//read result for each sale
			fscanf(infile_ptr, "%d", &SaleNumber);
			fscanf(infile_ptr, "%f", &Volume);
			fscanf(infile_ptr, "%d", &NumberofLinksinPath);

			if (NumberofLinksinPath <= 0)
			{
				AfxMessageBox("One of the sales could not be found.", MB_ICONEXCLAMATION);
				break;
			}
			else
			{
				for (j = 1; j <= NumberofLinksinPath; ++j)//NumberofLinksinPath has to be 4 (origin cell(C#R#), system(C#S#L#), landing(L#), and dummy destination). 
				{
					fscanf(infile_ptr, "%d", &tempNodeID);//right order, not in reverse order
					
					if (pDoc->Label_NumberToString[tempNodeID][0] == 'V')
					{
						k = 2;//skip the first letter which is 'VC'
						tempString = "";
						while (pDoc->Label_NumberToString[tempNodeID][k] != 'R' && pDoc->Label_NumberToString[tempNodeID][k] != 'S')//Row or System
						{
							tempString += pDoc->Label_NumberToString[tempNodeID][k];
							++k;
						}

						if (pDoc->Label_NumberToString[tempNodeID][k] == 'R')
						{
							column = atoi(tempString);
	
							tempString = "";
							++k;
							while (k < MAX_CHAR)
							{
								tempString += pDoc->Label_NumberToString[tempNodeID][k];
								++k;
							}
							row = atoi(tempString);
						}
					}
					else if (pDoc->Label_NumberToString[tempNodeID][0] == 'C')//system and landing
					{
						k = 1;//skip the first letter which is 'C'
						tempString = "";
						while (pDoc->Label_NumberToString[tempNodeID][k] != 'R' && pDoc->Label_NumberToString[tempNodeID][k] != 'S')//Row or System
						{
							tempString += pDoc->Label_NumberToString[tempNodeID][k];
							++k;
						}

						if (pDoc->Label_NumberToString[tempNodeID][k] == 'R')
						{
							rcolumn = atoi(tempString);
	
							tempString = "";
							++k;
							while (k < MAX_CHAR)
							{
								tempString += pDoc->Label_NumberToString[tempNodeID][k];
								++k;
							}
							rrow = atoi(tempString);
							DrawEachCell(rcolumn, rrow, RGB(0,255,0));
						}
						else//system and landing
						{
							cable = atoi(tempString);
								
							tempString = "";
							++k;
							while (pDoc->Label_NumberToString[tempNodeID][k] != 'L')
							{
								tempString += pDoc->Label_NumberToString[tempNodeID][k];
								++k;
							}
							system = atoi(tempString);

							tempString = "";
							++k;							
							while (k < MAX_CHAR)
							{
								tempString += pDoc->Label_NumberToString[tempNodeID][k];
								++k;
							}
							landing = atoi(tempString);
						}
					}
					else//landing
					{
						//we can skip landing number since we know the ID number in above.
					}
				}
			}
		}


		//drawing line from the cell to the cable way.



		//measuring distance from the cell to each cableway
		x1 = pDoc->DTMCoord[column][row].x;//cell coordinate
		y1 = pDoc->DTMCoord[column][row].y;
		m = pDoc->Cableways[landing][cable].slope;

		b = pDoc->Cableways[landing][cable].intercept;
		x = (y1 + 1.0/m*x1 - b)*m/(pow(m,2.0)+1);
		y = m*x + b;
		dist = sqrt(pow(x1-x, 2.0) + pow(y1-y, 2.0));

		maxX = pDoc->DTMCoord[pDoc->Cableways[landing][cable].tailcolumn][pDoc->Cableways[landing][cable].tailrow].x;
		minX = pDoc->DTMCoord[pDoc->Landings[landing].column][pDoc->Landings[landing].row].x;
		if (minX > maxX)
		{
			tempint = maxX;
			maxX = minX;
			minX = tempint;
		}
		maxY = pDoc->DTMCoord[pDoc->Cableways[landing][cable].tailcolumn][pDoc->Cableways[landing][cable].tailrow].y;
		minY = pDoc->DTMCoord[pDoc->Landings[landing].column][pDoc->Landings[landing].row].y;
		if (minY > maxY)
		{
			tempint = maxY;
			maxY = minY;
			minY = tempint;
		}
		
		feasibility = TRUE;
		if (x > maxX || x < minX)
			feasibility = FALSE;

		if (y > maxY || y < minY)
			feasibility = FALSE;
		
		if (dist > pDoc->CableSystems[system].maxlateraldist)//maximum lateral yarding distance.
			feasibility = FALSE;


		tempx = x;
		tempy = y;
		landingX = pDoc->DTMCoord[pDoc->Landings[landing].column][pDoc->Landings[landing].row].x;
		landingY = pDoc->DTMCoord[pDoc->Landings[landing].column][pDoc->Landings[landing].row].y;
		tailX = pDoc->DTMCoord[pDoc->Cableways[landing][cable].tailcolumn][pDoc->Cableways[landing][cable].tailrow].x;
		tailY = pDoc->DTMCoord[pDoc->Cableways[landing][cable].tailcolumn][pDoc->Cableways[landing][cable].tailrow].y;

		//keeps shortest distance from the current cable way to the current cell for not covered cells.
		if (feasibility == FALSE)
		{
			tempshortestdistfromanycables = 99999999999999.9;
							
			tempdist = sqrt(pow(x1-landingX, 2.0) + pow(y1-landingY, 2.0));
			if (tempshortestdistfromanycables > tempdist)
			{
				tempshortestdistfromanycables = tempdist;
				x = landingX;
				y = landingY;				
			}
							
			tempdist = sqrt(pow(x1-tailX, 2.0) + pow(y1-tailY, 2.0));
			if (tempshortestdistfromanycables > tempdist)
			{
				tempshortestdistfromanycables = tempdist;
				x = tailX;
				y = tailY;
			}

			if (tempx <= maxX && tempx >= minX && tempy <= maxY && tempy >= minY)//current cell is located between landing and tail spar but further than lateral yarding distance.
			{
				if (tempshortestdistfromanycables > dist)//dist is the distance from the cable way to the current cell with right angle.
				{
					x = tempx;
					y = tempy;//restore
				}
			}
		}

		//draw a line-cable way
		DrawFrom.x = (long)(x1 - pDoc->DTMminX + cpMargin.x);
		DrawFrom.y = (long)(y1 - pDoc->DTMmaxY - cpMargin.y);
		DrawTo.x = (long)(x - pDoc->DTMminX + cpMargin.x);
		DrawTo.y = (long)(y - pDoc->DTMmaxY - cpMargin.y);

		pDC->MoveTo(DrawFrom);
		pDC->LineTo(DrawTo);

		DrawTo.x = (long)(landingX - pDoc->DTMminX + cpMargin.x);
		DrawTo.y = (long)(landingY - pDoc->DTMmaxY - cpMargin.y);
		pDC->LineTo(DrawTo);
	}

	fclose(infile_ptr);//close file

	
	pDC->SelectObject(pOldPen);

	//delete CPen Object
	penColor[1].DeleteObject();

}





///////////////////////////////////////////////////////////////////////////////////////////////
//Show Results********************************************************************************
///////////////////////////////////////////////////////////////////////////////////////////////

void CCablanView::OnResultsShowcableways() 
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pDC = new CClientDC(this);
	OnPrepareDC(pDC);

	pDC->SetMapMode(MM_ISOTROPIC);
	pDC->SetWindowExt(ViewScale, ViewScale);

	int i;
	int landing, system, cable;
	int landingX, landingY;
	int tailX, tailY;

	penColor[1].CreatePen(PS_SOLID, 7, RGB(0,0,0));
	penColor[2].CreatePen(PS_SOLID, 15, RGB(255,0,0));
	CPen *pOldPen = pDC->SelectObject(&penColor[1]);
	

	//counting variables for report
	bool tempReportCableRoads[MAXLANDINGS][MAXCABLESYSTEMS][MAXCABLEWAYS];	
	int totalNumberofFinalCableroads = 0;
	int totalNumberofCableroadsperLanding[MAXLANDINGS];
	int totalNumberofTimberParcelsperLanding[MAXLANDINGS];
	int yardingsystemperLanding[MAXLANDINGS];

	memset(tempReportCableRoads, 0, sizeof (tempReportCableRoads));
	memset(totalNumberofCableroadsperLanding, 0, sizeof(totalNumberofCableroadsperLanding));
	memset(totalNumberofTimberParcelsperLanding, 0, sizeof(totalNumberofTimberParcelsperLanding));


	for (i = 1; i <= TotalNumberofFinalCableways; ++i)//TotalNumberofFinalCableways has total number of timber parcel... cable roads are duplicated.
	{
		cable = CableResults[i].cableway;
		landing = CableResults[i].landing;
		system = CableResults[i].system;
	
		if (system == 1)//in different color
			pDC->SelectObject(&penColor[1]);
		else
			pDC->SelectObject(&penColor[1]);


		++totalNumberofTimberParcelsperLanding[landing];


		if (tempReportCableRoads[landing][system][cable] == FALSE)
		{
			tempReportCableRoads[landing][system][cable] = TRUE;
			++totalNumberofFinalCableroads;
			++totalNumberofCableroadsperLanding[landing];
			yardingsystemperLanding[landing] = system;
		

			landingX = pDoc->DTMCoord[pDoc->Landings[landing].column][pDoc->Landings[landing].row].x;
			landingY = pDoc->DTMCoord[pDoc->Landings[landing].column][pDoc->Landings[landing].row].y;

			tailX = pDoc->DTMCoord[pDoc->Cableways[landing][cable].tailcolumn][pDoc->Cableways[landing][cable].tailrow].x;
			tailY = pDoc->DTMCoord[pDoc->Cableways[landing][cable].tailcolumn][pDoc->Cableways[landing][cable].tailrow].y;

			DrawFrom.x = (long)(landingX - pDoc->DTMminX + cpMargin.x);
			DrawFrom.y = (long)(landingY - pDoc->DTMmaxY - cpMargin.y);
			DrawTo.x = (long)(tailX - pDoc->DTMminX + cpMargin.x);
			DrawTo.y = (long)(tailY - pDoc->DTMmaxY - cpMargin.y);

			pDC->MoveTo(DrawFrom);
			pDC->LineTo(DrawTo);
		}
	}

	pDC->SelectObject(pOldPen);

	//delete CPen Object
	penColor[1].DeleteObject();

}

void CCablanView::OnResultsShowlandings() 
{
	pDC = new CClientDC(this);
	OnPrepareDC(pDC);

	CString tempString;

	pDC->SetMapMode(MM_ISOTROPIC);
	pDC->SetWindowExt(ViewScale, ViewScale);

	int i;
	
	for (i = 1; i <= TotalNumberofFinalLandings; ++i)
	{
		DrawEachCell(LandingLocationResults[i].col, LandingLocationResults[i].row, RGB(255,255,255));
//		//writing Tower ID
//		tempString.Format("%d", i);
//		WriteTextAboveGridCell(LandingLocationResults[i].col, LandingLocationResults[i].row, tempString);
	}
}

void CCablanView::OnResultsShowroads() 
{
	//This is good for only seperated optimization.
	pDC = new CClientDC(this);
	OnPrepareDC(pDC);

	pDC->SetMapMode(MM_ISOTROPIC);
	pDC->SetWindowExt(ViewScale, ViewScale);

	int i;
	
	for (i = 1; i <= TotalNumberofFinalRoadCells; ++i)
	{
		DrawEachCell(RoadCellResults[i].col, RoadCellResults[i].row, RGB(200,0,100));
	}	
}

void CCablanView::OnResultsShowtimbercells() 
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pDC = new CClientDC(this);
	OnPrepareDC(pDC);

	pDC->SetMapMode(MM_ISOTROPIC);
	pDC->SetWindowExt(ViewScale, ViewScale);

	int i, j;
	int cellX, cellY, cableX, cableY;

	penColor[1].CreatePen(PS_SOLID, 5, RGB(100,100,100));
	CPen *pOldPen = pDC->SelectObject(&penColor[1]);

	for (j = 1; j <= pDoc->DTMrows; ++j)
	{
		for(i = 1; i <= pDoc->DTMcolumns; ++i)
		{
			//draw cell with timber volume
			if (pDoc->Volume[i][j] > 0 && pDoc->CellsWithinUnitBoundary[i][j] == TRUE)
			{
				DrawEachCell(i, j, RGB(200,200,200));

				if (FromCellToCableResults[i][j].x > 0)
				{
					cellX = pDoc->DTMCoord[i][j].x;
					cellY = pDoc->DTMCoord[i][j].y;
					cableX = FromCellToCableResults[i][j].x;
					cableY = FromCellToCableResults[i][j].y;

					DrawFrom.x = (long)(cellX - pDoc->DTMminX + cpMargin.x);
					DrawFrom.y = (long)(cellY - pDoc->DTMmaxY - cpMargin.y);
					DrawTo.x = (long)(cableX - pDoc->DTMminX + cpMargin.x);
					DrawTo.y = (long)(cableY - pDoc->DTMmaxY - cpMargin.y);

					pDC->MoveTo(DrawFrom);
					pDC->LineTo(DrawTo);
				}
			}
			else
			{
				//give-up cells -- display them in red color
				if (pDoc->Volume[i][j] == -1.)
					DrawEachCell(i, j, RGB(255,0,0));
			}
		}
	}


	pDC->SelectObject(pOldPen);

	//delete CPen Object
	penColor[1].DeleteObject();
}


void CCablanView::OnResultsClearharvestunit() 
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pDC = new CClientDC(this);
	OnPrepareDC(pDC);

	pDC->SetMapMode(MM_ISOTROPIC);
	pDC->SetWindowExt(ViewScale, ViewScale);

	int i, j;

	//repaint with white brush
	for (j = pDoc->UnitBoundary.startRow; j <= pDoc->UnitBoundary.endRow; ++j)
	{
		for(i = pDoc->UnitBoundary.startCol; i <= pDoc->UnitBoundary.endCol; ++i)
		{
			if (pDoc->CellsWithinUnitBoundary[i][j] == TRUE)
				DrawEachCell(i, j, RGB(255,255,255));
		}
	}

	
}

void CCablanView::OnResultsShowallresults() 
{
	//OnResultsClearharvestunit();
	OnResultsShowtimbercells();
	OnResultsShowroads();
	OnResultsShowlandings();
	OnResultsShowcableways();
	
}



void CCablanView::OnNetworkHarvestingunit() 
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	penColor[1].CreatePen(PS_SOLID, 7, RGB(155,155,155));
	CPen *pOldPen = pDC->SelectObject(&penColor[1]);

	DrawFrom.x = long(pDoc->DTMCoord[1][1].x + cpMargin.x);
	DrawFrom.y = long(pDoc->DTMCoord[1][1].y - cpMargin.y);
	pDC->MoveTo(DrawFrom);

	DrawTo.x = long(pDoc->DTMCoord[100][1].x + cpMargin.x);
	DrawTo.y = long(pDoc->DTMCoord[100][1].y - cpMargin.y);
	pDC->LineTo(DrawTo);

	DrawTo.x = long(pDoc->DTMCoord[100][100].x + cpMargin.x);
	DrawTo.y = long(pDoc->DTMCoord[100][100].y - cpMargin.y);
	pDC->LineTo(DrawTo);

	DrawTo.x = long(pDoc->DTMCoord[1][100].x + cpMargin.x);
	DrawTo.y = long(pDoc->DTMCoord[1][100].y - cpMargin.y);
	pDC->LineTo(DrawTo);

	DrawTo.x = long(pDoc->DTMCoord[1][1].x + cpMargin.x);
	DrawTo.y = long(pDoc->DTMCoord[1][1].y - cpMargin.y);
	pDC->LineTo(DrawTo);

	pDC->SelectObject(pOldPen);

	//delete CPen Object
	penColor[1].DeleteObject();
}






/////////////////////////////////////////////////////////////////////////////
//DRAWING MAPS
/////////////////////////////////////////////////////////////////////////////
void CCablanView::OnLayersDemChangecontourinterval() 
{
	//the algorithm is from "www.ensu.ucalgary.ca/~nel-shei/lecture.htm"..
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CDialogContourMap dlg;

	int ContourInterval = pDoc->ContourInterval;
	int numberofcontourlines = (pDoc->DTMmaxZ - pDoc->DTMminZ) / ContourInterval + 1;
	int ContourElevation;//keep elevation for each contour line.
	CString tempString;
	
	int a, i, j;	
	
	
	//open the ContourMap Dialog box for inputting contour interval.
	dlg.m_ContourInterval = pDoc->ContourInterval;//initial value
	tempString.Format("%d", pDoc->DTMmaxZ);
	dlg.m_HighestElevation = tempString;
	tempString.Format("%d", pDoc->DTMminZ);
	dlg.m_LowestElevation = tempString;

	if (dlg.DoModal() == IDOK)
	{
		pDoc->ContourInterval = dlg.m_ContourInterval;
		ContourInterval = pDoc->ContourInterval;
		numberofcontourlines = (pDoc->DTMmaxZ - pDoc->DTMminZ) / ContourInterval + 1;
	

		//initializing
		for (j = 1; j <= pDoc->DTMrows; ++j)//not include pDoc->DTMrows since j+1 is used.
		{
			for (i = 1; i <= pDoc->DTMcolumns; ++i)
				pDoc->NumberofContourlinesWithinThisCell[i][j] = 0;
		}



		for (a = 1; a <= numberofcontourlines; ++a)
		{
			//current elevation
			ContourElevation = pDoc->DTMminZ + ContourInterval * a;
	
			memset(CellAlreadyChecked, 0, sizeof(CellAlreadyChecked));
			memset(CellsonContour, 0, sizeof(CellsonContour));
			DrawFrom.x = -1;//global variables
			DrawFrom.y = -1;
			DrawTo.x = -1;
			DrawTo.y = -1;


			//exclude boundaries.
			for (i = 0; i <= 1; ++i)
			{
				for (j = 0; j <= 1; ++j)
					CellsonContour[i][j] = TRUE;
			}

			for (j = 1; j <= pDoc->DTMrows - 1; ++j)//not include pDoc->DTMrows since j+1 is used.
			{
				for (i = 1; i <= pDoc->DTMcolumns - 1; ++i)
				{
					//check four cells {(i,j),(i+1,j),(i+1,j+1),(i,j+1)} to determine if the contour line passes through this cell or not.
					CheckContourCells(i, j, (double)ContourElevation + 0.1, 0);//enter ContourElevation as a double number to which is added 0.1 in order to get rid of the problems with having the same contour value in the grid cells.
					DrawFrom.x = -1;
					DrawFrom.y = -1;
				}
			}
		}

		pDoc->Contour[0][0][0].Start.x = 0;//indicates we already developed contour line data.	
	}

}



void CCablanView::OnLayersDemDrawcontourlines() 
{
	//the algorithm is from "www.ensu.ucalgary.ca/~nel-shei/lecture.htm"..
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pDC = new CClientDC(this);
	OnPrepareDC(pDC);

	pDC->SetMapMode(MM_ISOTROPIC);
	pDC->SetWindowExt(ViewScale, ViewScale);


	int i, j, k;
	

	//generate contour map and keep coordinates to draw the contour lines.
	if (pDoc->Contour[0][0][0].Start.x == -1)//no contour coordinate data
	{
		OnLayersDemChangecontourinterval();
	}
	else
	{
		//draw contour lines since we already have coordinates for all contour lines.
		for (j = 1; j <= pDoc->DTMrows - 1; ++j)//not include pDoc->DTMrows since j+1 is used.
		{
			for (i = 1; i <= pDoc->DTMcolumns - 1; ++i)
			{
				for (k = 0; k < pDoc->NumberofContourlinesWithinThisCell[i][j]; ++k)//if this cell has more than one contour lines passing through.
				{
					pDC->MoveTo(pDoc->Contour[i][j][k].Start);
					pDC->LineTo(pDoc->Contour[i][j][k].End);
				}
			}
		}
	}
}


bool CCablanView::CheckContourCells(int col, int row, double ContourElevation, int SideID)
{
	//check four cells {(i,j),(i+1,j),(i+1,j+1),(i,j+1)} to determine if the contour line passes through this cell or not.
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int colfrom, rowfrom, colto, rowto;
	double diffz1, diffz2;
	CPoint tempPoint;
	bool tempbool = FALSE;
	

	//this cell is already looked at for this contour line
	if (CellAlreadyChecked[col][row] == TRUE)
		return(FALSE);

	//the contour line hits the boundary
	if (col >= pDoc->DTMcolumns || row >= pDoc->DTMrows || col <= 1 || row <= 1)
	{
		DrawFrom.x = -1;
		return(FALSE);
	}

	//the contour line hits nodata cell
	if (pDoc->DTMCoord[col][row].z < 0)
	{
		DrawFrom.x = -1;
		return(FALSE);
	}
	
	//the contour line hits the cell already having contour line (closed loop)
	if (CellsonContour[col][row] == TRUE)
	{
		DrawFrom.x = -1;
		return(FALSE);
	}

	//check four cells with clock-wise direction.
	//exclude SideID lateral line of this grid cell, since we are from there. 
	if (SideID != 1)
	{
		//#1. 
		colfrom = col;
		rowfrom = row;
		colto = col + 1;
		rowto = row;

		//calculate elevation difference between grid cells and current contour elevation
		diffz1 = ContourElevation - pDoc->DTMCoord[colfrom][rowfrom].z;
		diffz2 = ContourElevation - pDoc->DTMCoord[colto][rowto].z;

		if (diffz1 * diffz2 < 0)//the contour line passes through between two selected cells.
		{
			//get exact location of path by linear interpolation
			tempPoint = LinearInterpolationforContourlines(colfrom, rowfrom, colto, rowto, ContourElevation);

			//if this is the start point, then keep the point coordinates and find another pass point within the grid cell.
			if (DrawFrom.x < 0)
			{
				DrawFrom = tempPoint;
				pDC->MoveTo(DrawFrom);
			}	
			else//this is not the starting point, then move the line following the contour line.
			{
				DrawTo = tempPoint;
				pDC->LineTo(DrawTo);
				CellAlreadyChecked[col][row] = TRUE;
				pDoc->Contour[col][row][pDoc->NumberofContourlinesWithinThisCell[col][row]].Start = DrawFrom;//keep the location where the line from before changing.
				pDoc->Contour[col][row][pDoc->NumberofContourlinesWithinThisCell[col][row]].End = tempPoint;
				pDoc->Contour[col][row][pDoc->NumberofContourlinesWithinThisCell[col][row]].Elevation = (int)ContourElevation;
				if (pDoc->NumberofContourlinesWithinThisCell[col][row] < MAXCONTOURLINES - 1)//if contour lines more than MAXCONTOURLINES pass through the cell, then keep only the last contour line.
					++pDoc->NumberofContourlinesWithinThisCell[col][row];
				++pDoc->NumberofContourlinesWithinThisCell[col][row];
				DrawFrom = tempPoint;//Change DrawFrom to current location of the line.
				//this contour line is heading from south to north-->check the three neighboring cells above this line --> col and row-1
				return(CheckContourCells(col, row-1, ContourElevation, 3));
			}
			
			tempbool = TRUE;
			CellsonContour[col][row] = TRUE;
		}
	}


	if (SideID != 2)
	{
		//#2. 
		colfrom = col + 1;
		rowfrom = row;
		colto = col + 1;
		rowto = row + 1;
	

		diffz1 = ContourElevation - pDoc->DTMCoord[colfrom][rowfrom].z;
		diffz2 = ContourElevation - pDoc->DTMCoord[colto][rowto].z;

		if (diffz1 * diffz2 < 0)//the contour line passes through between two above cells.
		{
			tempPoint = LinearInterpolationforContourlines(colfrom, rowfrom, colto, rowto, ContourElevation);
			if (DrawFrom.x < 0)
			{
				DrawFrom = tempPoint;
				pDC->MoveTo(DrawFrom);
			}	
			else//this is not the starting point, then move the line following the contour line.
			{
				DrawTo = tempPoint;
				pDC->LineTo(DrawTo);
				CellAlreadyChecked[col][row] = TRUE;
				pDoc->Contour[col][row][pDoc->NumberofContourlinesWithinThisCell[col][row]].Start = DrawFrom;//keep the location where the line from before changing.
				pDoc->Contour[col][row][pDoc->NumberofContourlinesWithinThisCell[col][row]].End = tempPoint;
				pDoc->Contour[col][row][pDoc->NumberofContourlinesWithinThisCell[col][row]].Elevation = (int)ContourElevation;
				if (pDoc->NumberofContourlinesWithinThisCell[col][row] < MAXCONTOURLINES - 1)//if contour lines more than MAXCONTOURLINES pass through the cell, then keep only the last contour line.
					++pDoc->NumberofContourlinesWithinThisCell[col][row];
				DrawFrom = tempPoint;//Change DrawFrom to current location of the line.
				//this contour line is heading from west to east-->check the three neighboring cells in right side of this line --> col+1 and row
				return(CheckContourCells(col+1, row, ContourElevation, 4));
			}
		
			tempbool = TRUE;
			CellsonContour[col][row] = TRUE;
		}
	}


	if (SideID != 3)
	{
		//#3. 
		colfrom = col + 1;
		rowfrom = row + 1;
		colto = col;
		rowto = row + 1;
	
		diffz1 = ContourElevation - pDoc->DTMCoord[colfrom][rowfrom].z;
		diffz2 = ContourElevation - pDoc->DTMCoord[colto][rowto].z;

		if (diffz1 * diffz2 < 0)//the contour line passes through between two above cells.
		{
			tempPoint = LinearInterpolationforContourlines(colfrom, rowfrom, colto, rowto, ContourElevation);
			if (DrawFrom.x < 0)
			{
				DrawFrom = tempPoint;
				pDC->MoveTo(DrawFrom);
			}	
			else//this is not the starting point, then move the line following the contour line.
			{
				DrawTo = tempPoint;
				pDC->LineTo(DrawTo);
				CellAlreadyChecked[col][row] = TRUE;
				pDoc->Contour[col][row][pDoc->NumberofContourlinesWithinThisCell[col][row]].Start = DrawFrom;//keep the location where the line from before changing.
				pDoc->Contour[col][row][pDoc->NumberofContourlinesWithinThisCell[col][row]].End = tempPoint;
				pDoc->Contour[col][row][pDoc->NumberofContourlinesWithinThisCell[col][row]].Elevation = (int)ContourElevation;
				if (pDoc->NumberofContourlinesWithinThisCell[col][row] < MAXCONTOURLINES - 1)//if contour lines more than MAXCONTOURLINES pass through the cell, then keep only the last contour line.
					++pDoc->NumberofContourlinesWithinThisCell[col][row];
				++pDoc->NumberofContourlinesWithinThisCell[col][row];
				DrawFrom = tempPoint;//Change DrawFrom to current location of the line.
				return(CheckContourCells(col, row+1, ContourElevation, 1));
			}
		
			tempbool = TRUE;
			CellsonContour[col][row] = TRUE;
		}
	}


	if (SideID != 4)
	{
		//#4. 
		colfrom = col;
		rowfrom = row + 1;
		colto = col;
		rowto = row;

		diffz1 = ContourElevation - pDoc->DTMCoord[colfrom][rowfrom].z;
		diffz2 = ContourElevation - pDoc->DTMCoord[colto][rowto].z;

		if (diffz1 * diffz2 <= 0)//the contour line passes through between two above cells.
		{
			tempPoint = LinearInterpolationforContourlines(colfrom, rowfrom, colto, rowto, ContourElevation);
			if (DrawFrom.x < 0)
			{
				DrawFrom = tempPoint;
				pDC->MoveTo(DrawFrom);
			}	
			else//this is not the starting point, then move the line following the contour line.
			{
				DrawTo = tempPoint;
				pDC->LineTo(DrawTo);
				CellAlreadyChecked[col][row] = TRUE;
				pDoc->Contour[col][row][pDoc->NumberofContourlinesWithinThisCell[col][row]].Start = DrawFrom;//keep the location where the line from before changing.
				pDoc->Contour[col][row][pDoc->NumberofContourlinesWithinThisCell[col][row]].End = tempPoint;
				pDoc->Contour[col][row][pDoc->NumberofContourlinesWithinThisCell[col][row]].Elevation = (int)ContourElevation;
				if (pDoc->NumberofContourlinesWithinThisCell[col][row] < MAXCONTOURLINES - 1)//if contour lines more than MAXCONTOURLINES pass through the cell, then keep only the last contour line.
					++pDoc->NumberofContourlinesWithinThisCell[col][row];
				++pDoc->NumberofContourlinesWithinThisCell[col][row];
				DrawFrom = tempPoint;//Change DrawFrom to current location of the line.
				return(CheckContourCells(col-1, row, ContourElevation, 2));
			}

			tempbool = TRUE;
			CellsonContour[col][row] = TRUE;
		}
	}

	//set TRUE to indicate this cell is ever looked at.
	CellAlreadyChecked[col][row] = TRUE;


	return (tempbool);
}

CPoint CCablanView::LinearInterpolationforContourlines(int col1, int row1, int col2, int row2, double ContourElevation)
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int tempcol, temprow;
	CPoint tempPoint;
	double xx, yy;
	bool tempbool = FALSE;

	if (pDoc->DTMCoord[col1][row1].z > pDoc->DTMCoord[col2][row2].z)
	{
		tempcol = col1;
		col1 = col2;
		col2 = tempcol;
		temprow = row1;
		row1 = row2;
		row2 = temprow;
	}

	xx = (double)pDoc->DTMCoord[col1][row1].x + (ContourElevation - (double)pDoc->DTMCoord[col1][row1].z) / (pDoc->DTMCoord[col2][row2].z - pDoc->DTMCoord[col1][row1].z) * (double)(pDoc->DTMCoord[col2][row2].x - pDoc->DTMCoord[col1][row1].x);
	yy = (double)pDoc->DTMCoord[col1][row1].y + (ContourElevation - (double)pDoc->DTMCoord[col1][row1].z) / (pDoc->DTMCoord[col2][row2].z - pDoc->DTMCoord[col1][row1].z) * (double)(pDoc->DTMCoord[col2][row2].y - pDoc->DTMCoord[col1][row1].y);

	xx = (xx - pDoc->DTMminX + cpMargin.x);
	yy = (yy - pDoc->DTMmaxY - cpMargin.y);
	
	tempPoint.x = (long)xx;
	tempPoint.y = (long)yy;

	return (tempPoint);
	
}



void CCablanView::OnLayersDemShowdem() 
{
	DrawDTM();
	
}







///////////////////////////////////////////////////////////////////////////////
//QUADTREE/////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
void CCablanView::SettingTimberVolumeCells()
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pDC = new CClientDC(this);
	OnPrepareDC(pDC);

	pDC->SetMapMode(MM_ISOTROPIC);
	pDC->SetWindowExt(ViewScale, ViewScale);

	boolTimberParcelAlreadySet = TRUE;//set boolean is TRUE


	int i, j, k, l;
	int ParcelDiameter;
	int starti, startj, endi, endj;
	float TotalVolume;
	int RandR,RandG,RandB;//color

	CDialogChokerLength dlg;


	//Step1: identify grid cells having less timber volume than Design Payload.
	//Results: bool VolumeCellLessthanDesignPayload[][]
	//DesignPayload is in volume (m3) and a global variable in View.
	//???????????????????????????Only Design payload from the first system is considered.
	DesignPayload = pDoc->CableSystems[1].gbUserMinPayload / pDoc->CableSystems[1].gbDensity;//(kg) / (kg/m3)
	IdentifyingVolumeCells();//this subroutine uses OriginalVolume()

	//Step2: Sort selected timber cells by volume
	//Results: SortedVolumeCells[i].col & SortedVolumeCells[i].row
	SortingVolumeCells();

	//Step3: Make a parcel of grid cells so total timber volume is greater than design payload.
	dlg.m_ChokerLength = pDoc->ChokerLength;
	if (dlg.DoModal() == IDOK)
		pDoc->ChokerLength = dlg.m_ChokerLength;

	ParcelDiameter = pDoc->ChokerLength / pDoc->DTMcellsize * 2;//in number of grid cells. Length/cellsize * 2 (diameter)
	if (ParcelDiameter < 1)
	{
		AfxMessageBox("Your DEM cell size is bigger than choker length. Clumping volume cells is not possible.", MB_ICONEXCLAMATION);
		return;
	}
	
	for (k = 1; k <= TotalNumberofVolumeCellsLessthanDesignPayload; ++k)
	{
		if (VolumeCellLessthanDesignPayload[SortedVolumeCells[k].col][SortedVolumeCells[k].row] == TRUE)
		{
			RandR = rand() % 200 + 25;
			RandG = rand() % 200 + 25;
			RandB = rand() % 200 + 25;
			DrawEachCell(SortedVolumeCells[k].col, SortedVolumeCells[k].row, RGB(255,255,255));

			TotalVolume = pDoc->Volume[SortedVolumeCells[k].col][SortedVolumeCells[k].row];
			VolumeCellLessthanDesignPayload[SortedVolumeCells[k].col][SortedVolumeCells[k].row] = FALSE;
	
			for (l = 1; l <= ParcelDiameter/2; ++l)
			{
				starti = SortedVolumeCells[k].col - l;
				if (starti < 1)
					starti = 1;
		
				endi = SortedVolumeCells[k].col + l;
				if (endi > pDoc->DTMcolumns)
					endi = pDoc->DTMcolumns;

				startj = SortedVolumeCells[k].row - l;
				if (startj < 1)
					startj = 1;
		
				endj = SortedVolumeCells[k].row + l;
				if (endj > pDoc->DTMrows)
					endj = pDoc->DTMrows;

				for (i = starti; i <= endi; ++i)
				{
					for (j = startj; j <= endj; ++j)
					{
						if (VolumeCellLessthanDesignPayload[i][j] == TRUE)
						{
							TotalVolume += pDoc->Volume[i][j];
							pDoc->Volume[i][j] = 0.;
							VolumeCellLessthanDesignPayload[i][j] = FALSE;
							
							DrawEachCell(i, j, RGB(RandR,RandG,RandB));

							if (TotalVolume >= DesignPayload)
							{
								pDoc->Volume[SortedVolumeCells[k].col][SortedVolumeCells[k].row] = TotalVolume;
								l = ParcelDiameter/2 + 1;
								i = endi + 1;//break i
								break;//break j
							}
						}
					}
				}

				//if there is no timber cell available in the neighborhood, then keep current Totalvolume although it is smaller than design payload.
				if (TotalVolume < DesignPayload)
					pDoc->Volume[SortedVolumeCells[k].col][SortedVolumeCells[k].row] = TotalVolume;				
			}
		}
	}//end of if

}



void CCablanView::DrawTimberParcels()
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pDC = new CClientDC(this);
	OnPrepareDC(pDC);

	pDC->SetMapMode(MM_ISOTROPIC);
	pDC->SetWindowExt(ViewScale, ViewScale);


	int i, j;

	for (j = pDoc->UnitBoundary.startRow; j <= pDoc->UnitBoundary.endRow; ++j)
	{
		for (i = pDoc->UnitBoundary.startCol; i <= pDoc->UnitBoundary.endCol; ++i)
		{
			if (pDoc->CellsWithinUnitBoundary[i][j] == TRUE && pDoc->Volume[i][j] > 0)
			{
				DrawEachCell(i, j, RGB(255,0,0));
			}
		}
	}
}


void CCablanView::IdentifyingVolumeCells()
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int i, j;

	memset (VolumeCellLessthanDesignPayload, 0, sizeof(VolumeCellLessthanDesignPayload));
	
	for (j = pDoc->UnitBoundary.startRow; j <= pDoc->UnitBoundary.endRow; ++j)
	{
		for(i = pDoc->UnitBoundary.startCol; i <= pDoc->UnitBoundary.endCol; ++i)
		{
			if (pDoc->Volume[i][j] < DesignPayload && pDoc->Volume[i][j] > 0)//Cautions!! this is using Volume[][] instead of OriginalVolume[][].
				VolumeCellLessthanDesignPayload[i][j] = TRUE;
		}
	}
}



void CCablanView::SortingVolumeCells()
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int i, j;
	float volume[MAXCOLUMNS * MAXROWS];//keep volume temporarily.
	int order[MAXCOLUMNS * MAXROWS];//keep order temporarily.
	int count = 0;

	SORTEDVOLUMECELLS tempSortedVolumeCells[MAXCOLUMNS * MAXROWS];
	
	
	for (j = pDoc->UnitBoundary.startRow; j <= pDoc->UnitBoundary.endRow; ++j)
	{
		for(i = pDoc->UnitBoundary.startCol; i <= pDoc->UnitBoundary.endCol; ++i)
		{
			if (VolumeCellLessthanDesignPayload[i][j] == TRUE)
			{
				++count;
				SortedVolumeCells[count].col = i;//count starts from 1.
				SortedVolumeCells[count].row = j;
				volume[count] = pDoc->OriginalVolume[i][j];
				order[count] = count;
			}
		}
	}

	TotalNumberofVolumeCellsLessthanDesignPayload = count;

	memcpy(tempSortedVolumeCells, SortedVolumeCells, sizeof(SortedVolumeCells));

	Shell_Sorting(volume, order, count);

	for (i = 1; i <= count; ++i)
	{
		SortedVolumeCells[i].col = tempSortedVolumeCells[order[i]].col;
		SortedVolumeCells[i].row = tempSortedVolumeCells[order[i]].row;
	}

}



void CCablanView::Shell_Sorting(float x[], int o[], int n)
{
	////////////////////////////////////////////////////////////////////////////
	//This subroutine is for
	//1. sorting Fromnode CSTring array.
	//2. keeping trace of order using Order array.
	//This routine is from Rojiani's book page 431.
	////////////////////////////////////////////////////////////////////////////
	// This routine is called by DataManager()

	float temp;
	int i;
	int last;
	bool is_sorted;
	int jump=n/2;

	int temp_o;

	while (jump >= 1)
	{
		last = n - jump;
		is_sorted = FALSE;

		while (!is_sorted)
		{
			is_sorted = TRUE;
			for (i = 1; i <= last; ++i)
			//i starts from 1 and end at last, since the x and o array start from [1] instead [0]
			{
				if (x[i] < x[i+jump])
				{
					temp = x[i];
					x[i] = x[i+jump];
					x[i+jump] = temp;
					is_sorted = FALSE;

					temp_o = o[i];
					o[i] = o[i+jump];
					o[i+jump] = temp_o;
				}
			}
		}
		jump = jump/2;
	}
}


void CCablanView::OnLayersVolumeClumpcells() 
{
	//temporary
	if (boolTimberParcelAlreadySet == FALSE)
	{
		SettingTimberVolumeCells();	
		DrawHarvestArea();
	}
	else
		DrawTimberParcels();


}

void CCablanView::OnLayersVolumeShowvolume() 
{
	DrawVolumeLayer();
}







//////////////////////////////////////////////////////////////////////////////////////
//BOUNDARY
//Setting Harvesting Unit Boundary
///////////////////////////////////////////////////////////////////////////////////////

void CCablanView::OnInputdataHarvestunitboundaryBymouse() 
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	boolSettingHarvestUnitBoundary = TRUE;
	pDoc->TotalNumberofUnitBoundaryVertices = 0;

}

void CCablanView::SettingHarvestingUnitBoundary()
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int i, j, k, l;
	int maxcol, maxrow, mincol, minrow;
	float slope;
	float intercept;
	CPoint StartingCell, EndingCell;
	float a[MAXUNITBOUNDARYVERTICES];//slope
	float b[MAXUNITBOUNDARYVERTICES];//intercept
	int minc[MAXUNITBOUNDARYVERTICES], maxc[MAXUNITBOUNDARYVERTICES];//range of column and row of each lateral side.
	int minr[MAXUNITBOUNDARYVERTICES], maxr[MAXUNITBOUNDARYVERTICES];
	int centercol, centerrow;//center of the boundary
	int interceptCount;//keep how many lateral sides are intercepted by the line between each cell and the center.
	float x, y;//intercept point
	float a1, b1;//slope and intercept of the line between cell and the center.
	int mincc, maxcc, minrr, maxrr;//range of column and row of the line between each cell and the center.
	float keptc, keptr;




	for (i = 1; i < pDoc->TotalNumberofUnitBoundaryVertices; ++i)//not include pDoc->TotalNumberofUnitBoundaryVertices
		DrawEachLine(pDoc->HarvestUnitBoundary[i].col, pDoc->HarvestUnitBoundary[i].row, pDoc->HarvestUnitBoundary[i+1].col, pDoc->HarvestUnitBoundary[i+1].row, 5, RGB(255,255,255));
	DrawEachLine(pDoc->HarvestUnitBoundary[i].col, pDoc->HarvestUnitBoundary[i].row, pDoc->HarvestUnitBoundary[1].col, pDoc->HarvestUnitBoundary[1].row, 5, RGB(255,255,255));

	//initialize
	memset(pDoc->CellsWithinUnitBoundary, FALSE, sizeof(pDoc->CellsWithinUnitBoundary));

	mincol = MAXCOLUMNS;
	minrow = MAXROWS;

	maxcol = 0;
	maxrow = 0;


	//if harvesting unit is already input from GIS layer -- mixing GIS layer with user-defined boundary
	//setting a large boundary for data from GIS layer
	for (j = 1; j <= pDoc->DTMrows; ++j)
	{
		for (i = 1; i <= pDoc->DTMcolumns; i++)
		{
			if (pDoc->HarvestArea[i][j] > 0)
			{
				pDoc->CellsWithinUnitBoundary[i][j] = TRUE;
				if (maxcol < i)
					maxcol = i;
				if (mincol > i)
					mincol = i;
				if (maxrow < j)
					maxrow = j;
				if (minrow > j)
					minrow = j;
			}
		}
	}



	//Step1: set a large boundary (rectagular shape)
	for (i = 1; i <= pDoc->TotalNumberofUnitBoundaryVertices; ++i)
	{
		if (maxcol < pDoc->HarvestUnitBoundary[i].col)
			maxcol = pDoc->HarvestUnitBoundary[i].col;
		if (mincol > pDoc->HarvestUnitBoundary[i].col)
			mincol = pDoc->HarvestUnitBoundary[i].col;
		if (maxrow < pDoc->HarvestUnitBoundary[i].row)
			maxrow = pDoc->HarvestUnitBoundary[i].row;
		if (minrow > pDoc->HarvestUnitBoundary[i].row)
			minrow = pDoc->HarvestUnitBoundary[i].row;
	}


	//keep this rectangle data in global variables.
	pDoc->UnitBoundary.startCol = mincol;
	pDoc->UnitBoundary.startRow = minrow;
	pDoc->UnitBoundary.endCol = maxcol;
	pDoc->UnitBoundary.endRow = maxrow;


	//Step2: Select grid cells falling right on the boundary.
	for (k = 1; k <= pDoc->TotalNumberofUnitBoundaryVertices; ++k)
	{
		if (k < pDoc->TotalNumberofUnitBoundaryVertices)
		{
			StartingCell.x = pDoc->HarvestUnitBoundary[k].col;
			StartingCell.y = pDoc->HarvestUnitBoundary[k].row;
			EndingCell.x = pDoc->HarvestUnitBoundary[k+1].col;
			EndingCell.y = pDoc->HarvestUnitBoundary[k+1].row;
		}
		else//if k = pDoc->TotalNumberofUnitBoundaryVertices, then connect to the first vertex.
		{
			StartingCell.x = pDoc->HarvestUnitBoundary[k].col;
			StartingCell.y = pDoc->HarvestUnitBoundary[k].row;
			EndingCell.x = pDoc->HarvestUnitBoundary[1].col;
			EndingCell.y = pDoc->HarvestUnitBoundary[1].row;
		}

	
		//draw a straight line to the EndingCell.
		if (StartingCell.x - EndingCell.x != 0)
		{
			slope = (float)(StartingCell.y - EndingCell.y) / (float)(StartingCell.x - EndingCell.x);
			intercept = (float)EndingCell.y - (float)EndingCell.x * slope;
		}
		else
			slope = (float)999999999.9;


		if (fabs(slope) > 1 && fabs(slope) < 10000.0)//select y-axis as an independent variable
		{
			if (StartingCell.y < EndingCell.y)
			{
				for (j = StartingCell.y; j <= EndingCell.y; ++j)//changing in y direction
				{
					i = (int)(((float)j - intercept)/slope);//find columne 
				
					pDoc->CellsWithinUnitBoundary[i][j] = TRUE;
				}
			}
			else
			{
				for (j = StartingCell.y; j >= EndingCell.y; --j)//changing in y direction
				{
					i = (int)(((float)j - intercept)/slope);//find columne 
				
					pDoc->CellsWithinUnitBoundary[i][j] = TRUE;
				}
			}
		}
		else if (fabs(slope) > 10000.0)//when slope is infinite ->North and South direction
		{
			if (StartingCell.y < EndingCell.y)
			{
				i = EndingCell.x;
				for (j = StartingCell.y; j <= EndingCell.y; ++j)//changing in y direction
				{
					pDoc->CellsWithinUnitBoundary[i][j] = TRUE;
				}
			}
			else
			{
				i = EndingCell.x;
				for (j = StartingCell.y; j >= EndingCell.y; --j)//changing in y direction
				{
					pDoc->CellsWithinUnitBoundary[i][j] = TRUE;
				}
			}
		}
		else//select x-axis as an independent variable
		{
			if (StartingCell.x < EndingCell.x)
			{
				for (i = StartingCell.x; i <= EndingCell.x; ++i)//changing in x direction
				{
					j = (int)((float)i * slope + intercept);//find columne 
					pDoc->CellsWithinUnitBoundary[i][j] = TRUE;
				}
			}
			else
			{
				for (i = StartingCell.x; i >= EndingCell.x; --i)//changing in x direction
				{
					j = (int)((float)i * slope + intercept);//find columne 
					pDoc->CellsWithinUnitBoundary[i][j] = TRUE;
				}
			}
		}
	}


	//Step3: Decide outside cell and inside cell of boundary
	//Step3-1:setting range of address and calculte slope and intercept of boundary lateral sides.
	for (k = 1; k <= pDoc->TotalNumberofUnitBoundaryVertices; ++k)
	{
		if (k < pDoc->TotalNumberofUnitBoundaryVertices)
		{
			StartingCell.x = pDoc->HarvestUnitBoundary[k].col;
			StartingCell.y = pDoc->HarvestUnitBoundary[k].row;
			EndingCell.x = pDoc->HarvestUnitBoundary[k+1].col;
			EndingCell.y = pDoc->HarvestUnitBoundary[k+1].row;
			if (pDoc->HarvestUnitBoundary[k].col < pDoc->HarvestUnitBoundary[k+1].col)
			{
				minc[k] = pDoc->HarvestUnitBoundary[k].col;
				maxc[k] = pDoc->HarvestUnitBoundary[k+1].col;
			}
			else
			{
				maxc[k] = pDoc->HarvestUnitBoundary[k].col;
				minc[k] = pDoc->HarvestUnitBoundary[k+1].col;
			}

			if (pDoc->HarvestUnitBoundary[k].row < pDoc->HarvestUnitBoundary[k+1].row)
			{
				minr[k] = pDoc->HarvestUnitBoundary[k].row;
				maxr[k] = pDoc->HarvestUnitBoundary[k+1].row;
			}
			else
			{
				maxr[k] = pDoc->HarvestUnitBoundary[k].row;
				minr[k] = pDoc->HarvestUnitBoundary[k+1].row;
			}
		}
		else//if k = pDoc->TotalNumberofUnitBoundaryVertices, then connect to the first vertex.
		{
			StartingCell.x = pDoc->HarvestUnitBoundary[k].col;
			StartingCell.y = pDoc->HarvestUnitBoundary[k].row;
			EndingCell.x = pDoc->HarvestUnitBoundary[1].col;
			EndingCell.y = pDoc->HarvestUnitBoundary[1].row;
			if (pDoc->HarvestUnitBoundary[k].col < pDoc->HarvestUnitBoundary[1].col)
			{
				minc[k] = pDoc->HarvestUnitBoundary[k].col;
				maxc[k] = pDoc->HarvestUnitBoundary[1].col;
			}
			else
			{
				maxc[k] = pDoc->HarvestUnitBoundary[k].col;
				minc[k] = pDoc->HarvestUnitBoundary[1].col;
			}

			if (pDoc->HarvestUnitBoundary[k].row < pDoc->HarvestUnitBoundary[1].row)
			{
				minr[k] = pDoc->HarvestUnitBoundary[k].row;
				maxr[k] = pDoc->HarvestUnitBoundary[1].row;
			}
			else
			{
				maxr[k] = pDoc->HarvestUnitBoundary[k].row;
				minr[k] = pDoc->HarvestUnitBoundary[1].row;
			}
		}

	
		//calculate slope and intercept of each later side of the boundary.
		if (StartingCell.x - EndingCell.x != 0)
		{
			a[k] = (float)(StartingCell.y - EndingCell.y) / (float)(StartingCell.x - EndingCell.x);
			b[k] = (float)EndingCell.y - (float)EndingCell.x * a[k];
		}
		else
		{
			a[k] = (float)999999999.9;
			b[k] = (float)StartingCell.x;
		}
	}

	//location of center point of the boundary.
	centercol = (maxcol-mincol)/2 + mincol;
	centerrow = (maxrow-minrow)/2 + minrow;

	//a loop for looking at each cell within the Rectangluar boundary.
	for (i = mincol; i <= maxcol; ++i)
	{
		for (j = minrow; j <= maxrow; ++j)
		{
			interceptCount = 0;
			keptc = 0.;
			keptr = 0.;
			//set range of the line between each cell and the center.
			if (centercol < i)
			{
				mincc = centercol;
				maxcc = i;
			}
			else
			{
				maxcc = centercol;
				mincc = i;
			}

			if (centerrow < j)
			{
				minrr = centerrow;
				maxrr = j;
			}
			else
			{
				maxrr = centerrow;
				minrr = j;
			}

			//linear equation from the grid cell to the center cell.
			if (i - centercol != 0)
			{
				a1 = (float)(j - centerrow) / (float)(i - centercol);
				b1 = (float)centerrow - (float)centercol * a1;
			}
			else
			{
				a1 = (float)999999999.9;
				b1 = (float)i;
			}
			
			
			//Step 3-2: Decide whether this cell is outside or inside of the boundary.
			for (k = 1; k <= pDoc->TotalNumberofUnitBoundaryVertices; ++k)
			{
				//finding intercepting point. 
				if (a[k] < 100000. && a1 < 100000.)
				{
					if (a1 != a[k])
					{
						x = (b[k] - b1)/(a1 - a[k]);
						y = a1 * x + b1;
					}
					else
					{
						x = 10000000.;
						y = 10000000.;
					}
				}
				else if (a[k] >= 100000. && a1 < 100000.)
				{
					x = b[k];
					y = a1 * x + b1;
				}
				else if (a1 >= 100000. && a[k] < 100000.)
				{
					x = b1;
					y = a[k] * x + b[k];
				}
				else
				{
					x = 10000000.;
					y = 10000000.;
				}
				

				//checking whether this intercept point falls within the range. If it does, then count this intercepting point. 
				if (x >= minc[k] && x <= maxc[k] && y >= minr[k] && y <= maxr[k])
				{
					if (x >= mincc && x <= maxcc && y >= minrr && y <= maxrr)
					{
						if (x != keptc || y != keptr)//preventing that intercepting point falls on crossing point between two later sides-->Counted two although the cell is outside.
						{
							++interceptCount;//counting how many lateral sides this cell-center cell will cross over. 
							keptc = x;
							keptr = y;
						}
					}
				}
			}

			//if the line of cell-center intercepts even number of later sides, then this cell is within the boundary.
			if (interceptCount % 2 == 0)
				pDoc->CellsWithinUnitBoundary[i][j] = TRUE;
		}
	}


	//Step 4: Checking
	//Step 4-1: some cells are not selected as being within the boundary because the line hit two boundary lateral sides but count only one since using keptc and keptr.
	//keptc and keptr prevents some cells outside boundary from being recognized as cells inside, but it also affects inside cells.
	//check all grid cells and if outside cell is surrounded by all inside cells, then this cell will be inside cell.
	int count;
	for (i = mincol; i <= maxcol; ++i)
	{
		for (j = minrow; j <= maxrow; ++j)
		{
			count = 0;
			if (pDoc->CellsWithinUnitBoundary[i][j] == FALSE)
			{
				for (k = i-1; k <= i+1; ++k)
				{
					for (l = j-1; l <= j+1; ++l)
					{
						if (pDoc->CellsWithinUnitBoundary[k][l] == TRUE)
							++count;
					}
				}
				if (count >= 7)
					pDoc->CellsWithinUnitBoundary[i][j] = TRUE;
			}
		}
	}

	//Step 4-2: the cells surrounded by all outside cells will be changed to outside cell.
	for (i = mincol; i <= maxcol; ++i)
	{
		for (j = minrow; j <= maxrow; ++j)
		{
			count = 0;
			if (pDoc->CellsWithinUnitBoundary[i][j] == TRUE)
			{
				for (k = i-1; k <= i+1; ++k)
				{
					for (l = j-1; l <= j+1; ++l)
					{
						if (pDoc->CellsWithinUnitBoundary[k][l] == FALSE)
							++count;
					}
				}
				if (count >= 7)
					pDoc->CellsWithinUnitBoundary[i][j] = FALSE;
			}
		}
	}


/*
	//Step 5: drawing cells within the boundary.
	for (i = mincol; i <= maxcol; ++i)
	{
		for (j = minrow; j <= maxrow; ++j)
		{
			if (pDoc->CellsWithinUnitBoundary[i][j] == TRUE)
				DrawEachCell(i, j, RGB(200,200,200));
		}
	}
*/

	boolSettingHarvestUnitBoundary = FALSE;
	DrawDTM();


}










//////////////////////////////////////////////////////////////////////////////////////
//FISH STREAMS
//
///////////////////////////////////////////////////////////////////////////////////////
void CCablanView::OnLayersStreamsCreatebuffer() 
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int i, j, k, l;
	int bufferwidth;//number of cells around fish stream-temporarily set??????????????????????????
	int colstart, colend, rowstart, rowend;
	
	CDialogFishStreamBuffer dlg;

	
	if (dlg.DoModal() == IDOK)
	{
		pDoc->FishStreamBufferWidth = dlg.m_FishStreamBufferWidth;
		memset(pDoc->FishStreamBuffer, 0, sizeof(pDoc->FishStreamBuffer));
		memcpy(pDoc->Volume, pDoc->OriginalVolume, sizeof(pDoc->Volume));
	}


	if (pDoc->FishStreamBufferWidth > 0)
	{
		//calculate the number of cells
		bufferwidth = pDoc->FishStreamBufferWidth / pDoc->DTMcellsize;
		bufferwidth /= 2;//make even number, since stream occupies one grid cell.
		bufferwidth *= 2; 
	
		for (i = 1; i <= pDoc->DTMcolumns; ++i)
		{
			for (j = 1; j <= pDoc->DTMrows; ++j)
			{
				if (pDoc->FishStreams[i][j] > 0)
				{
					colstart = i - bufferwidth/2;//bufferwidth is even number now.
					colend = i + bufferwidth/2;
					rowstart = j - bufferwidth/2;
					rowend = j + bufferwidth/2;

					if (colstart < 1)
						colstart = 1;
					if (rowstart < 1)
						rowstart = 1;
					if (colend > pDoc->DTMcolumns)
						colend = pDoc->DTMcolumns;
					if (rowend > pDoc->DTMrows)
						rowend = pDoc->DTMrows;
				
					for (k = colstart; k <= colend; ++k)
					{
						for (l = rowstart; l <= rowend; ++l)
						{
							pDoc->FishStreamBuffer[k][l] = pDoc->FishStreams[i][j];
							pDoc->Volume[k][l] = 0.;//put volume zero in the cells within the buffer zone.
						}
					}
				}
			}
		}
	}
	
	DrawDTM();
	DrawStreamBuffer();

}

void CCablanView::OnLayersStreamsShowstreams() 
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	if (pDoc->FishStreamBufferWidth <= 0)
		OnLayersStreamsCreatebuffer();
	else
		DrawStreamBuffer();
}


void CCablanView::DrawStreamBuffer()
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pDC = new CClientDC(this);
	OnPrepareDC(pDC);
	pDC->SetMapMode(MM_ISOTROPIC);
	pDC->SetWindowExt(ViewScale, ViewScale);

	CPoint cpFill;
	CBrush brsStream, brsStreamBuffer;

	int i, j;

	//create brush colored by elevation
	brsStream.CreateSolidBrush(RGB(0,0,150));
	brsStreamBuffer.CreateSolidBrush(RGB(0,0,250));
	
	CBrush *pOldBrush = pDC->SelectObject(&brsStream);

	for (j = 1; j <= pDoc->DTMrows; ++j)
	{
		for(i = 1; i <= pDoc->DTMcolumns; ++i)
		{
			if (pDoc->FishStreamBuffer[i][j] > 0)//data are existing
			{
				//generate coordinate to draw grids on the screen
				DrawFrom.x = (long)(pDoc->DTMCoord[i][j].x - pDoc->DTMminX - (pDoc->DTMcellsize/2.0) + cpMargin.x);// (CellHeight/2) integer problem??
				DrawFrom.y = (long)(pDoc->DTMCoord[i][j].y - pDoc->DTMmaxY - (pDoc->DTMcellsize/2.0) - cpMargin.y);
				DrawTo.x = (long)(pDoc->DTMCoord[i][j].x - pDoc->DTMminX + (pDoc->DTMcellsize/2.0) + cpMargin.x);
				DrawTo.y = (long)(pDoc->DTMCoord[i][j].y - pDoc->DTMmaxY + (pDoc->DTMcellsize/2.0) - cpMargin.y);

				cpFill.x = (long)(pDoc->DTMCoord[i][j].x - pDoc->DTMminX + cpMargin.x);
				cpFill.y = (long)(pDoc->DTMmaxY - pDoc->DTMCoord[i][j].y - cpMargin.y);
			
				// drawing
				CRect rcSquare(DrawFrom.x, DrawFrom.y, DrawTo.x, DrawTo.y);
			
				if (pDoc->FishStreams[i][j] > 0)
					pDC->FillRect(rcSquare, &brsStream);
				else
					pDC->FillRect(rcSquare, &brsStreamBuffer);
			}
		}
	}

	pDC->SelectObject(pOldBrush);

	//delete CBrush Object
	brsStream.DeleteObject();
	brsStreamBuffer.DeleteObject();


}


void CCablanView::OnLayersRoadsShowroads() 
{
	DrawExistingRoads();
		
}


void CCablanView::DrawExistingRoads()
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pDC = new CClientDC(this);
	OnPrepareDC(pDC);
	pDC->SetMapMode(MM_ISOTROPIC);
	pDC->SetWindowExt(ViewScale, ViewScale);

	CPoint cpFill;
	CBrush brsExistingRoads;

	int i, j;

	//create brush colored by elevation
	brsExistingRoads.CreateSolidBrush(RGB(0,200,250));
		
	CBrush *pOldBrush = pDC->SelectObject(&brsExistingRoads);

	for (j = 1; j <= pDoc->DTMrows; ++j)
	{
		for(i = 1; i <= pDoc->DTMcolumns; ++i)
		{
			if (pDoc->ExistingRoads[i][j] > 0)//data are existing
			{
				//generate coordinate to draw grids on the screen
				DrawFrom.x = (long)(pDoc->DTMCoord[i][j].x - pDoc->DTMminX - (pDoc->DTMcellsize/2.0) + cpMargin.x);// (CellHeight/2) integer problem??
				DrawFrom.y = (long)(pDoc->DTMCoord[i][j].y - pDoc->DTMmaxY - (pDoc->DTMcellsize/2.0) - cpMargin.y);
				DrawTo.x = (long)(pDoc->DTMCoord[i][j].x - pDoc->DTMminX + (pDoc->DTMcellsize/2.0) + cpMargin.x);
				DrawTo.y = (long)(pDoc->DTMCoord[i][j].y - pDoc->DTMmaxY + (pDoc->DTMcellsize/2.0) - cpMargin.y);

				cpFill.x = (long)(pDoc->DTMCoord[i][j].x - pDoc->DTMminX + cpMargin.x);
				cpFill.y = (long)(pDoc->DTMmaxY - pDoc->DTMCoord[i][j].y - cpMargin.y);
			
				// drawing
				CRect rcSquare(DrawFrom.x, DrawFrom.y, DrawTo.x, DrawTo.y);
			
				pDC->FillRect(rcSquare, &brsExistingRoads);
			}
		}
	}

	pDC->SelectObject(pOldBrush);

	//delete CBrush Object
	brsExistingRoads.DeleteObject();
}





void CCablanView::OnNetworkRoadsTimberexits() 
{
	boolTimberExits = TRUE;

}



void CCablanView::OnLandingsDeletelandinglocation() 
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int i;
	CDialogDeleteLandings dlg;
	int tempCurrentLanding;

	dlg.TotalNumberofLandings = pDoc->TotalNumberofLandings;
	dlg.CurrentLanding = 1;

	if (dlg.DoModal() == IDOK)
	{
		tempCurrentLanding = dlg.CurrentLanding;

		if (pDoc->TotalNumberofLandings == 1)
		{
			pDoc->Landings[1].row = pDoc->Landings[0].row;
			pDoc->Landings[1].column = pDoc->Landings[0].column;
			pDoc->Landings[1].CableSystem = pDoc->Landings[0].CableSystem;
			pDoc->LandingCosts[1] = pDoc->LandingCosts[0];
			pDoc->LandingSharingTowerID[1] = pDoc->LandingSharingTowerID[0];
			pDoc->boolLandingPayloadAnalysis[1] = pDoc->boolLandingPayloadAnalysis[0];

			memcpy(pDoc->Cableways[1], pDoc->Cableways[0], sizeof(pDoc->Cableways[1]));
			memcpy(pDoc->Payload[1], pDoc->Payload[0], sizeof(pDoc->Payload[1]));
			memcpy(pDoc->PayloadAtEachTP[1], pDoc->PayloadAtEachTP[0], sizeof(pDoc->PayloadAtEachTP[1]));
			memcpy(pDoc->CycleTimeAtEachTP[1], pDoc->CycleTimeAtEachTP[0], sizeof(pDoc->CycleTimeAtEachTP[1]));
			memcpy(pDoc->CycleTimeAtEachTP[1], pDoc->CycleTimeAtEachTP[0], sizeof(pDoc->CycleTimeAtEachTP[1]));
		}
		else if (tempCurrentLanding == pDoc->TotalNumberofLandings)//Sept.3,2002
		{
			pDoc->Landings[tempCurrentLanding].row = pDoc->Landings[0].row;
			pDoc->Landings[tempCurrentLanding].column = pDoc->Landings[0].column;
			pDoc->Landings[tempCurrentLanding].CableSystem = pDoc->Landings[0].CableSystem;
			pDoc->LandingCosts[tempCurrentLanding] = pDoc->LandingCosts[0];
			pDoc->LandingSharingTowerID[tempCurrentLanding] = pDoc->LandingSharingTowerID[0];
			pDoc->boolLandingPayloadAnalysis[tempCurrentLanding] = pDoc->boolLandingPayloadAnalysis[0];

			memcpy(pDoc->Cableways[tempCurrentLanding], pDoc->Cableways[0], sizeof(pDoc->Cableways[tempCurrentLanding]));
			memcpy(pDoc->Payload[tempCurrentLanding], pDoc->Payload[0], sizeof(pDoc->Payload[tempCurrentLanding]));
			memcpy(pDoc->PayloadAtEachTP[tempCurrentLanding], pDoc->PayloadAtEachTP[0], sizeof(pDoc->PayloadAtEachTP[tempCurrentLanding]));
			memcpy(pDoc->CycleTimeAtEachTP[tempCurrentLanding], pDoc->CycleTimeAtEachTP[0], sizeof(pDoc->CycleTimeAtEachTP[tempCurrentLanding]));
			memcpy(pDoc->CycleTimeAtEachTP[tempCurrentLanding], pDoc->CycleTimeAtEachTP[0], sizeof(pDoc->CycleTimeAtEachTP[tempCurrentLanding]));
		}
		else
		{
			for (i = 1; i < pDoc->TotalNumberofLandings; ++i)//not include pDoc->TotalNumberofLandings since it will erase one landing and reduce the totalnumberoflandings by one.
			{
				if (i >= tempCurrentLanding)
				{
					pDoc->Landings[i].row = pDoc->Landings[i+1].row;
					pDoc->Landings[i].column = pDoc->Landings[i+1].column;
					pDoc->Landings[i].CableSystem = pDoc->Landings[i+1].CableSystem;
					pDoc->LandingCosts[i] = pDoc->LandingCosts[i+1];
					pDoc->LandingSharingTowerID[i] = pDoc->LandingSharingTowerID[i+1];
					pDoc->boolLandingPayloadAnalysis[i] = pDoc->boolLandingPayloadAnalysis[i+1];

					memcpy(pDoc->Cableways[i], pDoc->Cableways[i+1], sizeof(pDoc->Cableways[i]));
					memcpy(pDoc->Payload[i], pDoc->Payload[i+1], sizeof(pDoc->Payload[i]));
					memcpy(pDoc->PayloadAtEachTP[i], pDoc->PayloadAtEachTP[i+1], sizeof(pDoc->PayloadAtEachTP[i]));
					memcpy(pDoc->CycleTimeAtEachTP[i], pDoc->CycleTimeAtEachTP[i+1], sizeof(pDoc->CycleTimeAtEachTP[i]));
					memcpy(pDoc->CycleTimeAtEachTP[i], pDoc->CycleTimeAtEachTP[i+1], sizeof(pDoc->CycleTimeAtEachTP[i]));

					//Sept.3,2002
					pDoc->Landings[i+1].row = pDoc->Landings[0].row;
					pDoc->Landings[i+1].column = pDoc->Landings[0].column;
					pDoc->Landings[i+1].CableSystem = pDoc->Landings[0].CableSystem;
					pDoc->LandingCosts[i+1] = pDoc->LandingCosts[0];
					pDoc->LandingSharingTowerID[i+1] = pDoc->LandingSharingTowerID[0];
					pDoc->boolLandingPayloadAnalysis[i+1] = pDoc->boolLandingPayloadAnalysis[0];

					memcpy(pDoc->Cableways[i+1], pDoc->Cableways[0], sizeof(pDoc->Cableways[i+1]));
					memcpy(pDoc->Payload[i+1], pDoc->Payload[0], sizeof(pDoc->Payload[i+1]));
					memcpy(pDoc->PayloadAtEachTP[i+1], pDoc->PayloadAtEachTP[0], sizeof(pDoc->PayloadAtEachTP[i+1]));
					memcpy(pDoc->CycleTimeAtEachTP[i+1], pDoc->CycleTimeAtEachTP[0], sizeof(pDoc->CycleTimeAtEachTP[i+1]));
					memcpy(pDoc->CycleTimeAtEachTP[i+1], pDoc->CycleTimeAtEachTP[0], sizeof(pDoc->CycleTimeAtEachTP[i+1]));
				}
			}
		}
		--pDoc->TotalNumberofLandings;
	}			
}





void CCablanView::OnInputdataLandingcosts() 
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	int i;
	CDialogLandingCosts dlg;

	memcpy(dlg.LandingCosts, pDoc->LandingCosts, sizeof(pDoc->LandingCosts));
	memcpy(dlg.LandingSharingTowerID, pDoc->LandingSharingTowerID, sizeof(pDoc->LandingSharingTowerID));
	dlg.TotalNumberofLandings = pDoc->TotalNumberofLandings;
	dlg.CurrentLanding = 1;//always shows 1 as the default landing ID.


	if (dlg.DoModal() == IDOK)
	{
		memcpy(pDoc->LandingCosts, dlg.LandingCosts, sizeof(pDoc->LandingCosts));
		memcpy(pDoc->LandingSharingTowerID, dlg.LandingSharingTowerID, sizeof(pDoc->LandingSharingTowerID));
		for (i = 1; i <= pDoc->TotalNumberofLandings; ++i)
		{
			if (pDoc->LandingSharingTowerID[i] > 0)//if there is any sharing tower,
				//put the same amount of landing costs as the sharing tower has.
				pDoc->LandingCosts[i] = pDoc->LandingCosts[pDoc->LandingSharingTowerID[i]];
		}
	}

}






void CCablanView::SaveResultstoFile()
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int i, j;

	FILE *outfile_ptr;
	CString tempString;
	
	tempString = "e:/thesis/output/ResultFile.dat";
	outfile_ptr = fopen(tempString, "w");
	
	fprintf(outfile_ptr, "%d\n", TotalNumberofFinalLandings);
	for (i = 1; i <= TotalNumberofFinalLandings; ++i)
		fprintf(outfile_ptr, "%d %d %d\n", LandingLocationResults[i].col, LandingLocationResults[i].row, LandingLocationResults[i].nodeno);

	fprintf(outfile_ptr, "%d\n", TotalNumberofFinalCableways);
	for (i = 1; i <= TotalNumberofFinalCableways; ++i)
		fprintf(outfile_ptr, "%d %d %d\n", CableResults[i].landing, CableResults[i].system, CableResults[i].cableway);

	fprintf(outfile_ptr, "%d\n", TotalNumberofFinalRoadCells);
	for (i = 1; i <= TotalNumberofFinalRoadCells; ++i)
		fprintf(outfile_ptr, "%d %d %d\n", RoadCellResults[i].col, RoadCellResults[i].row, RoadCellResults[i].nodeno);


	for (j = 1; j <= pDoc->DTMrows; ++j)
	{
		for(i = 1; i <= pDoc->DTMcolumns; ++i)
			fprintf(outfile_ptr, "%f %d %d %d\n", pDoc->Volume[i][j], pDoc->CellsWithinUnitBoundary[i][j], FromCellToCableResults[i][j].x, FromCellToCableResults[i][j].y);
	}

	fclose(outfile_ptr);
}


void CCablanView::OpenResultstoFile()
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int i, j;

	FILE *infile_ptr;
	CString tempString;
	
	//tempString = "e:/thesis/output/kollerall27/output/ResultFile.dat";
	tempString = ResultFile;
	infile_ptr = fopen(tempString, "rb");
	
	fscanf(infile_ptr, "%d\n", &TotalNumberofFinalLandings);
	for (i = 1; i <= TotalNumberofFinalLandings; ++i)
		fscanf(infile_ptr, "%d %d %d\n", &LandingLocationResults[i].col, &LandingLocationResults[i].row, &LandingLocationResults[i].nodeno);

	fscanf(infile_ptr, "%d\n", &TotalNumberofFinalCableways);
	for (i = 1; i <= TotalNumberofFinalCableways; ++i)
		fscanf(infile_ptr, "%d %d %d\n", &CableResults[i].landing, &CableResults[i].system, &CableResults[i].cableway);

	fscanf(infile_ptr, "%d\n", &TotalNumberofFinalRoadCells);
	for (i = 1; i <= TotalNumberofFinalRoadCells; ++i)
		fscanf(infile_ptr, "%d %d %d\n", &RoadCellResults[i].col, &RoadCellResults[i].row, &RoadCellResults[i].nodeno);

	for (j = 1; j <= pDoc->DTMrows; ++j)
	{
		for(i = 1; i <= pDoc->DTMcolumns; ++i)
			fscanf(infile_ptr, "%f %d %d %d\n", &pDoc->Volume[i][j], &pDoc->CellsWithinUnitBoundary[i][j], &FromCellToCableResults[i][j].x, &FromCellToCableResults[i][j].y);
	}

	fclose(infile_ptr);
}

void CCablanView::OnResultsFilemanagerOpenfile() 
{
	OpenResultstoFile();
	
}

void CCablanView::OnResultsFilemanagerSavefile() 
{
	SaveResultstoFile();
	
}




void CCablanView::OnResultsRoadreports() 
{
	CCablanDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	int i, j;
	int totalnumberofsales;
	int Method;
	int SaleNumber;
	float Volume;
	int NumberofLinksinPath;
	int beginNode, endNode, tempNodeID, tempLinkID;
	float totaldist;
	float slope0_5, slope5_10, slope10_15, slope15_20, slope20_25, slope25_30;
	float sideslope0_15, sideslope15_30, sideslope30_45, sideslope45_60, sideslope60_75, sideslope75_90;

	float grandslope0_5, grandslope5_10, grandslope10_15, grandslope15_20, grandslope20_25, grandslope25_30;
	float grandsideslope0_15, grandsideslope15_30, grandsideslope30_45, grandsideslope45_60, grandsideslope60_75, grandsideslope75_90;

	bool boolLinks[MAX_LINKS];

	float haulingcostperlanding;
	float roadcostperlanding;

	float grandtotalvolume = 0.;
	float grandtotalhaulingcost = 0.;
	float grandtotalroadcost = 0.;
	float grandtotallength = 0.;

	FILE *infile_ptr, *outfile_ptr;
	CString FileName1 = "E:\\thesis\\output\\bestroadpath.txt";//define file name for keeping the best route
	infile_ptr = fopen(FileName1, "rb");
	
	CString FileName2 = "E:\\thesis\\output\\RoadMileage.txt";//define file name for keeping the best route
	outfile_ptr = fopen(FileName2, "w");
	
	
	GenerateRoadNetwork(-1);//generate all link data//if -1 than keep distance.
	GenerateRoadSaleData(LandingVolume, 0);
	pDoc->DataManager();//sorting and numbering user's input data
	memset(boolLinks, 0, sizeof(boolLinks));

	//read total number of sales including excluded sales
	fscanf(infile_ptr, "%d", &Method);//which algorithm found this solution->Global variable
	fscanf(infile_ptr, "%d", &totalnumberofsales);//local temporary variable -> just for skipping this data in file

	grandslope0_5 = 0.;
	grandslope5_10 = 0.;
	grandslope10_15 = 0.;
	grandslope15_20 = 0.;
	grandslope20_25 = 0.;
	grandslope25_30 = 0.;

	grandsideslope0_15 = 0.;
	grandsideslope15_30 = 0.;
	grandsideslope30_45 = 0.;
	grandsideslope45_60 = 0.;
	grandsideslope60_75 = 0.;
	grandsideslope75_90 = 0.;



	//First; count volume on AccumulatedVolumeperLink
	for (i = 1; i <= totalnumberofsales; ++i)
	{
		if (pDoc->SaleExclusion[i] != TRUE)
		{
			//read result for each sale
			fscanf(infile_ptr, "%d", &SaleNumber);
			fscanf(infile_ptr, "%f", &Volume);
			fscanf(infile_ptr, "%d", &NumberofLinksinPath);

			fprintf(outfile_ptr, "Sale: %d\n", SaleNumber);
			fprintf(outfile_ptr, "Volume: %f\n", Volume);
			fprintf(outfile_ptr, "Number of links: %d\n", NumberofLinksinPath);
			
			fprintf(outfile_ptr, "Dist	Slope	SideSlope\n");
			totaldist = 0.0;
			slope0_5 = 0.;
			slope5_10 = 0.;
			slope10_15 = 0.;
			slope15_20 = 0.;
			slope20_25 = 0.;
			slope25_30 = 0.;

			sideslope0_15 = 0.;
			sideslope15_30 = 0.;
			sideslope30_45 = 0.;
			sideslope45_60 = 0.;
			sideslope60_75 = 0.;
			sideslope75_90 = 0.;

			haulingcostperlanding = 0.;
			roadcostperlanding = 0.;

			if (NumberofLinksinPath <= 0)
			{
				AfxMessageBox("One of the sales could not be found.", MB_ICONEXCLAMATION);
				break;
			}
			else
			{
				for (j = 1; j <= NumberofLinksinPath; ++j)
				{
					beginNode = endNode;
					fscanf(infile_ptr, "%d", &tempNodeID);//right order, not in reverse order
					endNode = tempNodeID;
					
					//find link ID
					if (j >= 2)//from the second node
					{
						tempLinkID = pDoc->FindLinkNumberbetweenTwoNodes(beginNode, endNode);

						if (boolLinks[tempLinkID] == FALSE)
						{
							boolLinks[tempLinkID] = TRUE;
							
							haulingcostperlanding += pDoc->Links[tempLinkID].VariableCost * Volume;
							roadcostperlanding += pDoc->Links[tempLinkID].FixedCost;

	
							fprintf(outfile_ptr, "%d %f %f %f\n", j, pDoc->tempDist[tempLinkID], pDoc->tempSlope[tempLinkID], pDoc->tempSideSlope[tempLinkID]);
							totaldist += pDoc->tempDist[tempLinkID];
							if (pDoc->tempSlope[tempLinkID] >= .25)
								slope25_30 += pDoc->tempDist[tempLinkID];
							else if (pDoc->tempSlope[tempLinkID] >= .20)
								slope20_25 += pDoc->tempDist[tempLinkID];
							else if (pDoc->tempSlope[tempLinkID] >= .15)
								slope15_20 += pDoc->tempDist[tempLinkID];
							else if (pDoc->tempSlope[tempLinkID] >= .10)
								slope10_15 += pDoc->tempDist[tempLinkID];
							else if (pDoc->tempSlope[tempLinkID] >= .05)
								slope5_10 += pDoc->tempDist[tempLinkID];
							else
								slope0_5 += pDoc->tempDist[tempLinkID];

							if (pDoc->tempSideSlope[tempLinkID] >= .75)
								sideslope75_90 += pDoc->tempDist[tempLinkID];
							else if (pDoc->tempSlope[tempLinkID] >= .60)
								sideslope60_75 += pDoc->tempDist[tempLinkID];
							else if (pDoc->tempSlope[tempLinkID] >= .45)
								sideslope45_60 += pDoc->tempDist[tempLinkID];
							else if (pDoc->tempSlope[tempLinkID] >= .30)
								sideslope30_45 += pDoc->tempDist[tempLinkID];
							else if (pDoc->tempSlope[tempLinkID] >= .15)
								sideslope15_30 += pDoc->tempDist[tempLinkID];
							else
								sideslope0_15 += pDoc->tempDist[tempLinkID];
						}
					}

				}

				fprintf(outfile_ptr, "haulingcosts: %f\n", haulingcostperlanding);
				fprintf(outfile_ptr, "roadcosts: %f\n", roadcostperlanding);

				fprintf(outfile_ptr, "total dist: %f\n", totaldist);

				fprintf(outfile_ptr, "slope0-5: %f\n", slope0_5);
				fprintf(outfile_ptr, "slope5-10: %f\n", slope5_10);
				fprintf(outfile_ptr, "slope10-15: %f\n", slope10_15);
				fprintf(outfile_ptr, "slope15-20: %f\n", slope15_20);
				fprintf(outfile_ptr, "slope20-25: %f\n", slope20_25);
				fprintf(outfile_ptr, "slope25-30: %f\n", slope25_30);

				fprintf(outfile_ptr, "sideslope0-15: %f\n", sideslope0_15);
				fprintf(outfile_ptr, "sideslope15-30: %f\n", sideslope15_30);
				fprintf(outfile_ptr, "sideslope30-45: %f\n", sideslope30_45);
				fprintf(outfile_ptr, "sideslope45-60: %f\n", sideslope45_60);
				fprintf(outfile_ptr, "sideslope60-75: %f\n", sideslope60_75);
				fprintf(outfile_ptr, "sideslope75-90: %f\n", sideslope75_90);

				grandslope0_5 += slope0_5;
				grandslope5_10 += slope5_10;
				grandslope10_15 += slope10_15;
				grandslope15_20 += slope15_20;
				grandslope20_25 += slope20_25;
				grandslope25_30 += slope25_30;

				grandsideslope0_15 += sideslope0_15;
				grandsideslope15_30 += sideslope15_30;
				grandsideslope30_45 += sideslope30_45;
				grandsideslope45_60 += sideslope45_60;
				grandsideslope60_75 += sideslope60_75;
				grandsideslope75_90 += sideslope75_90;
			
				grandtotallength += totaldist;
				grandtotalhaulingcost += haulingcostperlanding;
				grandtotalroadcost += roadcostperlanding;

				grandtotalvolume += Volume;

				fprintf(outfile_ptr, "\n\n");
				fprintf(outfile_ptr, "Grand total volume: %f\n", grandtotalvolume);
				fprintf(outfile_ptr, "Grand total road length: %f\n", grandtotallength);
				fprintf(outfile_ptr, "grandslope0_5: %f\n", grandslope0_5);
				fprintf(outfile_ptr, "grandslope5_10: %f\n", grandslope5_10);
				fprintf(outfile_ptr, "grandslope10_15: %f\n", grandslope10_15);
				fprintf(outfile_ptr, "grandslope15_20: %f\n", grandslope15_20);
				fprintf(outfile_ptr, "grandslope20_25: %f\n", grandslope20_25);
				fprintf(outfile_ptr, "grandslope25_30: %f\n", grandslope25_30);

				fprintf(outfile_ptr, "grandsideslope0_15: %f\n", grandsideslope0_15);
				fprintf(outfile_ptr, "grandsideslope15_30: %f\n", grandsideslope15_30);
				fprintf(outfile_ptr, "grandsideslope30_45: %f\n", grandsideslope30_45);
				fprintf(outfile_ptr, "grandsideslope45_60: %f\n", grandsideslope45_60);
				fprintf(outfile_ptr, "grandsideslope60_75: %f\n", grandsideslope60_75);
				fprintf(outfile_ptr, "grandsideslope75_90: %f\n", grandsideslope75_90);


//				fprintf(outfile_ptr, "Grand total hauling cost: %f\n", grandtotalhaulingcost);
//				fprintf(outfile_ptr, "Grand total road cost: %f\n", grandtotalroadcost);
			}
		}
	}

	fclose(infile_ptr);
	fclose(outfile_ptr);

}

